@@ -2,6 +2,7 @@
 #define BOOLEXP_H
 #include "copyrite.h"
 #include "chunk.h"
+#include "dbio.h"
 
 typedef chunk_reference_t boolexp;
 
@@ -24,8 +25,8 @@ extern boolexp parse_boolexp(dbref player, const char *buf, lock_type ltype);
 extern boolexp parse_boolexp_d(dbref player, const char *buf, lock_type ltype,
                                int derefs);
 extern void free_boolexp(boolexp b);
-boolexp getboolexp(FILE * f, const char *ltype);
-void putboolexp(FILE * f, boolexp b);
+boolexp getboolexp(PENNFILE *f, const char *ltype);
+void putboolexp(PENNFILE *f, boolexp b);
 enum u_b_f {
   UB_ALL, /**< Use names of objects */
   UB_DBREF, /**< Use dbrefs */
