@@ -24,12 +24,12 @@ struct bufferq {
 #define BufferQLast(b) ((b)->last_string)
 #define BufferQLastType(b) ((b)->last_type)
 
-extern BUFFERQ *allocate_bufferq(int lines);
-extern BUFFERQ *reallocate_bufferq(BUFFERQ * bq, int lines);
-extern void free_bufferq(BUFFERQ * bq);
-extern void add_to_bufferq(BUFFERQ * bq, int type, dbref player,
-                           const char *msg);
-extern char *iter_bufferq(BUFFERQ * bq, char **p, dbref *player, int *type,
-                          time_t * timestamp);
-extern int bufferq_lines(BUFFERQ * bq);
-extern int isempty_bufferq(BUFFERQ * bq);
+BUFFERQ *allocate_bufferq(int lines);
+BUFFERQ *reallocate_bufferq(BUFFERQ *bq, int lines);
+void free_bufferq(BUFFERQ *bq);
+void add_to_bufferq(BUFFERQ *bq, int type, dbref player, const char *msg);
+char *iter_bufferq(BUFFERQ *bq, char **p, dbref *player, int *type,
+                   time_t * timestamp);
+int bufferq_lines(BUFFERQ *bq);
+int bufferq_blocks(BUFFERQ *bq);
+bool isempty_bufferq(BUFFERQ *bq);
