@@ -3,7 +3,7 @@
 #include <ctype.h>
 #include "config.h"
 
-#ifdef HAS_SAFE_TOUPPER
+#ifdef HAVE_SAFE_TOUPPER
 #define DOWNCASE(x)     tolower((unsigned char)x)
 #define UPCASE(x)       toupper((unsigned char)x)
 #else
