@@ -1,24 +1,24 @@
 /* Must be included after mushtype.h to get dbref typedef */
-
 #ifndef _CHUNK_H_
 #define _CHUNK_H_
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
 #undef LOG_CHUNK_STATS
 
-typedef unsigned short u_int_16;
-typedef unsigned int u_int_32;
-
-typedef u_int_32 chunk_reference_t;
+typedef uint32_t chunk_reference_t;
 #define NULL_CHUNK_REFERENCE 0
 
-chunk_reference_t chunk_create(unsigned char const *data, u_int_16 len,
-                               unsigned char derefs);
+chunk_reference_t chunk_create(unsigned char const *data, uint16_t len,
+                               uint8_t derefs);
 void chunk_delete(chunk_reference_t reference);
-u_int_16 chunk_fetch(chunk_reference_t reference,
-                     unsigned char *buffer, u_int_16 buffer_len);
-u_int_16 chunk_len(chunk_reference_t reference);
-unsigned char chunk_derefs(chunk_reference_t reference);
-void chunk_migration(int count, chunk_reference_t ** references);
+uint16_t chunk_fetch(chunk_reference_t reference,
+                     unsigned char *buffer, uint16_t buffer_len);
+uint16_t chunk_len(chunk_reference_t reference);
+uint8_t chunk_derefs(chunk_reference_t reference);
+void chunk_migration(int count, chunk_reference_t **references);
 int chunk_num_swapped(void);
 void chunk_init(void);
 enum chunk_stats_type { CSTATS_SUMMARY, CSTATS_REGIONG, CSTATS_PAGINGG,
