@@ -1,11 +1,13 @@
 /* conf.h */
-
-#ifndef __CONF_H
-#define __CONF_H
+#ifndef __PENNCONF_H
+#define __PENNCONF_H
 
 #include "copyrite.h"
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -146,6 +148,7 @@ struct options_table {
   int max_player_chans;         /**< Number of channels a player can create */
   int max_channels;             /**< Total maximum allowed channels */
   int chan_cost;                /**< Cost to create a channel */
+  int noisy_cemit;              /**< Is @cemit noisy by default? */
   char connect_file[2][256];    /**< Names of text and html connection files */
   char motd_file[2][256];       /**< Names of text and html motd files */
   char wizmotd_file[2][256];    /**< Names of text and html wizmotd files */
@@ -245,8 +248,8 @@ struct options_table {
   int player_name_len;  /**< Maximum length of player names */
   int queue_entry_cpu_time;     /**< Maximum cpu time allowed per queue entry */
   int ascii_names;      /**< Are object names restricted to ascii characters? */
-  int max_global_fns;   /**< Maximum number of functions */
   char chunk_swap_file[256];    /**< Name of the attribute swap file */
+  int chunk_swap_initial; /**< Disc space to reserve for the swap file, in kibibytes */
   int chunk_cache_memory;       /**< Memory to use for the attribute cache */
   int chunk_migrate_amount;     /**< Number of attrs to migrate each second */
   int read_remote_desc; /**< Can players read DESCRIBE attribute remotely? */
@@ -265,13 +268,11 @@ struct options_table {
   char command_log[256]; /**< File to log suspect commands */
   char trace_log[256]; /**< File to log trace data */
   char checkpt_log[256]; /**< File to log checkpoint data */
-#ifdef HAS_MYSQL
   char sql_platform[256]; /**< Type of SQL server, or "disabled" */
   char sql_host[256]; /**< Hostname of sql server */
   char sql_username[256]; /**< Username for sql */
   char sql_password[256]; /**< Password for sql */
   char sql_database[256]; /**< Database for sql */
-#endif
 };
 
 extern OPTTAB options;
@@ -281,17 +282,21 @@ extern PENNCONF *add_config(const char *name, config_func handler, void *loc,
                             int max, const char *group);
 extern PENNCONF *new_config(void);
 extern PENNCONF *get_config(const char *name);
+extern void validate_config(void);
+
 
 int cf_bool(const char *opt, const char *val, void *loc, int maxval,
-            int source);
-int cf_str(const char *opt, const char *val, void *loc, int maxval, int source);
-int cf_int(const char *opt, const char *val, void *loc, int maxval, int source);
+            int from_cmd);
+int cf_str(const char *opt, const char *val, void *loc, int maxval,
+           int from_cmd);
+int cf_int(const char *opt, const char *val, void *loc, int maxval,
+           int from_cmd);
 int cf_dbref(const char *opt, const char *val, void *loc, int maxval,
-             int source);
+             int from_cmd);
 int cf_flag(const char *opt, const char *val, void *loc, int maxval,
-            int source);
+            int from_cmd);
 int cf_time(const char *opt, const char *val, void *loc, int maxval,
-            int source);
+            int from_cmd);
 
 
 #define NUMQ    36
@@ -422,7 +427,6 @@ int cf_time(const char *opt, const char *val, void *loc, int maxval,
 #define TINY_MATH (options.tiny_math)
 #define NEWLINE_ONE_CHAR (options.newline_one_char)
 #define ONLY_ASCII_NAMES (options.ascii_names)
-#define MAX_GLOBAL_FNS (options.max_global_fns)
 #define USE_QUOTA (options.use_quota)
 #define EMPTY_ATTRS (options.empty_attrs)
 #define FUNCTION_SIDE_EFFECTS (options.function_side_effects)
@@ -432,13 +436,11 @@ int cf_time(const char *opt, const char *val, void *loc, int maxval,
 #define CMDLOG (options.command_log)
 #define TRACELOG (options.trace_log)
 #define CHECKLOG (options.checkpt_log)
-#ifdef HAS_MYSQL
 #define SQL_PLATFORM (options.sql_platform)
 #define SQL_HOST (options.sql_host)
 #define SQL_DB (options.sql_database)
 #define SQL_USER (options.sql_username)
 #define SQL_PASS (options.sql_password)
-#endif
 
 #define CHUNK_SWAP_FILE (options.chunk_swap_file)
 #define CHUNK_CACHE_MEMORY (options.chunk_cache_memory)
@@ -463,4 +465,4 @@ struct globals_table {
 
 extern GLOBALTAB globals;
 
-#endif                          /* __CONF_H */
+#endif                          /* __PENN_CONF_H */
