@@ -8,8 +8,8 @@
 #include "config.h"
 #include "confmagic.h"
 
-
 typedef struct priv_info PRIV;
+
 /** Privileges.
  * This structure represents a privilege and its associated data.
  * Privileges tables are used to provide a unified way to parse
@@ -18,8 +18,8 @@ typedef struct priv_info PRIV;
 struct priv_info {
   const char *name;     /**< Name of the privilege */
   char letter;          /**< One-letter abbreviation */
-  long int bits_to_set; /**< Bitflags required to set this privilege */
-  long int bits_to_show;        /**< Bitflags required to see this privilege */
+  privbits bits_to_set; /**< Bitflags required to set this privilege */
+  privbits bits_to_show;        /**< Bitflags required to see this privilege */
 };
 
 #define PrivName(x)     ((x)->name)
@@ -27,12 +27,12 @@ struct priv_info {
 #define PrivSetBits(x)  ((x)->bits_to_set)
 #define PrivShowBits(x) ((x)->bits_to_show)
 
-extern int string_to_privs(PRIV *table, const char *str, long int origprivs);
-extern int list_to_privs(PRIV *table, const char *str, long int origprivs);
-extern int string_to_privsets(PRIV *table, const char *str, int *setprivs,
-                              int *clrprivs);
-extern int letter_to_privs(PRIV *table, const char *str, long int origprivs);
-extern const char *privs_to_string(PRIV *table, int privs);
-extern const char *privs_to_letters(PRIV *table, int privs);
+privbits string_to_privs(PRIV *table, const char *str, privbits origprivs);
+privbits list_to_privs(PRIV *table, const char *str, privbits origprivs);
+int string_to_privsets(PRIV *table, const char *str, privbits *setprivs,
+                       privbits *clrprivs);
+privbits letter_to_privs(PRIV *table, const char *str, privbits origprivs);
+extern const char *privs_to_string(PRIV *table, privbits privs);
+extern const char *privs_to_letters(PRIV *table, privbits privs);
 
 #endif                          /* __PRIVTAB_H */
