@@ -9,10 +9,10 @@ struct ptab_entry;
  * data is looked up by the best matching prefix of the given key.
  */
 typedef struct ptab {
-  int state;                    /**< Internal table state */
-  int len;                      /**< Table size */
-  int maxlen;                   /**< Maximum table size */
-  int current;                  /**< Internal table state */
+  bool state;                   /**< Internal table state */
+  size_t len;                   /**< Table size */
+  size_t maxlen;                /**< Maximum table size */
+  size_t current;                       /**< Internal table state */
   struct ptab_entry **tab;      /**< Pointer to array of entries */
 } PTAB;
 
@@ -25,6 +25,7 @@ void ptab_delete(PTAB *, const char *);
 void ptab_start_inserts(PTAB *);
 void ptab_end_inserts(PTAB *);
 void ptab_insert(PTAB *, const char *, void *);
+void ptab_insert_one(PTAB *, const char *, void *);
 void ptab_stats_header(dbref);
 void ptab_stats(dbref, PTAB *, const char *);
 void *ptab_firstentry_new(PTAB *, char *key);
