@@ -1,11 +1,9 @@
 #ifndef _SHS_H
 #define _SHS_H
 
-#include "config.h"
-#ifdef I_STDINT
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
-#include "confmagic.h"
 
 /* -------------- SHS.H --------------------------- */
 
@@ -13,17 +11,8 @@
 #define LONG SHS_LONG
 #endif
 
-#ifdef __MINGW32__
-typedef unsigned char BYTE;
-typedef long LONG;
-#else
-typedef unsigned char BYTE;
-#ifdef HAS_UINT32_T
-typedef uint32_t LONG;
-#else
-typedef unsigned long LONG;
-#endif
-#endif
+typedef uint8_t BYTE;
+typedef uint32_t LONG;          /* Ick. */
 
 #define SHS_BLOCKSIZE 64
 
@@ -40,6 +29,6 @@ typedef struct {
 } SHS_INFO;
 
 void shsInit(SHS_INFO *shsInfo);
-void shsUpdate(SHS_INFO *shsInfo, const BYTE * buffer, int count);
+void shsUpdate(SHS_INFO *shsInfo, const BYTE *buffer, int count);
 void shsFinal(SHS_INFO *shsInfo);
 #endif
