@@ -36,6 +36,8 @@ char const *st_insert_perm(char const *s, StrTree *root);
 char const *st_find(char const *s, StrTree *root);
 void st_delete(char const *s, StrTree *root);
 void st_print(StrTree *root);
+typedef void (*STFunc) (const char *, int, void *);
+void st_walk(StrTree *, STFunc, void *);
 void st_flush(StrTree *root);
 
 extern long st_count;
