@@ -66,6 +66,9 @@
 #include <fcntl.h>
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -74,6 +77,7 @@
 #endif
 #include "conf.h"
 #include "externs.h"
+#include "mypcre.h"
 #include "access.h"
 #include "mymalloc.h"
 #include "match.h"
@@ -84,7 +88,6 @@
 #include "flags.h"
 #include "confmagic.h"
 
-
 /** An access flag. */
 typedef struct a_acsflag acsflag;
 /** An access flag.
@@ -92,8 +95,8 @@ typedef struct a_acsflag acsflag;
  */
 struct a_acsflag {
   const char *name;             /**< Name of the access flag */
-  int toggle;                   /**< Is this a negatable flag? */
-  int flag;                     /**< Bitmask of the flag */
+  bool toggle;                   /**< Is this a negatable flag? */
+  uint32_t flag;                     /**< Bitmask of the flag */
 };
 static acsflag acslist[] = {
   {"connect", 1, ACS_CONNECT},
@@ -111,31 +114,73 @@ static acsflag acslist[] = {
 };
 
 static struct access *access_top;
-static int add_access_node
-  (const char *host, const dbref who, const int can, const int cant,
-   const char *comment);
 static void free_access_list(void);
 
-static int
-add_access_node(const char *host, const dbref who, const int can,
-                const int cant, const char *comment)
-{
-  struct access *end;
-  struct access *tmp;
+extern const unsigned char *tables;
 
-  tmp = (struct access *) mush_malloc(sizeof(struct access), "struct_access");
-  if (!tmp)
-    return 0;
+static void
+sitelock_free(struct access *ap)
+{
+  mush_free(ap->host, "sitelock.rule.pattern");
+  if (ap->comment)
+    mush_free(ap->comment, "sitelock.rule.comment");
+  if (ap->re)
+    free(ap);
+  mush_free(ap, "sitelock.rule");
+}
+
+static struct access *
+sitelock_alloc(const char *host, dbref who,
+               uint32_t can, uint32_t cant,
+               const char *comment, const char **errptr)
+  __attribute_malloc__;
+
+    static struct access *sitelock_alloc(const char *host, dbref who,
+                                         uint32_t can, uint32_t cant,
+                                         const char *comment,
+                                         const char **errptr)
+{
+  struct access *tmp;
+  tmp = mush_malloc(sizeof(struct access), "sitelock.rule");
+  if (!tmp) {
+    static const char *memerr = "unable to allocate memory";
+    if (errptr)
+      *errptr = memerr;
+    return NULL;
+  }
   tmp->who = who;
   tmp->can = can;
   tmp->cant = cant;
-  strcpy(tmp->host, host);
-  if (comment)
-    strcpy(tmp->comment, comment);
+  tmp->host = mush_strdup(host, "sitelock.rule.pattern");
+  if (comment && *comment)
+    tmp->comment = mush_strdup(comment, "sitelock.rule.comment");
   else
-    tmp->comment[0] = '\0';
+    tmp->comment = NULL;
   tmp->next = NULL;
 
+  if (can & ACS_REGEXP) {
+    int erroffset = 0;
+    tmp->re = pcre_compile(host, 0, errptr, &erroffset, tables);
+    if (!tmp->re) {
+      sitelock_free(tmp);
+      return NULL;
+    }
+  } else
+    tmp->re = NULL;
+
+  return tmp;
+}
+
+static bool
+add_access_node(const char *host, dbref who, uint32_t can,
+                uint32_t cant, const char *comment, const char **errptr)
+{
+  struct access *end, *tmp;
+
+  tmp = sitelock_alloc(host, who, can, cant, comment, errptr);
+  if (!tmp)
+    return false;
+
   if (!access_top) {
     /* Add to the beginning */
     access_top = tmp;
@@ -145,25 +190,25 @@ add_access_node(const char *host, const dbref who, const int can,
       end = end->next;
     end->next = tmp;
   }
-
-  return 1;
+  return true;
 }
 
 
 /** Read the access.cnf file.
  * Initialize the access rules linked list and read in the access.cnf file.
- * Return 1 if successful, 0 if not
+ * \return true if successful, false if not
  */
-int
+bool
 read_access_file(void)
 {
   FILE *fp;
   char buf[BUFFER_LEN];
   char *p;
-  int can, cant;
+  uint32_t can, cant;
   int retval;
   dbref who;
   char *comment;
+  const char *errptr = NULL;
 
   if (access_top) {
     /* We're reloading the file, so we've got to delete any current 
@@ -176,7 +221,7 @@ read_access_file(void)
   release_fd();
   fp = fopen(ACCESS_FILE, FOPEN_READ);
   if (!fp) {
-    do_log(LT_ERR, GOD, GOD, T("No %s file found."), ACCESS_FILE);
+    do_rawlog(LT_ERR, T("Access file %s not found."), ACCESS_FILE);
     retval = 0;
   } else {
     do_rawlog(LT_ERR, "Reading %s", ACCESS_FILE);
@@ -195,8 +240,10 @@ read_access_file(void)
         comment = NULL;
         /* Is this the @sitelock entry? */
         if (!strncasecmp(p, "@sitelock", 9)) {
-          if (!add_access_node("@sitelock", AMBIGUOUS, ACS_SITELOCK, 0, ""))
-            do_log(LT_ERR, GOD, GOD, T("Failed to add sitelock node!"));
+          if (!add_access_node("@sitelock", AMBIGUOUS, ACS_SITELOCK, 0, "",
+                               &errptr))
+            do_log(LT_ERR, GOD, GOD, T("Failed to add sitelock node: %s"),
+                   errptr);
         } else {
           if ((comment = strchr(p, '#'))) {
             *comment++ = '\0';
@@ -211,8 +258,9 @@ read_access_file(void)
           if (!parse_access_options(p, &who, &can, &cant, NOTHING))
             /* Nothing listed, so assume we can't do anything! */
             cant = ACS_DEFAULT;
-          if (!add_access_node(buf, who, can, cant, comment))
-            do_log(LT_ERR, GOD, GOD, T("Failed to add access node!"));
+          if (!add_access_node(buf, who, can, cant, comment, &errptr))
+            do_log(LT_ERR, GOD, GOD, T("Failed to add access node: %s"),
+                   errptr);
         }
       }
     }
@@ -234,7 +282,7 @@ write_access_file(void)
   struct access *ap;
   acsflag *c;
 
-  sprintf(tmpf, "%s.tmp", ACCESS_FILE);
+  snprintf(tmpf, BUFFER_LEN, "%s.tmp", ACCESS_FILE);
   /* Be sure we have a file descriptor */
   release_fd();
   fp = fopen(tmpf, FOPEN_WRITE);
@@ -269,10 +317,10 @@ write_access_file(void)
             fprintf(fp, "!%s ", c->name);
         break;
       }
-      if (ap->comment && *ap->comment)
+      if (ap->comment)
         fprintf(fp, "# %s\n", ap->comment);
       else
-        fprintf(fp, "\n");
+        fputc('\n', fp);
     }
     fclose(fp);
     rename_file(tmpf, ACCESS_FILE);
@@ -314,12 +362,12 @@ ip4_to_ip6(const char *addr)
  *   flags (can't register, isn't suspect)
  * \endverbatim
  */
-int
-site_can_access(const char *hname, int flag, dbref who)
+bool
+site_can_access(const char *hname, uint32_t flag, dbref who)
 {
   struct access *ap;
   acsflag *c;
-  char *p;
+  const char *p;
 
   if (!hname || !*hname)
     return 0;
@@ -328,22 +376,13 @@ site_can_access(const char *hname, int flag, dbref who)
     p++;
 
   for (ap = access_top; ap; ap = ap->next) {
-    if (!(ap->can & ACS_SITELOCK)
-        && ((ap->can & ACS_REGEXP)
-            ? (quick_regexp_match(ap->host, hname, 0)
-               || (p && quick_regexp_match(ap->host, p, 0))
-#ifdef FORCE_IPV4
-               || quick_regexp_match(ip4_to_ip6(ap->host), hname, 0)
-               || (p && quick_regexp_match(ip4_to_ip6(ap->host), p, 0))
-#endif
-            )
-            : (quick_wild(ap->host, hname)
-               || (p && quick_wild(ap->host, p))
-#ifdef FORCE_IPV4
-               || quick_wild(ip4_to_ip6(ap->host), hname)
-               || (p && quick_wild(ip4_to_ip6(ap->host), p))
-#endif
-            ))
+    if (ap->can & ACS_SITELOCK)
+      continue;
+    if ((ap->can & ACS_REGEXP)
+        ? (qcomp_regexp_match(ap->re, hname)
+           || (p && qcomp_regexp_match(ap->re, p)))
+        : (quick_wild(ap->host, hname)
+           || (p && quick_wild(ap->host, p)))
         && (ap->who == AMBIGUOUS || ap->who == who)) {
       /* Got one */
       if (flag & ACS_CONNECT) {
@@ -387,7 +426,7 @@ struct access *
 site_check_access(const char *hname, dbref who, int *rulenum)
 {
   struct access *ap;
-  char *p;
+  const char *p;
 
   *rulenum = 0;
   if (!hname || !*hname)
@@ -398,22 +437,13 @@ site_check_access(const char *hname, dbref who, int *rulenum)
 
   for (ap = access_top; ap; ap = ap->next) {
     (*rulenum)++;
-    if (!(ap->can & ACS_SITELOCK)
-        && ((ap->can & ACS_REGEXP)
-            ? (quick_regexp_match(ap->host, hname, 0)
-               || (p && quick_regexp_match(ap->host, p, 0))
-#ifdef FORCE_IPV4
-               || quick_regexp_match(ip4_to_ip6(ap->host), hname, 0)
-               || (p && quick_regexp_match(ip4_to_ip6(ap->host), p, 0))
-#endif
-            )
-            : (quick_wild(ap->host, hname)
-               || (p && quick_wild(ap->host, p))
-#ifdef FORCE_IPV4
-               || quick_wild(ip4_to_ip6(ap->host), hname)
-               || (p && quick_wild(ip4_to_ip6(ap->host), p))
-#endif
-            ))
+    if (ap->can & ACS_SITELOCK)
+      continue;
+    if (((ap->can & ACS_REGEXP)
+         ? (qcomp_regexp_match(ap->re, hname)
+            || (p && qcomp_regexp_match(ap->re, p)))
+         : (quick_wild(ap->host, hname)
+            || (p && quick_wild(ap->host, p))))
         && (ap->who == AMBIGUOUS || ap->who == who)) {
       /* Got one */
       return ap;
@@ -431,7 +461,7 @@ site_check_access(const char *hname, dbref who, int *rulenum)
  * This function provides an appealing display of an access rule
  * in the list.
  */
-int
+void
 format_access(struct access *ap, int rulenum,
               dbref who __attribute__ ((__unused__)), char *buff, char **bp)
 {
@@ -472,7 +502,6 @@ format_access(struct access *ap, int rulenum,
   } else {
     safe_str(T("No matching access rule"), buff, bp);
   }
-  return 0;
 }
 
 
@@ -491,29 +520,28 @@ format_access(struct access *ap, int rulenum,
  * Build an appropriate comment based on the player and date
  * \endverbatim
  */
-int
-add_access_sitelock(dbref player, const char *host, dbref who, int can,
-                    int cant)
+bool
+add_access_sitelock(dbref player, const char *host, dbref who, uint32_t can,
+                    uint32_t cant)
 {
   struct access *end;
   struct access *tmp;
+  const char *errptr = NULL;
 
-  tmp = (struct access *) mush_malloc(sizeof(struct access), "struct_access");
-  if (!tmp)
-    return 0;
 
-  tmp->who = who;
-  tmp->can = can;
-  tmp->cant = cant;
-  strcpy(tmp->host, host);
-  sprintf(tmp->comment, "By %s(#%d) on %s", Name(player), player,
-          show_time(mudtime, 0));
-  tmp->next = NULL;
+  tmp = sitelock_alloc(host, who, can, cant, "", &errptr);
+
+  if (!tmp) {
+    notify_format(player, T("Unable to add sitelock entry: %s"), errptr);
+    return false;
+  }
 
   if (!access_top) {
     /* Add to the beginning, but first add a sitelock marker */
-    if (!add_access_node("@sitelock", AMBIGUOUS, ACS_SITELOCK, 0, ""))
+    if (!add_access_node("@sitelock", AMBIGUOUS, ACS_SITELOCK, 0, "", &errptr)) {
+      notify_format(player, T("Unable to add @sitelock separator: %s"), errptr);
       return 0;
+    }
     access_top->next = tmp;
   } else {
     end = access_top;
@@ -522,8 +550,12 @@ add_access_sitelock(dbref player, const char *host, dbref who, int can,
     /* Now, either we're at the sitelock or the end */
     if (end->can != ACS_SITELOCK) {
       /* We're at the end and there's no sitelock marker. Add one */
-      if (!add_access_node("@sitelock", AMBIGUOUS, ACS_SITELOCK, 0, ""))
+      if (!add_access_node("@sitelock", AMBIGUOUS, ACS_SITELOCK, 0, "",
+                           &errptr)) {
+        notify_format(player, T("Unable to add @sitelock separator: %s"),
+                      errptr);
         return 0;
+      }
       end = end->next;
     } else {
       /* We're in the middle, so be sure we keep the list linked */
@@ -537,35 +569,43 @@ add_access_sitelock(dbref player, const char *host, dbref who, int can,
 /** Remove an access rule from the linked list.
  * \param pattern access rule host pattern to match.
  * \return number of rule removed.
- * \verbatim
  * This function removes an access rule from the list.
  * Only rules that appear after the "@sitelock" rule can be
  * removed with this function.
- * \endverbatim
  */
 int
 remove_access_sitelock(const char *pattern)
 {
   struct access *ap, *next, *prev = NULL;
   int n = 0;
+  int rulenum = 0, deletethis = -1;
+
+  if (is_strict_integer(pattern))
+    deletethis = parse_integer(pattern);
 
   /* We only want to be able to delete entries added with @sitelock */
-  for (ap = access_top; ap; ap = ap->next)
+  for (ap = access_top; ap; ap = ap->next) {
+    rulenum++;
     if (strcmp(ap->host, "@sitelock") == 0) {
       prev = ap;
       ap = ap->next;
       break;
     }
+  }
 
   while (ap) {
+    rulenum++;
     next = ap->next;
-    if (strcasecmp(pattern, ap->host) == 0) {
+    if (deletethis == -1 ? (strcasecmp(pattern, ap->host) == 0)
+        : deletethis == rulenum) {
       n++;
-      mush_free(ap, "struct_access");
+      sitelock_free(ap);
       if (prev)
         prev->next = next;
       else
         access_top = next;
+      if (deletethis >= 0)
+        break;
     } else {
       prev = ap;
     }
@@ -583,7 +623,7 @@ free_access_list(void)
   ap = access_top;
   while (ap) {
     next = ap->next;
-    mush_free((Malloc_t) ap, "struct_access");
+    sitelock_free(ap);
     ap = next;
   }
   access_top = NULL;
@@ -624,7 +664,7 @@ do_list_access(dbref player)
       notify_format(player,
                     "%3d SITE: %-20s  DBREF: %-6s FLAGS:%s", rulenum,
                     ap->host, unparse_dbref(ap->who), flaglist);
-      notify_format(player, " COMMENT: %s", ap->comment);
+      notify_format(player, " COMMENT: %s", ap->comment ? ap->comment : "");
     } else {
       notify(player,
              T
@@ -632,6 +672,9 @@ do_list_access(dbref player)
     }
 
   }
+  if (rulenum == 0) {
+    notify(player, T("There are no access rules."));
+  }
 }
 
 /** Parse access options into fields.
@@ -646,8 +689,8 @@ do_list_access(dbref player)
  * This makes a copy of the options string, so it's not modified.
  */
 int
-parse_access_options(const char *opts, dbref *who, int *can, int *cant,
-                     dbref player)
+parse_access_options(const char *opts, dbref *who, uint32_t *can,
+                     uint32_t *cant, dbref player)
 {
   char myopts[BUFFER_LEN];
   char *p;
