@@ -39,10 +39,6 @@ StrTree atr_names;
 extern PRIV attr_privs_set[];
 extern PRIV attr_privs_view[];
 
-static int real_atr_clr(dbref thinking, char const *atr, dbref player,
-                        int we_are_wiping);
-
-
 /** A string to hold the name of a missing prefix branch, set by
  * can_write_attr_internal.  Again, gross and ugly.  Please fix.
  */
@@ -53,23 +49,17 @@ static char missing_name[ATTRIBUTE_NAME_LIMIT + 1];
 /** How many attributes go in a "page" of attribute memory? */
 #define ATTRS_PER_PAGE (200)
 
-/** A page of memory for attributes.
- * This structure is a collection of attribute memory. Rather than
- * allocate new attributes one at a time, we allocate them in pages,
- * and build a linked free list from the allocated page.
- */
-typedef struct atrpage {
-  ATTR atrs[ATTRS_PER_PAGE];    /**< Array of attribute structures */
-} ATTRPAGE;
+slab *attrib_slab = NULL;
 
-static ATTR *atr_free_list = NULL;
-static ATTR *alloc_atr(void);
-static ATTR *pop_free_list(void);
-static void push_free_list(ATTR *);
+static int real_atr_clr(dbref thinking, char const *atr, dbref player,
+                        int we_are_wiping);
+
+static ATTR *alloc_atr(const void *hint);
+static void free_atr(ATTR *);
 static void atr_free_one(ATTR *);
 static ATTR *find_atr_pos_in_list(ATTR ***pos, char const *name);
-static atr_err can_create_attr(dbref player, dbref obj, char const *atr_name,
-                               unsigned int flags);
+static int can_create_attr(dbref player, dbref obj, char const *atr_name,
+                           uint32_t flags);
 static ATTR *find_atr_in_list(ATTR *atr, char const *name);
 static ATTR *atr_get_with_parent(dbref obj, char const *atrname, dbref *parent);
 
@@ -149,13 +139,68 @@ atr_sub_branch(ATTR *branch)
 
 /** Scan an attribute list for an attribute with the specified name.
  * This continues from whatever start point is passed in.
- * \param atr the start of the list to search from
- * \param name the attribute name to look for
- * \return the matching attribute, or NULL
+ * 
+ * Attributes are stored as sorted linked lists. This means that
+ * search is O(N) in the worst case. An unsuccessful search is usually
+ * better than that, because we don't have to look at every attribute
+ * unless you're looking for something greater than all the attributes
+ * on the object. There are a couple of possibilities I've been
+ * mulling over for... um... years... about ways to improve this.
+ *
+ * Option 1 is to change the data structure. I'd use a hybrid between
+ * a standard linked list and a skip list. Most objects have under 5
+ * attributes on them. With this few, a linear linked list is
+ * fine. When more attribute are added, though, it would turn into a
+ * skip list, with all current attributes having a depth of 1, and
+ * further attributes having a randomly chosen depth with a cap of,
+ * say, 5 (I'll have to work out the math to find the optimum
+ * size). This will provide O(lg N) searches on objects with lots of
+ * attributes and yet not take up lots of extra memory on objects with
+ * only a few attributes -- a problem with using a tree structure. All
+ * the code for this is in my head; I just need to sit down and write
+ * it.
+ *
+ * Option 2 is to speed up the current lookup. There are a lot of
+ * string comparisions that we don't strictly need: All attributes
+ * with the same name use the same underlying storage from a string
+ * pool. You can look up an attribute name and then just compare
+ * pointers, saving a lot of calls to strcoll(). The string pool is
+ * implemented using a red-black tree, so it needs O(lg P) string
+ * comparisions + O(N) pointer equality comparisions (P is the number
+ * of unique attribute names in the pool, N the number of attributes
+ * on the object). Hmm. I'm not so sure that's much of an improvement
+ * after all...  Let's try it out anyways and see what happens. Don't
+ * expect this to be permanent, though.
+ *
+ * \param atr the start of the list to search from \param name the
+ * attribute name to look for \return the matching attribute, or NULL
  */
 static ATTR *
 find_atr_in_list(ATTR *atr, char const *name)
 {
+#define ATR_PTR_CMP
+#ifdef ATR_PTR_CMP
+  /* New way; pointer comparisions */
+  const char *memoized;
+
+  memoized = st_find(name, &atr_names);
+  if (!memoized)                /* This attribute name doesn't exist on any object */
+    return NULL;
+
+  while (atr) {
+    if (AL_NAME(atr) == memoized)
+      return atr;
+#if 0
+    /* Unfortunately, this will break under many locales, since
+       attribute list sorting uses strcoll() and not strcmp().  */
+    else if (*memoized < *AL_NAME(atr))
+      return NULL;              /* Can't be any of the remaining attributes */
+#endif
+    else
+      atr = AL_NEXT(atr);
+  }
+#else
+  /* Old way; lots of string comparisions */
   int comp;
 
   while (atr) {
@@ -166,11 +211,12 @@ find_atr_in_list(ATTR *atr, char const *name)
       return atr;
     atr = AL_NEXT(atr);
   }
+#endif
 
   return NULL;
 }
 
-/** Find the place to insert an attribute with the specified name.
+/** Find the place to insert/delete an attribute with the specified name.
  * \param pos a pointer to the ATTR ** holding the list position
  * \param name the attribute name to look for
  * \return the matching attribute, or NULL if no matching attribute
@@ -198,12 +244,13 @@ find_atr_pos_in_list(ATTR ***pos, char const *name)
  * all of those flags.
  * \param player the dbref to use for privilege checks.
  * \param p a space-separated string of attribute flags.
- * \return an attribute flag bitmask.
+ * \param bits pointer to a privbits to store the attribute flags mask.
+ * \return 0 on success, -1 on error
  */
 int
-string_to_atrflag(dbref player, char const *p)
+string_to_atrflag(dbref player, char const *p, privbits *bits)
 {
-  int f;
+  privbits f;
   f = string_to_privs(attr_privs_set, p, 0);
   if (!f)
     return -1;
@@ -212,7 +259,8 @@ string_to_atrflag(dbref player, char const *p)
   if (!See_All(player) && (f & AF_WIZARD))
     return -1;
   f &= ~AF_INTERNAL;
-  return f;
+  *bits = f;
+  return 0;
 }
 
 /** Convert a string of attribute flags to a pair of bitmasks.
@@ -223,10 +271,11 @@ string_to_atrflag(dbref player, char const *p)
  * \param p a space-separated string of attribute flags.
  * \param setbits pointer to address of bitmask to set.
  * \param clrbits pointer to address of bitmask to clear.
- * \return setbits or -1 on error.
+ * \return 0 on success or -1 on error.
  */
 int
-string_to_atrflagsets(dbref player, char const *p, int *setbits, int *clrbits)
+string_to_atrflagsets(dbref player, char const *p, privbits *setbits,
+                      privbits *clrbits)
 {
   int f;
   *setbits = *clrbits = 0;
@@ -237,8 +286,7 @@ string_to_atrflagsets(dbref player, char const *p, int *setbits, int *clrbits)
     return -1;
   if (!See_All(player) && ((*setbits & AF_WIZARD) || (*clrbits & AF_WIZARD)))
     return -1;
-  f &= ~AF_INTERNAL;
-  return *setbits;
+  return 0;
 }
 
 /** Convert an attribute flag bitmask into a list of the full
@@ -247,7 +295,7 @@ string_to_atrflagsets(dbref player, char const *p, int *setbits, int *clrbits)
  * \return a pointer to a static buffer with the full names of the flags.
  */
 const char *
-atrflag_to_string(int mask)
+atrflag_to_string(privbits mask)
 {
   return privs_to_string(attr_privs_view, mask);
 }
@@ -397,14 +445,32 @@ can_write_attr_internal(dbref player, dbref obj, ATTR *atr, int safe)
   return 1;
 }
 
+/** If the attribute exists on the object, see if the player can modify it.
+ * Otherwise, see if they can create it.
+ * \param player the player trying to set the attr
+ * \param thing the object to check
+ * \param attrname the name of the attribute to check
+ * \retval 0 attribute cannot be changed by player
+ * \retval 1 attribute can be changed by player
+ */
+bool
+can_edit_attr(dbref player, dbref thing, const char *attrname)
+{
+  ATTR *ptr = find_atr_in_list(List(thing), attrname);
+  if (ptr)
+    return Can_Write_Attr(player, thing, ptr);
+  else
+    return can_create_attr(player, thing, attrname, 0) == AE_OKAY;
+}
+
 /** Utility define for atr_add and can_create_attr */
 #define set_default_flags(atr,flags) \
   do { \
     ATTR *std = atr_match(AL_NAME((atr))); \
-    if (std && !(flags) && !strcmp(AL_NAME(std), AL_NAME((atr)))) {     \
-      AL_FLAGS(atr) = AL_FLAGS(std); \
+    if (std && !strcmp(AL_NAME(std), AL_NAME((atr)))) {	\
+      AL_FLAGS(atr) = AL_FLAGS(std) | flags; \
     } else { \
-      AL_FLAGS(atr) = flags; \
+        AL_FLAGS(atr) = flags; \
     } \
   } while (0)
 
@@ -415,14 +481,12 @@ can_write_attr_internal(dbref player, dbref obj, ATTR *atr, int safe)
  * \param player the player trying to do the write.
  * \param obj the object targetted for the write.
  * \param atr the attribute being interrogated.
- * \param flags the default flags to add to the attribute. 
- *              0 for default flags if it's a builtin attribute.
+ * \param flags the default flags to add to the attribute.
  * \retval 0 if the player cannot write the attribute.
  * \retval 1 if the player can write the attribute.
  */
 static int
-can_create_attr(dbref player, dbref obj, char const *atr_name,
-                unsigned int flags)
+can_create_attr(dbref player, dbref obj, char const *atr_name, uint32_t flags)
 {
   char *p;
   ATTR tmpatr, *atr;
@@ -484,7 +548,7 @@ can_create_attr(dbref player, dbref obj, char const *atr_name,
  * \param atr_name the name for the attribute
  */
 static ATTR *
-create_atr(dbref thing, char const *atr_name)
+create_atr(dbref thing, char const *atr_name, const ATTR *hint)
 {
   ATTR *ptr, **ins;
   char const *name;
@@ -495,7 +559,7 @@ create_atr(dbref thing, char const *atr_name)
     return NULL;
 
   /* allocate a new page, if needed */
-  ptr = pop_free_list();
+  ptr = alloc_atr(hint);
   if (ptr == NULL) {
     st_delete(name, &atr_names);
     return NULL;
@@ -530,7 +594,7 @@ create_atr(dbref thing, char const *atr_name)
  */
 void
 atr_new_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
-            dbref player, unsigned int flags, unsigned char derefs)
+            dbref player, uint32_t flags, uint8_t derefs)
 {
   ATTR *ptr;
   char *p, root_name[ATTRIBUTE_NAME_LIMIT + 1];
@@ -543,7 +607,7 @@ atr_new_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
     do_rawlog(LT_ERR, T("Bad attribute name %s on object %s"), atr,
               unparse_dbref(thing));
 
-  ptr = create_atr(thing, atr);
+  ptr = create_atr(thing, atr, List(thing));
   if (!ptr)
     return;
 
@@ -555,7 +619,7 @@ atr_new_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
     if (!root) {
       do_rawlog(LT_ERR, T("Missing root attribute '%s' on object #%d!\n"),
                 root_name, thing);
-      root = create_atr(thing, root_name);
+      root = create_atr(thing, root_name, ptr);
       set_default_flags(root, 0);
       AL_FLAGS(root) |= AF_ROOT;
       AL_CREATOR(root) = player;
@@ -568,7 +632,7 @@ atr_new_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
         free(t);
       }
     } else {
-      if (!AL_FLAGS(root) & AF_ROOT)    /* Upgrading old database */
+      if (!(AL_FLAGS(root) & AF_ROOT))  /* Upgrading old database */
         AL_FLAGS(root) |= AF_ROOT;
     }
   }
@@ -611,7 +675,7 @@ atr_new_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
  */
 atr_err
 atr_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
-        dbref player, unsigned int flags)
+        dbref player, uint32_t flags)
 {
   ATTR *ptr, *root = NULL;
   char *p;
@@ -645,7 +709,7 @@ atr_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
       root = find_atr_in_list(ptr, missing_name);
 
       if (!root) {
-        root = create_atr(thing, missing_name);
+        root = create_atr(thing, missing_name, ptr);
         if (!root)
           return AE_TREE;
 
@@ -671,7 +735,7 @@ atr_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
       *p = '`';
     }
 
-    ptr = create_atr(thing, atr);
+    ptr = create_atr(thing, atr, root ? root : List(thing));
     if (!ptr)
       return AE_ERROR;
 
@@ -710,7 +774,6 @@ atr_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
   return AE_OKAY;
 }
 
-
 /** Remove all child attributes from root attribute that can be.
  * \param player object doing a @wipe.
  * \param thing object being @wiped.
@@ -993,7 +1056,7 @@ atr_iter_get(dbref player, dbref thing, const char *name, int mortal,
 {
   ATTR *ptr, **indirect;
   int result;
-  int len;
+  size_t len;
 
   result = 0;
   if (!name || !*name)
@@ -1036,7 +1099,7 @@ atr_pattern_count(dbref player, dbref thing, const char *name,
 {
   ATTR *ptr, **indirect;
   int result;
-  int len;
+  size_t len;
   dbref parent = NOTHING;
 
   result = 0;
@@ -1101,7 +1164,7 @@ atr_iter_get_parent(dbref player, dbref thing, const char *name, int mortal,
 {
   ATTR *ptr, **indirect;
   int result;
-  int len;
+  size_t len;
   dbref parent = NOTHING;
 
   result = 0;
@@ -1162,7 +1225,12 @@ atr_free_all(dbref thing)
 
   while ((ptr = List(thing))) {
     List(thing) = AL_NEXT(ptr);
-    atr_free_one(ptr);
+
+    if (ptr->data)
+      chunk_delete(ptr->data);
+    st_delete(AL_NAME(ptr), &atr_names);
+
+    free_atr(ptr);
   }
 }
 
@@ -1187,7 +1255,6 @@ atr_cpy(dbref dest, dbref source)
         && (AttrCount(dest) < max_attrs)) {
       atr_new_add(dest, AL_NAME(ptr), atr_value(ptr),
                   AL_CREATOR(ptr), AL_FLAGS(ptr), AL_DEREFS(ptr));
-      AttrCount(dest)++;
     }
 }
 
@@ -1216,7 +1283,7 @@ typedef struct used_attr {
  *           **prev now points to the entry for it.
  */
 static int
-find_attr(UsedAttr *** prev, char const *name)
+find_attr(UsedAttr ***prev, char const *name)
 {
   int comp;
 
@@ -1240,7 +1307,7 @@ find_attr(UsedAttr *** prev, char const *name)
  *         the one inserted.
  */
 static UsedAttr **
-use_attr(UsedAttr ** prev, char const *name, int no_prog)
+use_attr(UsedAttr **prev, char const *name, uint32_t no_prog)
 {
   int found;
   UsedAttr *used;
@@ -1281,9 +1348,10 @@ use_attr(UsedAttr ** prev, char const *name, int no_prog)
  */
 int
 atr_comm_match(dbref thing, dbref player, int type, int end, char const *str,
-               int just_match, char *atrname, char **abp, dbref *errobj)
+               int just_match, int check_locks,
+               char *atrname, char **abp, dbref *errobj)
 {
-  int flag_mask;
+  uint32_t flag_mask;
   ATTR *ptr;
   int parent_depth;
   char tbuf1[BUFFER_LEN];
@@ -1294,13 +1362,15 @@ atr_comm_match(dbref thing, dbref player, int type, int end, char const *str,
   UsedAttr *used_list, **prev;
   ATTR *skip[ATTRIBUTE_NAME_LIMIT / 2];
   int skipcount;
-  int lock_checked = 0;
+  int lock_checked = !check_locks;
   char match_space[BUFFER_LEN * 2];
-  int match_space_len = BUFFER_LEN * 2;
+  ssize_t match_space_len = BUFFER_LEN * 2;
 
   /* check for lots of easy ways out */
-  if ((type != '$' && type != '^') || !GoodObject(thing) || Halted(thing)
-      || (type == '$' && NoCommand(thing)))
+  if (type != '$' && type != '^')
+    return 0;
+  if (check_locks && (!GoodObject(thing) || Halted(thing)
+                      || (type == '$' && NoCommand(thing))))
     return 0;
 
   if (type == '$') {
@@ -1504,8 +1574,8 @@ atr_comm_match(dbref thing, dbref player, int type, int end, char const *str,
           match++;
           if (!just_match)
             wild_match_case_r(tbuf2 + 1, str, AF_Case(ptr),
-                              global_eval_context.wnxt, 10,
-                              match_space, match_space_len);
+                              global_eval_context.wnxt, 10, match_space,
+                              match_space_len);
         }
       }
       if (match_found) {
@@ -1571,7 +1641,7 @@ one_comm_match(dbref thing, dbref player, const char *atr, const char *str)
   char tbuf2[BUFFER_LEN];
   char *s;
   char match_space[BUFFER_LEN * 2];
-  int match_space_len = BUFFER_LEN * 2;
+  ssize_t match_space_len = BUFFER_LEN * 2;
 
   /* check for lots of easy ways out */
   if (!GoodObject(thing) || Halted(thing) || NoCommand(thing))
@@ -1606,14 +1676,10 @@ one_comm_match(dbref thing, dbref player, const char *atr, const char *str)
 
   if (AF_Regexp(ptr) ?
       regexp_match_case_r(tbuf2 + 1, str, AF_Case(ptr),
-                          global_eval_context.wnxt, 10, match_space,
-                          match_space_len) : wild_match_case_r(tbuf2 + 1, str,
-                                                               AF_Case(ptr),
-                                                               global_eval_context.
-                                                               wnxt, 10,
-                                                               match_space,
-                                                               match_space_len))
-  {
+                          global_eval_context.wnxt, 10,
+                          match_space, match_space_len) :
+      wild_match_case_r(tbuf2 + 1, str, AF_Case(ptr), global_eval_context.wnxt,
+                        10, match_space, match_space_len)) {
     if (!eval_lock(player, thing, Command_Lock)
         || !eval_lock(player, thing, Use_Lock))
       return 0;
@@ -1643,7 +1709,7 @@ one_comm_match(dbref thing, dbref player, const char *atr, const char *str)
  */
 int
 do_set_atr(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
-           dbref player, unsigned int flags)
+           dbref player, uint32_t flags)
 {
   ATTR *old;
   char name[BUFFER_LEN];
@@ -1744,9 +1810,7 @@ do_set_atr(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
 
   was_hearer = Hearer(thing);
   was_listener = Listener(thing);
-  res =
-    s ? atr_add(thing, name, s, player,
-                (flags & 0x02) ? AF_NOPROG : AF_EMPTY_FLAGS)
+  res = s ? atr_add(thing, name, s, player, (flags & 0x02) ? AF_NOPROG : 0)
     : atr_clr(thing, name, player);
   switch (res) {
   case AE_SAFE:
@@ -1768,7 +1832,7 @@ do_set_atr(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
       return 0;
     }
   case AE_BADNAME:
-    notify_format(player, T("That's not a very good name for an attribute."));
+    notify(player, T("That's not a very good name for an attribute."));
     return 0;
   case AE_ERROR:
     if (*missing_name) {
@@ -1833,10 +1897,10 @@ do_set_atr(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
  * \param arg2 the desired lock status ('on' or 'off').
  */
 void
-do_atrlock(dbref player, const char *arg1, const char *arg2)
+do_atrlock(dbref player, const char *xarg1, const char *arg2)
 {
   dbref thing;
-  char *p;
+  char *p, *arg1;
   ATTR *ptr;
   int status;
   if (!arg2 || !*arg2)
@@ -1850,20 +1914,27 @@ do_atrlock(dbref player, const char *arg1, const char *arg2)
       status = 0;
   }
 
-  if (!arg1 || !*arg1) {
+  if (!xarg1 || !*xarg1) {
     notify(player, T("You need to give an object/attribute pair."));
     return;
   }
+
+  arg1 = mush_strdup(xarg1, "atrlock.string");
+
   if (!(p = strchr(arg1, '/')) || !(*(p + 1))) {
     notify(player, T("You need to give an object/attribute pair."));
+    mush_free(arg1, "atrlock.string");
     return;
   }
   *p++ = '\0';
   if ((thing = noisy_match_result(player, arg1, NOTYPE, MAT_EVERYTHING)) ==
-      NOTHING)
+      NOTHING) {
+    mush_free(arg1, "atrlock.string");
     return;
+  }
   if (!controls(player, thing)) {
     notify(player, T("Permission denied."));
+    mush_free(arg1, "atrlock.string");
     return;
   }
 
@@ -1872,29 +1943,34 @@ do_atrlock(dbref player, const char *arg1, const char *arg2)
     if (!status) {
       notify_format(player, T("That attribute is %slocked."),
                     AF_Locked(ptr) ? "" : "un");
+      mush_free(arg1, "atrlock.string");
       return;
     } else if (!Can_Write_Attr(player, thing, ptr)) {
       notify(player,
              T("You need to be able to set the attribute to change its lock."));
+      mush_free(arg1, "atrlock.string");
       return;
     } else {
       if (status == 1) {
         AL_FLAGS(ptr) |= AF_LOCKED;
         AL_CREATOR(ptr) = Owner(player);
         notify(player, T("Attribute locked."));
+        mush_free(arg1, "atrlock.string");
         return;
       } else if (status == 2) {
         AL_FLAGS(ptr) &= ~AF_LOCKED;
         notify(player, T("Attribute unlocked."));
+        mush_free(arg1, "atrlock.string");
         return;
       } else {
         notify(player, T("Invalid status on atrlock.. Notify god."));
+        mush_free(arg1, "atrlock.string");
         return;
       }
     }
   } else
     notify(player, T("No such attribute."));
-  return;
+  mush_free(arg1, "atrlock.string");
 }
 
 /** Change ownership of an attribute.
@@ -1906,25 +1982,32 @@ do_atrlock(dbref player, const char *arg1, const char *arg2)
  * \param arg2 the name of the new owner (or "me").
  */
 void
-do_atrchown(dbref player, const char *arg1, const char *arg2)
+do_atrchown(dbref player, const char *xarg1, const char *arg2)
 {
   dbref thing, new_owner;
-  char *p;
+  char *p, *arg1;
   ATTR *ptr;
-  if (!arg1 || !*arg1) {
+  if (!xarg1 || !*xarg1) {
     notify(player, T("You need to give an object/attribute pair."));
     return;
   }
+
+  arg1 = mush_strdup(xarg1, "atrchown.string");
+
   if (!(p = strchr(arg1, '/')) || !(*(p + 1))) {
     notify(player, T("You need to give an object/attribute pair."));
+    mush_free(arg1, "atrchown.string");
     return;
   }
   *p++ = '\0';
   if ((thing = noisy_match_result(player, arg1, NOTYPE, MAT_EVERYTHING)) ==
-      NOTHING)
+      NOTHING) {
+    mush_free(arg1, "atrchown.string");
     return;
+  }
   if (!controls(player, thing)) {
     notify(player, T("Permission denied."));
+    mush_free(arg1, "atrchown.string");
     return;
   }
 
@@ -1934,6 +2017,7 @@ do_atrchown(dbref player, const char *arg1, const char *arg2)
     new_owner = lookup_player(arg2);
   if (new_owner == NOTHING) {
     notify(player, T("I can't find that player"));
+    mush_free(arg1, "atrchown.string");
     return;
   }
 
@@ -1942,68 +2026,48 @@ do_atrchown(dbref player, const char *arg1, const char *arg2)
     if (Can_Write_Attr(player, thing, ptr)) {
       if (new_owner != Owner(player) && !Wizard(player)) {
         notify(player, T("You can only chown an attribute to yourself."));
+        mush_free(arg1, "atrchown.string");
         return;
       }
       AL_CREATOR(ptr) = Owner(new_owner);
       notify(player, T("Attribute owner changed."));
+      mush_free(arg1, "atrchown.string");
       return;
     } else {
       notify(player, T("You don't have the permission to chown that."));
+      mush_free(arg1, "atrchown.string");
       return;
     }
   } else
     notify(player, T("No such attribute."));
+  mush_free(arg1, "atrchown.string");
 }
 
 
-/** Return the head of the attribute free list.
- * This function returns the head of the attribute free list. If there
- * are no more ATTR's on the free list, allocate a new page.
+/** Allocate a new ATTR from a slab allocator.
  * \return the pointer to the head of the attribute free list.
  */
 static ATTR *
-alloc_atr(void)
+alloc_atr(const void *hint)
 {
-  if (!atr_free_list) {
-    ATTRPAGE *page;
-    int j;
-    page = (ATTRPAGE *) mush_malloc(sizeof(ATTRPAGE), "ATTRPAGE");
-    if (!page)
-      mush_panic("Couldn't allocate memory in alloc_attr");
-    for (j = 0; j < ATTRS_PER_PAGE - 1; j++)
-      AL_NEXT(page->atrs + j) = page->atrs + j + 1;
-    AL_NEXT(page->atrs + ATTRS_PER_PAGE - 1) = NULL;
-    atr_free_list = page->atrs;
+  if (!attrib_slab) {
+    attrib_slab = slab_create("attributes", sizeof(ATTR));
+    slab_set_opt(attrib_slab, SLAB_ALLOC_BEST_FIT, 1);
+    slab_set_opt(attrib_slab, SLAB_HINTLESS_THRESHOLD, 10);
   }
-  return atr_free_list;
+
+  return slab_malloc(attrib_slab, hint);
 }
 
-/** Pop an empty attribute off of the free list for use on
- * an object.
- * \return the pointer to an attribute, or NULL on error.
- */
-static ATTR *
-pop_free_list(void)
-{
-  ATTR *ptr;
-  ptr = alloc_atr();
-  if (!ptr)
-    return NULL;
-  atr_free_list = AL_NEXT(ptr);
-  AL_NEXT(ptr) = NULL;
-  return ptr;
-}
 
-/** Push a now-unused attribute onto the free list
+/** Free an unused attribute struct.
  * \param An attribute that's been deleted from an object and
  * had its chunk reference deleted.
  */
 static void
-push_free_list(ATTR *a)
+free_atr(ATTR *a)
 {
-  memset(a, 0, sizeof(*a));
-  AL_NEXT(a) = atr_free_list;
-  atr_free_list = a;
+  slab_free(attrib_slab, a);
 }
 
 /** Delete one attribute, deallocating its name and data.
@@ -2020,7 +2084,7 @@ atr_free_one(ATTR *a)
   st_delete(AL_NAME(a), &atr_names);
   if (a->data)
     chunk_delete(a->data);
-  push_free_list(a);
+  free_atr(a);
 }
 
 /** Return the compressed data for an attribute.
@@ -2032,12 +2096,13 @@ unsigned char const *
 atr_get_compressed_data(ATTR *atr)
 {
   static unsigned char buffer[BUFFER_LEN * 2];
-  unsigned int len;
+  static unsigned char const empty_string[] = { 0 };
+  size_t len;
   if (!atr->data)
-    return (unsigned char *) "";
+    return empty_string;
   len = chunk_fetch(atr->data, buffer, sizeof(buffer));
   if (len > sizeof(buffer))
-    return (unsigned char *) "";
+    return empty_string;
   buffer[len] = '\0';
   return buffer;
 }
