@@ -21,7 +21,7 @@
  * boolexp. Now, it turns the parse tree into bytecode that can be
  * stored in the chunk manager. It probably also evaluates faster, but
  * no profiling has been done to support this claim. It certainly
- * involves less non-tail recursion.
+ * involves less non-tail recursion and better cache behavior.
  *
  * It's a three-stage process. First, the lock string is turned into a
  * parse tree. Second, the tree is walked and "assembler" instructions
@@ -77,6 +77,10 @@
  * the logic when ands, ors and nots are being used together. For
  * example, !a|!b can become !(a&b).
  *
+ * The only optimization done right now is thread jumping: If a jump
+ * would move the program counter to another jump operation, it instead
+ * goes to that jump's destination.
+ * 
  * There's more useful room for improvement in the lock
  * @warnings. Checking things like flag and power keys for valid flags
  * comes to mind.
@@ -88,8 +92,11 @@
 
 #include <ctype.h>
 #include <string.h>
-
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 #include "conf.h"
+#include "dbdefs.h"
 #include "mushdb.h"
 #include "match.h"
 #include "externs.h"
@@ -97,10 +104,10 @@
 #include "parse.h"
 #include "attrib.h"
 #include "flags.h"
-#include "dbdefs.h"
 #include "log.h"
 #include "extchat.h"
 #include "strtree.h"
+#include "mymalloc.h"
 #include "confmagic.h"
 
 #ifdef WIN32
@@ -212,7 +219,7 @@ struct bvm_asmnode {
  * the bytecode. The nodes are part of a linked list.  */
 struct bvm_strnode {
   char *s; /**< The string */
-  int len; /**< Its length */
+  size_t len; /**< Its length */
   struct bvm_strnode *next; /**< Pointer to the next node */
 };
 
@@ -224,7 +231,7 @@ struct bvm_asm {
   struct bvm_strnode *shead; /**< The start of the list of strings */
   struct bvm_strnode *stail; /**< The end of the list */
   int label; /**< The current label id to use */
-  int strcount; /**< The number of nodes in the string list */
+  size_t strcount;      /**< The number of nodes in the string list */
 };
 
 /** The flag lock key (A^B) only allows a few values for A. This
@@ -245,14 +252,14 @@ static struct flag_lock_types flag_locks[] = {
   {"OBJID", OP_TOBJID},
   {"IP", OP_TIP},
   {"HOSTNAME", OP_THOSTNAME},
-  {"DBREFLIST", OP_TDBREFLIST}, {
-                                 NULL, 0}
+  {"DBREFLIST", OP_TDBREFLIST},
+  {NULL, OP_RET}
 };
 
-static unsigned char *
+static uint8_t *
 safe_get_bytecode(boolexp b)
   __attribute_malloc__;
-    static unsigned char *get_bytecode(boolexp b, u_int_16 * storelen);
+    static uint8_t *get_bytecode(boolexp b, uint16_t * storelen);
     static struct boolexp_node *alloc_bool(void) __attribute_malloc__;
     static struct boolatr *alloc_atr(const char *name,
                                      const char *s) __attribute_malloc__;
@@ -278,8 +285,8 @@ safe_get_bytecode(boolexp b)
  __attribute_malloc__;
     static void generate_bvm_asm1(struct bvm_asm *a, struct boolexp_node *b,
                                   boolexp_type outer);
-    static int pos_of_label(struct bvm_asm *a, int label);
-    static int offset_to_string(struct bvm_asm *a, int c);
+    static size_t pos_of_label(struct bvm_asm *a, int label);
+    static size_t offset_to_string(struct bvm_asm *a, int c);
     static struct bvm_asmnode *insn_after_label(struct bvm_asm *a, int label);
     static void opt_thread_jumps(struct bvm_asm *a);
     static void optimize_bvm_asm(struct bvm_asm *a);
@@ -311,10 +318,10 @@ safe_get_bytecode(boolexp b)
  * \param b the boolexp to retrieve.
  * \return a malloced copy of the bytecode.
  */
-    static unsigned char *safe_get_bytecode(boolexp b)
+    static uint8_t *safe_get_bytecode(boolexp b)
 {
-  unsigned char *bytecode;
-  u_int_16 len;
+  uint8_t *bytecode;
+  uint16_t len;
 
   len = chunk_len(b);
   bytecode = mush_malloc(len, "boolexp.bytecode");
@@ -324,13 +331,14 @@ safe_get_bytecode(boolexp b)
 
 /** Given a chunk id, return the bytecode for a boolexp.  
  * \param b The boolexp to retrieve.
+ * \param storelen the length of the bytecode.
  * \return a static copy of the bytecode.
  */
-static unsigned char *
-get_bytecode(boolexp b, u_int_16 * storelen)
+static uint8_t *
+get_bytecode(boolexp b, uint16_t * storelen)
 {
-  static unsigned char bytecode[BUFFER_LEN * 2];
-  u_int_16 len;
+  static uint8_t bytecode[BUFFER_LEN * 2];
+  uint16_t len;
 
   len = chunk_fetch(b, bytecode, sizeof bytecode);
   if (storelen)
@@ -350,8 +358,8 @@ boolexp
 dup_bool(boolexp b)
 {
   boolexp r;
-  unsigned char *bytecode;
-  u_int_16 len = 0;
+  uint8_t *bytecode;
+  uint16_t len = 0;
 
   if (b == TRUE_BOOLEXP)
     return TRUE_BOOLEXP;
@@ -386,7 +394,7 @@ sizeof_boolexp(boolexp b)
   if (b == TRUE_BOOLEXP)
     return 0;
   else
-    return chunk_len(b);
+    return (int) chunk_len(b);
 }
 
 /** Evaluate a boolexp.
@@ -421,12 +429,12 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
     ATTR *a;
     int r = 0;
     char *s = NULL;
-    unsigned char *bytecode, *pc;
+    uint8_t *bytecode, *pc;
 
     bytecode = pc = safe_get_bytecode(b);
 
     while (1) {
-      op = (bvm_opcode) * pc;
+      op = (bvm_opcode) *pc;
       memcpy(&arg, pc + 1, sizeof arg);
       pc += INSN_LEN;
       switch (op) {
@@ -673,11 +681,11 @@ unparse_boolexp(dbref player, boolexp b, enum u_b_f flag)
     bytecode = pc = get_bytecode(b, NULL);
 
     while (1) {
-      op = (bvm_opcode) * pc;
+      op = (bvm_opcode) *pc;
       memcpy(&arg, pc + 1, sizeof arg);
       pc += INSN_LEN;
       /* Handle most negation cases */
-      if (op != OP_RET && (bvm_opcode) * pc == OP_NEGR && op != OP_PAREN)
+      if (op != OP_RET && (bvm_opcode) *pc == OP_NEGR && op != OP_PAREN)
         safe_chr('!', boolexp_buf, &buftop);
       switch (op) {
       case OP_JMPT:
@@ -705,7 +713,7 @@ unparse_boolexp(dbref player, boolexp b, enum u_b_f flag)
           int pstack = 1, parg;
           unsigned char *tpc = pc;
           while (1) {
-            if ((bvm_opcode) * tpc == OP_PAREN) {
+            if ((bvm_opcode) *tpc == OP_PAREN) {
               memcpy(&parg, tpc + 1, sizeof parg);
               if (parg)
                 pstack--;
@@ -718,7 +726,7 @@ unparse_boolexp(dbref player, boolexp b, enum u_b_f flag)
             }
             tpc += INSN_LEN;
           }
-          if ((bvm_opcode) * tpc == OP_NEGR)
+          if ((bvm_opcode) *tpc == OP_NEGR)
             safe_strl("!(", 2, boolexp_buf, &buftop);
           else
             safe_chr('(', boolexp_buf, &buftop);
@@ -833,7 +841,7 @@ alloc_bool(void)
 {
   struct boolexp_node *b;
 
-  b = mush_malloc(sizeof *b, "boolexp_node");
+  b = mush_malloc(sizeof *b, "boolexp.node");
 
   b->data.sub.a = NULL;
   b->data.sub.b = NULL;
@@ -848,7 +856,7 @@ alloc_bool(void)
 static void
 free_bool(struct boolexp_node *b)
 {
-  mush_free(b, "boolexp_node");
+  mush_free(b, "boolexp.node");
 }
 
 /** Free a boolexp ast node.
@@ -1258,6 +1266,8 @@ gen_label_id(struct bvm_asm *a)
   return l;
 }
 
+slab *bvm_asmnode_slab = NULL;
+
 /** Add an instruction to the assembler list.
  * \param a the assembler list.
  * \param op the opcode of the instruction.
@@ -1271,7 +1281,8 @@ append_insn(struct bvm_asm *a, bvm_opcode op, int arg, const char *s)
 
   if (s) {
     struct bvm_strnode *newstr;
-    int count = 0, found = 0;
+    uint32_t count = 0;
+    bool found = 0;
 
     /* Look for an existing string */
     for (newstr = a->shead; newstr; newstr = newstr->next, count++) {
@@ -1302,7 +1313,9 @@ append_insn(struct bvm_asm *a, bvm_opcode op, int arg, const char *s)
     }
   }
 
-  newop = mush_malloc(sizeof *newop, "bvm.asmnode");
+  if (bvm_asmnode_slab == NULL)
+    bvm_asmnode_slab = slab_create("bvm.asmnode", sizeof *newop);
+  newop = slab_malloc(bvm_asmnode_slab, NULL);
   if (!newop)
     mush_panic(T("Unable to allocate memory for boolexp asm node!"));
   newop->op = op;
@@ -1429,16 +1442,13 @@ generate_bvm_asm(struct boolexp_node *b)
 static void
 free_bvm_asm(struct bvm_asm *a)
 {
-  struct bvm_asmnode *i, *tmp1;
   struct bvm_strnode *s, *tmp2;
 
   if (!a)
     return;
 
-  for (i = a->head; i; i = tmp1) {
-    tmp1 = i->next;
-    mush_free(i, "bvm.asmnode");
-  }
+  slab_destroy(bvm_asmnode_slab);
+  bvm_asmnode_slab = NULL;
 
   for (s = a->shead; s; s = tmp2) {
     tmp2 = s->next;
@@ -1453,10 +1463,10 @@ free_bvm_asm(struct bvm_asm *a)
  * \param the id of the label to find.
  * \return the number of instructions before the label.
  */
-static int
+static size_t
 pos_of_label(struct bvm_asm *as, int label)
 {
-  int offset = 0;
+  size_t offset = 0;
   struct bvm_asmnode *a;
   for (a = as->head; a; a = a->next) {
     if (a->op == OP_LABEL && a->arg == label)
@@ -1472,10 +1482,10 @@ pos_of_label(struct bvm_asm *as, int label)
  * \param c The c-th string is the one that's wanted.
  * \return the distance from the start of the string section to the start of the c-th string.
  */
-static int
+static size_t
 offset_to_string(struct bvm_asm *a, int c)
 {
-  int offset = 0;
+  size_t offset = 0;
   int n = 0;
   struct bvm_strnode *s;
 
@@ -1533,7 +1543,7 @@ opt_thread_jumps(struct bvm_asm *a)
           /* Avoid useless conditional jumps on different conditions by
              jumping to the next instruction after. Ex: a&b|c */
           struct bvm_asmnode *newlbl;
-          newlbl = mush_malloc(sizeof *newlbl, "bvm.asmnode");
+          newlbl = slab_malloc(bvm_asmnode_slab, NULL);
           if (!newlbl)
             mush_panic(T("Unable to allocate memory for boolexp asm node!"));
           newlbl->op = OP_LABEL;
@@ -1575,7 +1585,7 @@ emit_bytecode(struct bvm_asm *a, int derefs)
   struct bvm_asmnode *i;
   struct bvm_strnode *s;
   unsigned char *pc, *bytecode;
-  u_int_16 len, blen;
+  uint16_t len, blen;
 
   if (!a)
     return TRUE_BOOLEXP;
@@ -1792,7 +1802,7 @@ print_bytecode(boolexp b)
   printf("Total length of bytecode+strings: %d bytes\n", len);
 
   while (1) {
-    op = (bvm_opcode) * pc;
+    op = (bvm_opcode) *pc;
     memcpy(&arg, pc + 1, sizeof arg);
     pc += INSN_LEN;
     printf("%-5d ", pos);
@@ -1917,7 +1927,7 @@ check_lock(dbref player, dbref i, const char *name, boolexp be)
   bytecode = pc = get_bytecode(be, NULL);
 
   while (1) {
-    op = (bvm_opcode) * pc;
+    op = (bvm_opcode) *pc;
     memcpy(&arg, pc + 1, sizeof arg);
     pc += INSN_LEN;
     switch (op) {
