@@ -11,8 +11,8 @@
  * on top of malloc(), with significantly reduced overhead and the ability
  * to rearrange allocations to actively control fragmentation and increase
  * locality.
- * 
- * 
+ *
+ *
  * <h3>Basic operation:</h3>
  * The managed memory pool is divided into regions of approximately 64KB.
  * These regions contain variable-size chunks representing allocated and
@@ -22,14 +22,14 @@
  * the used/free status, the size of the chunk, and the number of
  * dereferences for the chunk), and each region has additional overhead
  * of about 42 bytes.
- * 
+ *
  * Allocations are made with the chunk_create() call, which is given
  * the size of the data, the data value to be stored, and an initial
  * dereference count to be assigned to the chunk.  Once created, the
  * value of a chunk cannot be changed; the storage is immutable.
  * chunk_create() returns an integral reference value that can be
  * used to retrieve or free the allocation.
- * 
+ *
  * Allocations are accessed with the chunk_fetch(), chunk_len(), and
  * chunk_derefs() calls.  Each of these if given a reference (as
  * returned by chunk_create()), and chunk_fetch() is additionally
@@ -37,10 +37,10 @@
  * chunk_fetch() and chunk_len() increment a chunk's dereference
  * count (up to the maximum of 255), which is used in migration to
  * improve locality.
- * 
+ *
  * Allocations are freed with the chunk_delete() call, which also
  * requires a reference as input.
- * 
+ *
  * Finally, allocations are allowed to rearrange themselves with the
  * chunk_migration() call.  chunk_migration() takes an array of
  * pointers to chunk references as input, and examines each of the
@@ -53,8 +53,8 @@
  * chunk_migration(); however, _all_ allocations made with chunk_create()
  * must eventually be submitted for migration in order to maintain the
  * memory pool in a non-fragmented state.
- * 
- * 
+ *
+ *
  * <h3>Migration:</h3>
  * Under normal conditions, extended use of this chunk allocation system
  * would lead to a significantly fragmented datastore, unless there was
@@ -68,14 +68,14 @@
  * incrementally, giving permission to move a small number of chunks
  * at any one time, and spreading out the cost of defragmenting the
  * data store.
- * 
+ *
  * Just because you give permission to move a chunk doesn't mean that it
  * will be moved.  The chunk may be perfectly happy where it is, with
  * no need to move it elsewhere.  Chunks are only moved when their
  * personal happiness would be improved by a move.  In general, maximizing
  * the happiness of individual chunks will improve the happiness of the
  * whole.
- * 
+ *
  * There are two things that factor into a chunk's happiness.
  * The things that make a chunk unhappy are:
  * <ul>
@@ -87,7 +87,7 @@
  * Neither of these factors are absolute; both of them have different
  * weights that add into a general unhappiness for the chunk.  The lower
  * the unhappiness, the better.
- * 
+ *
  * Over time and usage, the dereference counts for chunks will increase
  * and eventually reach a maximum value of 255.  (The count is limited
  * by the fact that it's stored in a single byte for each chunk.)  If
@@ -102,8 +102,8 @@
  * up incrementing the dereference count on all chunks, and thus all
  * regions).  Given a dump frequency of once per hour (the default), there
  * should be a period change about every 2.6 days.
- * 
- * 
+ *
+ *
  * <h3>Statistics:</h3>
  * The chunk memory management system keeps several statistics about
  * the allocation pool, both to maintain good operation through active
@@ -111,7 +111,7 @@
  * using the system (and its designer ;-)).  These statistics are
  * reported (in PennMUSH) through the use of the @stats command,
  * with /chunks switch.
- * 
+ *
  * @stats/chunks generates output similar to this:
  * \verbatim
  * Chunks:         99407 allocated (   8372875 bytes,     223808 ( 2%) overhead)
@@ -122,7 +122,7 @@
  * Regions:          147 total,       16 cached
  * Paging:        158686 out,     158554 in
  * Storage:      9628500 total (86% saturation)
- *  
+ *
  * Period:             1 (   5791834 accesses so far,       1085 chunks at max)
  * Migration:     245543 moves this period
  *                  145536 slide
@@ -139,16 +139,16 @@
  * is the number of free chunks, their total size, and the amount of
  * fragmented free space (free space not in the largest free chunk for
  * its region is considered fragmented).
- * 
+ *
  * Next comes statistics on regions: the number of regions in use
  * and the number held in the memory cache.  All regions not in the
  * cache are paged out to disk.  Paging statistics follow, listing
- * the number of times a region has been moved out of or into 
+ * the number of times a region has been moved out of or into
  * memory cache.  After that, the total amount of storage (in memory
  * or on disk) used is given, along with the saturation rate (where
  * saturation is indicated by what fraction of the used space is
  * actually allocated in chunks).
- * 
+ *
  * Finally comes statistics on migration and the migration period.
  * The period number is listed, along with the total number of
  * dereferences in the period and how many chunks have the maximum
@@ -160,14 +160,14 @@
  * are when an allocation is moved in order to fill in a free space;
  * the space can be either exactly filled by the move, or inexactly
  * filled (leaving some remaining free space).
- * 
- * 
+ *
+ *
  * <h3>Histograms:</h3>
  * The chunk memory management system can also display a few
  * histograms about itself.  These histograms are reported (in PennMUSH)
  * through the use of the @stats command, with the /regions, /freespace,
  * or /paging switches.
- * 
+ *
  * All of @stats/regions, @stats/freespace, and @stas/paging produce
  * histograms vs. region average dereference count.  The histograms
  * use buckets four counts wide, so all regions from 0-3 will be in
@@ -176,7 +176,7 @@
  * different, then the highest spikes will be allowed to extend off
  * the top of the histogram (with their real values labeled in
  * parenthesis next to them).
- * 
+ *
  * @stats/regions is a histogram of how many regions at each count
  * currently exist.  In a healthy game, there should be a large spike
  * at some dereference count between 64 and 128 (representing the
@@ -187,11 +187,11 @@
  * areas (above the large spike).  New migration periods occur when
  * the large spike would pass 128, at which point everything is halved
  * and the spike is pushed back down to 64.
- * 
+ *
  * @stats/freespace is a histogram of how much free space exists in
  * regions at each dereference count.  This histogram is included
  * to aid in diagnosis of the cause for dropping saturation rates.
- * 
+ *
  * @stats/paging is a histogram of the number of regions being paged
  * in or out at each dereference count.  As of this writing, a very
  * unhealthy behaviour is observed, wherein the histogram shows a
@@ -211,20 +211,26 @@
 #include "config.h"
 #include "conf.h"
 
+#define _XOPEN_SOURCE 600
 #include <limits.h>
 #include <string.h>
 #include <stdarg.h>
 #include <stdlib.h>
-
 #include <fcntl.h>
 #include <assert.h>
 #include <sys/types.h>
 #ifdef WIN32
+#include <wtypes.h>
 #include <io.h>
 #else
+#define __USE_UNIX98
 #include <unistd.h>
 #endif
 #include <errno.h>
+#ifdef I_SYS_STAT
+#include <sys/stat.h>
+#endif
+
 
 #include "externs.h"
 #include "chunk.h"
@@ -529,8 +535,8 @@ static int ignore;      /**< Used to shut up compiler warnings when not assertin
  * the rest of the 64K bytes of the region contain chunks.
  */
 typedef struct region_header {
-  u_int_16 region_id;           /**< will be INVALID_REGION_ID if not in use */
-  u_int_16 first_free;          /**< offset of 1st free chunk */
+  uint16_t region_id;           /**< will be INVALID_REGION_ID if not in use */
+  uint16_t first_free;          /**< offset of 1st free chunk */
   struct region_header *prev;   /**< linked list prev for LRU cache */
   struct region_header *next;   /**< linked list next for LRU cache */
 } RegionHeader;
@@ -539,16 +545,16 @@ typedef struct region_header {
 
 /** In-memory (never paged) region info.  */
 typedef struct region {
-  u_int_16 used_count;          /**< number of used chunks */
-  u_int_16 free_count;          /**< number of free chunks */
-  u_int_16 free_bytes;          /**< number of free bytes (with headers) */
-  u_int_16 largest_free_chunk;  /**< largest single free chunk */
-  unsigned int total_derefs;    /**< total of all used chunk derefs */
-  unsigned int period_last_touched;     /**< "this" period, for deref counts;
+  uint16_t used_count;          /**< number of used chunks */
+  uint16_t free_count;          /**< number of free chunks */
+  uint16_t free_bytes;          /**< number of free bytes (with headers) */
+  uint16_t largest_free_chunk;  /**< largest single free chunk */
+  uint32_t total_derefs;        /**< total of all used chunk derefs */
+  uint32_t period_last_touched; /**< "this" period, for deref counts;
                                            we don't page in regions to update
                                            counts on period change! */
   RegionHeader *in_memory;      /**< cache entry; NULL if paged out */
-  u_int_16 oddballs[NUM_ODDBALLS];      /**< chunk offsets with odd derefs */
+  uint16_t oddballs[NUM_ODDBALLS];      /**< chunk offsets with odd derefs */
 } Region;
 
 #define RegionDerefs(region) \
@@ -581,19 +587,19 @@ static char child_filename[300];
 /** Deref scale control.
  * When the deref counts get too big, the current period is incremented
  * and all derefs are divided by 2. */
-static u_int_32 curr_period;
+static uint32_t curr_period;
 
 /*
  * Info about all regions
  */
-static u_int_32 region_count;   /**< regions in use */
-static u_int_32 region_array_len;       /**< length of regions array */
+static uint32_t region_count;   /**< regions in use */
+static uint32_t region_array_len;       /**< length of regions array */
 static Region *regions;         /**< regions array, realloced as (rarely) needed */
 
 /*
  * regions presently in memory
  */
-static u_int_32 cached_region_count;    /**< number of regions in cache */
+static uint32_t cached_region_count;    /**< number of regions in cache */
 static RegionHeader *cache_head;        /**< most recently used region */
 static RegionHeader *cache_tail;        /**< least recently used region */
 
@@ -636,7 +642,7 @@ static int noisy_log = 0;
 /*
  * Forward decls
  */
-static void find_oddballs(u_int_16 region);
+static void find_oddballs(uint16_t region);
 
 /*
  * Debug routines
@@ -686,7 +692,7 @@ dump_debug_log(FILE * fp)
 
 /** Test if a chunk is migratable. */
 static int
-migratable(u_int_16 region, u_int_16 offset)
+migratable(uint16_t region, uint16_t offset)
 {
   chunk_reference_t ref = ChunkReference(region, offset);
   int j;
@@ -705,11 +711,11 @@ migratable(u_int_16 region, u_int_16 offset)
  * \param fp the FILE* to output to.
  */
 static void
-debug_dump_region(u_int_16 region, FILE * fp)
+debug_dump_region(uint16_t region, FILE * fp)
 {
   Region *rp = regions + region;
   RegionHeader *rhp;
-  u_int_16 offset, count;
+  uint16_t offset, count;
 
   ASSERT(region < region_count);
   rhp = rp->in_memory;
@@ -758,9 +764,9 @@ debug_dump_region(u_int_16 region, FILE * fp)
  * \param offset the offset to verify.
  */
 static void
-verify_used_chunk(u_int_16 region, u_int_16 offset)
+verify_used_chunk(uint16_t region, uint16_t offset)
 {
-  u_int_16 pos;
+  uint16_t pos;
 
   ASSERT(region < region_count);
 
@@ -783,7 +789,7 @@ verify_used_chunk(u_int_16 region, u_int_16 offset)
  * \return true if the region is valid.
  */
 static int
-region_is_valid(u_int_16 region)
+region_is_valid(uint16_t region)
 {
   int result;
   Region *rp;
@@ -796,8 +802,8 @@ region_is_valid(u_int_16 region)
   int len;
   int was_free;
   int dump;
-  u_int_16 next_free;
-  u_int_16 offset;
+  uint16_t next_free;
+  uint16_t offset;
 
   if (region >= region_count) {
     do_rawlog(LT_ERR, "region 0x%04x is not valid: region_count is 0x%04x",
@@ -955,9 +961,8 @@ region_is_valid(u_int_16 region)
  * \param derefs the deref count to set on the chunk.
  */
 static void
-write_used_chunk(u_int_16 region, u_int_16 offset, u_int_16 full_len,
-                 unsigned char const *data, u_int_16 data_len,
-                 unsigned char derefs)
+write_used_chunk(uint16_t region, uint16_t offset, uint16_t full_len,
+                 unsigned char const *data, uint16_t data_len, uint8_t derefs)
 {
   unsigned char *cptr = ChunkPointer(region, offset);
   if (full_len <= MAX_SHORT_CHUNK_LEN + CHUNK_SHORT_DATA_OFFSET) {
@@ -968,14 +973,14 @@ write_used_chunk(u_int_16 region, u_int_16 offset, u_int_16 full_len,
     memcpy(cptr + CHUNK_SHORT_DATA_OFFSET, data, data_len);
   } else if (full_len <= MAX_MEDIUM_CHUNK_LEN + CHUNK_MEDIUM_DATA_OFFSET) {
     /* chunk is medium */
-    u_int_16 len = full_len - CHUNK_MEDIUM_DATA_OFFSET;
+    uint16_t len = full_len - CHUNK_MEDIUM_DATA_OFFSET;
     cptr[0] = (len >> 8) + CHUNK_USED + CHUNK_TAG1_MEDIUM + CHUNK_TAG2_MEDIUM;
     cptr[CHUNK_DEREF_OFFSET] = derefs;
     cptr[CHUNK_MEDIUM_LEN_LSB_OFFSET] = len & 0xff;
     memcpy(cptr + CHUNK_MEDIUM_DATA_OFFSET, data, data_len);
   } else {
     /* chunk is long */
-    u_int_16 len = full_len - CHUNK_LONG_DATA_OFFSET;
+    uint16_t len = full_len - CHUNK_LONG_DATA_OFFSET;
     cptr[0] = CHUNK_USED + CHUNK_TAG1_LONG + CHUNK_TAG2_LONG;
     cptr[CHUNK_DEREF_OFFSET] = derefs;
     cptr[CHUNK_LONG_LEN_MSB_OFFSET] = len >> 8;
@@ -991,8 +996,8 @@ write_used_chunk(u_int_16 region, u_int_16 offset, u_int_16 full_len,
  * \param next the offset for the next free chunk.
  */
 static void
-write_free_chunk(u_int_16 region, u_int_16 offset, u_int_16 full_len,
-                 u_int_16 next)
+write_free_chunk(uint16_t region, uint16_t offset, uint16_t full_len,
+                 uint16_t next)
 {
   unsigned char *cptr = ChunkPointer(region, offset);
   if (full_len <= MAX_SHORT_CHUNK_LEN + CHUNK_SHORT_DATA_OFFSET) {
@@ -1003,14 +1008,14 @@ write_free_chunk(u_int_16 region, u_int_16 offset, u_int_16 full_len,
     cptr[CHUNK_DEREF_OFFSET] = next & 0xff;
   } else if (full_len <= MAX_MEDIUM_CHUNK_LEN + CHUNK_MEDIUM_DATA_OFFSET) {
     /* chunk is medium */
-    u_int_16 len = full_len - CHUNK_MEDIUM_DATA_OFFSET;
+    uint16_t len = full_len - CHUNK_MEDIUM_DATA_OFFSET;
     cptr[0] = (len >> 8) + CHUNK_FREE + CHUNK_TAG1_MEDIUM + CHUNK_TAG2_MEDIUM;
     cptr[CHUNK_MEDIUM_LEN_LSB_OFFSET] = len & 0xff;
     cptr[CHUNK_MEDIUM_DATA_OFFSET] = next >> 8;
     cptr[CHUNK_DEREF_OFFSET] = next & 0xff;
   } else {
     /* chunk is long */
-    u_int_16 len = full_len - CHUNK_LONG_DATA_OFFSET;
+    uint16_t len = full_len - CHUNK_LONG_DATA_OFFSET;
     cptr[0] = CHUNK_FREE + CHUNK_TAG1_LONG + CHUNK_TAG2_LONG;
     cptr[CHUNK_LONG_LEN_MSB_OFFSET] = len >> 8;
     cptr[CHUNK_LONG_LEN_LSB_OFFSET] = len & 0xff;
@@ -1025,7 +1030,7 @@ write_free_chunk(u_int_16 region, u_int_16 offset, u_int_16 full_len,
  * \param next the offset for the next free chunk.
  */
 static void
-write_next_free(u_int_16 region, u_int_16 offset, u_int_16 next)
+write_next_free(uint16_t region, uint16_t offset, uint16_t next)
 {
   unsigned char *cptr = ChunkPointer(region, offset);
   if (ChunkIsShort(region, offset)) {
@@ -1049,10 +1054,10 @@ write_next_free(u_int_16 region, u_int_16 offset, u_int_16 next)
  * \param offset the offset of the left-hand chunk to coalesce.
  */
 static void
-coalesce_frees(u_int_16 region, u_int_16 offset)
+coalesce_frees(uint16_t region, uint16_t offset)
 {
   Region *rp = regions + region;
-  u_int_16 full_len, next;
+  uint16_t full_len, next;
   full_len = ChunkFullLen(region, offset);
   next = ChunkNextFree(region, offset);
   if (offset + full_len == next) {
@@ -1070,10 +1075,10 @@ coalesce_frees(u_int_16 region, u_int_16 offset)
  * \param offset the offset of the chunk to free.
  */
 static void
-free_chunk(u_int_16 region, u_int_16 offset)
+free_chunk(uint16_t region, uint16_t offset)
 {
   Region *rp = regions + region;
-  u_int_16 full_len, left;
+  uint16_t full_len, left;
 
   full_len = ChunkFullLen(region, offset);
   rp->total_derefs -= ChunkDerefs(region, offset);
@@ -1107,7 +1112,7 @@ free_chunk(u_int_16 region, u_int_16 offset)
     rp->in_memory->first_free = offset;
     left = 0;
   } else {
-    u_int_16 next;
+    uint16_t next;
     next = ChunkNextFree(region, left);
     while (next && next < offset) {
       left = next;
@@ -1125,11 +1130,11 @@ free_chunk(u_int_16 region, u_int_16 offset)
  * \param region the region to search for a large hole in.
  * \return the size of the largest free chunk.
  */
-static u_int_16
-largest_hole(u_int_16 region)
+static uint16_t
+largest_hole(uint16_t region)
 {
-  u_int_16 size;
-  u_int_16 offset;
+  uint16_t size;
+  uint16_t offset;
   size = 0;
   for (offset = regions[region].in_memory->first_free;
        offset; offset = ChunkNextFree(region, offset))
@@ -1148,11 +1153,11 @@ largest_hole(u_int_16 region)
  * \param align the alignment to use: 0 = easiest, 1 = left, 2 = right.
  * \return the offset of the allocated space.
  */
-static u_int_16
-split_hole(u_int_16 region, u_int_16 offset, u_int_16 full_len, int align)
+static uint16_t
+split_hole(uint16_t region, uint16_t offset, uint16_t full_len, int align)
 {
   Region *rp = regions + region;
-  u_int_16 hole_len = ChunkFullLen(region, offset);
+  uint16_t hole_len = ChunkFullLen(region, offset);
 
   rp->used_count++;
   if (full_len <= MAX_SHORT_CHUNK_LEN + CHUNK_SHORT_DATA_OFFSET) {
@@ -1175,7 +1180,7 @@ split_hole(u_int_16 region, u_int_16 offset, u_int_16 full_len, int align)
     if (rp->in_memory->first_free == offset)
       rp->in_memory->first_free = ChunkNextFree(region, offset);
     else {
-      u_int_16 hole;
+      uint16_t hole;
       for (hole = rp->in_memory->first_free;
            hole; hole = ChunkNextFree(region, hole))
         if (ChunkNextFree(region, hole) == offset)
@@ -1202,7 +1207,7 @@ split_hole(u_int_16 region, u_int_16 offset, u_int_16 full_len, int align)
     if (rp->in_memory->first_free == offset)
       rp->in_memory->first_free += full_len;
     else {
-      u_int_16 hole;
+      uint16_t hole;
       for (hole = rp->in_memory->first_free;
            hole; hole = ChunkNextFree(region, hole))
         if (ChunkNextFree(region, hole) == offset)
@@ -1233,16 +1238,17 @@ split_hole(u_int_16 region, u_int_16 offset, u_int_16 full_len, int align)
  * \param region region to read
  */
 static void
-read_cache_region(fd_type fd, RegionHeader * rhp, u_int_16 region)
+read_cache_region(fd_type fd, RegionHeader *rhp, uint16_t region)
 {
   off_t file_offset = region * REGION_SIZE;
   int j;
   char *pos;
   size_t remaining;
-  int done;
+  ssize_t done;
 
   debug_log("read_cache_region %04x", region);
 
+#ifndef HAVE_PREAD
   /* Try to seek up to 3 times... */
   for (j = 0; j < 3; j++)
 #ifdef WIN32
@@ -1258,28 +1264,26 @@ read_cache_region(fd_type fd, RegionHeader * rhp, u_int_16 region)
 #else
     mush_panicf("chunk swap file seek, errno %d: %s", errno, strerror(errno));
 #endif
+#endif                          /* !HAVE_PREAD */
   pos = (char *) rhp;
   remaining = REGION_SIZE;
   for (j = 0; j < 10; j++) {
-#ifdef WIN32
-#ifndef __MINGW32__
+#if defined(HAVE_PREAD)
+    done = pread(fd, pos, remaining, file_offset);
+#elif defined(WIN32) && !defined(__MINGW32__)
     if (!ReadFile(fd, pos, remaining, &done, NULL)) {
       /* nothing */
     }
-#endif
 #else
     done = read(fd, pos, remaining);
 #endif
     if (done >= 0) {
       remaining -= done;
       pos += done;
+      file_offset += done;
       if (!remaining)
         return;
     }
-#ifndef WIN32
-    if (done == -1 && errno == EAGAIN)
-      sleep(0);
-#endif
   }
 #ifdef WIN32
   mush_panicf("chunk swap file read, %lu remaining, GetLastError %d",
@@ -1296,16 +1300,17 @@ read_cache_region(fd_type fd, RegionHeader * rhp, u_int_16 region)
  * \param region region to write
  */
 static void
-write_cache_region(fd_type fd, RegionHeader * rhp, u_int_16 region)
+write_cache_region(fd_type fd, RegionHeader *rhp, uint16_t region)
 {
   off_t file_offset = region * REGION_SIZE;
   int j;
   char *pos;
   size_t remaining;
-  int done;
+  ssize_t done;
 
   debug_log("write_cache_region %04x", region);
 
+#ifndef HAVE_PWRITE
   /* Try to seek up to 3 times... */
   for (j = 0; j < 3; j++)
 #ifdef WIN32
@@ -1321,30 +1326,32 @@ write_cache_region(fd_type fd, RegionHeader * rhp, u_int_16 region)
 #else
     mush_panicf("chunk swap file seek, errno %d: %s", errno, strerror(errno));
 #endif
+#endif                          /* !HAVE_PWRITE */
   pos = (char *) rhp;
   remaining = REGION_SIZE;
+
+  /* SW: I'm not sure why Talek has this loop so many times -- the
+     only recoverable way the writes should fail is if they're
+     interrupted by a signal and can't be restarted for some
+     reason. */
   for (j = 0; j < 10; j++) {
-#ifdef WIN32
-#ifndef __MINGW32__
+
+#if defined(HAVE_PWRITE)
+    done = pwrite(fd, pos, remaining, file_offset);
+#elif defined(WIN32) && !defined(__MINGW32__)
     if (!WriteFile(fd, pos, remaining, &done, NULL)) {
       /* nothing */
     }
-#else
-    done = write(fd, pos, remaining);
-#endif
 #else
     done = write(fd, pos, remaining);
 #endif
     if (done >= 0) {
       remaining -= done;
       pos += done;
+      file_offset += done;
       if (!remaining)
         return;
     }
-#ifndef WIN32
-    if (done == -1 && errno == EAGAIN)
-      sleep(0);
-#endif
   }
 #ifdef WIN32
   mush_panicf("chunk swap file write, %lu remaining, GetLastError %d",
@@ -1359,7 +1366,7 @@ write_cache_region(fd_type fd, RegionHeader * rhp, u_int_16 region)
  * \param rhp the cached region to keep around.
  */
 static void
-touch_cache_region(RegionHeader * rhp)
+touch_cache_region(RegionHeader *rhp)
 {
   debug_log("touch_cache_region %04x", rhp->region_id);
 
@@ -1436,11 +1443,11 @@ find_available_cache_region(void)
  * \param region the region to bring in.
  */
 static void
-bring_in_region(u_int_16 region)
+bring_in_region(uint16_t region)
 {
   Region *rp = regions + region;
   RegionHeader *rhp, *prev, *next;
-  u_int_32 offset;
+  uint32_t offset;
   unsigned int shift;
 
   debug_log("bring_in_region %04x", region);
@@ -1503,10 +1510,10 @@ bring_in_region(u_int_16 region)
  * Recycle an empty region if possible.
  * \return the region id for the new region.
  */
-static u_int_16
+static uint16_t
 create_region(void)
 {
-  u_int_16 region;
+  uint16_t region;
 
   for (region = 0; region < region_count; region++)
     if (regions[region].used_count == 0)
@@ -1548,11 +1555,11 @@ create_region(void)
  * \param region the region to search in.
  */
 static void
-find_oddballs(u_int_16 region)
+find_oddballs(uint16_t region)
 {
   Region *rp = regions + region;
   int j, d1, d2;
-  u_int_16 offset, len;
+  uint16_t offset, len;
   int mean;
 
   for (j = 0; j < NUM_ODDBALLS; j++)
@@ -1596,10 +1603,10 @@ find_oddballs(u_int_16 region)
  * \param old_region the region the chunk was in before (if any).
  * \return the region id for the least unhappy region.
  */
-static u_int_16
-find_best_region(u_int_16 full_len, int derefs, u_int_16 old_region)
+static uint16_t
+find_best_region(uint16_t full_len, int derefs, uint16_t old_region)
 {
-  u_int_16 best_region, region;
+  uint16_t best_region, region;
   int best_score, score;
   int free_bytes;
   Region *rp;
@@ -1654,11 +1661,11 @@ find_best_region(u_int_16 full_len, int derefs, u_int_16 old_region)
  * \param old_region the region the chunk was in before (if any).
  * \param old_offset the offset the chunk was at before (if any).
  */
-static u_int_16
-find_best_offset(u_int_16 full_len, u_int_16 region,
-                 u_int_16 old_region, u_int_16 old_offset)
+static uint16_t
+find_best_offset(uint16_t full_len, uint16_t region,
+                 uint16_t old_region, uint16_t old_offset)
 {
-  u_int_16 fits, offset;
+  uint16_t fits, offset;
 
   bring_in_region(region);
 
@@ -1741,7 +1748,7 @@ chunk_statistics(dbref player)
   int free_large = 0;
   int used_count = 0;
   int used_bytes = 0;
-  u_int_16 rid;
+  uint16_t rid;
 
   for (rid = 0; rid < region_count; rid++) {
     free_count += regions[rid].free_count;
@@ -1820,7 +1827,7 @@ chunk_page_stats(dbref player)
 static void
 chunk_region_statistics(dbref player)
 {
-  u_int_16 rid;
+  uint16_t rid;
   const char *s;
 
   if (!GoodObject(player)) {
@@ -1952,10 +1959,10 @@ migrate_sort(void)
  * \param which the index (in the migration arrays) of the chunk to move.
  */
 static void
-migrate_slide(u_int_16 region, u_int_16 offset, int which)
+migrate_slide(uint16_t region, uint16_t offset, int which)
 {
   Region *rp = regions + region;
-  u_int_16 o_len, len, next, other, prev, o_off, o_oth;
+  uint16_t o_len, len, next, other, prev, o_off, o_oth;
 
   debug_log("migrate_slide %d (%08x) to %04x%04x",
             which, m_references[which][0], region, offset);
@@ -2020,10 +2027,10 @@ migrate_slide(u_int_16 region, u_int_16 offset, int which)
  * \param which the index (in the migration arrays) of the chunk to move.
  */
 static void
-migrate_move(u_int_16 region, u_int_16 offset, int which, int align)
+migrate_move(uint16_t region, uint16_t offset, int which, int align)
 {
   Region *rp = regions + region;
-  u_int_16 s_reg, s_off, s_len, o_off, length;
+  uint16_t s_reg, s_off, s_len, o_off, length;
   Region *srp;
 
   debug_log("migrate_move %d (%08x) to %04x%04x, alignment %d",
@@ -2082,11 +2089,11 @@ migrate_move(u_int_16 region, u_int_16 offset, int which, int align)
 }
 
 static void
-migrate_region(u_int_16 region)
+migrate_region(uint16_t region)
 {
   chunk_reference_t high, low;
   int j, derefs;
-  u_int_16 offset, length, best_region, best_offset;
+  uint16_t offset, length, best_region, best_offset;
 
   bring_in_region(region);
 
@@ -2121,9 +2128,9 @@ migrate_region(u_int_16 region)
  * \return the chunk reference for retrieving (or deleting) the data.
  */
 chunk_reference_t
-chunk_create(unsigned char const *data, u_int_16 len, unsigned char derefs)
+chunk_create(unsigned char const *data, uint16_t len, uint8_t derefs)
 {
-  u_int_16 full_len, region, offset;
+  uint16_t full_len, region, offset;
 
   if (len < MIN_CHUNK_LEN || len > MAX_CHUNK_LEN)
     mush_panicf("Illegal chunk length requested: %d bytes", len);
@@ -2156,7 +2163,7 @@ chunk_create(unsigned char const *data, u_int_16 len, unsigned char derefs)
 void
 chunk_delete(chunk_reference_t reference)
 {
-  u_int_16 region, offset;
+  uint16_t region, offset;
   region = ChunkReferenceToRegion(reference);
   offset = ChunkReferenceToOffset(reference);
   ASSERT(region < region_count);
@@ -2183,11 +2190,11 @@ chunk_delete(chunk_reference_t reference)
  * \param buffer_len the length of the buffer.
  * \return the length of the data.
  */
-u_int_16
+uint16_t
 chunk_fetch(chunk_reference_t reference,
-            unsigned char *buffer, u_int_16 buffer_len)
+            unsigned char *buffer, uint16_t buffer_len)
 {
-  u_int_16 region, offset, len;
+  uint16_t region, offset, len;
   region = ChunkReferenceToRegion(reference);
   offset = ChunkReferenceToOffset(reference);
   ASSERT(region < region_count);
@@ -2216,7 +2223,7 @@ chunk_fetch(chunk_reference_t reference,
  * \param reference the reference to the chunk to be queried.
  * \return the length of the data.
  */
-u_int_16
+uint16_t
 chunk_len(chunk_reference_t reference)
 {
   return chunk_fetch(reference, NULL, 0);
@@ -2231,7 +2238,7 @@ chunk_len(chunk_reference_t reference)
 unsigned char
 chunk_derefs(chunk_reference_t reference)
 {
-  u_int_16 region, offset;
+  uint16_t region, offset;
   region = ChunkReferenceToRegion(reference);
   offset = ChunkReferenceToOffset(reference);
   ASSERT(region < region_count);
@@ -2243,17 +2250,17 @@ chunk_derefs(chunk_reference_t reference)
 }
 
 /** Migrate allocated chunks around.
- * 
+ *
  * \param count the number of chunks to move.
  * \param references an array of pointers to chunk references,
  * which will be updated in place if necessary.
  */
 void
-chunk_migration(int count, chunk_reference_t ** references)
+chunk_migration(int count, chunk_reference_t **references)
 {
   int k, l;
   unsigned total;
-  u_int_16 region, offset;
+  uint16_t region, offset;
 
   debug_log("*** chunk_migration starts, count = %d", count);
 
@@ -2318,7 +2325,7 @@ int
 chunk_num_swapped(void)
 {
   int count;
-  u_int_16 region;
+  uint16_t region;
   count = 0;
   for (region = 0; region < region_count; region++)
     if (!regions[region].in_memory)
@@ -2351,12 +2358,18 @@ chunk_init(void)
   cache_head = NULL;
   cache_tail = NULL;
 
+#ifdef HAVE_POSIX_FALLOCATE
+  /* Reserve some space for the swap file to start with. */
+  if (options.chunk_swap_initial > 0)
+    posix_fallocate(swap_fd, 0, (options.chunk_swap_initial * 1024));
+#endif
+
   region_count = 0;
   region_array_len = FIXME_INIT_REGION_LEN;
 #ifdef DEBUG_CHUNK_MALLOC
   do_rawlog(LT_TRACE, "CHUNK: malloc()ing initial region array");
 #endif
-  regions = mush_malloc(region_array_len * sizeof(Region), "chunk region list");
+  regions = mush_calloc(region_array_len, sizeof(Region), "chunk region list");
   if (!regions)
     mush_panic("cannot malloc space for chunk region list");
 
@@ -2410,7 +2423,7 @@ chunk_new_period(void)
 {
   RegionHeader *rhp;
   Region *rp;
-  u_int_16 region, offset;
+  uint16_t region, offset;
   int shift;
 
 #ifdef LOG_CHUNK_STATS
@@ -2482,6 +2495,19 @@ chunk_fork_file(void)
     j++;
   }
 
+#ifdef HAVE_POSIX_FALLOCATE
+  /* Try to reserve all the space needed for the child's copy of the chunk file all at once. */
+  {
+    struct stat fsize;
+    if (fstat(swap_fd, &fsize) == 0)
+      posix_fallocate(swap_fd_child, 0, fsize.st_size);
+  }
+#endif
+
+#ifdef HAVE_POSIX_FADVISE
+  posix_fadvise(swap_fd, 0, 0, POSIX_FADV_SEQUENTIAL);
+#endif
+
   rhp = find_available_cache_region();
   prev = rhp->prev;
   next = rhp->next;
@@ -2496,6 +2522,10 @@ chunk_fork_file(void)
   rhp->prev = prev;
   rhp->next = next;
 
+#ifdef HAVE_POSIX_FADVISE
+  posix_fadvise(swap_fd, 0, 0, POSIX_FADV_RANDOM);
+#endif
+
   return 1;
 }
 
@@ -2520,6 +2550,11 @@ chunk_fork_child(void)
     return;
 
   close(swap_fd);
+
+#ifdef HAVE_POSIX_FADVISE
+  posix_fadvise(swap_fd_child, 0, 0, POSIX_FADV_RANDOM);
+#endif
+
   swap_fd = swap_fd_child;
   swap_fd_child = -1;
 }
