@@ -22,7 +22,6 @@
 #include "attrib.h"
 #include "extmail.h"
 #include "malias.h"
-#include "getpgsiz.h"
 #include "parse.h"
 #include "access.h"
 #include "version.h"
@@ -53,19 +52,23 @@ extern int config_set(const char *opt, char *val, int source, int restrictions);
 /** Is there a right-hand side of the equal sign? From command.c */
 extern int rhs_present;
 
-COMMAND (cmd_allhalt) {
+COMMAND(cmd_allhalt)
+{
   do_allhalt(player);
 }
 
-COMMAND (cmd_allquota) {
+COMMAND(cmd_allquota)
+{
   do_allquota(player, arg_left, SW_ISSET(sw, SWITCH_QUIET));
 }
 
-COMMAND (cmd_atrlock) {
+COMMAND(cmd_atrlock)
+{
   do_atrlock(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_attribute) {
+COMMAND(cmd_attribute)
+{
   if (SW_ISSET(sw, SWITCH_ACCESS))
     do_attribute_access(player, arg_left, arg_right,
                         SW_ISSET(sw, SWITCH_RETROACTIVE));
@@ -77,11 +80,13 @@ COMMAND (cmd_attribute) {
     do_attribute_info(player, arg_left);
 }
 
-COMMAND (cmd_atrchown) {
+COMMAND(cmd_atrchown)
+{
   do_atrchown(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_boot) {
+COMMAND(cmd_boot)
+{
   if (SW_ISSET(sw, SWITCH_PORT))
     do_boot(player, arg_left, BOOT_DESC);
   else if (SW_ISSET(sw, SWITCH_ME))
@@ -92,7 +97,8 @@ COMMAND (cmd_boot) {
     do_boot(player, arg_left, BOOT_NAME);
 }
 
-COMMAND (cmd_break) {
+COMMAND(cmd_break)
+{
   if (parse_boolean(arg_left)) {
     global_eval_context.break_called = 1;
     if (arg_right && *arg_right) {
@@ -106,7 +112,8 @@ COMMAND (cmd_break) {
   }
 }
 
-COMMAND (cmd_assert) {
+COMMAND(cmd_assert)
+{
   if (!parse_boolean(arg_left)) {
     global_eval_context.break_called = 1;
     if (arg_right && *arg_right) {
@@ -120,23 +127,28 @@ COMMAND (cmd_assert) {
   }
 }
 
-COMMAND (cmd_chownall) {
+COMMAND(cmd_chownall)
+{
   do_chownall(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_PRESERVE));
 }
 
-COMMAND (cmd_chown) {
+COMMAND(cmd_chown)
+{
   do_chown(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_PRESERVE));
 }
 
-COMMAND (cmd_chzoneall) {
+COMMAND(cmd_chzoneall)
+{
   do_chzoneall(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_chzone) {
+COMMAND(cmd_chzone)
+{
   (void) do_chzone(player, arg_left, arg_right, 1);
 }
 
-COMMAND (cmd_config) {
+COMMAND(cmd_config)
+{
   int lc;
   lc = SW_ISSET(sw, SWITCH_LOWERCASE);
   if (SW_ISSET(sw, SWITCH_GLOBALS))
@@ -157,7 +169,7 @@ COMMAND (cmd_config) {
     do_list_flags("FLAG", player, arg_left, lc, T("Flags"));
   else if (SW_ISSET(sw, SWITCH_POWERS))
     do_list_flags("POWER", player, arg_left, lc, T("Powers"));
-  else if (SW_ISSET(sw, SWITCH_SET)) {
+  else if (SW_ISSET(sw, SWITCH_SET) || SW_ISSET(sw, SWITCH_SAVE)) {
     if (!Wizard(player)) {
       notify(player, T("You can't remake the world in your image."));
       return;
@@ -166,35 +178,70 @@ COMMAND (cmd_config) {
       notify(player, T("What did you want to set?"));
       return;
     }
-    if (!config_set(arg_left, arg_right, 1, 0)
-        && !config_set(arg_left, arg_right, 1, 1))
-      notify(player, T("Couldn't set that option"));
-    else
-      notify(player, T("Option set."));
+    {
+      int source = SW_ISSET(sw, SWITCH_SAVE) ? 2 : 1;
+      if (source == 2) {
+	if (!God(player)) {
+	/* Only god can alter the original config file. */
+	  notify(player, T("You can't remake the world in your image."));
+	  return;
+	}     
+      }
+      if (!config_set(arg_left, arg_right, source, 0)
+          && !config_set(arg_left, arg_right, source, 1))
+        notify(player, T("Couldn't set that option"));
+      else {
+        if (source == 2) {
+#ifdef HAVE_ED
+          notify(player, T("Option set and saved."));
+#else
+          notify(player, T("Option set but not saved (Saves disabled.)"));
+#endif
+        } else
+          notify(player, T("Option set."));
+      }
+    }
   } else
     do_config_list(player, arg_left, lc);
 }
 
-COMMAND (cmd_cpattr) {
+COMMAND(cmd_cpattr)
+{
   do_cpattr(player, arg_left, args_right, 0, SW_ISSET(sw, SWITCH_NOFLAGCOPY));
 }
 
-COMMAND (cmd_create) {
-  do_create(player, arg_left, parse_integer(arg_right));
+COMMAND(cmd_create)
+{
+
+  int cost = 0;
+  char *newdbref;
+
+  if (args_right[1] && *args_right[1])
+    cost = parse_integer(args_right[1]);
+
+  if (args_right[2] && *args_right[2])
+    newdbref = args_right[2];
+  else
+    newdbref = (char *) NULL;
+
+  do_create(player, arg_left, cost, newdbref);
 }
 
-COMMAND (cmd_clone) {
+COMMAND(cmd_clone)
+{
   if (SW_ISSET(sw, SWITCH_PRESERVE))
     do_clone(player, arg_left, arg_right, SWITCH_PRESERVE);
   else
     do_clone(player, arg_left, arg_right, SWITCH_NONE);
 }
 
-COMMAND (cmd_dbck) {
+COMMAND(cmd_dbck)
+{
   do_dbck(player);
 }
 
-COMMAND (cmd_decompile) {
+COMMAND(cmd_decompile)
+{
   char prefix[BUFFER_LEN];
   int sd = SW_ISSET(sw, SWITCH_SKIPDEFAULTS);
   *prefix = '\0';
@@ -220,30 +267,36 @@ COMMAND (cmd_decompile) {
     do_decompile(player, arg_left, prefix, DEC_NORMAL, sd);
 }
 
-COMMAND (cmd_teach) {
+COMMAND(cmd_teach)
+{
   do_teach(player, cause, arg_left);
 }
 
-COMMAND (cmd_destroy) {
+COMMAND(cmd_destroy)
+{
   do_destroy(player, arg_left, (SW_ISSET(sw, SWITCH_OVERRIDE)));
 }
 
-COMMAND (cmd_dig) {
+COMMAND(cmd_dig)
+{
   do_dig(player, arg_left, args_right, (SW_ISSET(sw, SWITCH_TELEPORT)));
 }
 
-COMMAND (cmd_disable) {
+COMMAND(cmd_disable)
+{
   do_enable(player, arg_left, 0);
 }
 
-COMMAND (cmd_doing) {
+COMMAND(cmd_doing)
+{
   if (SW_ISSET(sw, SWITCH_HEADER))
     do_poll(player, arg_left, 0);
   else
     do_doing(player, arg_left);
 }
 
-COMMAND (cmd_dolist) {
+COMMAND(cmd_dolist)
+{
   unsigned int flags = 0;
   if (SW_ISSET(sw, SWITCH_NOTIFY))
     flags |= DOL_NOTIFY;
@@ -252,7 +305,8 @@ COMMAND (cmd_dolist) {
   do_dolist(player, arg_left, arg_right, cause, flags);
 }
 
-COMMAND (cmd_dump) {
+COMMAND(cmd_dump)
+{
   if (SW_ISSET(sw, SWITCH_PARANOID))
     do_dump(player, arg_left, DUMP_PARANOID);
   else if (SW_ISSET(sw, SWITCH_DEBUG))
@@ -261,17 +315,20 @@ COMMAND (cmd_dump) {
     do_dump(player, arg_left, DUMP_NORMAL);
 }
 
-COMMAND (cmd_edit) {
+COMMAND(cmd_edit)
+{
   do_gedit(player, arg_left, args_right,
            SW_ISSET(sw, SWITCH_FIRST) ? EDIT_FIRST : EDIT_ALL,
            SW_ISSET(sw, SWITCH_CHECK) ? 0 : 1);
 }
 
-COMMAND (cmd_elock) {
+COMMAND(cmd_elock)
+{
   do_lock(player, arg_left, arg_right, Enter_Lock);
 }
 
-COMMAND (cmd_emit) {
+COMMAND(cmd_emit)
+{
   int spflags = !strcmp(cmd->name, "@NSEMIT") ? PEMIT_SPOOF : 0;
   SPOOF(player, cause, sw);
 
@@ -282,11 +339,13 @@ COMMAND (cmd_emit) {
     do_emit(player, arg_left, spflags);
 }
 
-COMMAND (cmd_enable) {
+COMMAND(cmd_enable)
+{
   do_enable(player, arg_left, 1);
 }
 
-COMMAND (cmd_entrances) {
+COMMAND(cmd_entrances)
+{
   if (SW_ISSET(sw, SWITCH_EXITS))
     do_entrances(player, arg_left, args_right, ENT_EXITS);
   else if (SW_ISSET(sw, SWITCH_THINGS))
@@ -299,19 +358,23 @@ COMMAND (cmd_entrances) {
     do_entrances(player, arg_left, args_right, ENT_ALL);
 }
 
-COMMAND (cmd_eunlock) {
+COMMAND(cmd_eunlock)
+{
   do_unlock(player, arg_left, Enter_Lock);
 }
 
-COMMAND (cmd_find) {
+COMMAND(cmd_find)
+{
   do_find(player, arg_left, args_right);
 }
 
-COMMAND (cmd_firstexit) {
+COMMAND(cmd_firstexit)
+{
   do_firstexit(player, arg_left);
 }
 
-COMMAND (cmd_flag) {
+COMMAND(cmd_flag)
+{
   if (SW_ISSET(sw, SWITCH_LIST))
     do_list_flags("FLAG", player, arg_left, 0, T("Flags"));
   else if (SW_ISSET(sw, SWITCH_ADD))
@@ -334,11 +397,13 @@ COMMAND (cmd_flag) {
     do_flag_info("FLAG", player, arg_left);
 }
 
-COMMAND (cmd_force) {
+COMMAND(cmd_force)
+{
   do_force(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_function) {
+COMMAND(cmd_function)
+{
   if (SW_ISSET(sw, SWITCH_DELETE))
     do_function_delete(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_ENABLE))
@@ -346,7 +411,8 @@ COMMAND (cmd_function) {
   else if (SW_ISSET(sw, SWITCH_DISABLE))
     do_function_toggle(player, arg_left, 0);
   else if (SW_ISSET(sw, SWITCH_RESTRICT))
-    do_function_restrict(player, arg_left, args_right[1]);
+    do_function_restrict(player, arg_left, args_right[1],
+                         SW_ISSET(sw, SWITCH_BUILTIN));
   else if (SW_ISSET(sw, SWITCH_RESTORE))
     do_function_restore(player, arg_left);
   else {
@@ -378,28 +444,34 @@ COMMAND (cmd_function) {
 }
 
 
-COMMAND (cmd_grep) {
+COMMAND(cmd_grep)
+{
   do_grep(player, arg_left, arg_right, ((SW_ISSET(sw, SWITCH_IPRINT))
                                         || (SW_ISSET(sw, SWITCH_PRINT))),
           ((SW_ISSET(sw, SWITCH_IPRINT))
            || (SW_ISSET(sw, SWITCH_ILIST))));
 }
 
-COMMAND (cmd_halt) {
+COMMAND(cmd_halt)
+{
   if (SW_ISSET(sw, SWITCH_ALL))
     do_allhalt(player);
+  else if (SW_BY_NAME(sw, "PID"))
+    do_haltpid(player, arg_left);
   else
     do_halt1(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_hide) {
+COMMAND(cmd_hide)
+{
   hide_player(player, !(SW_ISSET(sw, SWITCH_NO) || SW_ISSET(sw, SWITCH_OFF)));
 }
 
-COMMAND (cmd_hook) {
+COMMAND(cmd_hook)
+{
   enum hook_type flags;
 
-  if (!Wizard(player)) {
+  if (!(Wizard(player) || has_power_by_name(player, "HOOK", NOTYPE))) {
     notify(player, T("You need a fishing license to use that hook."));
     return;
   }
@@ -422,36 +494,43 @@ COMMAND (cmd_hook) {
   do_hook(player, arg_left, args_right[1], args_right[2], flags);
 }
 
-COMMAND (cmd_huh_command) {
+COMMAND(cmd_huh_command)
+{
   notify(player, T("Huh?  (Type \"help\" for help.)"));
 }
 
-COMMAND (cmd_home) {
+COMMAND(cmd_home)
+{
   if (!Mobile(player))
     return;
   do_move(player, "home", MOVE_NORMAL);
 }
 
-COMMAND (cmd_kick) {
+COMMAND(cmd_kick)
+{
   do_kick(player, arg_left);
 }
 
-COMMAND (cmd_lemit) {
+COMMAND(cmd_lemit)
+{
   int spflags = !strcmp(cmd->name, "@NSLEMIT") ? PEMIT_SPOOF : 0;
   SPOOF(player, cause, sw);
   do_lemit(player, arg_left,
            (SW_ISSET(sw, SWITCH_SILENT) * PEMIT_SILENT) | spflags);
 }
 
-COMMAND (cmd_link) {
+COMMAND(cmd_link)
+{
   do_link(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_PRESERVE));
 }
 
-COMMAND (cmd_listmotd) {
+COMMAND(cmd_listmotd)
+{
   do_motd(player, MOTD_LIST, "");
 }
 
-COMMAND (cmd_list) {
+COMMAND(cmd_list)
+{
   int lc;
   lc = SW_ISSET(sw, SWITCH_LOWERCASE);
   if (SW_ISSET(sw, SWITCH_MOTD))
@@ -472,14 +551,16 @@ COMMAND (cmd_list) {
     do_list(player, arg_left, lc);
 }
 
-COMMAND (cmd_lock) {
+COMMAND(cmd_lock)
+{
   if ((switches) && (switches[0]))
     do_lock(player, arg_left, arg_right, switches);
   else
     do_lock(player, arg_left, arg_right, Basic_Lock);
 }
 
-COMMAND (cmd_log) {
+COMMAND(cmd_log)
+{
   if (SW_ISSET(sw, SWITCH_CHECK))
     do_writelog(player, arg_left, LT_CHECK);
   else if (SW_ISSET(sw, SWITCH_CMD))
@@ -496,7 +577,8 @@ COMMAND (cmd_log) {
     do_writelog(player, arg_left, LT_CMD);
 }
 
-COMMAND (cmd_logwipe) {
+COMMAND(cmd_logwipe)
+{
   if (SW_ISSET(sw, SWITCH_CHECK))
     do_logwipe(player, LT_CHECK, arg_left);
   else if (SW_ISSET(sw, SWITCH_CMD))
@@ -511,11 +593,13 @@ COMMAND (cmd_logwipe) {
     do_logwipe(player, LT_ERR, arg_left);
 }
 
-COMMAND (cmd_lset) {
+COMMAND(cmd_lset)
+{
   do_lset(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_mail) {
+COMMAND(cmd_mail)
+{
   int urgent = SW_ISSET(sw, SWITCH_URGENT);
   int silent = SW_ISSET(sw, SWITCH_SILENT);
   int nosig = SW_ISSET(sw, SWITCH_NOSIG);
@@ -569,7 +653,8 @@ COMMAND (cmd_mail) {
 }
 
 
-COMMAND (cmd_malias) {
+COMMAND(cmd_malias)
+{
   if (SW_ISSET(sw, SWITCH_LIST))
     do_malias_list(player);
   else if (SW_ISSET(sw, SWITCH_ALL))
@@ -604,14 +689,54 @@ COMMAND (cmd_malias) {
     do_malias(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_map) {
+COMMAND(cmd_map)
+{
   unsigned int flags = DOL_MAP;
   if (SW_ISSET(sw, SWITCH_DELIMIT))
     flags |= DOL_DELIM;
   do_dolist(player, arg_left, arg_right, cause, flags);
 }
 
-COMMAND (cmd_motd) {
+COMMAND(cmd_message)
+{
+  char *message;
+  char *attrib;
+  unsigned int flags = 0;
+  int numargs, i;
+  char *args[10];
+
+  if (!(SW_ISSET(sw, SWITCH_SPOOF) && (controls(player, cause)
+                                       || Can_Nspemit(player)))) {
+    cause = player;
+  }
+
+  for (numargs = 1; args_right[numargs] && numargs < 13; numargs++) ;
+
+  switch (numargs) {
+  case 1:
+    notify(player, T("@message them with what?"));
+    return;
+  case 2:
+    notify(player, T("Use what attribute for the @message?"));
+    return;
+  }
+  if (!*arg_left) {
+    notify(player, T("@message who?"));
+    return;
+  }
+
+  message = args_right[1];
+  attrib = args_right[2];
+
+  for (i = 0; (i + 3) < numargs; i++) {
+    args[i] = args_right[i + 3];
+  }
+
+  do_message_list(player, cause, arg_left, attrib, message, flags, i, args);
+}
+
+COMMAND(cmd_motd)
+{
   if (SW_ISSET(sw, SWITCH_CONNECT))
     do_motd(player, MOTD_MOTD, arg_left);
   else if (SW_ISSET(sw, SWITCH_LIST))
@@ -626,45 +751,62 @@ COMMAND (cmd_motd) {
     do_motd(player, MOTD_MOTD, arg_left);
 }
 
-COMMAND (cmd_mvattr) {
+COMMAND(cmd_mvattr)
+{
   do_cpattr(player, arg_left, args_right, 1, SW_ISSET(sw, SWITCH_NOFLAGCOPY));
 }
 
-COMMAND (cmd_name) {
+COMMAND(cmd_name)
+{
   do_name(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_newpassword) {
+COMMAND(cmd_newpassword)
+{
   do_newpassword(player, cause, arg_left, arg_right);
 }
 
-COMMAND (cmd_nuke) {
+COMMAND(cmd_nuke)
+{
   do_destroy(player, arg_left, 1);
 }
 
-COMMAND (cmd_oemit) {
+COMMAND(cmd_oemit)
+{
   int spflags = !strcmp(cmd->name, "@NSOEMIT") ? PEMIT_SPOOF : 0;
   SPOOF(player, cause, sw);
   do_oemit_list(player, arg_left, arg_right, spflags);
 }
 
-COMMAND (cmd_open) {
+COMMAND(cmd_open)
+{
   do_open(player, arg_left, args_right);
 }
 
-COMMAND (cmd_parent) {
+COMMAND(cmd_parent)
+{
   do_parent(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_password) {
+COMMAND(cmd_password)
+{
   do_password(player, cause, arg_left, arg_right);
 }
 
-COMMAND (cmd_pcreate) {
-  do_pcreate(player, arg_left, arg_right);
+COMMAND(cmd_pcreate)
+{
+  const char *newdbref;
+
+  if (args_right[2] && *args_right[2])
+    newdbref = args_right[2];
+  else
+    newdbref = NULL;
+
+  do_pcreate(player, arg_left, args_right[1], newdbref);
 }
 
-COMMAND (cmd_pemit) {
+COMMAND(cmd_pemit)
+{
   int flags;
 
   SPOOF(player, cause, sw);
@@ -685,7 +827,8 @@ COMMAND (cmd_pemit) {
     do_pemit(player, arg_left, arg_right, flags);
 }
 
-COMMAND (cmd_prompt) {
+COMMAND(cmd_prompt)
+{
   int flags;
   SPOOF(player, cause, sw);
   if (SW_ISSET(sw, SWITCH_SILENT))
@@ -699,15 +842,18 @@ COMMAND (cmd_prompt) {
   do_pemit_list(player, arg_left, arg_right, flags | PEMIT_PROMPT);
 }
 
-COMMAND (cmd_poll) {
+COMMAND(cmd_poll)
+{
   do_poll(player, arg_left, SW_ISSET(sw, SWITCH_CLEAR));
 }
 
-COMMAND (cmd_poor) {
+COMMAND(cmd_poor)
+{
   do_poor(player, arg_left);
 }
 
-COMMAND (cmd_power) {
+COMMAND(cmd_power)
+{
   if (SW_ISSET(sw, SWITCH_LIST))
     do_list_flags("POWER", player, arg_left, 0, T("Powers"));
   else if (SW_ISSET(sw, SWITCH_ADD))
@@ -730,7 +876,8 @@ COMMAND (cmd_power) {
     do_power(player, arg_left, args_right[1]);
 }
 
-COMMAND (cmd_ps) {
+COMMAND(cmd_ps)
+{
   if (SW_ISSET(sw, SWITCH_ALL))
     do_queue(player, arg_left, QUEUE_ALL);
   else if (SW_ISSET(sw, SWITCH_SUMMARY) || SW_ISSET(sw, SWITCH_COUNT))
@@ -741,11 +888,13 @@ COMMAND (cmd_ps) {
     do_queue(player, arg_left, QUEUE_NORMAL);
 }
 
-COMMAND (cmd_purge) {
+COMMAND(cmd_purge)
+{
   do_purge(player);
 }
 
-COMMAND (cmd_quota) {
+COMMAND(cmd_quota)
+{
   if (SW_ISSET(sw, SWITCH_ALL))
     do_allquota(player, arg_left, (SW_ISSET(sw, SWITCH_QUIET)));
   else if (SW_ISSET(sw, SWITCH_SET))
@@ -754,11 +903,13 @@ COMMAND (cmd_quota) {
     do_quota(player, arg_left, "", 0);
 }
 
-COMMAND (cmd_readcache) {
+COMMAND(cmd_readcache)
+{
   do_readcache(player);
 }
 
-COMMAND (cmd_remit) {
+COMMAND(cmd_remit)
+{
   int flags;
   SPOOF(player, cause, sw);
   if (SW_ISSET(sw, SWITCH_SILENT))
@@ -774,23 +925,27 @@ COMMAND (cmd_remit) {
   do_remit(player, arg_left, arg_right, flags);
 }
 
-COMMAND (cmd_rejectmotd) {
+COMMAND(cmd_rejectmotd)
+{
   do_motd(player, MOTD_DOWN, arg_left);
 }
 
-COMMAND (cmd_restart) {
+COMMAND(cmd_restart)
+{
   if (SW_ISSET(sw, SWITCH_ALL))
     do_allrestart(player);
   else
     do_restart_com(player, arg_left);
 }
 
-COMMAND (cmd_rwall) {
+COMMAND(cmd_rwall)
+{
   do_wall(player, arg_left, WALL_RW, SW_ISSET(sw, SWITCH_EMIT));
 }
 
 
-COMMAND (cmd_scan) {
+COMMAND(cmd_scan)
+{
   if (SW_ISSET(sw, SWITCH_ROOM))
     do_scan(player, arg_left, CHECK_NEIGHBORS | CHECK_HERE);
   else if (SW_ISSET(sw, SWITCH_SELF))
@@ -804,20 +959,24 @@ COMMAND (cmd_scan) {
             CHECK_SELF | CHECK_HERE | CHECK_ZONE | CHECK_GLOBAL);
 }
 
-COMMAND (cmd_search) {
+COMMAND(cmd_search)
+{
   do_search(player, arg_left, args_right);
 }
 
-COMMAND (cmd_select) {
+COMMAND(cmd_select)
+{
   do_switch(player, arg_left, args_right, cause, 1,
             SW_ISSET(sw, SWITCH_NOTIFY), SW_ISSET(sw, SWITCH_REGEXP));
 }
 
-COMMAND (cmd_set) {
+COMMAND(cmd_set)
+{
   do_set(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_shutdown) {
+COMMAND(cmd_shutdown)
+{
   enum shutdown_type paranoid;
   if (!Wizard(player)) {
     notify(player, T("You don't have the authority to do that!"));
@@ -832,7 +991,8 @@ COMMAND (cmd_shutdown) {
     do_shutdown(player, paranoid);
 }
 
-COMMAND (cmd_sitelock) {
+COMMAND(cmd_sitelock)
+{
   if (SW_ISSET(sw, SWITCH_BAN))
     do_sitelock(player, arg_left, NULL, NULL, SITELOCK_BAN);
   else if (SW_ISSET(sw, SWITCH_REGISTER))
@@ -849,7 +1009,8 @@ COMMAND (cmd_sitelock) {
     do_sitelock(player, arg_left, args_right[1], args_right[2], SITELOCK_ADD);
 }
 
-COMMAND (cmd_stats) {
+COMMAND(cmd_stats)
+{
   if (SW_ISSET(sw, SWITCH_TABLES))
     do_list_memstats(player);
   else if (SW_ISSET(sw, SWITCH_CHUNKS)) {
@@ -867,7 +1028,8 @@ COMMAND (cmd_stats) {
     do_stats(player, arg_left);
 }
 
-COMMAND (cmd_sweep) {
+COMMAND(cmd_sweep)
+{
   if (SW_ISSET(sw, SWITCH_CONNECTED))
     do_sweep(player, "connected");
   else if (SW_ISSET(sw, SWITCH_HERE))
@@ -880,17 +1042,20 @@ COMMAND (cmd_sweep) {
     do_sweep(player, arg_left);
 }
 
-COMMAND (cmd_switch) {
+COMMAND(cmd_switch)
+{
   do_switch(player, arg_left, args_right, cause, SW_ISSET(sw, SWITCH_FIRST),
             SW_ISSET(sw, SWITCH_NOTIFY), SW_ISSET(sw, SWITCH_REGEXP));
 }
 
-COMMAND (cmd_squota) {
+COMMAND(cmd_squota)
+{
   do_quota(player, arg_left, arg_right, 1);
 }
 
 
-COMMAND (cmd_teleport) {
+COMMAND(cmd_teleport)
+{
   if (rhs_present && !*arg_right)
     notify(player, T("You can't teleport to nothing!"));
   else
@@ -898,58 +1063,74 @@ COMMAND (cmd_teleport) {
                 (SW_ISSET(sw, SWITCH_INSIDE)));
 }
 
-COMMAND (cmd_trigger) {
+COMMAND(cmd_trigger)
+{
   do_trigger(player, arg_left, args_right);
 }
 
-COMMAND (cmd_ulock) {
+COMMAND(cmd_ulock)
+{
   do_lock(player, arg_left, arg_right, Use_Lock);
 }
 
-COMMAND (cmd_undestroy) {
+COMMAND(cmd_undestroy)
+{
   do_undestroy(player, arg_left);
 }
 
-COMMAND (cmd_unlink) {
+COMMAND(cmd_unlink)
+{
   do_unlink(player, arg_left);
 }
 
-COMMAND (cmd_unlock) {
+COMMAND(cmd_unlock)
+{
   if ((switches) && (switches[0]))
     do_unlock(player, arg_left, switches);
   else
     do_unlock(player, arg_left, Basic_Lock);
 }
 
-COMMAND (cmd_uptime) {
+COMMAND(cmd_uptime)
+{
   do_uptime(player, SW_ISSET(sw, SWITCH_MORTAL));
 }
 
-COMMAND (cmd_uunlock) {
+COMMAND(cmd_uunlock)
+{
   do_unlock(player, arg_left, Use_Lock);
 }
 
-COMMAND (cmd_verb) {
+COMMAND(cmd_verb)
+{
   do_verb(player, cause, arg_left, args_right);
 }
 
-COMMAND (cmd_version) {
+COMMAND(cmd_version)
+{
   do_version(player);
 }
 
-COMMAND (cmd_wait) {
-  do_wait(player, cause, arg_left, arg_right, SW_ISSET(sw, SWITCH_UNTIL));
+COMMAND(cmd_wait)
+{
+  if (SW_BY_NAME(sw, "PID"))
+    do_waitpid(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_UNTIL));
+  else
+    do_wait(player, cause, arg_left, arg_right, SW_ISSET(sw, SWITCH_UNTIL));
 }
 
-COMMAND (cmd_wall) {
+COMMAND(cmd_wall)
+{
   do_wall(player, arg_left, WALL_ALL, SW_ISSET(sw, SWITCH_EMIT));
 }
 
-COMMAND (cmd_warnings) {
+COMMAND(cmd_warnings)
+{
   do_warnings(player, arg_left, arg_right);
 }
 
-COMMAND (cmd_wcheck) {
+COMMAND(cmd_wcheck)
+{
   if (SW_ISSET(sw, SWITCH_ALL))
     do_wcheck_all(player);
   else if (SW_ISSET(sw, SWITCH_ME))
@@ -958,36 +1139,44 @@ COMMAND (cmd_wcheck) {
     do_wcheck(player, arg_left);
 }
 
-COMMAND (cmd_whereis) {
+COMMAND(cmd_whereis)
+{
   do_whereis(player, arg_left);
 }
 
-COMMAND (cmd_wipe) {
+COMMAND(cmd_wipe)
+{
   do_wipe(player, arg_left);
 }
 
-COMMAND (cmd_wizwall) {
+COMMAND(cmd_wizwall)
+{
   do_wall(player, arg_left, WALL_WIZ, SW_ISSET(sw, SWITCH_EMIT));
 }
 
-COMMAND (cmd_wizmotd) {
+COMMAND(cmd_wizmotd)
+{
   do_motd(player, MOTD_WIZ, arg_left);
 }
 
-COMMAND (cmd_zemit) {
+COMMAND(cmd_zemit)
+{
   int flags = !strcmp(cmd->name, "@NSZEMIT") ? PEMIT_SPOOF : 0;
   do_zemit(player, arg_left, arg_right, flags);
 }
 
-COMMAND (cmd_brief) {
+COMMAND(cmd_brief)
+{
   do_examine(player, arg_left, EXAM_BRIEF, 0, 0);
 }
 
-COMMAND (cmd_drop) {
+COMMAND(cmd_drop)
+{
   do_drop(player, arg_left);
 }
 
-COMMAND (cmd_examine) {
+COMMAND(cmd_examine)
+{
   int all = SW_ISSET(sw, SWITCH_ALL);
   int parent = SW_ISSET(sw, SWITCH_PARENT);
   if (SW_ISSET(sw, SWITCH_BRIEF))
@@ -1000,35 +1189,43 @@ COMMAND (cmd_examine) {
     do_examine(player, arg_left, EXAM_NORMAL, all, parent);
 }
 
-COMMAND (cmd_empty) {
+COMMAND(cmd_empty)
+{
   do_empty(player, arg_left);
 }
 
-COMMAND (cmd_enter) {
+COMMAND(cmd_enter)
+{
   do_enter(player, arg_left);
 }
 
-COMMAND (cmd_dismiss) {
+COMMAND(cmd_dismiss)
+{
   do_dismiss(player, arg_left);
 }
 
-COMMAND (cmd_desert) {
+COMMAND(cmd_desert)
+{
   do_desert(player, arg_left);
 }
 
-COMMAND (cmd_follow) {
+COMMAND(cmd_follow)
+{
   do_follow(player, arg_left);
 }
 
-COMMAND (cmd_unfollow) {
+COMMAND(cmd_unfollow)
+{
   do_unfollow(player, arg_left);
 }
 
-COMMAND (cmd_get) {
+COMMAND(cmd_get)
+{
   do_get(player, arg_left);
 }
 
-COMMAND (cmd_buy) {
+COMMAND(cmd_buy)
+{
   char *from = NULL;
   char *forwhat = NULL;
   int price = -1;
@@ -1061,31 +1258,38 @@ COMMAND (cmd_buy) {
   do_buy(player, arg_left, from, price);
 }
 
-COMMAND (cmd_give) {
+COMMAND(cmd_give)
+{
   do_give(player, arg_left, arg_right, (SW_ISSET(sw, SWITCH_SILENT)));
 }
 
-COMMAND (cmd_goto) {
+COMMAND(cmd_goto)
+{
   move_wrapper(player, arg_left);
 }
 
-COMMAND (cmd_inventory) {
+COMMAND(cmd_inventory)
+{
   do_inventory(player);
 }
 
-COMMAND (cmd_kill) {
+COMMAND(cmd_kill)
+{
   do_kill(player, arg_left, atol(arg_right), 0);
 }
 
-COMMAND (cmd_look) {
+COMMAND(cmd_look)
+{
   do_look_at(player, arg_left, (SW_ISSET(sw, SWITCH_OUTSIDE)));
 }
 
-COMMAND (cmd_leave) {
+COMMAND(cmd_leave)
+{
   do_leave(player);
 }
 
-COMMAND (cmd_page) {
+COMMAND(cmd_page)
+{
   if (SW_ISSET(sw, SWITCH_PORT))
     do_page_port(player, arg_left, arg_right);
   else
@@ -1095,45 +1299,55 @@ COMMAND (cmd_page) {
             SW_ISSET(sw, SWITCH_OVERRIDE), rhs_present);
 }
 
-COMMAND (cmd_pose) {
+COMMAND(cmd_pose)
+{
   do_pose(player, arg_left, (SW_ISSET(sw, SWITCH_NOSPACE)));
 }
 
-COMMAND (cmd_say) {
+COMMAND(cmd_say)
+{
   do_say(player, arg_left);
 }
 
-COMMAND (cmd_score) {
+COMMAND(cmd_score)
+{
   do_score(player);
 }
 
-COMMAND (cmd_semipose) {
+COMMAND(cmd_semipose)
+{
   do_pose(player, arg_left, 1);
 }
 
-COMMAND (cmd_slay) {
+COMMAND(cmd_slay)
+{
   do_kill(player, arg_left, 0, 1);
 }
 
-COMMAND (cmd_take) {
+COMMAND(cmd_take)
+{
   do_get(player, arg_left);
 }
 
-COMMAND (cmd_think) {
+COMMAND(cmd_think)
+{
   do_think(player, arg_left);
 }
 
-COMMAND (cmd_whisper) {
+COMMAND(cmd_whisper)
+{
   do_whisper(player, arg_left, arg_right,
              (SW_ISSET(sw, SWITCH_NOISY) ||
               (!SW_ISSET(sw, SWITCH_SILENT) && NOISY_WHISPER)));
 }
 
-COMMAND (cmd_use) {
+COMMAND(cmd_use)
+{
   do_use(player, arg_left);
 }
 
-COMMAND (command_atrset) {
+COMMAND(command_atrset)
+{
   dbref thing;
 
   if ((thing = match_controlled(player, arg_left)) == NOTHING)
@@ -1144,15 +1358,17 @@ COMMAND (command_atrset) {
     do_set_atr(thing, switches, arg_right, player,
                0x1 | (SW_ISSET(sw, SWITCH_NOEVAL) ? 0 : 0x02));
   } else {
-    do_set_atr(thing, switches, NULL, player, 1);
+    do_set_atr(thing, switches, NULL, player, 0x1);
   }
 }
 
-COMMAND (cmd_null) {
+COMMAND(cmd_null)
+{
   return;
 }
 
-COMMAND (cmd_warn_on_missing) {
+COMMAND(cmd_warn_on_missing)
+{
   notify_format(Owner(player),
                 T
                 ("No command found in code by %s - don't start code with functions."),
