@@ -13,6 +13,8 @@
 #include "config.h"
 
 #include <string.h>
+#include <assert.h>
+#include <stdlib.h>
 
 #include "conf.h"
 #include "externs.h"
@@ -22,17 +24,18 @@
 #include "match.h"
 #include "attrib.h"
 #include "extmail.h"
-#include "getpgsiz.h"
 #include "parse.h"
 #include "access.h"
 #include "version.h"
 #include "ptab.h"
 #include "htab.h"
+#include "strtree.h"
 #include "function.h"
 #include "command.h"
 #include "mymalloc.h"
 #include "flags.h"
 #include "log.h"
+#include "sort.h"
 #include "cmds.h"
 #include "confmagic.h"
 
@@ -41,17 +44,33 @@ PTAB ptab_command_perms;        /**< Prefix table for command permissions */
 
 HASHTAB htab_reserved_aliases;  /**< Hash table for reserved command aliases */
 
+slab *command_slab = NULL; /**< slab for command_info structs */
+
 static const char *command_isattr(char *command);
 static int command_check(dbref player, COMMAND_INFO *cmd);
-static int switch_find(COMMAND_INFO *cmd, char *sw);
+static int switch_find(COMMAND_INFO *cmd, const char *sw);
 static void strccat(char *buff, char **bp, const char *from);
 static int has_hook(struct hook_data *hook);
-extern int global_fun_invocations;      /**< Counter for function invocations */
+extern int global_fun_invocations;       /**< Counter for function invocations */
 extern int global_fun_recursions;       /**< Counter for function recursion */
 
+SWITCH_VALUE *dyn_switch_list = NULL;
+int switch_bytes = 0;
+size_t num_switches = 0;
+
+enum command_load_state { CMD_LOAD_BUILTIN,
+  CMD_LOAD_LOCAL,
+  CMD_LOAD_DONE
+};
+static enum command_load_state command_state = CMD_LOAD_BUILTIN;
+static StrTree switch_names;
+
 int run_hook(dbref player, dbref cause, struct hook_data *hook,
              char *saveregs[], int save);
 
+int run_hook_override(COMMAND_INFO *cmd, dbref player, const char *commandraw);
+
+
 /** The list of standard commands. Additional commands can be added
  * at runtime with add_command().
  */
@@ -74,10 +93,10 @@ COMLIST commands[] = {
   {"@BOOT", "PORT ME SILENT", cmd_boot, CMD_T_ANY, 0, 0},
   {"@BREAK", NULL, cmd_break, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE, 0,
    0},
-  {"@CEMIT", "NOEVAL NOISY SPOOF", cmd_cemit,
+  {"@CEMIT", "NOEVAL NOISY SILENT SPOOF", cmd_cemit,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
   {"@CHANNEL",
-   "LIST ADD DELETE RENAME NAME PRIVS QUIET NOISY DECOMPILE DESCRIBE CHOWN WIPE MUTE UNMUTE GAG UNGAG HIDE UNHIDE WHAT TITLE BRIEF RECALL BUFFER SET",
+   "LIST ADD DELETE RENAME MOGRIFIER NAME PRIVS QUIET NOISY DECOMPILE DESCRIBE CHOWN WIPE MUTE UNMUTE GAG UNGAG HIDE UNHIDE WHAT TITLE BRIEF RECALL BUFFER SET",
    cmd_channel,
    CMD_T_ANY | CMD_T_SWITCHES | CMD_T_EQSPLIT | CMD_T_NOGAGGED | CMD_T_RS_ARGS,
    0, 0},
@@ -92,12 +111,13 @@ COMLIST commands[] = {
   {"@CHZONE", NULL, cmd_chzone,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
   {"@CONFIG",
-   "SET LOWERCASE LIST GLOBALS DEFAULTS COSTS FLAGS FUNCTIONS COMMANDS ATTRIBS",
+   "SET SAVE LOWERCASE LIST GLOBALS DEFAULTS COSTS FLAGS FUNCTIONS COMMANDS ATTRIBS",
    cmd_config, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@CPATTR", "CONVERT NOFLAGCOPY", cmd_cpattr,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS,
    0, 0},
-  {"@CREATE", NULL, cmd_create, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED,
+  {"@CREATE", NULL, cmd_create,
+   CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS | CMD_T_NOGAGGED,
    0, 0},
   {"@CLONE", "PRESERVE", cmd_clone, CMD_T_ANY | CMD_T_NOGAGGED | CMD_T_EQSPLIT,
    0, 0},
@@ -143,15 +163,16 @@ COMLIST commands[] = {
 
   {"@FORCE", "NOEVAL", cmd_force, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED,
    0, 0},
-  {"@FUNCTION", "DELETE ENABLE DISABLE PRESERVE RESTORE RESTRICT", cmd_function,
+  {"@FUNCTION", "BUILTIN DELETE ENABLE DISABLE PRESERVE RESTORE RESTRICT",
+   cmd_function,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS | CMD_T_NOGAGGED, 0, 0},
   {"@GREP", "LIST PRINT ILIST IPRINT", cmd_grep,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE | CMD_T_NOGAGGED, 0, 0},
-  {"@HALT", "ALL", cmd_halt, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
+  {"@HALT", "ALL PID", cmd_halt, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@HIDE", "NO OFF YES ON", cmd_hide, CMD_T_ANY, 0, 0},
   {"@HOOK", "LIST AFTER BEFORE IGNORE OVERRIDE", cmd_hook,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS,
-   "WIZARD", 0},
+   "WIZARD", "hook"},
   {"@KICK", NULL, cmd_kick, CMD_T_ANY, "WIZARD", 0},
 
   {"@LEMIT", "NOEVAL SILENT SPOOF", cmd_lemit,
@@ -181,6 +202,8 @@ COMLIST commands[] = {
 
   {"@MAP", "DELIMIT", cmd_map, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE,
    0, 0},
+  {"@MESSAGE", "NOEVAL SPOOF", cmd_message,
+   CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS, 0, 0},
   {"@MOTD", "CONNECT LIST WIZARD DOWN FULL", cmd_motd,
    CMD_T_ANY | CMD_T_NOGAGGED, 0, 0},
   {"@MVATTR", "CONVERT NOFLAGCOPY", cmd_mvattr,
@@ -216,7 +239,8 @@ COMLIST commands[] = {
   {"@PARENT", NULL, cmd_parent, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@PASSWORD", NULL, cmd_password, CMD_T_PLAYER | CMD_T_EQSPLIT
    | CMD_T_NOPARSE | CMD_T_RS_NOPARSE | CMD_T_NOGUEST, 0, 0},
-  {"@PCREATE", NULL, cmd_pcreate, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
+  {"@PCREATE", NULL, cmd_pcreate, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS, 0,
+   0},
 
   {"@PEMIT", "LIST CONTENTS SILENT NOISY NOEVAL SPOOF", cmd_pemit,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
@@ -245,7 +269,7 @@ COMLIST commands[] = {
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS | CMD_T_RS_NOPARSE, 0, 0},
   {"@SET", NULL, cmd_set, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
   {"@SHUTDOWN", "PANIC REBOOT PARANOID", cmd_shutdown, CMD_T_ANY, "WIZARD", 0},
-#ifdef HAS_MYSQL
+#if defined(HAVE_MYSQL) || defined(HAVE_POSTGRESQL) || defined(HAVE_SQLITE3)
   {"@SQL", NULL, cmd_sql, CMD_T_ANY, "WIZARD", "SQL_OK"},
 #else
   {"@SQL", NULL, cmd_unimplemented, CMD_T_ANY, "WIZARD", "SQL_OK"},
@@ -277,7 +301,7 @@ COMLIST commands[] = {
   {"@UUNLOCK", NULL, cmd_uunlock, CMD_T_ANY | CMD_T_NOGAGGED, 0, 0},
   {"@VERB", NULL, cmd_verb, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS, 0, 0},
   {"@VERSION", NULL, cmd_version, CMD_T_ANY, 0, 0},
-  {"@WAIT", "UNTIL", cmd_wait, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE,
+  {"@WAIT", "PID UNTIL", cmd_wait, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE,
    0, 0},
   {"@WALL", "NOEVAL EMIT", cmd_wall, CMD_T_ANY, "WIZARD ROYALTY", "ANNOUNCE"},
 
@@ -390,7 +414,6 @@ struct command_perms_t command_perms[] = {
   {NULL, 0}
 };
 
-
 static void
 strccat(char *buff, char **bp, const char *from)
 {
@@ -398,35 +421,61 @@ strccat(char *buff, char **bp, const char *from)
     safe_str(", ", buff, bp);
   safe_str(from, buff, bp);
 }
+
+/* Comparison function for bsearch() */
 static int
-switch_find(COMMAND_INFO *cmd, char *sw)
+switch_cmp(const void *a, const void *b)
+{
+  const char *name = a;
+  const SWITCH_VALUE *sw = b;
+  return strcmp(name, sw->name);
+}
+
+/* This has different semantics than a prefix table, or we'd use that. */
+static int
+switch_find(COMMAND_INFO *cmd, const char *sw)
 {
   SWITCH_VALUE *sw_val;
-  int i = 0;
-  int len;
-  if (!sw || !*sw)
+
+  if (!sw || !*sw || !dyn_switch_list)
     return 0;
-  len = strlen(sw);
-  /* Special case, for init */
-  sw_val = switch_list;
+
   if (!cmd) {
-    while (sw_val->name) {
-      if (strcmp(sw_val->name, sw) == 0)
-        return sw_val->value;
-      sw_val++;
-    }
-    return 0;
+    sw_val = bsearch(sw, dyn_switch_list, num_switches, sizeof(SWITCH_VALUE),
+                     switch_cmp);
+    if (sw_val)
+      return sw_val->value;
+    else
+      return 0;
   } else {
+    size_t len = strlen(sw);
+    if (!cmd->sw.mask)
+      return 0;
+    sw_val = dyn_switch_list;
     while (sw_val->name) {
-      i++;
-      if (SW_ISSET(cmd->sw, i) && (strncmp(sw_val->name, sw, len) == 0))
-        return i;
+      if (SW_ISSET(cmd->sw.mask, sw_val->value)
+          && (strncmp(sw_val->name, sw, len) == 0))
+        return sw_val->value;
       sw_val++;
     }
   }
   return 0;
 }
 
+/** Test if a particular switch was given, using name
+ * \param sw the switch mask to test
+ * \param name the name of the switch to test for.
+ */
+bool
+SW_BY_NAME(switch_mask sw, const char *name)
+{
+  int idx = switch_find(NULL, name);
+  if (idx)
+    return SW_ISSET(sw, idx);
+  else
+    return false;
+}
+
 /** Allocate and populate a COMMAND_INFO structure.
  * This function generates a new COMMAND_INFO structure, populates it
  * with given values, and returns a pointer. It should not be used
@@ -442,10 +491,10 @@ switch_find(COMMAND_INFO *cmd, char *sw)
 COMMAND_INFO *
 make_command(const char *name, int type,
              object_flag_type flagmask, object_flag_type powers,
-             switch_mask *sw, command_func func)
+             const char *sw, command_func func)
 {
   COMMAND_INFO *cmd;
-  cmd = (COMMAND_INFO *) mush_malloc(sizeof(COMMAND_INFO), "command");
+  cmd = slab_malloc(command_slab, NULL);
   memset(cmd, 0, sizeof(COMMAND_INFO));
   cmd->name = name;
   cmd->restrict_message = NULL;
@@ -453,10 +502,31 @@ make_command(const char *name, int type,
   cmd->type = type;
   cmd->flagmask = flagmask;
   cmd->powers = powers;
-  if (sw)
-    memcpy(cmd->sw, sw, sizeof(switch_mask));
-  else
-    SW_ZERO(cmd->sw);
+  switch (command_state) {
+  case CMD_LOAD_BUILTIN:
+    cmd->sw.names = sw;
+    break;
+  case CMD_LOAD_LOCAL:{
+      char sw_copy[BUFFER_LEN];
+      char *pos;
+      cmd->sw.names = sw;
+      mush_strncpy(sw_copy, sw, BUFFER_LEN);
+      pos = sw_copy;
+      while (pos) {
+        char *thisone = split_token(&pos, ' ');
+        st_insert(thisone, &switch_names);
+      }
+      break;
+    }
+  case CMD_LOAD_DONE:{
+      switch_mask mask = switchmask(sw);
+      if (mask) {
+        cmd->sw.mask = SW_ALLOC();
+        SW_COPY(cmd->sw.mask, mask);
+      } else
+        cmd->sw.mask = NULL;
+    }
+  }
   cmd->hooks.before.obj = NOTHING;
   cmd->hooks.before.attrname = NULL;
   cmd->hooks.after.obj = NOTHING;
@@ -483,16 +553,13 @@ command_add(const char *name, int type, const char *flagstr,
             const char *powerstr, const char *switchstr, command_func func)
 {
   object_flag_type flagmask = NULL, powers = NULL;
-  switch_mask *sw = switchmask(switchstr);
 
   if (flagstr)
     flagmask = string_to_bits("FLAG", flagstr);
   if (powerstr)
     powers = string_to_bits("POWER", powerstr);
-  ptab_start_inserts(&ptab_command);
-  ptab_insert(&ptab_command, name,
-              make_command(name, type, flagmask, powers, sw, func));
-  ptab_end_inserts(&ptab_command);
+  ptab_insert_one(&ptab_command, name,
+                  make_command(name, type, flagmask, powers, switchstr, func));
   return command_find(name);
 }
 
@@ -552,7 +619,7 @@ command_find_exact(const char *name)
 COMMAND_INFO *
 command_modify(const char *name, int type,
                object_flag_type flagmask, object_flag_type powers,
-               switch_mask *sw, command_func func)
+               switch_mask sw, command_func func)
 {
   COMMAND_INFO *cmd;
   cmd = command_find(name);
@@ -565,7 +632,7 @@ command_modify(const char *name, int type,
   if (powers)
     cmd->powers = powers;
   if (sw)
-    memcpy(cmd->sw, sw, sizeof(switch_mask));
+    SW_COPY(cmd->sw.mask, sw);
   if (func)
     cmd->func = func;
   return cmd;
@@ -577,13 +644,20 @@ command_modify(const char *name, int type,
  * \param switches list of switches as a string.
  * \return pointer to a static switch mask.
  */
-switch_mask *
+switch_mask
 switchmask(const char *switches)
 {
-  static switch_mask sw;
+  static switch_mask sw = NULL;
+  static int sm_bytes = 0;
   char buff[BUFFER_LEN];
   char *p, *s;
   int switchnum;
+
+  if (sm_bytes < switch_bytes) {
+    sw = mush_realloc(sw, switch_bytes, "cmd.switch.vector");
+    sm_bytes = switch_bytes;
+  }
+
   SW_ZERO(sw);
   if (!switches || !switches[0])
     return NULL;
@@ -597,7 +671,7 @@ switchmask(const char *switches)
     else
       SW_SET(sw, switchnum);
   }
-  return &sw;
+  return sw;
 }
 
 /** Add an alias to the table of reserved aliases.
@@ -614,6 +688,8 @@ reserve_alias(const char *a)
   hashadd(strupper(a), (void *) placeholder, &htab_reserved_aliases);
 }
 
+static StrTree switch_names;
+
 /** Initialize command tables (before reading config file).
  * This function performs command table initialization that should take place
  * before the configuration file has been read. It initializes the
@@ -628,21 +704,41 @@ command_init_preconfig(void)
 {
   struct command_perms_t *c;
   COMLIST *cmd;
+  SWITCH_VALUE *sv;
   static int done = 0;
   if (done == 1)
     return;
   done = 1;
 
   ptab_init(&ptab_command);
-  hashinit(&htab_reserved_aliases, 16, sizeof(COMMAND_INFO));
+  hashinit(&htab_reserved_aliases, 16);
+
+  /* Build initial switch table. */
+  st_init(&switch_names);
+  for (sv = switch_list; sv->name; sv++)
+    st_insert(sv->name, &switch_names);
+
+  command_slab = slab_create("commands", sizeof(COMMAND_INFO));
   reserve_aliases();
+
   ptab_start_inserts(&ptab_command);
+  command_state = CMD_LOAD_BUILTIN;
   for (cmd = commands; cmd->name; cmd++) {
+    if (cmd->switches) {
+      char sw_copy[BUFFER_LEN];
+      char *pos;
+      strcpy(sw_copy, cmd->switches);
+      pos = sw_copy;
+      while (pos) {
+        char *sw = split_token(&pos, ' ');
+        st_insert(sw, &switch_names);
+      }
+    }
     ptab_insert(&ptab_command, cmd->name,
                 make_command(cmd->name, cmd->type,
                              string_to_bits("FLAG", cmd->flagstr),
                              string_to_bits("POWER", cmd->powers),
-                             switchmask(cmd->switches), cmd->func));
+                             cmd->switches, cmd->func));
   }
   ptab_end_inserts(&ptab_command);
 
@@ -652,9 +748,34 @@ command_init_preconfig(void)
     ptab_insert(&ptab_command_perms, c->name, c);
   ptab_end_inserts(&ptab_command_perms);
 
+  command_state = CMD_LOAD_LOCAL;
   local_commands();
 }
 
+struct bst_data {
+  SWITCH_VALUE *table;
+  size_t n;
+  size_t start;
+};
+
+static void
+build_switch_table(const char *sw, int count __attribute__ ((__unused__)),
+                   void *d)
+{
+  SWITCH_VALUE *s;
+  struct bst_data *data = d;
+
+  for (s = switch_list; s->name; s++) {
+    if (strcmp(s->name, sw) == 0) {
+      data->table[data->n++] = *s;
+      return;
+    }
+  }
+  /* Not in switchinc.c table */
+  data->table[data->n].value = data->start++;
+  data->table[data->n++].name = mush_strdup(sw, "switch.name");
+}
+
 /** Initialize commands (after reading config file).
  * This function performs command initialization that should take place
  * after the configuration file has been read.
@@ -663,6 +784,36 @@ command_init_preconfig(void)
 void
 command_init_postconfig(void)
 {
+  struct bst_data sw_data;
+  COMMAND_INFO *c;
+  size_t sl_size;
+
+  command_state = CMD_LOAD_DONE;
+
+  /* First make the switch table */
+  dyn_switch_list = mush_calloc(switch_names.count + 2, sizeof(SWITCH_VALUE),
+                                "cmd_switch_table");
+  if (!dyn_switch_list)
+    mush_panic(T("Unable to allocate command switch table"));
+  sw_data.table = dyn_switch_list;
+  sw_data.n = 0;
+  sw_data.start = sizeof switch_list / sizeof(SWITCH_VALUE);
+  sl_size = sw_data.start - 2;
+  st_walk(&switch_names, build_switch_table, &sw_data);
+  num_switches = sw_data.start;
+  dyn_switch_list[sw_data.n].name = NULL;
+  st_flush(&switch_names);
+  switch_bytes = ceil((double) num_switches / 8.0);
+
+  /* Then convert the list of switch names in all commands to masks */
+  for (c = ptab_firstentry(&ptab_command); c; c = ptab_nextentry(&ptab_command)) {
+    const char *switchstr = c->sw.names;
+    if (switchstr) {
+      c->sw.mask = SW_ALLOC();
+      SW_COPY(c->sw.mask, switchmask(switchstr));
+    }
+  }
+
   return;
 }
 
@@ -688,9 +839,7 @@ alias_command(const char *command, const char *alias)
   if (!cmd)
     return 0;
 
-  ptab_start_inserts(&ptab_command);
-  ptab_insert(&ptab_command, strupper(alias), cmd);
-  ptab_end_inserts(&ptab_command);
+  ptab_insert_one(&ptab_command, strupper(alias), cmd);
   return 1;
 }
 
@@ -765,8 +914,10 @@ command_argparse(dbref player, dbref cause, char **from, char *to,
     aold = t;
     while (*f == ' ')
       f++;
-    process_expression(to, &t, (const char **) &f, player, cause, cause,
-                       parse, (split | args), NULL);
+    if (process_expression(to, &t, (const char **) &f, player, cause, cause,
+                           parse, (split | args), NULL)) {
+      done = 1;
+    }
     *t = '\0';
     if (args) {
       argv[i] = aold;
@@ -866,7 +1017,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
   char command2[BUFFER_LEN];
   char b;
   int switchnum;
-  switch_mask sw;
+  switch_mask sw = NULL;
   char switch_err[BUFFER_LEN], *se;
   int noeval;
   int noevtoken = 0;
@@ -874,11 +1025,11 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
 
   rhs_present = 0;
 
-  command = (char *) mush_malloc(BUFFER_LEN, "string");
-  swtch = (char *) mush_malloc(BUFFER_LEN, "string");
-  ls = (char *) mush_malloc(BUFFER_LEN, "string");
-  rs = (char *) mush_malloc(BUFFER_LEN, "string");
-  switches = (char *) mush_malloc(BUFFER_LEN, "string");
+  command = mush_malloc(BUFFER_LEN, "string");
+  swtch = mush_malloc(BUFFER_LEN, "string");
+  ls = mush_malloc(BUFFER_LEN, "string");
+  rs = mush_malloc(BUFFER_LEN, "string");
+  switches = mush_malloc(BUFFER_LEN, "string");
   if (!command || !swtch || !ls || !rs || !switches)
     mush_panic("Couldn't allocate memory in command_parse");
   p = string;
@@ -1059,7 +1210,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
     return NULL;
   }
   /* Parse out any switches */
-  SW_ZERO(sw);
+  sw = SW_ALLOC();
   swp = switches;
   *swp = '\0';
   se = switch_err;
@@ -1198,9 +1349,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
     /* If we have a hook/ignore that returns false, we don't do the command */
     if (run_hook(player, cause, &cmd->hooks.ignore, saveregs, 1)) {
       /* If we have a hook/override, we use that instead */
-      if (!has_hook(&cmd->hooks.override) ||
-          !one_comm_match(cmd->hooks.override.obj, player,
-                          cmd->hooks.override.attrname, commandraw)) {
+      if (!run_hook_override(cmd, player, commandraw)) {
         /* Otherwise, we do hook/before, the command, and hook/after */
         /* But first, let's see if we had an invalid switch */
         if (*switch_err) {
@@ -1224,6 +1373,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
     free_global_regs("hook.regs", saveregs);
   }
 
+  SW_FREE(sw);
   command_parse_free_args;
   return retval;
 }
@@ -1289,24 +1439,25 @@ generic_command_failure(dbref player, dbref cause, char *string)
  * \retval 0 failure (unable to find command name).
  */
 int
-restrict_command(const char *name, const char *restriction)
+restrict_command(const char *name, const char *xrestriction)
 {
   COMMAND_INFO *command;
   struct command_perms_t *c;
-  char *message;
+  char *message, *restriction, *rsave;
   int clear;
   FLAG *mask;
   FLAG *powers;
   char *tp;
 
-  if (!name || !*name || !restriction || !*restriction ||
+  if (!name || !*name || !xrestriction || !*xrestriction ||
       !(command = command_find(name)))
     return 0;
 
   if (command->restrict_message) {
-    mush_free((Malloc_t) command->restrict_message, "cmd_restrict_message");
+    mush_free((void *) command->restrict_message, "cmd_restrict_message");
     command->restrict_message = NULL;
   }
+  rsave = restriction = mush_strdup(xrestriction, "rc.string");
   message = strchr(restriction, '"');
   if (message) {
     *(message++) = '\0';
@@ -1367,6 +1518,7 @@ restrict_command(const char *name, const char *restriction)
     }
     restriction = tp;
   }
+  mush_free(rsave, "rc.string");
   return 1;
 }
 
@@ -1374,7 +1526,8 @@ restrict_command(const char *name, const char *restriction)
  * This does nothing more than notify the player
  * with "This command has not been implemented"
  */
-COMMAND (cmd_unimplemented) {
+COMMAND(cmd_unimplemented)
+{
   char *saveregs[NUMQ];
 
   if (strcmp(cmd->name, "UNIMPLEMENTED_COMMAND") != 0 &&
@@ -1483,8 +1636,8 @@ do_command_delete(dbref player, char *name)
         } else
           cptr = ptab_nextentry_new(&ptab_command, alias);
       }
-      mush_free((Malloc_t) command->name, "command_add");
-      mush_free((Malloc_t) command, "command");
+      mush_free((void *) command->name, "command_add");
+      slab_free(command_slab, command);
       if (acount > 1)
         notify_format(player, T("Removed %s and aliases from command table."),
                       name);
@@ -1502,7 +1655,8 @@ do_command_delete(dbref player, char *name)
  * This is the only command which should be defined in this
  * file, because it uses variables from this file, etc.
  */
-COMMAND (cmd_command) {
+COMMAND(cmd_command)
+{
   COMMAND_INFO *command;
   SWITCH_VALUE *sw_val;
   char buff[BUFFER_LEN];
@@ -1613,12 +1767,15 @@ COMMAND (cmd_command) {
     notify_format(player, "Restrict   : %s", buff);
     buff[0] = '\0';
     notify(player, show_command_flags(command->flagmask, command->powers));
-    bp = buff;
-    for (sw_val = switch_list; sw_val->name; sw_val++)
-      if (SW_ISSET(command->sw, sw_val->value))
-        strccat(buff, &bp, sw_val->name);
-    *bp = '\0';
-    notify_format(player, "Switches   : %s", buff);
+    if (command->sw.mask) {
+      bp = buff;
+      for (sw_val = dyn_switch_list; sw_val->name; sw_val++)
+        if (SW_ISSET(command->sw.mask, sw_val->value))
+          strccat(buff, &bp, sw_val->name);
+      *bp = '\0';
+      notify_format(player, "Switches   : %s", buff);
+    } else
+      notify(player, "Switches   :");
     buff[0] = '\0';
     bp = buff;
     if (command->type & CMD_T_LS_ARGS) {
@@ -1662,7 +1819,7 @@ void
 do_list_commands(dbref player, int lc)
 {
   char *b = list_commands();
-  notify_format(player, "Commands: %s", lc ? strlower(b) : b);
+  notify_format(player, T("Commands: %s"), lc ? strlower(b) : b);
 }
 
 /** Return a list of defined commands.
@@ -1672,24 +1829,22 @@ char *
 list_commands(void)
 {
   COMMAND_INFO *command;
-  const char *ptrs[BUFFER_LEN / 2];
+  char *ptrs[BUFFER_LEN / 2];
   static char buff[BUFFER_LEN];
   char *bp;
   int nptrs = 0, i;
   command = (COMMAND_INFO *) ptab_firstentry(&ptab_command);
   while (command) {
-    ptrs[nptrs] = command->name;
+    ptrs[nptrs] = (char *) command->name;
     nptrs++;
     command = (COMMAND_INFO *) ptab_nextentry(&ptab_command);
   }
 
-  do_gensort((dbref) 0, (char **) ptrs, NULL, nptrs, ALPHANUM_LIST);
+  do_gensort(0, ptrs, NULL, nptrs, ALPHANUM_LIST);
   bp = buff;
   safe_str(ptrs[0], buff, &bp);
   for (i = 1; i < nptrs; i++) {
-    if (gencomp
-        ((dbref) 0, (char *) ptrs[i], (char *) ptrs[i - 1],
-         ALPHANUM_LIST) > 0) {
+    if (gencomp((dbref) 0, ptrs[i], ptrs[i - 1], ALPHANUM_LIST) > 0) {
       safe_chr(' ', buff, &bp);
       safe_str(ptrs[i], buff, &bp);
     }
@@ -1797,8 +1952,7 @@ command_check_byname(dbref player, const char *name)
 static int
 has_hook(struct hook_data *hook)
 {
-  if (!hook || !GoodObject(hook->obj) || IsGarbage(hook->obj)
-      || !hook->attrname)
+  if (!hook || !GoodObject(hook->obj) || IsGarbage(hook->obj))
     return 0;
   return 1;
 }
@@ -1855,6 +2009,22 @@ run_hook(dbref player, dbref cause, struct hook_data *hook, char *saveregs[],
   return parse_boolean(buff);
 }
 
+int
+run_hook_override(COMMAND_INFO *cmd, dbref player, const char *commandraw)
+{
+
+  if (!has_hook(&cmd->hooks.override))
+    return 0;
+
+  if (cmd->hooks.override.attrname) {
+    return one_comm_match(cmd->hooks.override.obj, player,
+                          cmd->hooks.override.attrname, commandraw);
+  } else {
+    return atr_comm_match(cmd->hooks.override.obj, player, '$', ':', commandraw,
+                          0, 1, NULL, NULL, NULL);
+  }
+}
+
 /** Set up or remove a command hook.
  * \verbatim
  * This is the top-level function for @hook. If an object and attribute
@@ -1901,8 +2071,13 @@ do_hook(dbref player, char *command, char *obj, char *attrname,
     h->obj = NOTHING;
     mush_free(h->attrname, "hook.attr");
     h->attrname = NULL;
-  } else if (!obj || !*obj || !attrname || !*attrname) {
-    notify(player, T("You must give both an object and attribute."));
+  } else if (!obj || !*obj
+             || (flag != HOOK_OVERRIDE && (!attrname || !*attrname))) {
+    if (flag == HOOK_OVERRIDE) {
+      notify(player, T("You must give an object."));
+    } else {
+      notify(player, T("You must give both an object and attribute."));
+    }
   } else {
     dbref objdb = match_thing(player, obj);
     if (!GoodObject(objdb)) {
@@ -1912,7 +2087,11 @@ do_hook(dbref player, char *command, char *obj, char *attrname,
     h->obj = objdb;
     if (h->attrname)
       mush_free(h->attrname, "hook.attr");
-    h->attrname = mush_strdup(strupper(attrname), "hook.attr");
+    if (!attrname || !*attrname) {
+      h->attrname = NULL;
+    } else {
+      h->attrname = mush_strdup(strupper(attrname), "hook.attr");
+    }
     notify_format(player, T("Hook set for %s"), cmd->name);
   }
 }
@@ -1936,18 +2115,18 @@ do_hook_list(dbref player, char *command)
     notify(player, T("No such command."));
     return;
   }
-  if (Wizard(player)) {
+  if (Wizard(player) || has_power_by_name(player, "HOOK", NOTYPE)) {
     if (GoodObject(cmd->hooks.before.obj))
-      notify_format(player, "@hook/before: #%d/%s",
+      notify_format(player, T("@hook/before: #%d/%s"),
                     cmd->hooks.before.obj, cmd->hooks.before.attrname);
     if (GoodObject(cmd->hooks.after.obj))
-      notify_format(player, "@hook/after: #%d/%s", cmd->hooks.after.obj,
+      notify_format(player, T("@hook/after: #%d/%s"), cmd->hooks.after.obj,
                     cmd->hooks.after.attrname);
     if (GoodObject(cmd->hooks.ignore.obj))
-      notify_format(player, "@hook/ignore: #%d/%s",
+      notify_format(player, T("@hook/ignore: #%d/%s"),
                     cmd->hooks.ignore.obj, cmd->hooks.ignore.attrname);
     if (GoodObject(cmd->hooks.override.obj))
-      notify_format(player, "@hook/override: #%d/%s",
+      notify_format(player, T("@hook/override: #%d/%s"),
                     cmd->hooks.override.obj, cmd->hooks.override.attrname);
   }
 }
