@@ -14,15 +14,20 @@
 #include <stdio.h>
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <errno.h>
 
 #include "conf.h"
 #include "externs.h"
+#include "ansi.h"
 #include "pueblo.h"
 #include "mushdb.h"
 #include "parse.h"
@@ -39,8 +44,7 @@
 time_t mudtime;                 /**< game time, in seconds */
 
 static void show_compile_options(dbref player);
-static char *config_list_helper(dbref player, PENNCONF * cp, int lc);
-static char *config_list_helper2(dbref player, PENNCONF * cp, int lc);
+static char *config_to_string(dbref player, PENNCONF *cp, int lc);
 
 OPTTAB options;         /**< The table of configuration options */
 HASHTAB local_options;  /**< Hash table for local config options */
@@ -159,13 +163,13 @@ PENNCONF conftable[] = {
   ,
   {"zone_control_zmp_only", cf_bool, &options.zone_control, 2, 0, "db"}
   ,
-  {"ancestor_room", cf_int, &options.ancestor_room, 100000, 0, "db"}
+  {"ancestor_room", cf_dbref, &options.ancestor_room, 100000, 0, "db"}
   ,
-  {"ancestor_exit", cf_int, &options.ancestor_exit, 100000, 0, "db"}
+  {"ancestor_exit", cf_dbref, &options.ancestor_exit, 100000, 0, "db"}
   ,
-  {"ancestor_thing", cf_int, &options.ancestor_thing, 100000, 0, "db"}
+  {"ancestor_thing", cf_dbref, &options.ancestor_thing, 100000, 0, "db"}
   ,
-  {"ancestor_player", cf_int, &options.ancestor_player, 100000, 0, "db"}
+  {"ancestor_player", cf_dbref, &options.ancestor_player, 100000, 0, "db"}
   ,
   {"mud_name", cf_str, options.mud_name, 128, 0, "net"}
   ,
@@ -191,7 +195,6 @@ PENNCONF conftable[] = {
   ,
   {"pueblo", cf_bool, &options.support_pueblo, 2, 0, "net"}
   ,
-#ifdef HAS_MYSQL
   {"sql_platform", cf_str, options.sql_platform, sizeof options.sql_platform, 0,
    "net"}
   ,
@@ -206,8 +209,6 @@ PENNCONF conftable[] = {
   {"sql_database", cf_str, options.sql_database, sizeof options.sql_database, 0,
    NULL}
   ,
-#endif
-
   {"forking_dump", cf_bool, &options.forking_dump, 2, 0, "dump"}
   ,
   {"dump_message", cf_str, options.dump_message, sizeof options.dump_message, 0,
@@ -342,8 +343,6 @@ PENNCONF conftable[] = {
   {"queue_entry_cpu_time", cf_int, &options.queue_entry_cpu_time, 100000, 0,
    "limits"}
   ,
-  {"max_global_fns", cf_int, &options.max_global_fns, 2000, 0, 0}
-  ,
   {"use_quota", cf_bool, &options.use_quota, 2, 0, "limits"}
   ,
   {"max_channels", cf_int, &options.max_channels, 1000, 0, "chat"}
@@ -352,7 +351,8 @@ PENNCONF conftable[] = {
   ,
   {"chan_cost", cf_int, &options.chan_cost, 10000, 0, "chat"}
   ,
-
+  {"noisy_cemit", cf_bool, &options.noisy_cemit, 2, 0, "chat"}
+  ,
   {"log_commands", cf_bool, &options.log_commands, 2, 0, "log"}
   ,
   {"log_forces", cf_bool, &options.log_forces, 2, 0, "log"}
@@ -480,6 +480,9 @@ PENNCONF conftable[] = {
   {"chunk_swap_file", cf_str, options.chunk_swap_file,
    sizeof options.chunk_swap_file, 0, "files"}
   ,
+  {"chunk_swap_initial_size", cf_int, &options.chunk_swap_initial, 1000000, 0,
+   "files"}
+  ,
   {"chunk_cache_memory", cf_int, &options.chunk_cache_memory,
    1000000000, 65510 * 2, "files"}
   ,
@@ -535,6 +538,29 @@ PENNCONFGROUP confgroups[] = {
   {NULL, NULL, 0}
 };
 
+#if 0
+/* Just to mark these strings for translation */
+PENNCONFGROUP dummy[] = {
+  {"attribs", T("Options affecting attributes"), 0},
+  {"chat", T("Chat system options"), 0},
+  {"cmds", T("Options affecting command behavior"), 0},
+  {"compile", T("Compile-time options"), 0},
+  {"cosmetic", T("Cosmetic options"), 0},
+  {"costs", T("Costs"), 0},
+  {"db", T("Database options"), 0},
+  {"dump", T("Options affecting dumps and other periodic processes"), 0},
+  {"files", T("Files used by the MUSH"), CGP_GOD},
+  {"flags", T("Default flags for new objects"), 0},
+  {"funcs", T("Options affecting function behavior"), 0},
+  {"limits", T("Limits and other constants"), 0},
+  {"log", T("Logging options"), 0},
+  {"messages", T("Message files sent by the MUSH"), CGP_GOD},
+  {"net", T("Networking and connection-related options"), 0},
+  {"tiny", T("TinyMUSH compatibility options"), 0},
+  {NULL, NULL, 0}
+};
+#endif
+
 /** Returns a pointer to a newly allocated PENNCONF object.
  * \return pointer to newly allocated PENNCONF object.
  */
@@ -591,13 +617,13 @@ get_config(const char *name)
  * \param val value of the option.
  * \param loc address to store the value.
  * \param maxval (unused).
- * \param source 0 if read from config file; 1 if from command.
+ * \param from_cmd 0 if read from config file; 1 if from command.
  * \retval 0 failure (unable to parse val).
  * \retval 1 success.
  */
 int
 cf_bool(const char *opt, const char *val, void *loc,
-        int maxval __attribute__ ((__unused__)), int source)
+        int maxval __attribute__ ((__unused__)), int from_cmd)
 {
   /* enter boolean parameter */
 
@@ -608,8 +634,8 @@ cf_bool(const char *opt, const char *val, void *loc,
            !strcasecmp(val, "0"))
     *((int *) loc) = 0;
   else {
-    if (source == 0) {
-      do_rawlog(LT_ERR, T("CONFIG: option %s value %s invalid.\n"), opt, val);
+    if (from_cmd == 0) {
+      do_rawlog(LT_ERR, T("CONFIG: option %s value %s invalid."), opt, val);
     }
     return 0;
   }
@@ -622,20 +648,20 @@ cf_bool(const char *opt, const char *val, void *loc,
  * \param val value of the option.
  * \param loc address to store the value.
  * \param maxval maximum length of value string.
- * \param source 0 if read from config file; 1 if from command.
+ * \param from_cmd 0 if read from config file; 1 if from command.
  * \retval 0 failure (unable to parse val).
  * \retval 1 success.
  */
 int
-cf_str(const char *opt, const char *val, void *loc, int maxval, int source)
+cf_str(const char *opt, const char *val, void *loc, int maxval, int from_cmd)
 {
   /* enter string parameter */
   size_t len = strlen(val);
 
   /* truncate if necessary */
   if (len >= (size_t) maxval) {
-    if (source == 0) {
-      do_rawlog(LT_ERR, T("CONFIG: option %s value truncated\n"), opt);
+    if (from_cmd == 0) {
+      do_rawlog(LT_ERR, T("CONFIG: option %s value truncated"), opt);
     }
     len = maxval - 1;
   }
@@ -645,19 +671,22 @@ cf_str(const char *opt, const char *val, void *loc, int maxval, int source)
 }
 
 /** Parse a dbref configuration option.
+ * dbrefs can be a raw number N or #N, even though # normally starts a comment
+ * in the config file. #-1 is allowed. Limits are -1 to db_max for @config,
+ * -1 to INT_MAX for reading mush.cnf (And validated elsewhere).
  * \param opt name of the configuration option.
  * \param val value of the option.
  * \param loc address to store the value.
  * \param maxval maximum dbref.
- * \param source 0 if read from config file; 1 if from command.
+ * \param from_cmd 0 if read from config file; 1 if from command.
  * \retval 0 failure (unable to parse val).
  * \retval 1 success.
  */
 int
-cf_dbref(const char *opt, const char *val, void *loc, int maxval, int source)
+cf_dbref(const char *opt, const char *val, void *loc,
+         int maxval __attribute__ ((__unused__)), int from_cmd)
 {
   /* enter dbref or integer parameter */
-
   int n;
   size_t offset = 0;
 
@@ -667,14 +696,14 @@ cf_dbref(const char *opt, const char *val, void *loc, int maxval, int source)
   n = parse_integer(val + offset);
 
   /* enforce limits */
-  if ((maxval >= 0) && (n > maxval)) {
-    n = maxval;
-    if (source == 0) {
-      do_rawlog(LT_ERR, T("CONFIG: option %s value limited to #%d\n"), opt,
+  if (n < NOTHING) {
+    n = NOTHING;
+    if (from_cmd == 0) {
+      do_rawlog(LT_ERR, T("CONFIG: option %s value limited to #%d"), opt,
                 maxval);
     }
   }
-  if (source && (!GoodObject(n) || IsGarbage(n))) {
+  if (from_cmd && ((!GoodObject(n) && n != NOTHING) || (n > 0 && IsGarbage(n)))) {
     do_rawlog(LT_ERR,
               T("CONFIG: attempt to set option %s to a bad dbref (#%d)"),
               opt, n);
@@ -689,12 +718,12 @@ cf_dbref(const char *opt, const char *val, void *loc, int maxval, int source)
  * \param val value of the option.
  * \param loc address to store the value.
  * \param maxval maximum value.
- * \param source 0 if read from config file; 1 if from command.
+ * \param from_cmd 0 if read from config file; 1 if from command.
  * \retval 0 failure (unable to parse val).
  * \retval 1 success.
  */
 int
-cf_int(const char *opt, const char *val, void *loc, int maxval, int source)
+cf_int(const char *opt, const char *val, void *loc, int maxval, int from_cmd)
 {
   /* enter integer parameter */
 
@@ -708,8 +737,8 @@ cf_int(const char *opt, const char *val, void *loc, int maxval, int source)
   /* enforce limits */
   if ((maxval >= 0) && (n > maxval)) {
     n = maxval;
-    if (source == 0) {
-      do_rawlog(LT_ERR, T("CONFIG: option %s value limited to %d\n"), opt,
+    if (from_cmd == 0) {
+      do_rawlog(LT_ERR, T("CONFIG: option %s value limited to %d"), opt,
                 maxval);
     }
   }
@@ -723,12 +752,12 @@ cf_int(const char *opt, const char *val, void *loc, int maxval, int source)
  * \param val value of the option.
  * \param loc address to store the value.
  * \param maxval maximum value.
- * \param source 0 if read from config file; 1 if from command.
+ * \param from_cmd 0 if read from config file; 1 if from command.
  * \retval 0 failure (unable to parse val).
  * \retval 1 success.
  */
 int
-cf_time(const char *opt, const char *val, void *loc, int maxval, int source)
+cf_time(const char *opt, const char *val, void *loc, int maxval, int from_cmd)
 {
   /* enter time parameter */
   char *end = NULL;
@@ -740,7 +769,7 @@ cf_time(const char *opt, const char *val, void *loc, int maxval, int source)
     in_minutes = 1;
 
   while (val && *val) {
-    n = strtol(val, &end, 10);
+    n = parse_int(val, &end, 10);
 
     switch (*end) {
     case '\0':
@@ -762,7 +791,7 @@ cf_time(const char *opt, const char *val, void *loc, int maxval, int source)
       secs += n * 3600;
       break;
     default:
-      if (source == 0)
+      if (from_cmd == 0)
         do_rawlog(LT_ERR, T("CONFIG: Unknown time interval in option %s"), opt);
       return 0;
     }
@@ -772,8 +801,8 @@ done:
   /* enforce limits */
   if ((maxval >= 0) && (secs > maxval)) {
     secs = maxval;
-    if (source == 0) {
-      do_rawlog(LT_ERR, T("CONFIG: option %s value limited to %d\n"), opt,
+    if (from_cmd == 0) {
+      do_rawlog(LT_ERR, T("CONFIG: option %s value limited to %d"), opt,
                 maxval);
     }
   }
@@ -788,12 +817,12 @@ done:
  * \param val value of the option.
  * \param loc address to store the value.
  * \param maxval maximum length of value.
- * \param source 0 if read from config file; 1 if from command.
+ * \param from_cmd 0 if read from config file; 1 if from command.
  * \retval 0 failure (unable to parse val).
  * \retval 1 success.
  */
 int
-cf_flag(const char *opt, const char *val, void *loc, int maxval, int source)
+cf_flag(const char *opt, const char *val, void *loc, int maxval, int from_cmd)
 {
   size_t len = strlen(val);
   size_t total = strlen((char *) loc);
@@ -802,17 +831,95 @@ cf_flag(const char *opt, const char *val, void *loc, int maxval, int source)
   if (len + total + 1 >= (size_t) maxval) {
     len = maxval - total - 1;
     if (len <= 0) {
-      if (source == 0)
-        do_rawlog(LT_ERR, T("CONFIG: option %s value overflow\n"), opt);
+      if (from_cmd == 0)
+        do_rawlog(LT_ERR, T("CONFIG: option %s value overflow"), opt);
       return 0;
     }
-    if (source == 0)
-      do_rawlog(LT_ERR, T("CONFIG: option %s value truncated\n"), opt);
+    if (from_cmd == 0)
+      do_rawlog(LT_ERR, T("CONFIG: option %s value truncated"), opt);
   }
   sprintf((char *) loc, "%s %s", (char *) loc, val);
   return 1;
 }
 
+/** Validate config options after reading the database.
+ * Used mostly for dbref options to make sure they point to valid
+ * objects.
+ */
+void
+validate_config(void)
+{
+
+#define VALIDATE_ROOM(opt) do { \
+    if (!(GoodObject((opt)) && IsRoom((opt)))) { \
+      opt = 0; \
+      do_rawlog(LT_ERR, T("CONFIG: option %s not a valid room!"), #opt); \
+    } \
+  } while (0)
+
+  VALIDATE_ROOM(PLAYER_START);
+  VALIDATE_ROOM(MASTER_ROOM);
+  VALIDATE_ROOM(BASE_ROOM);
+  VALIDATE_ROOM(DEFAULT_HOME);
+
+#undef VALIDATE_ROOM
+
+#define VALIDATE(opt) do { \
+    if (!GoodObject((opt)) && (opt) != NOTHING) { \
+      opt = 0; \
+      do_rawlog(LT_ERR, T("CONFIG: option %s not a valid dbref or -1!"), \
+                #opt); \
+    } \
+   } while (0)
+
+  VALIDATE(ANCESTOR_ROOM);
+  VALIDATE(ANCESTOR_THING);
+  VALIDATE(ANCESTOR_PLAYER);
+  VALIDATE(ANCESTOR_EXIT);
+
+#undef VALIDATE
+}
+
+static char *toplevel_cfile = NULL;
+
+static void
+append_restriction(const char *r, const char *what, const char *opts)
+{
+  FILE *out;
+
+  out = fopen(toplevel_cfile, "a");
+  if (out) {
+    fprintf(out, "# Added by @config/save\n%s %s %s\n\n", r, what, opts);
+    fclose(out);
+  }
+}
+
+/* Not perfect -- things defined in included config files won't
+   get changed -- but it'll catch most cases.
+*/
+static void
+save_config_option(PENNCONF *cp)
+{
+#if defined(HAVE_ED)
+  FILE *ed;
+
+  /* ed is the standard! Why reinvent it? */
+
+  ed = popen(ED_PATH, "w");
+  if (!ed) {
+    do_rawlog(LT_ERR, "Unable to open ed: %s", strerror(errno));
+    return;
+  }
+
+  fprintf(ed, "e %s\n", toplevel_cfile);
+  fprintf(ed, ",s/^[[:space:]]*%s[[:space:]].*$/%s/\n", cp->name,
+          trim_space_sep(config_to_string(GOD, cp, 1), ' '));
+  fputs("wq\n", ed);
+  pclose(ed);
+#endif
+
+}
+
 /** Set a configuration option.
  * This function sets a runtime configuration option. During the load
  * of the configuration file, it gets run twice - once to set the
@@ -820,7 +927,7 @@ cf_flag(const char *opt, const char *val, void *loc, int maxval, int source)
  * that require having the flag table available.
  * \param opt name of the option.
  * \param val value to set the option to.
- * \param source 0 if being set from mush.cnf, 1 from softcode.
+ * \param source 0 if being set from mush.cnf, 1 from softcode, 2 from softcode and saving.
  * \param restrictions 1 if we're setting restriction options, 0 for others.
  * \retval 1 success.
  * \retval 0 failure.
@@ -844,7 +951,7 @@ config_set(const char *opt, char *val, int source, int restrictions)
       if (!restrict_command(val, p)) {
         if (source == 0) {
           do_rawlog(LT_ERR,
-                    T("CONFIG: Invalid command or restriction for %s.\n"), val);
+                    T("CONFIG: Invalid command or restriction for %s."), val);
         }
         return 0;
       }
@@ -852,11 +959,13 @@ config_set(const char *opt, char *val, int source, int restrictions)
       if (source == 0) {
         do_rawlog(LT_ERR,
                   T
-                  ("CONFIG: restrict_command %s requires a restriction value.\n"),
+                  ("CONFIG: restrict_command %s requires a restriction value."),
                   val);
       }
       return 0;
     }
+    if (source == 2)
+      append_restriction("restrict_command", val, p);
     return 1;
   } else if (!strcasecmp(opt, "restrict_function")) {
     if (!restrictions)
@@ -867,8 +976,7 @@ config_set(const char *opt, char *val, int source, int restrictions)
       if (!restrict_function(val, p)) {
         if (source == 0) {
           do_rawlog(LT_ERR,
-                    T("CONFIG: Invalid function or restriction for %s.\n"),
-                    val);
+                    T("CONFIG: Invalid function or restriction for %s."), val);
         }
         return 0;
       }
@@ -876,11 +984,13 @@ config_set(const char *opt, char *val, int source, int restrictions)
       if (source == 0) {
         do_rawlog(LT_ERR,
                   T
-                  ("CONFIG: restrict_function %s requires a restriction value.\n"),
+                  ("CONFIG: restrict_function %s requires a restriction value."),
                   val);
       }
       return 0;
     }
+    if (source == 2)
+      append_restriction("restrict_function", val, p);
     return 1;
   } else if (!strcasecmp(opt, "reserve_alias")) {
     if (!restrictions)
@@ -895,17 +1005,19 @@ config_set(const char *opt, char *val, int source, int restrictions)
       *p++ = '\0';
       if (!alias_command(val, p)) {
         if (source == 0) {
-          do_rawlog(LT_ERR, T("CONFIG: Couldn't alias %s to %s.\n"), p, val);
+          do_rawlog(LT_ERR, T("CONFIG: Couldn't alias %s to %s."), p, val);
         }
         return 0;
       }
     } else {
       if (source == 0) {
         do_rawlog(LT_ERR,
-                  T("CONFIG: command_alias %s requires an alias.\n"), val);
+                  T("CONFIG: command_alias %s requires an alias."), val);
       }
       return 0;
     }
+    if (source == 2)
+      append_restriction("reserve_alias", val, p);
     return 1;
   } else if (!strcasecmp(opt, "attribute_alias")) {
     if (!restrictions)
@@ -915,17 +1027,19 @@ config_set(const char *opt, char *val, int source, int restrictions)
       *p++ = '\0';
       if (!alias_attribute(val, p)) {
         if (source == 0) {
-          do_rawlog(LT_ERR, T("CONFIG: Couldn't alias %s to %s.\n"), p, val);
+          do_rawlog(LT_ERR, T("CONFIG: Couldn't alias %s to %s."), p, val);
         }
         return 0;
       }
     } else {
       if (source == 0) {
         do_rawlog(LT_ERR,
-                  T("CONFIG: attribute_alias %s requires an alias.\n"), val);
+                  T("CONFIG: attribute_alias %s requires an alias."), val);
       }
       return 0;
     }
+    if (source == 2)
+      append_restriction("attribute_alias", val, p);
     return 1;
   } else if (!strcasecmp(opt, "function_alias")) {
     if (!restrictions)
@@ -935,17 +1049,19 @@ config_set(const char *opt, char *val, int source, int restrictions)
       *p++ = '\0';
       if (!alias_function(val, p)) {
         if (source == 0) {
-          do_rawlog(LT_ERR, T("CONFIG: Couldn't alias %s to %s.\n"), p, val);
+          do_rawlog(LT_ERR, T("CONFIG: Couldn't alias %s to %s."), p, val);
         }
         return 0;
       }
     } else {
       if (source == 0) {
         do_rawlog(LT_ERR,
-                  T("CONFIG: function_alias %s requires an alias.\n"), val);
+                  T("CONFIG: function_alias %s requires an alias."), val);
       }
       return 0;
     }
+    if (source == 2)
+      append_restriction("function_alias", val, p);
     return 1;
   } else if (!strcasecmp(opt, "help_command")
              || !strcasecmp(opt, "ahelp_command")) {
@@ -954,12 +1070,12 @@ config_set(const char *opt, char *val, int source, int restrictions)
     if (!restrictions)
       return 0;
     /* Add a new help-like command */
-    if (source == 1)
+    if (source >= 1)            /* Can't do this from @config/set */
       return 0;
     if (!val || !*val) {
       do_rawlog(LT_ERR,
                 T
-                ("CONFIG: help_command requires a command name and file name.\n"));
+                ("CONFIG: help_command requires a command name and file name."));
       return 0;
     }
     comm = val;
@@ -971,7 +1087,7 @@ config_set(const char *opt, char *val, int source, int restrictions)
     } else {
       do_rawlog(LT_ERR,
                 T
-                ("CONFIG: help_command requires a command name and file name.\n"));
+                ("CONFIG: help_command requires a command name and file name."));
       return 0;
     }
   } else if (restrictions) {
@@ -987,26 +1103,32 @@ config_set(const char *opt, char *val, int source, int restrictions)
                      && strcmp(cp->group, "messages") != 0))
         && !strcasecmp(cp->name, opt)) {
       i = cp->handler(opt, val, cp->loc, cp->max, source);
-      if (i)
+      if (i) {
         cp->overridden = 1;
+        if (source == 2)
+          save_config_option(cp);
+      }
       return i;
     }
   }
-  for (cp = (PENNCONF *) hash_firstentry(&local_options); cp;
-       cp = (PENNCONF *) hash_nextentry(&local_options)) {
+  for (cp = hash_firstentry(&local_options); cp;
+       cp = hash_nextentry(&local_options)) {
     int i = 0;
     if ((!source || (cp->group && strcmp(cp->group, "files") != 0
                      && strcmp(cp->group, "messages") != 0))
         && !strcasecmp(cp->name, opt)) {
       i = cp->handler(opt, val, cp->loc, cp->max, source);
-      if (i)
+      if (i) {
         cp->overridden = 1;
+        if (source == 2)
+          save_config_option(cp);
+      }
       return i;
     }
   }
 
   if (source == 0) {
-    do_rawlog(LT_ERR, T("CONFIG: directive '%s' in cnf file ignored.\n"), opt);
+    do_rawlog(LT_ERR, T("CONFIG: directive '%s' in cnf file ignored."), opt);
   }
   return 0;
 }
@@ -1024,6 +1146,7 @@ conf_default_set(void)
   strcpy(options.crash_db, "data/PANIC.db");
   strcpy(options.chatdb, "data/chatdb");
   options.chan_cost = 1000;
+  options.noisy_cemit = 0;
   options.max_player_chans = 3;
   options.max_channels = 200;
   strcpy(options.mail_db, "data/maildb");
@@ -1166,7 +1289,6 @@ conf_default_set(void)
   options.max_guest_pennies = 100000;
   options.max_depth = 10;
   options.max_parents = 10;
-  options.max_global_fns = 50;
   options.purge_interval = 601;
   options.dbck_interval = 599;
   options.max_attrcount = 2048;
@@ -1177,6 +1299,7 @@ conf_default_set(void)
   options.ascii_names = 1;
   options.call_lim = 10000;
   strcpy(options.chunk_swap_file, "data/chunkswap");
+  options.chunk_swap_initial = 2048;
   options.chunk_cache_memory = 1000000;
   options.chunk_migrate_amount = 50;
   options.read_remote_desc = 0;
@@ -1185,14 +1308,14 @@ conf_default_set(void)
   strcpy(options.ssl_ca_file, "");
   options.ssl_require_client_cert = 0;
 #endif
+  /* Set this to 1 so that allocations made before reading the config file
+     will be tracked. */
   options.mem_check = 1;
-#ifdef HAS_MYSQL
   strcpy(options.sql_platform, "disabled");
   strcpy(options.sql_database, "");
   strcpy(options.sql_username, "");
   strcpy(options.sql_password, "");
   strcpy(options.sql_host, "127.0.0.1");
-#endif
 }
 
 /* Limit how many files we can nest */
@@ -1220,8 +1343,8 @@ config_file_startup(const char *conf, int restrictions)
   PENNCONF *cp;
   char tbuf1[BUFFER_LEN];
   char *p, *q, *s;
-
   static char cfile[BUFFER_LEN];        /* Remember the last one */
+
   if (conf_recursion == 0) {
     if (conf && *conf)
       strcpy(cfile, conf);
@@ -1231,6 +1354,8 @@ config_file_startup(const char *conf, int restrictions)
       return 0;
     }
     do_rawlog(LT_ERR, "Reading %s", cfile);
+    if (toplevel_cfile == NULL)
+      toplevel_cfile = mush_strdup(cfile, "config.file");
   } else {
     if (conf && *conf)
       fp = fopen(conf, FOPEN_READ);
@@ -1370,7 +1495,7 @@ do_config_list(dbref player, const char *type, int lc)
   PENNCONF *cp;
 
   if (SUPPORT_PUEBLO)
-    notify_noenter(player, tprintf("%cSAMP%c", TAG_START, TAG_END));
+    notify_noenter(player, open_tag("SAMP"));
   if (type && *type) {
     /* Look up the type in the group table */
     int found = 0;
@@ -1385,7 +1510,7 @@ do_config_list(dbref player, const char *type, int lc)
       /* It wasn't a group. Is is one or more specific options? */
       for (cp = conftable; cp->name; cp++) {
         if (cp->group && string_prefix(cp->name, type)) {
-          notify(player, config_list_helper(player, cp, lc));
+          notify(player, config_to_string(player, cp, lc));
           found = 1;
         }
       }
@@ -1394,7 +1519,7 @@ do_config_list(dbref player, const char *type, int lc)
         for (cp = (PENNCONF *) hash_firstentry(&local_options); cp;
              cp = (PENNCONF *) hash_nextentry(&local_options)) {
           if (cp->group && !strcasecmp(cp->name, type)) {
-            notify(player, config_list_helper(player, cp, lc));
+            notify(player, config_to_string(player, cp, lc));
             found = 1;
           }
         }
@@ -1415,13 +1540,13 @@ do_config_list(dbref player, const char *type, int lc)
       else {
         for (cp = conftable; cp->name; cp++) {
           if (cp->group && !strcmp(cp->group, cgp->name)) {
-            notify(player, config_list_helper(player, cp, lc));
+            notify(player, config_to_string(player, cp, lc));
           }
         }
         for (cp = (PENNCONF *) hash_firstentry(&local_options); cp;
              cp = (PENNCONF *) hash_nextentry(&local_options)) {
           if (cp->group && !strcasecmp(cp->group, cgp->name)) {
-            notify(player, config_list_helper(player, cp, lc));
+            notify(player, config_to_string(player, cp, lc));
           }
         }
       }
@@ -1436,14 +1561,14 @@ do_config_list(dbref player, const char *type, int lc)
     }
   }
   if (SUPPORT_PUEBLO)
-    notify_noenter(player, tprintf("%c/SAMP%c", TAG_START, TAG_END));
+    notify_noenter(player, close_tag("SAMP"));
 }
 
 /** Lowercase a string if we've been asked to */
 #define MAYBE_LC(x) (lc ? strlower(x) : x)
 static char *
-config_list_helper(dbref player
-                   __attribute__ ((__unused__)), PENNCONF * cp, int lc)
+config_to_string(dbref player
+                 __attribute__ ((__unused__)), PENNCONF *cp, int lc)
 {
   static char result[BUFFER_LEN];
   char *bp = result;
@@ -1483,9 +1608,9 @@ config_list_helper(dbref player
 
 /* This one doesn't return the names */
 static char *
-config_list_helper2(dbref player
-                    __attribute__ ((__unused__)), PENNCONF * cp, int lc
-                    __attribute__ ((__unused__)))
+config_to_string2(dbref player
+                  __attribute__ ((__unused__)), PENNCONF *cp, int lc
+                  __attribute__ ((__unused__)))
 {
   static char result[BUFFER_LEN];
   char *bp = result;
@@ -1529,14 +1654,14 @@ FUNCTION(fun_config)
   if (args[0] && *args[0]) {
     for (cp = conftable; cp->name; cp++) {
       if (cp->group && !strcasecmp(cp->name, args[0])) {
-        safe_str(config_list_helper2(executor, cp, 0), buff, bp);
+        safe_str(config_to_string2(executor, cp, 0), buff, bp);
         return;
       }
     }
     for (cp = (PENNCONF *) hash_firstentry(&local_options); cp;
          cp = (PENNCONF *) hash_nextentry(&local_options)) {
       if (cp->group && !strcasecmp(cp->name, args[0])) {
-        safe_str(config_list_helper2(executor, cp, 0), buff, bp);
+        safe_str(config_to_string2(executor, cp, 0), buff, bp);
         return;
       }
     }
@@ -1610,13 +1735,19 @@ show_compile_options(dbref player)
   notify(player, T(" Attributes are 8-bit word compressed in memory."));
 #endif
 
-#ifdef HAS_OPENSSL
+#ifdef HAVE_SSL
   notify(player, T(" The MUSH was compiled with SSL support."));
 #endif
 
-#ifdef HAS_MYSQL
+#ifdef HAVE_MYSQL
   notify(player, T(" The MUSH was compiled with MySQL support."));
 #endif
+#ifdef HAVE_POSTGRESQL
+  notify(player, T(" The MUSH was compiled with Postgresql support."));
+#endif
+#ifdef HAVE_SQLITE3
+  notify(player, T(" The MUSH was compiled with Sqlite3 support."));
+#endif
 
 #ifdef INFO_SLAVE
   notify(player, T(" DNS and ident lookups are handled by a slave process."));
@@ -1624,8 +1755,6 @@ show_compile_options(dbref player)
   notify(player, T(" DNS and ident lookups are handled by the MUSH process."));
 #endif
 
-  notify(player, T(" BSD sockets networking in use."));
-
 #ifdef HAS_GETDATE
   notify(player, T(" Extended convtime() is supported."));
 #else
@@ -1638,4 +1767,26 @@ show_compile_options(dbref player)
   notify(player, T(" CPU usage limiting is NOT supported."));
 #endif
 
+#ifdef HAVE_INOTIFY
+  notify(player, T(" Changed help files will be automatically reindexed."));
+#endif
+
+#ifdef HAVE_SSE2
+  notify(player, T(" SSE2 instructions are being used."));
+#endif
+
+#ifdef HAVE_SSE3
+  notify(player, T(" SSE3 instructions are being used."));
+#endif
+
+#ifdef HAVE_ALTIVEC
+  notify(player, T(" Altivec instructions are being used."));
+#endif
+
+#ifdef HAVE_ED
+  notify(player, T(" @config/save is enabled."));
+#else
+  notify(player, T(" @config/save is disabled."));
+#endif
+
 }
