@@ -16,9 +16,15 @@
 #include <string.h>
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 
 #include "conf.h"
 #include "command.h"
@@ -33,6 +39,7 @@
 #include "flags.h"
 #include "dbdefs.h"
 #include "log.h"
+#include "intmap.h"
 #include "confmagic.h"
 
 
@@ -53,13 +60,19 @@ typedef struct bque {
   char *env[10];                /**< environment, from wild match */
   char *rval[NUMQ];             /**< environment, from setq() */
   char *comm;                   /**< command to be executed */
+  uint32_t pid;                 /**< Process id of this entry */
 } BQUE;
 
+intmap *queue_map = NULL; /**< Intmap for looking up queue entries by pid */
+static uint32_t top_pid = 1;
+#define MAX_PID (1U << 15)
+
 static BQUE *qfirst = NULL, *qlast = NULL, *qwait = NULL;
 static BQUE *qlfirst = NULL, *qllast = NULL;
 static BQUE *qsemfirst = NULL, *qsemlast = NULL;
 
-static int add_to_generic(dbref player, int am, const char *name, int flags);
+static int add_to_generic(dbref player, int am, const char *name,
+                          uint32_t flags);
 static int add_to(dbref player, int am);
 static int add_to_sem(dbref player, int am, const char *name);
 static int queue_limit(dbref player);
@@ -83,6 +96,14 @@ extern sig_atomic_t cpu_time_limit_hit; /**< Have we used too much CPU? */
  */
 #define SEMAPHORE_FLAGS (AF_LOCKED | AF_PRIVATE | AF_NOCOPY | AF_NODUMP)
 
+/** Queue initializtion function. Must  be called before anything
+ * is added to the queue.
+ */
+void
+init_queue(void)
+{
+  queue_map = im_new();
+}
 
 /* Returns true if the attribute on thing can be used as a semaphore.
  * atr should be given in UPPERCASE. 
@@ -114,7 +135,7 @@ waitable_attr(dbref thing, const char *atr)
 }
 
 static int
-add_to_generic(dbref player, int am, const char *name, int flags)
+add_to_generic(dbref player, int am, const char *name, uint32_t flags)
 {
   int num = 0;
   ATTR *a;
@@ -127,25 +148,24 @@ add_to_generic(dbref player, int am, const char *name, int flags)
    * it later, because clearing it may fail (perhaps someone's also
    * foolishly using it as a branch in an attribute tree)
    */
-  sprintf(buff, "%d", num);
+  snprintf(buff, sizeof buff, "%d", num);
   (void) atr_add(player, name, buff, GOD, flags);
   if (!num) {
     (void) atr_clr(player, name, GOD);
   }
-  return (num);
+  return num;
 }
 
 static int
 add_to(dbref player, int am)
 {
-  return (add_to_generic(player, am, "QUEUE", 0));
+  return add_to_generic(player, am, "QUEUE", NOTHING);
 }
 
 static int
 add_to_sem(dbref player, int am, const char *name)
 {
-  return (add_to_generic
-          (player, am, name ? name : "SEMAPHORE", SEMAPHORE_FLAGS));
+  return add_to_generic(player, am, name ? name : "SEMAPHORE", SEMAPHORE_FLAGS);
 }
 
 static int
@@ -171,17 +191,18 @@ free_qentry(BQUE *point)
   int a;
   for (a = 0; a < 10; a++)
     if (point->env[a]) {
-      mush_free((Malloc_t) point->env[a], "bqueue_env");
+      mush_free(point->env[a], "cqueue.env");
     }
   for (a = 0; a < NUMQ; a++)
     if (point->rval[a]) {
-      mush_free((Malloc_t) point->rval[a], "bqueue_rval");
+      mush_free(point->rval[a], "cqueue.rval");
     }
   if (point->semattr)
-    mush_free((Malloc_t) point->semattr, "bqueue_semattr");
+    mush_free(point->semattr, "cqueue.semattr");
   if (point->comm)
-    mush_free((Malloc_t) point->comm, "bqueue_comm");
-  mush_free((Malloc_t) point, "BQUE");
+    mush_free(point->comm, "cqueue.comm");
+  im_delete(queue_map, point->pid);
+  mush_free(point, "cqueue");
 }
 
 static int
@@ -190,7 +211,7 @@ pay_queue(dbref player, const char *command)
   int estcost;
   estcost =
     QUEUE_COST +
-    (QUEUE_LOSS ? ((get_random_long(0, QUEUE_LOSS - 1) == 0) ? 1 : 0) : 0);
+    (QUEUE_LOSS ? ((get_random32(0, QUEUE_LOSS - 1) == 0) ? 1 : 0) : 0);
   if (!quiet_payfor(player, estcost)) {
     notify(Owner(player), T("Not enough money to queue command."));
     return 0;
@@ -231,6 +252,31 @@ pay_queue(dbref player, const char *command)
   return 1;
 }
 
+static uint32_t
+next_pid(void)
+{
+  uint32_t pid = top_pid;
+
+  if (im_count(queue_map) >= (int) MAX_PID) {
+    do_rawlog(LT_ERR,
+              T
+              ("There are %ld queue entries! That's too many. Failing to add another."),
+              (long) im_count(queue_map));
+    return 0;
+  }
+
+  while (1) {
+    if (pid > MAX_PID)
+      pid = 1;
+    if (im_exists(queue_map, pid))
+      pid++;
+    else {
+      top_pid = pid + 1;
+      return pid;
+    }
+  }
+}
+
 /** Add a new entry onto the player or object command queues.
  * This function adds a new entry to the back of the player or
  * object command queues (depending on whether the call was
@@ -244,12 +290,20 @@ parse_que(dbref player, const char *command, dbref cause)
 {
   int a;
   BQUE *tmp;
+  int pid;
   if (!IsPlayer(player) && (Halted(player)))
     return;
   if (!pay_queue(player, command))      /* make sure player can afford to do it */
     return;
-  tmp = (BQUE *) mush_malloc(sizeof(BQUE), "BQUE");
-  tmp->comm = mush_strdup(command, "bqueue_comm");
+  pid = next_pid();
+  if (pid == 0) {
+    /* Too many queue entries */
+    notify(player, T("Queue entry table full. Try again later."));
+    return;
+  }
+  tmp = mush_malloc(sizeof *tmp, "cqueue");
+  tmp->pid = pid;
+  tmp->comm = mush_strdup(command, "cqueue.comm");
   tmp->semattr = NULL;
   tmp->player = player;
   tmp->queued = QUEUE_PER_OWNER ? Owner(player) : player;
@@ -260,13 +314,13 @@ parse_que(dbref player, const char *command, dbref cause)
     if (!global_eval_context.wnxt[a])
       tmp->env[a] = NULL;
     else {
-      tmp->env[a] = mush_strdup(global_eval_context.wnxt[a], "bqueue_env");
+      tmp->env[a] = mush_strdup(global_eval_context.wnxt[a], "cqueue.env");
     }
   for (a = 0; a < NUMQ; a++)
     if (!global_eval_context.rnxt[a] || !global_eval_context.rnxt[a][0])
       tmp->rval[a] = NULL;
     else {
-      tmp->rval[a] = mush_strdup(global_eval_context.rnxt[a], "bqueue_rval");
+      tmp->rval[a] = mush_strdup(global_eval_context.rnxt[a], "cqueue.rval");
     }
   if (IsPlayer(cause)) {
     if (qlast) {
@@ -281,6 +335,7 @@ parse_que(dbref player, const char *command, dbref cause)
     } else
       qllast = qlfirst = tmp;
   }
+  im_insert(queue_map, tmp->pid, tmp);
 }
 
 
@@ -357,6 +412,7 @@ wait_que(dbref player, int waittill, char *command, dbref cause, dbref sem,
 {
   BQUE *tmp;
   int a;
+  int pid;
   if (waittill == 0) {
     if (sem != NOTHING)
       add_to_sem(sem, -1, semattr);
@@ -365,8 +421,14 @@ wait_que(dbref player, int waittill, char *command, dbref cause, dbref sem,
   }
   if (!pay_queue(player, command))      /* make sure player can afford to do it */
     return;
-  tmp = (BQUE *) mush_malloc(sizeof(BQUE), "BQUE");
-  tmp->comm = mush_strdup(command, "bqueue_comm");
+  pid = next_pid();
+  if (pid == 0) {
+    notify(player, T("Queue entry table full. Try again later."));
+    return;
+  }
+  tmp = mush_malloc(sizeof *tmp, "cqueue");
+  tmp->comm = mush_strdup(command, "cqueue.comm");
+  tmp->pid = pid;
   tmp->player = player;
   tmp->queued = QUEUE_PER_OWNER ? Owner(player) : player;
   tmp->cause = cause;
@@ -376,19 +438,19 @@ wait_que(dbref player, int waittill, char *command, dbref cause, dbref sem,
     if (!global_eval_context.wnxt[a])
       tmp->env[a] = NULL;
     else {
-      tmp->env[a] = mush_strdup(global_eval_context.wnxt[a], "bqueue_env");
+      tmp->env[a] = mush_strdup(global_eval_context.wnxt[a], "cqueue.env");
     }
   }
   for (a = 0; a < NUMQ; a++) {
     if (!global_eval_context.rnxt[a] || !global_eval_context.rnxt[a][0])
       tmp->rval[a] = NULL;
     else {
-      tmp->rval[a] = mush_strdup(global_eval_context.rnxt[a], "bqueue_rval");
+      tmp->rval[a] = mush_strdup(global_eval_context.rnxt[a], "cqueue.rval");
     }
   }
 
   if (until) {
-    tmp->left = waittill;
+    tmp->left = (time_t) waittill;
   } else {
     if (waittill >= 0)
       tmp->left = mudtime + waittill;
@@ -413,7 +475,7 @@ wait_que(dbref player, int waittill, char *command, dbref cause, dbref sem,
 
     /* Put it on the end of the semaphore queue */
     tmp->semattr =
-      mush_strdup(semattr ? semattr : "SEMAPHORE", "bqueue_semattr");
+      mush_strdup(semattr ? semattr : "SEMAPHORE", "cqueue.semattr");
     if (qsemlast != NULL) {
       qsemlast->next = tmp;
       qsemlast = tmp;
@@ -421,6 +483,7 @@ wait_que(dbref player, int waittill, char *command, dbref cause, dbref sem,
       qsemfirst = qsemlast = tmp;
     }
   }
+  im_insert(queue_map, tmp->pid, tmp);
 }
 
 /** Once-a-second check for queued commands.
@@ -725,7 +788,8 @@ dequeue_semaphores(dbref thing, char const *aname, int count, int all,
     add_to_sem(thing, -count, aname);
 }
 
-COMMAND (cmd_notify_drain) {
+COMMAND(cmd_notify_drain)
+{
   int drain;
   char *pos;
   char const *aname;
@@ -812,12 +876,12 @@ COMMAND (cmd_notify_drain) {
  * \endverbatim
  * \param player the enactor
  * \param cause the object causing the command to be added.
- * \param arg1 the wait time, semaphore object/attribute, or both.
+ * \param arg1 the wait time, semaphore object/attribute, or both. Modified!
  * \param cmd command to queue.
  * \param until if 1, wait until an absolute time.
  */
 void
-do_wait(dbref player, dbref cause, char *arg1, char *cmd, int until)
+do_wait(dbref player, dbref cause, char *arg1, const char *cmd, bool until)
 {
   dbref thing;
   char *tcount = NULL, *aname = NULL;
@@ -896,6 +960,114 @@ do_wait(dbref player, dbref cause, char *arg1, char *cmd, int until)
   mush_free(arg2, "strip_braces.buff");
 }
 
+/** Interface to @wait/pid; modifies the wait times of queue
+ * entries.
+ * \param player the object doing the command.
+ * \param pidstr the process id to modify.
+ * \param timestr the new timeout.
+ * \param until true if timeout is an absolute time.
+ */
+void
+do_waitpid(dbref player, const char *pidstr, const char *timestr, bool until)
+{
+  uint32_t pid;
+  BQUE *q, *tmp, *last;
+  bool found;
+
+  if (!is_uinteger(pidstr)) {
+    notify(player, T("That is not a valid pid!"));
+    return;
+  }
+
+  pid = parse_uint32(pidstr, NULL, 10);
+  q = im_find(queue_map, pid);
+
+  if (!q) {
+    notify(player, T("That is not a valid pid!"));
+    return;
+  }
+
+  if (!controls(player, q->player) && !HaltAny(player)) {
+    notify(player, T("Permission denied."));
+    return;
+  }
+
+  if (q->sem != NOTHING && q->left == 0) {
+    notify(player,
+           T("You cannot adjust the timeout of an indefinite semaphore."));
+    return;
+  }
+
+  if (!is_strict_integer(timestr)) {
+    notify(player, T("That is not a valid timestamp."));
+    return;
+  }
+
+  if (until) {
+    int when;
+
+    when = parse_integer(timestr);
+
+    if (when < 0)
+      when = 0;
+
+    q->left = (time_t) when;
+
+  } else {
+    int offset = parse_integer(timestr);
+
+    /* If timestr looks like +NNN or -NNN, add or subtract a number
+       of seconds to the current timeout. Otherwise, change timeout. 
+     */
+    if (timestr[0] == '+' || timestr[0] == '-')
+      q->left += offset;
+    else
+      q->left = mudtime + offset;
+
+    if (q->left < 0)
+      q->left = 0;
+  }
+
+  /* Now adjust it in the wait queue. Not a clever approach, but I
+     wrote it at 3 am and clever was not an option. */
+  found = false;
+  for (tmp = qwait, last = NULL; tmp; last = tmp, tmp = tmp->next) {
+    if (tmp == q) {
+      if (last)
+        last->next = q->next;
+      else
+        qwait = qwait->next;
+      found = true;
+      break;
+    }
+  }
+  if (found) {
+    found = false;
+    for (tmp = qwait, last = NULL; tmp; last = tmp, tmp = tmp->next) {
+      if (tmp->left > q->left) {
+        if (last) {
+          last->next = q;
+          q->next = tmp;
+        } else {
+          q->next = qwait;
+          qwait = q;
+        }
+        found = true;
+        break;
+      }
+    }
+    if (!found) {
+      if (last)
+        last->next = q;
+      else
+        qwait = q;
+      q->next = NULL;
+    }
+  }
+
+  notify_format(player, T("Queue entry with pid %u updated."), pid);
+}
+
 static void
 show_queue(dbref player, dbref victim, int q_type, int q_quiet, int q_all,
            BQUE *q_ptr, int *tot, int *self, int *del)
@@ -913,25 +1085,26 @@ show_queue(dbref player, dbref victim, int q_type, int q_quiet, int q_all,
           continue;
         switch (q_type) {
         case 1:                /* wait queue */
-          notify_format(player, "[%ld]%s: %s",
+          notify_format(player, "(Pid: %u) [%ld]%s: %s", tmp->pid,
                         (long) difftime(tmp->left, mudtime),
                         unparse_object(player, tmp->player), tmp->comm);
           break;
         case 2:                /* semaphore queue */
           if (tmp->left != 0) {
-            notify_format(player, "[#%d/%s/%ld]%s: %s", tmp->sem,
-                          tmp->semattr, (long) difftime(tmp->left, mudtime),
+            notify_format(player, "(Pid: %u) [#%d/%s/%ld]%s: %s", tmp->pid,
+                          tmp->sem, tmp->semattr, (long) difftime(tmp->left,
+                                                                  mudtime),
                           unparse_object(player, tmp->player), tmp->comm);
           } else {
-            notify_format(player, "[#%d/%s]%s: %s", tmp->sem,
-                          tmp->semattr, unparse_object(player, tmp->player),
+            notify_format(player, "(Pid: %u) [#%d/%s]%s: %s", tmp->pid,
+                          tmp->sem, tmp->semattr, unparse_object(player,
+                                                                 tmp->player),
                           tmp->comm);
           }
           break;
         default:               /* player or object queue */
-          notify_format(player, "%s: %s", unparse_object(player,
-                                                         tmp->player),
-                        tmp->comm);
+          notify_format(player, "(Pid: %u) %s: %s", tmp->pid,
+                        unparse_object(player, tmp->player), tmp->comm);
         }
       }
     }
@@ -1002,8 +1175,9 @@ do_queue(dbref player, const char *what, enum queue_type flag)
     if (!quick)
       notify(player, T("------------  Queue Done  ------------"));
     notify_format(player,
-                  "Totals: Player...%d/%d[%ddel]  Object...%d/%d[%ddel]  Wait...%d/%d  Semaphore...%d/%d",
-                  pq, tpq, dpq, oq, toq, doq, wq, twq, sq, tsq);
+                  T
+                  ("Totals: Player...%d/%d[%ddel]  Object...%d/%d[%ddel]  Wait...%d/%d[%ddel]  Semaphore...%d/%d"),
+                  pq, tpq, dpq, oq, toq, doq, wq, twq, dwq, sq, tsq);
   }
 }
 
@@ -1143,6 +1317,69 @@ do_halt1(dbref player, const char *arg1, const char *arg2)
   }
 }
 
+/** Halt a particular pid
+ * \param player the enactor.
+ * \param arg1 string representing the pid to halt.
+ */
+void
+do_haltpid(dbref player, const char *arg1)
+{
+  uint32_t pid;
+  BQUE *q;
+  dbref victim;
+
+  if (!is_uinteger(arg1)) {
+    notify(player, T("That is not a valid pid!"));
+    return;
+  }
+
+  pid = parse_uint32(arg1, NULL, 10);
+
+  q = im_find(queue_map, pid);
+
+  if (!q) {
+    notify(player, T("That is not a valid pid!"));
+    return;
+  }
+
+  victim = q->player;
+  if (!controls(player, victim) && !HaltAny(player)) {
+    notify(player, T("Permission denied."));
+    return;
+  }
+
+  /* Instead of trying to track what queue this entry currently
+     belongs too, flag it as halted and just not execute it when its
+     turn comes up (Or show it in @ps, etc.).  Exception is for
+     semaphores, which otherwise might wait forever. */
+  q->player = NOTHING;
+
+  if (q->semattr) {
+    BQUE *last = NULL, *tmp;
+
+    for (tmp = qsemfirst; tmp; last = tmp, tmp = tmp->next) {
+      if (tmp == q) {
+        if (last)
+          last->next = tmp->next;
+        else
+          qsemfirst = tmp->next;
+
+        if (qsemlast == tmp)
+          qsemlast = last;
+
+        break;
+      }
+    }
+
+    giveto(victim, QUEUE_COST);
+    add_to_sem(q->sem, -1, q->semattr);
+    free_qentry(q);
+  }
+
+  notify_format(player, T("Queue entry with pid %u halted."), pid);
+}
+
+
 /** Halt all objects in the database.
  * \param player the enactor.
  */
