@@ -43,7 +43,7 @@ parse_linkable_room(dbref player, const char *room_name)
   } else if (!strcasecmp(room_name, "home")) {
     return HOME;                /* HOME is always linkable */
   } else {
-    room = parse_dbref(room_name);
+    room = parse_objid(room_name);
   }
 
   /* check room */
@@ -424,10 +424,11 @@ do_dig(dbref player, const char *name, char **argv, int tport)
  * \param player the enactor.
  * \param name name of thing to create.
  * \param cost pennies spent in creation.
+ * \paran newdbref the (unparsed) dbref to give the object, or NULL to use the next free
  * \return dbref of new thing, or NOTHING.
  */
 dbref
-do_create(dbref player, char *name, int cost)
+do_create(dbref player, char *name, int cost, char *newdbref)
 {
   dbref loc;
   dbref thing;
@@ -441,6 +442,25 @@ do_create(dbref player, char *name, int cost)
   } else if (cost < OBJECT_COST) {
     cost = OBJECT_COST;
   }
+
+  if (newdbref && *newdbref) {
+    /* move newdbref to the start of the free list */
+    if (!has_flag_by_name(player, "WIZARD", NOTYPE)) {
+      notify(player, T("Permission denied."));
+      return NOTHING;
+    }
+    thing = parse_dbref(newdbref);
+    if (thing == NOTHING || !GoodObject(thing) || !IsGarbage(thing)) {
+      notify(player, T("That is not a valid dbref."));
+      return NOTHING;
+    }
+
+    if (!make_first_free(thing)) {
+      notify(player, T("Unable to create object with that dbref."));
+      return NOTHING;
+    }
+  }
+
   if (can_pay_fees(player, cost)) {
     /* create the object */
     thing = new_object();
@@ -472,7 +492,7 @@ do_create(dbref player, char *name, int cost)
       PUSH(thing, Contents(Source(player)));
 
     /* and we're done */
-    notify_format(player, "Created: Object %s.", unparse_dbref(thing));
+    notify_format(player, T("Created: Object %s."), unparse_dbref(thing));
     current_state.things++;
     local_data_create(thing);
     return thing;
@@ -490,12 +510,12 @@ clone_object(dbref player, dbref thing, const char *newname, int preserve)
 
   memcpy(REFDB(clone), REFDB(thing), sizeof(struct object));
   Owner(clone) = Owner(player);
-  Name(clone) = NULL;
   if (newname && *newname)
     set_name(clone, newname);
   else
     set_name(clone, Name(thing));
   s_Pennies(clone, Pennies(thing));
+  AttrCount(clone) = 0;
   atr_cpy(clone, thing);
   Locks(clone) = NULL;
   clone_locks(player, thing, clone);
