@@ -8,16 +8,21 @@
 
 #include "copyrite.h"
 #include "config.h"
-
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
 #include <stdlib.h>
+#include <errno.h>
+#include <stdarg.h>
 #include "conf.h"
 #include "dbio.h"
 #include "externs.h"
@@ -33,8 +38,10 @@
 #include "parse.h"
 #include "privtab.h"
 #include "htab.h"
+#include "extchat.h"
 #include "extmail.h"
 #include "confmagic.h"
+#include "ansi.h"
 
 #ifdef WIN32
 #pragma warning( disable : 4761)        /* disable warning re conversion */
@@ -74,18 +81,18 @@ HASHTAB htab_objdata_keys;    /**< Object data keys hash table */
 
 static void db_grow(dbref newtop);
 
-static void db_write_obj_basic(FILE * f, dbref i, struct object *o);
-int db_paranoid_write_object(FILE * f, dbref i, int flag);
-int db_write_object(FILE * f, dbref i);
-void putlocks(FILE * f, lock_list *l);
-void getlocks(dbref i, FILE * f);
-void get_new_locks(dbref i, FILE * f, int c);
-void db_read_attrs(FILE * f, dbref i, int c);
-int get_list(FILE * f, dbref i);
+static void db_write_obj_basic(PENNFILE *f, dbref i, struct object *o);
+int db_paranoid_write_object(PENNFILE *f, dbref i, int flag);
+int db_write_object(PENNFILE *f, dbref i);
+void putlocks(PENNFILE *f, lock_list *l);
+void getlocks(dbref i, PENNFILE *f);
+void get_new_locks(dbref i, PENNFILE *f, int c);
+void db_read_attrs(PENNFILE *f, dbref i, int c);
+int get_list(PENNFILE *f, dbref i);
 void db_free(void);
 static void init_objdata_htab(int size, void (*free_data) (void *));
-static void db_write_flags(FILE * f);
-static dbref db_read_oldstyle(FILE * f);
+static void db_write_flags(PENNFILE *f);
+static dbref db_read_oldstyle(PENNFILE *f);
 
 StrTree object_names;       /**< String tree of object names */
 extern StrTree atr_names;
@@ -218,11 +225,11 @@ new_object(void)
   o->zone = NOTHING;
   o->penn = 0;
   o->type = TYPE_GARBAGE;
-  /* Flags are set by the functions that call this */
-  o->powers = new_flag_bitmask("POWER");
   o->warnings = 0;
   o->modification_time = o->creation_time = mudtime;
   o->attrcount = 0;
+  /* Flags are set by the functions that call this */
+  o->powers = new_flag_bitmask("POWER");
   if (current_state.garbage)
     current_state.garbage--;
   return newobj;
@@ -233,9 +240,9 @@ new_object(void)
  * \param ref value to write.
  */
 void
-putref(FILE * f, long int ref)
+putref(PENNFILE *f, long int ref)
 {
-  OUTPUT(fprintf(f, "%ld\n", ref));
+  penn_fprintf(f, "%ld\n", ref);
 }
 
 /** Output a string to a file.
@@ -245,22 +252,22 @@ putref(FILE * f, long int ref)
  * \param s value to write.
  */
 void
-putstring(FILE * f, const char *s)
+putstring(PENNFILE *f, const char *s)
 {
-  OUTPUT(putc('"', f));
+  penn_fputc('"', f);
   while (*s) {
     switch (*s) {
     case '\\':
     case '"':
-      OUTPUT(putc('\\', f));
+      penn_fputc('\\', f);
       /* FALL THROUGH */
     default:
-      OUTPUT(putc(*s, f));
+      penn_fputc(*s, f);
     }
     s++;
   }
-  OUTPUT(putc('"', f));
-  OUTPUT(putc('\n', f));
+  penn_fputc('"', f);
+  penn_fputc('\n', f);
 }
 
 /** Read a labeled entry from a database.
@@ -274,7 +281,7 @@ putstring(FILE * f, const char *s)
  * buffer containing the value that was read.
  */
 void
-db_read_labeled_string(FILE * f, char **label, char **value)
+db_read_labeled_string(PENNFILE *f, char **label, char **value)
 {
   static char lbuf[BUFFER_LEN], vbuf[BUFFER_LEN];
   int c;
@@ -288,14 +295,14 @@ db_read_labeled_string(FILE * f, char **label, char **value)
   dbline++;
 
   do {
-    c = getc(f);
+    c = penn_fgetc(f);
     while (isspace(c)) {
       if (c == '\n')
         dbline++;
-      c = getc(f);
+      c = penn_fgetc(f);
     }
     if (c == '#') {
-      while ((c = getc(f)) != '\n' && c != EOF) {
+      while ((c = penn_fgetc(f)) != '\n' && c != EOF) {
         /* nothing */
       }
       if (c == '\n')
@@ -319,7 +326,7 @@ db_read_labeled_string(FILE * f, char **label, char **value)
       longjmp(db_err, 1);
     }
     safe_chr(c, lbuf, &p);
-    c = getc(f);
+    c = penn_fgetc(f);
   } while (c != EOF && !isspace(c));
   *p++ = '\0';
   if (p >= lbuf + BUFFER_LEN)
@@ -327,7 +334,7 @@ db_read_labeled_string(FILE * f, char **label, char **value)
 
   /* suck up separating whitespace */
   while (c != '\n' && c != EOF && isspace(c))
-    c = getc(f);
+    c = penn_fgetc(f);
 
   /* check for presence of a value, which we must have. */
   if (c == EOF || c == '\n') {
@@ -347,11 +354,11 @@ db_read_labeled_string(FILE * f, char **label, char **value)
     int sline;
     sline = dbline;
     for (;;) {
-      c = getc(f);
+      c = penn_fgetc(f);
       if (c == '"')
         break;
       if (c == '\\')
-        c = getc(f);
+        c = penn_fgetc(f);
       if (c == EOF) {
         do_rawlog(LT_ERR, "DB: Unclosed quoted string starting on line %d",
                   sline);
@@ -366,7 +373,7 @@ db_read_labeled_string(FILE * f, char **label, char **value)
       safe_chr(c, vbuf, &p);
     }
     do {
-      c = getc(f);
+      c = penn_fgetc(f);
       if (c != EOF && !isspace(c)) {
         do_rawlog(LT_ERR, "DB: Garbage after quoted string, line %d", dbline);
         longjmp(db_err, 1);
@@ -382,7 +389,7 @@ db_read_labeled_string(FILE * f, char **label, char **value)
         longjmp(db_err, 1);
       }
       safe_chr(c, vbuf, &p);
-      c = getc(f);
+      c = penn_fgetc(f);
     } while (c != EOF && c != '\n');
     if (c == '\n' && (p - vbuf >= 2) && (*(p - 2) == '\r')) {
       /* Oops, we read in \r\n at the end of this value. Drop the \r */
@@ -406,7 +413,7 @@ db_read_labeled_string(FILE * f, char **label, char **value)
  * buffer containing the value that was read.
  */
 void
-db_read_this_labeled_string(FILE * f, const char *label, char **value)
+db_read_this_labeled_string(PENNFILE *f, const char *label, char **value)
 {
   char *readlabel;
 
@@ -420,7 +427,7 @@ db_read_this_labeled_string(FILE * f, const char *label, char **value)
   }
 }
 
-/** Read an integer with a given label.
+/** Read an int with a given label.
  * If the label read is different than the one being checked, the 
  * database load will abort with an error.
  * \param f the file to read from.
@@ -428,7 +435,7 @@ db_read_this_labeled_string(FILE * f, const char *label, char **value)
  * \param value pointer to update to the number that was read.
  */
 void
-db_read_this_labeled_number(FILE * f, const char *label, int *value)
+db_read_this_labeled_int(PENNFILE *f, const char *label, int *value)
 {
   char *readlabel;
   char *readvalue;
@@ -445,20 +452,61 @@ db_read_this_labeled_number(FILE * f, const char *label, int *value)
   *value = parse_integer(readvalue);
 }
 
-/** Read an integer and label.
+/** Read an int and label.
  * \param f the file to read from.
  * \param label pointer to update to the address of a static
  * buffer containing the label that was read.
  * \param value pointer to update to the number that was read.
  */
 void
-db_read_labeled_number(FILE * f, char **label, int *value)
+db_read_labeled_int(PENNFILE *f, char **label, int *value)
 {
   char *readvalue;
   db_read_labeled_string(f, label, &readvalue);
   *value = parse_integer(readvalue);
 }
 
+
+/** Read a uint32_t with a given label.
+ * If the label read is different than the one being checked, the 
+ * database load will abort with an error.
+ * \param f the file to read from.
+ * \param label the label that should be read.
+ * \param value pointer to update to the number that was read.
+ */
+void
+db_read_this_labeled_uint32(PENNFILE *f, const char *label, uint32_t *value)
+{
+  char *readlabel;
+  char *readvalue;
+
+  db_read_labeled_string(f, &readlabel, &readvalue);
+
+  if (strcmp(readlabel, label)) {
+    do_rawlog(LT_ERR,
+              T("DB: error: Got label '%s', expected label '%s' at line %d"),
+              readlabel, label, dbline);
+    longjmp(db_err, 1);
+  }
+
+  *value = parse_uint32(readvalue, NULL, 10);
+}
+
+/** Read a uint32_t and label.
+ * \param f the file to read from.
+ * \param label pointer to update to the address of a static
+ * buffer containing the label that was read.
+ * \param value pointer to update to the number that was read.
+ */
+void
+db_read_labeled_uint32(PENNFILE *f, char **label, uint32_t *value)
+{
+  char *readvalue;
+  db_read_labeled_string(f, label, &readvalue);
+  *value = parse_uint32(readvalue, NULL, 10);
+}
+
+
 /** Read a dbref with a given label.
  * If the label read is different than the one being checked, the 
  * database load will abort with an error.
@@ -467,7 +515,7 @@ db_read_labeled_number(FILE * f, char **label, int *value)
  * \param value pointer to update to the dbref that was read.
  */
 void
-db_read_this_labeled_dbref(FILE * f, const char *label, dbref *val)
+db_read_this_labeled_dbref(PENNFILE *f, const char *label, dbref *val)
 {
   char *readlabel;
   char *readvalue;
@@ -490,7 +538,7 @@ db_read_this_labeled_dbref(FILE * f, const char *label, dbref *val)
  * \param value pointer to update to the dbref that was read.
  */
 void
-db_read_labeled_dbref(FILE * f, char **label, dbref *val)
+db_read_labeled_dbref(PENNFILE *f, char **label, dbref *val)
 {
   char *readvalue;
   db_read_labeled_string(f, label, &readvalue);
@@ -498,29 +546,29 @@ db_read_labeled_dbref(FILE * f, char **label, dbref *val)
 }
 
 static void
-db_write_label(FILE * f, char const *l)
+db_write_label(PENNFILE *f, char const *l)
 {
-  OUTPUT(fputs(l, f));
-  OUTPUT(putc(' ', f));
+  penn_fputs(l, f);
+  penn_fputc(' ', f);
 }
 
 void
-db_write_labeled_string(FILE * f, char const *label, char const *value)
+db_write_labeled_string(PENNFILE *f, char const *label, char const *value)
 {
   db_write_label(f, label);
   putstring(f, value);
 }
 
 void
-db_write_labeled_number(FILE * f, char const *label, int value)
+db_write_labeled_int(PENNFILE *f, char const *label, int value)
 {
-  OUTPUT(fprintf(f, "%s %d\n", label, value));
+  penn_fprintf(f, "%s %d\n", label, value);
 }
 
 void
-db_write_labeled_dbref(FILE * f, char const *label, dbref value)
+db_write_labeled_dbref(PENNFILE *f, char const *label, dbref value)
 {
-  OUTPUT(fprintf(f, "%s #%d\n", label, value));
+  penn_fprintf(f, "%s #%d\n", label, value);
 }
 
 /** Write a boolexp to a file in unparsed (text) form.
@@ -528,7 +576,7 @@ db_write_labeled_dbref(FILE * f, char const *label, dbref value)
  * \param b pointer to boolexp to write.
  */
 void
-putboolexp(FILE * f, boolexp b)
+putboolexp(PENNFILE *f, boolexp b)
 {
   db_write_labeled_string(f, "  key", unparse_boolexp(GOD, b, UB_DBREF));
 }
@@ -538,18 +586,18 @@ putboolexp(FILE * f, boolexp b)
  * \param l pointer to lock_list to write.
  */
 void
-putlocks(FILE * f, lock_list *l)
+putlocks(PENNFILE *f, lock_list *l)
 {
   lock_list *ll;
   int count = 0;
   for (ll = l; ll; ll = ll->next)
     count++;
-  db_write_labeled_number(f, "lockcount", count);
+  db_write_labeled_int(f, "lockcount", count);
   for (ll = l; ll; ll = ll->next) {
     db_write_labeled_string(f, " type", ll->type);
     db_write_labeled_dbref(f, "  creator", L_CREATOR(ll));
     db_write_labeled_string(f, "  flags", lock_flags_long(ll));
-    db_write_labeled_number(f, "  derefs", chunk_derefs(L_KEY(ll)));
+    db_write_labeled_int(f, "  derefs", chunk_derefs(L_KEY(ll)));
     putboolexp(f, ll->key);
     /* putboolexp adds a '\n', so we won't. */
   }
@@ -564,7 +612,7 @@ putlocks(FILE * f, lock_list *l)
  * \param o pointer to object to write.
  */
 static void
-db_write_obj_basic(FILE * f, dbref i, struct object *o)
+db_write_obj_basic(PENNFILE *f, dbref i, struct object *o)
 {
   db_write_labeled_string(f, "name", o->name);
   db_write_labeled_dbref(f, "location", o->location);
@@ -575,15 +623,15 @@ db_write_obj_basic(FILE * f, dbref i, struct object *o)
   putlocks(f, Locks(i));
   db_write_labeled_dbref(f, "owner", o->owner);
   db_write_labeled_dbref(f, "zone", o->zone);
-  db_write_labeled_number(f, "pennies", Pennies(i));
-  db_write_labeled_number(f, "type", Typeof(i));
+  db_write_labeled_int(f, "pennies", Pennies(i));
+  db_write_labeled_int(f, "type", Typeof(i));
   db_write_labeled_string(f, "flags",
                           bits_to_string("FLAG", o->flags, GOD, NOTHING));
   db_write_labeled_string(f, "powers",
                           bits_to_string("POWER", o->powers, GOD, NOTHING));
   db_write_labeled_string(f, "warnings", unparse_warnings(o->warnings));
-  db_write_labeled_number(f, "created", (int) o->creation_time);
-  db_write_labeled_number(f, "modified", (int) o->modification_time);
+  db_write_labeled_int(f, "created", (int) o->creation_time);
+  db_write_labeled_int(f, "modified", (int) o->modification_time);
 }
 
 /** Write out an object.
@@ -592,7 +640,7 @@ db_write_obj_basic(FILE * f, dbref i, struct object *o)
  * \param i dbref of object to write.
  */
 int
-db_write_object(FILE * f, dbref i)
+db_write_object(PENNFILE *f, dbref i)
 {
   struct object *o;
   ALIST *list;
@@ -609,7 +657,7 @@ db_write_object(FILE * f, dbref i)
       continue;
     count++;
   }
-  db_write_labeled_number(f, "attrcount", count);
+  db_write_labeled_int(f, "attrcount", count);
 
   for (list = o->list; list; list = AL_NEXT(list)) {
     if (AF_Nodump(list))
@@ -617,7 +665,7 @@ db_write_object(FILE * f, dbref i)
     db_write_labeled_string(f, " name", AL_NAME(list));
     db_write_labeled_dbref(f, "  owner", Owner(AL_CREATOR(list)));
     db_write_labeled_string(f, "  flags", atrflag_to_string(AL_FLAGS(list)));
-    db_write_labeled_number(f, "  derefs", AL_DEREFS(list));
+    db_write_labeled_int(f, "  derefs", AL_DEREFS(list));
     db_write_labeled_string(f, "  value", atr_value(list));
   }
   return 0;
@@ -640,7 +688,7 @@ db_write_object(FILE * f, dbref i)
  * \return the number of objects in the database (db_top)
  */
 dbref
-db_write(FILE * f, int flag)
+db_write(PENNFILE *f, int flag)
 {
   dbref i;
   int dbflag;
@@ -670,14 +718,15 @@ db_write(FILE * f, int flag)
   dbflag += DBF_NEW_POWERS;
   dbflag += DBF_POWERS_LOGGED;
   dbflag += DBF_LABELS;
+  dbflag += DBF_SPIFFY_AF_ANSI;
 
-  OUTPUT(fprintf(f, "+V%d\n", dbflag * 256 + 2));
+  penn_fprintf(f, "+V%d\n", dbflag * 256 + 2);
 
   db_write_labeled_string(f, "savedtime", show_time(mudtime, 1));
 
   db_write_flags(f);
 
-  OUTPUT(fprintf(f, "~%d\n", db_top));
+  penn_fprintf(f, "~%d\n", db_top);
 
   for (i = 0; i < db_top; i++) {
 #ifdef WIN32SERVICES
@@ -687,19 +736,19 @@ db_write(FILE * f, int flag)
 #endif
     if (IsGarbage(i))
       continue;
-    OUTPUT(fprintf(f, "!%d\n", i));
+    penn_fprintf(f, "!%d\n", i);
     db_write_object(f, i);
   }
-  OUTPUT(fputs(EOD, f));
+  penn_fputs(EOD, f);
   return db_top;
 }
 
 static void
-db_write_flags(FILE * f)
+db_write_flags(PENNFILE *f)
 {
-  OUTPUT(fprintf(f, "+FLAGS LIST\n"));
+  penn_fprintf(f, "+FLAGS LIST\n");
   flag_write_all(f, "FLAG");
-  OUTPUT(fprintf(f, "+POWER LIST\n"));
+  penn_fprintf(f, "+POWER LIST\n");
   flag_write_all(f, "POWER");
 }
 
@@ -713,7 +762,7 @@ db_write_flags(FILE * f)
  * \param flag 1 = debug, 0 = normal
  */
 int
-db_paranoid_write_object(FILE * f, dbref i, int flag)
+db_paranoid_write_object(PENNFILE *f, dbref i, int flag)
 {
   struct object *o;
   ALIST *list, *next;
@@ -723,7 +772,6 @@ db_paranoid_write_object(FILE * f, dbref i, int flag)
   char *p;
   char lastp;
   dbref owner;
-  int flags;
   int fixmemdb = 0;
   int count = 0;
   int attrcount = 0;
@@ -739,7 +787,7 @@ db_paranoid_write_object(FILE * f, dbref i, int flag)
     attrcount++;
   }
 
-  db_write_labeled_number(f, "attrcount", attrcount);
+  db_write_labeled_int(f, "attrcount", attrcount);
 
   for (list = o->list; list; list = next) {
     next = AL_NEXT(list);
@@ -762,10 +810,10 @@ db_paranoid_write_object(FILE * f, dbref i, int flag)
         count = 0;
         do {
           name[BUFFER_LEN - 6] = '\0';
-          sprintf(tbuf1, "%s%d", name, count);
+          snprintf(tbuf1, BUFFER_LEN, "%s%d", name, count);
           count++;
         } while (count < 10000 && atr_get_noparent(i, tbuf1));
-        strcpy(name, tbuf1);
+        mush_strncpy(name, tbuf1, BUFFER_LEN);
       }
       do_rawlog(LT_CHECK,
                 T(" * Bad attribute name on #%d. Changing name to %s.\n"),
@@ -786,7 +834,7 @@ db_paranoid_write_object(FILE * f, dbref i, int flag)
     db_write_labeled_string(f, " name", name);
     db_write_labeled_dbref(f, "  owner", owner);
     db_write_labeled_string(f, "  flags", atrflag_to_string(AL_FLAGS(list)));
-    db_write_labeled_number(f, "  derefs", AL_DEREFS(list));
+    db_write_labeled_int(f, "  derefs", AL_DEREFS(list));
 
     /* now check the attribute */
     strcpy(tbuf1, atr_value(list));
@@ -811,7 +859,7 @@ db_paranoid_write_object(FILE * f, dbref i, int flag)
     db_write_labeled_string(f, "  value", tbuf1);
     if (flag && fixmemdb) {
       /* Fix the db in memory */
-      flags = AL_FLAGS(list);
+      privbits flags = AL_FLAGS(list);
       atr_clr(i, AL_NAME(list), owner);
       (void) atr_add(i, name, tbuf1, owner, flags);
       list = atr_get_noparent(i, name);
@@ -838,7 +886,7 @@ db_paranoid_write_object(FILE * f, dbref i, int flag)
  * \return the number of objects in the database (db_top)
  */
 dbref
-db_paranoid_write(FILE * f, int flag)
+db_paranoid_write(PENNFILE *f, int flag)
 {
   dbref i;
   int dbflag;
@@ -865,14 +913,15 @@ db_paranoid_write(FILE * f, int flag)
   dbflag += DBF_NEW_POWERS;
   dbflag += DBF_POWERS_LOGGED;
   dbflag += DBF_LABELS;
+  dbflag += DBF_SPIFFY_AF_ANSI;
 
   do_rawlog(LT_CHECK, "PARANOID WRITE BEGINNING...\n");
 
-  OUTPUT(fprintf(f, "+V%d\n", dbflag * 256 + 2));
+  penn_fprintf(f, "+V%d\n", dbflag * 256 + 2);
 
   db_write_labeled_string(f, "savedtime", show_time(mudtime, 1));
   db_write_flags(f);
-  OUTPUT(fprintf(f, "~%d\n", db_top));
+  penn_fprintf(f, "~%d\n", db_top);
 
   /* write out each object */
   for (i = 0; i < db_top; i++) {
@@ -883,13 +932,13 @@ db_paranoid_write(FILE * f, int flag)
 #endif
     if (IsGarbage(i))
       continue;
-    OUTPUT(fprintf(f, "!%d\n", i));
+    penn_fprintf(f, "!%d\n", i);
     db_paranoid_write_object(f, i, flag);
     /* print out a message every so many objects */
     if (i % globals.paranoid_checkpt == 0)
       do_rawlog(LT_CHECK, T("\t...wrote up to object #%d\n"), i);
   }
-  OUTPUT(fputs(EOD, f));
+  penn_fputs(EOD, f);
   do_rawlog(LT_CHECK, T("\t...finished at object #%d\n"), i - 1);
   do_rawlog(LT_CHECK, "END OF PARANOID WRITE.\n");
   return db_top;
@@ -901,15 +950,15 @@ db_paranoid_write(FILE * f, int flag)
  * \return long int read.
  */
 long int
-getref(FILE * f)
+getref(PENNFILE *f)
 {
   static char buf[BUFFER_LEN];
-  if (!fgets(buf, sizeof(buf), f)) {
+  if (!penn_fgets(buf, sizeof(buf), f)) {
     do_rawlog(LT_ERR, T("Unexpected EOF at line %d"), dbline);
     longjmp(db_err, 1);
   }
   dbline++;
-  return strtol(buf, NULL, 10);
+  return parse_integer(buf);
 }
 
 
@@ -921,15 +970,15 @@ getref(FILE * f)
  * \param f file pointer to read from.
  * \return pointer to static buffer containing string read.
  */
-const char *
-getstring_noalloc(FILE * f)
+char *
+getstring_noalloc(PENNFILE *f)
 {
   static char buf[BUFFER_LEN];
   char *p;
   int c;
 
   p = buf;
-  c = fgetc(f);
+  c = penn_fgetc(f);
   if (c == EOF) {
     do_rawlog(LT_ERR, T("Unexpected EOF at line %d"), dbline);
     longjmp(db_err, 1);
@@ -943,26 +992,26 @@ getstring_noalloc(FILE * f)
         return buf;
       }
       safe_chr(c, buf, &p);
-      c = fgetc(f);
+      c = penn_fgetc(f);
     }
   } else {
     for (;;) {
-      c = fgetc(f);
+      c = penn_fgetc(f);
       if (c == '"') {
         /* It's a closing quote if it's followed by \r or \n */
-        c = fgetc(f);
+        c = penn_fgetc(f);
         if (c == '\r') {
           /* Get a possible \n, too */
-          if ((c = fgetc(f)) != '\n')
-            ungetc(c, f);
+          if ((c = penn_fgetc(f)) != '\n')
+            penn_ungetc(c, f);
           else
             dbline++;
         } else if (c != '\n')
-          ungetc(c, f);
+          penn_ungetc(c, f);
         *p = '\0';
         return buf;
       } else if (c == '\\') {
-        c = fgetc(f);
+        c = penn_fgetc(f);
       }
       if ((c == '\0') || (c == EOF)) {
         *p = '\0';
@@ -981,7 +1030,7 @@ getstring_noalloc(FILE * f)
  * \return pointer to boolexp read.
  */
 boolexp
-getboolexp(FILE * f, const char *type)
+getboolexp(PENNFILE *f, const char *type)
 {
   char *val;
   db_read_this_labeled_string(f, "key", &val);
@@ -998,11 +1047,11 @@ extern PRIV lock_privs[];
  * \param c number of locks, or -1 if not yet known.
  */
 void
-get_new_locks(dbref i, FILE * f, int c)
+get_new_locks(dbref i, PENNFILE *f, int c)
 {
   char *val, *key;
   dbref creator;
-  int flags;
+  privbits flags;
   char type[BUFFER_LEN];
   boolexp b;
   int count = c, derefs = 0, found = 0;
@@ -1015,8 +1064,8 @@ get_new_locks(dbref i, FILE * f, int c)
   for (;;) {
     int ch;
 
-    ch = fgetc(f);
-    ungetc(ch, f);
+    ch = penn_fgetc(f);
+    penn_ungetc(ch, f);
 
     if (ch != ' ')
       break;
@@ -1030,10 +1079,10 @@ get_new_locks(dbref i, FILE * f, int c)
       db_read_this_labeled_dbref(f, "creator", &creator);
       db_read_this_labeled_string(f, "flags", &val);
       flags = string_to_privs(lock_privs, val, 0);
-      db_read_this_labeled_number(f, "derefs", &derefs);
+      db_read_this_labeled_int(f, "derefs", &derefs);
     } else {
-      db_read_this_labeled_number(f, "creator", &creator);
-      db_read_this_labeled_number(f, "flags", &flags);
+      db_read_this_labeled_int(f, "creator", &creator);
+      db_read_this_labeled_uint32(f, "flags", &flags);
     }
     /* boolexp */
     db_read_this_labeled_string(f, "key", &key);
@@ -1059,15 +1108,15 @@ get_new_locks(dbref i, FILE * f, int c)
  * \param f file pointer to read from.
  */
 void
-getlocks(dbref i, FILE * f)
+getlocks(dbref i, PENNFILE *f)
 {
   /* Assumes it begins at the beginning of a line. */
   int c;
   boolexp b;
   char buf[BUFFER_LEN], *p;
-  while ((c = getc(f)), c != EOF && c == '_') {
+  while ((c = penn_fgetc(f)), c != EOF && c == '_') {
     p = buf;
-    while ((c = getc(f)), c != EOF && c != '|') {
+    while ((c = penn_fgetc(f)), c != EOF && c != '|') {
       *p++ = c;
     }
     *p = '\0';
@@ -1080,10 +1129,10 @@ getlocks(dbref i, FILE * f)
       /* getboolexp() would already have complained. */
       return;
     } else {
-      add_lock_raw(Owner(i), i, buf, b, -1);
+      add_lock_raw(Owner(i), i, buf, b, LF_DEFAULT);
     }
   }
-  ungetc(c, f);
+  penn_ungetc(c, f);
   return;
 }
 
@@ -1116,24 +1165,27 @@ db_free(void)
  * \param i dbref for the attribute list.
  */
 int
-get_list(FILE * f, dbref i)
+get_list(PENNFILE *f, dbref i)
 {
   int c;
   char *p, *q;
   char tbuf1[BUFFER_LEN + 150];
-  int flags;
+  privbits flags;
   int count = 0;
-  unsigned char derefs;
+  uint8_t derefs;
+  ansi_string *as;
+  char tbuf2[BUFFER_LEN];
+  char *tb2;
 
   List(i) = NULL;
   tbuf1[0] = '\0';
   while (1)
-    switch (c = getc(f)) {
+    switch (c = penn_fgetc(f)) {
     case ']':                  /* new style attribs, read name then value */
       /* Using getstring_noalloc here will cause problems with attribute
          names starting with ". This is probably a better fix than just
          disallowing " in attribute names. */
-      fgets(tbuf1, BUFFER_LEN + 150, f);
+      penn_fgets(tbuf1, BUFFER_LEN + 150, f);
       if (!(p = strchr(tbuf1, '^'))) {
         do_rawlog(LT_ERR, T("ERROR: Bad format on new attributes. object #%d"),
                   i);
@@ -1147,7 +1199,7 @@ get_list(FILE * f, dbref i)
         return -1;
       }
       *q++ = '\0';
-      flags = atoi(q);
+      flags = parse_uinteger(q);
       /* Remove obsolete AF_NUKED flag and AF_STATIC, just in case */
       flags &= ~AF_NUKED;
       flags &= ~AF_STATIC;
@@ -1161,14 +1213,24 @@ get_list(FILE * f, dbref i)
          present. */
       q = strchr(q, '^');
       if (q++)
-        derefs = atoi(q);
+        derefs = parse_uinteger(q);
       else
         derefs = 0;
       /* We add the attribute assuming that atoi(p) is an ok dbref
        * since we haven't loaded the whole db and can't really tell
        * if it is or not. We'll fix this up at the end of the load 
        */
-      atr_new_add(i, tbuf1, getstring_noalloc(f), atoi(p), flags, derefs);
+      tb2 = getstring_noalloc(f);
+      if (strchr(tb2, TAG_START) || strchr(tb2, ESC_CHAR)) {
+        as = parse_ansi_string(tb2);
+        tb2 = tbuf2;
+        safe_ansi_string(as, 0, as->len, tbuf2, &tb2);
+        *(tb2) = '\0';
+        tb2 = tbuf2;
+        free_ansi_string(as);
+      }
+
+      atr_new_add(i, tbuf1, tb2, atoi(p), flags, derefs);
       count++;
       /* Check removed for atoi(q) == 0  (which results in NOTHING for that
        * parameter, and thus no flags), since this eliminates 'visual'
@@ -1180,7 +1242,7 @@ get_list(FILE * f, dbref i)
       return -1;
       break;
     case '<':                  /* end of list */
-      if ('\n' != getc(f)) {
+      if ('\n' != penn_fgetc(f)) {
         do_rawlog(LT_ERR, T("ERROR: no line feed after < on object %d"), i);
         return -1;
       }
@@ -1212,23 +1274,24 @@ extern PRIV attr_privs_view[];
  * \param count the number of attributes to read.
  */
 void
-db_read_attrs(FILE * f, dbref i, int count)
+db_read_attrs(PENNFILE *f, dbref i, int count)
 {
   char name[ATTRIBUTE_NAME_LIMIT + 1];
   char value[BUFFER_LEN + 1];
   dbref owner;
   int derefs;
-  int flags;
+  privbits flags;
   char *tmp;
   int found = 0;
+  ansi_string *as;
 
   List(i) = NULL;
 
   for (;;) {
     int c;
 
-    c = fgetc(f);
-    ungetc(c, f);
+    c = penn_fgetc(f);
+    penn_ungetc(c, f);
 
     if (c != ' ')
       break;
@@ -1240,9 +1303,18 @@ db_read_attrs(FILE * f, dbref i, int count)
     db_read_this_labeled_dbref(f, "owner", &owner);
     db_read_this_labeled_string(f, "flags", &tmp);
     flags = string_to_privs(attr_privs_view, tmp, 0);
-    db_read_this_labeled_number(f, "derefs", &derefs);
+    db_read_this_labeled_int(f, "derefs", &derefs);
     db_read_this_labeled_string(f, "value", &tmp);
     strcpy(value, tmp);
+    if (!(globals.indb_flags & DBF_SPIFFY_AF_ANSI)) {
+      if (strchr(value, ESC_CHAR) || strchr(value, TAG_START)) {
+        char *vp = value;
+        as = parse_ansi_string(value);
+        safe_ansi_string(as, 0, as->len, value, &vp);
+        *vp = '\0';
+        free_ansi_string(as);
+      }
+    }
     atr_new_add(i, name, value, owner, flags, derefs);
   }
 
@@ -1259,7 +1331,7 @@ db_read_attrs(FILE * f, dbref i, int count)
  * \return number of objects in the database
  */
 static dbref
-db_read_oldstyle(FILE * f)
+db_read_oldstyle(PENNFILE *f)
 {
   int c;
   dbref i;
@@ -1270,7 +1342,7 @@ db_read_oldstyle(FILE * f)
   for (i = 0;; i++) {
     /* Loop invariant: we always begin at the beginning of a line. */
     errobj = i;
-    c = getc(f);
+    c = penn_fgetc(f);
     switch (c) {
       /* make sure database is at least this big *1.5 */
     case '~':
@@ -1279,7 +1351,7 @@ db_read_oldstyle(FILE * f)
       break;
       /* Use the MUSH 2.0 header stuff to see what's in this db */
     case '+':
-      c = getc(f);              /* Skip the V */
+      c = penn_fgetc(f);        /* Skip the V */
       if (c == 'F') {
         (void) getstring_noalloc(f);
         flag_read_all(f, "FLAG");
@@ -1352,18 +1424,16 @@ db_read_oldstyle(FILE * f)
         break;
       }
 
-      if (IsPlayer(i) && (strlen(o->name) > (Size_t) PLAYER_NAME_LIMIT)) {
+      if (IsPlayer(i) && (strlen(o->name) > (size_t) PLAYER_NAME_LIMIT)) {
         char buff[BUFFER_LEN + 1];      /* The name plus a NUL */
-        strncpy(buff, o->name, PLAYER_NAME_LIMIT);
-        buff[PLAYER_NAME_LIMIT] = '\0';
+        mush_strncpy(buff, o->name, PLAYER_NAME_LIMIT);
         set_name(i, buff);
         do_rawlog(LT_CHECK,
                   T(" * Name of #%d is longer than the maximum, truncating.\n"),
                   i);
       } else if (!IsPlayer(i) && (strlen(o->name) > OBJECT_NAME_LIMIT)) {
         char buff[OBJECT_NAME_LIMIT + 1];       /* The name plus a NUL */
-        strncpy(buff, o->name, OBJECT_NAME_LIMIT);
-        buff[OBJECT_NAME_LIMIT] = '\0';
+        mush_strncpy(buff, o->name, OBJECT_NAME_LIMIT);
         set_name(i, buff);
         do_rawlog(LT_CHECK,
                   T(" * Name of #%d is longer than the maximum, truncating.\n"),
@@ -1443,8 +1513,8 @@ db_read_oldstyle(FILE * f)
     case '*':
       {
         char buff[80];
-        ungetc('*', f);
-        fgets(buff, sizeof buff, f);
+        penn_ungetc('*', f);
+        penn_fgets(buff, sizeof buff, f);
         if (strcmp(buff, EOD) != 0) {
           do_rawlog(LT_ERR, T("ERROR: No end of dump after object #%d"), i - 1);
           return -1;
@@ -1471,7 +1541,7 @@ db_read_oldstyle(FILE * f)
  * \return number of objects in the database.
  */
 dbref
-db_read(FILE * f)
+db_read(PENNFILE *f)
 {
   int c;
   dbref i = 0;
@@ -1489,12 +1559,12 @@ db_read(FILE * f)
   db_free();
   globals.indb_flags = 1;
 
-  c = fgetc(f);
+  c = penn_fgetc(f);
   if (c != '+') {
     do_rawlog(LT_ERR, T("Database does not start with a version string"));
     return -1;
   }
-  c = fgetc(f);
+  c = penn_fgetc(f);
   if (c != 'V') {
     do_rawlog(LT_ERR, T("Database does not start with a version string"));
     return -1;
@@ -1517,10 +1587,10 @@ db_read(FILE * f)
 
   do_rawlog(LT_ERR, T("Loading database saved on %s UTC"), db_timestamp);
 
-  while ((c = fgetc(f)) != EOF) {
+  while ((c = penn_fgetc(f)) != EOF) {
     switch (c) {
     case '+':
-      c = fgetc(f);
+      c = penn_fgetc(f);
       if (c == 'F') {
         (void) getstring_noalloc(f);
         flag_read_all(f, "FLAG");
@@ -1582,8 +1652,8 @@ db_read(FILE * f)
         db_grow(i + 1);
         o = db + i;
         while (1) {
-          c = fgetc(f);
-          ungetc(c, f);
+          c = penn_fgetc(f);
+          penn_ungetc(c, f);
           /* At the start of another object or the EOD marker */
           if (c == '!' || c == '*')
             break;
@@ -1709,8 +1779,8 @@ db_read(FILE * f)
     case '*':
       {
         char buff[80];
-        ungetc('*', f);
-        fgets(buff, sizeof buff, f);
+        penn_ungetc('*', f);
+        penn_fgets(buff, sizeof buff, f);
         if (strcmp(buff, EOD) != 0) {
           do_rawlog(LT_ERR, T("ERROR: No end of dump after object #%d"), i - 1);
           return -1;
@@ -1734,10 +1804,10 @@ db_read(FILE * f)
 static void
 init_objdata_htab(int size, void (*free_data) (void *))
 {
-  if (size < 128)
-    size = 128;
-  hash_init(&htab_objdata, size, 4, free_data);
-  hashinit(&htab_objdata_keys, 8, 32);
+  if (size < 10)
+    size = 10;
+  hash_init(&htab_objdata, size, free_data);
+  hashinit(&htab_objdata_keys, 8);
 }
 
 /** Add data to the object data hashtable. 
@@ -1745,7 +1815,9 @@ init_objdata_htab(int size, void (*free_data) (void *))
  * that is built at database load and isn't saved to disk, but it
  * can be used for other purposes as well - it's a good general
  * tool for hackers who want to add their own data to objects.
- * This function adds data to the hashtable.
+ * This function adds data to the hashtable. NULL data cleared
+ * that particular keybase/object entry. It does not free the
+ * data pointer.
  * \param thing dbref of object to associate the data with.
  * \param keybase base string for type of data.
  * \param data pointer to the data to store.
@@ -1754,13 +1826,16 @@ init_objdata_htab(int size, void (*free_data) (void *))
 void *
 set_objdata(dbref thing, const char *keybase, void *data)
 {
-  hashdelete(tprintf("%s_#%d", keybase, thing), &htab_objdata);
+  char keyname[BUFFER_LEN];
+
+  mush_strncpy(keyname, tprintf("%s_#%d", keybase, thing), BUFFER_LEN);
+  hashdelete(keyname, &htab_objdata);
   if (data) {
-    if (hashadd(tprintf("%s_#%d", keybase, thing), data, &htab_objdata) < 0)
+    if (!hashadd(keyname, data, &htab_objdata))
       return NULL;
     if (hash_find(&htab_objdata_keys, keybase) == NULL) {
-      char *newkey = strdup(keybase);
-      hashadd(keybase, (void *) &newkey, &htab_objdata_keys);
+      char *newkey = mush_strdup(keyname, "objdata.key");
+      hashadd(keybase, newkey, &htab_objdata_keys);
     }
   }
   return data;
@@ -1798,14 +1873,13 @@ void
 create_minimal_db(void)
 {
   dbref start_room, god, master_room;
-
-  int desc_flags = AF_VISUAL | AF_NOPROG | AF_PREFIXMATCH;
+  uint32_t desc_flags = AF_VISUAL | AF_NOPROG | AF_PREFIXMATCH;
 
   start_room = new_object();    /* #0 */
   god = new_object();           /* #1 */
   master_room = new_object();   /* #2 */
 
-  init_objdata_htab(DB_INITIAL_SIZE, NULL);
+  init_objdata_htab(128, NULL);
 
   set_name(start_room, "Room Zero");
   Type(start_room) = TYPE_ROOM;
@@ -1823,9 +1897,11 @@ create_minimal_db(void)
   Owner(god) = god;
   CreTime(god) = mudtime;
   ModTime(god) = (time_t) 0;
-  add_lock(god, god, Basic_Lock, parse_boolexp(god, "=me", Basic_Lock), -1);
-  add_lock(god, god, Enter_Lock, parse_boolexp(god, "=me", Enter_Lock), -1);
-  add_lock(god, god, Use_Lock, parse_boolexp(god, "=me", Use_Lock), -1);
+  add_lock(god, god, Basic_Lock, parse_boolexp(god, "=me", Basic_Lock),
+           LF_DEFAULT);
+  add_lock(god, god, Enter_Lock, parse_boolexp(god, "=me", Enter_Lock),
+           LF_DEFAULT);
+  add_lock(god, god, Use_Lock, parse_boolexp(god, "=me", Use_Lock), LF_DEFAULT);
   atr_new_add(god, "DESCRIBE", "You see Number One.", god, desc_flags, 1);
   atr_new_add(god, "MAILCURF", "0", god, AF_LOCKED | AF_NOPROG | AF_WIZARD, 1);
   add_folder_name(god, 0, "inbox");
@@ -1844,4 +1920,207 @@ create_minimal_db(void)
               "This is the master room. Any exit in here is considered global. The same is true for objects with $-commands placed here.",
               god, desc_flags, 1);
   current_state.rooms++;
+
+
+  init_chatdb();
+  mail_init();
+}
+
+
+/** Run a function, and jump if error */
+#define OUTPUT(fun) do { if ((fun) < 0) longjmp(db_err, 1); } while (0)
+
+/* Wrapper for fopen for use in reboot code. */
+PENNFILE *
+penn_fopen(const char *filename, const char *mode)
+{
+  PENNFILE *pf;
+
+  pf = mush_malloc(sizeof *pf, "pennfile");
+  pf->type = PFT_FILE;
+  pf->handle.f = fopen(filename, mode);
+  if (!pf->handle.f) {
+    do_rawlog(LT_ERR, "Unable to open %s in mode '%s': %s",
+              filename, mode, strerror(errno));
+    mush_free(pf, "pennfile");
+    return NULL;
+  }
+  return pf;
+}
+
+/* Close a db file, which may really be a pipe */
+void
+penn_fclose(PENNFILE *pf)
+{
+  switch (pf->type) {
+  case PFT_PIPE:
+#ifndef WIN32
+    pclose(pf->handle.f);
+#endif
+    break;
+  case PFT_FILE:
+    fclose(pf->handle.f);
+    break;
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    gzclose(pf->handle.g);
+#endif
+    break;
+  }
+  mush_free(pf, "pennfile");
+}
+
+
+int
+penn_fgetc(PENNFILE *f)
+{
+  switch (f->type) {
+  case PFT_FILE:
+  case PFT_PIPE:
+    return fgetc(f->handle.f);
+    break;
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    return gzgetc(f->handle.g);
+#endif
+    break;
+  }
+  return 0;
+}
+
+char *
+penn_fgets(char *buf, int len, PENNFILE *pf)
+{
+  switch (pf->type) {
+  case PFT_FILE:
+  case PFT_PIPE:
+    return fgets(buf, len, pf->handle.f);
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    return gzgets(pf->handle.g, buf, len);
+#endif
+    break;
+  }
+  return NULL;
+}
+
+int
+penn_fputc(int c, PENNFILE *f)
+{
+  switch (f->type) {
+  case PFT_FILE:
+  case PFT_PIPE:
+    OUTPUT(fputc(c, f->handle.f));
+    break;
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    OUTPUT(gzputc(f->handle.g, c));
+#endif
+    break;
+  }
+  return 0;
+}
+
+int
+penn_fputs(const char *s, PENNFILE *f)
+{
+  switch (f->type) {
+  case PFT_FILE:
+  case PFT_PIPE:
+    OUTPUT(fputs(s, f->handle.f));
+    break;
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    OUTPUT(gzputs(f->handle.g, s));
+#endif
+    break;
+  }
+  return 0;
+}
+
+int
+penn_fprintf(PENNFILE *f, const char *fmt, ...)
+{
+  va_list ap;
+  int r = -1;
+
+  switch (f->type) {
+  case PFT_FILE:
+  case PFT_PIPE:
+    va_start(ap, fmt);
+    r = vfprintf(f->handle.f, fmt, ap);
+    va_end(ap);
+    if (r < 0)
+      longjmp(db_err, 1);
+    break;
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    /* No equivalent to vfprintf in zlib... */
+#ifdef HAVE_VASPRINTF
+    {                           /* Safe GNU/BSD way */
+      char *line = NULL;
+      va_start(ap, fmt);
+      r = vasprintf(&line, fmt, ap);
+      va_end(ap);
+      if (r > -1) {
+        OUTPUT(gzputs(f->handle.g, line));
+        free(line);
+      } else
+        longjmp(db_err, 1);
+    }
+#else
+    {
+      char line[BUFFER_LEN * 2];
+
+      va_start(ap, fmt);
+#ifdef HAVE_VSNPRINTF
+      r = vsnprintf(line, sizeof line, fmt, ap);
+#else
+      r = vsprintf(line, fmt, ap);
+#endif
+      va_end(ap, fmt);
+      if (r > -1)
+        OUTPUT(gzputs(f->handle.g, line));
+      else
+        longjmp(db_err, 1);
+    }
+#endif
+#endif
+    break;
+  }
+  return r;
+}
+
+int
+penn_ungetc(int c, PENNFILE *f)
+{
+  switch (f->type) {
+  case PFT_FILE:
+  case PFT_PIPE:
+    OUTPUT(ungetc(c, f->handle.f));
+    break;
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    OUTPUT(gzungetc(c, f->handle.g));
+#endif
+    break;
+  }
+  return c;
+
+}
+
+int
+penn_feof(PENNFILE *pf)
+{
+  switch (pf->type) {
+  case PFT_FILE:
+  case PFT_PIPE:
+    return feof(pf->handle.f);
+  case PFT_GZFILE:
+#ifdef HAVE_LIBZ
+    return gzeof(pf->handle.g);
+#endif
+    break;
+  }
+  return 0;
 }
