@@ -54,7 +54,6 @@
 #include "config.h"
 
 #include <ctype.h>
-#include <assert.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
@@ -736,9 +735,6 @@ free_object(dbref thing)
 
   current_state.garbage++;
 
-#ifdef HAS_ASSERT
-  assert(IsGarbage(thing));
-#endif
 }
 
 static void
@@ -883,6 +879,32 @@ clear_exit(dbref thing)
   giveto(Owner(thing), EXIT_COST);
 }
 
+/** If object is in the free list, move it to the very beginning.
+ * \param object dbref of object to move
+ * \return 1 if object is moved successfully, 0 otherwise
+ */
+int
+make_first_free(dbref object)
+{
+  dbref curr;
+  dbref prev = NOTHING;
+
+  if (first_free == NOTHING || !GoodObject(object) || !IsGarbage(object))
+    return 0;                   /* no garbage, or object isn't garbage */
+  else if (first_free == object)
+    return 1;                   /* object is already at the head of the queue */
+  for (curr = first_free; Next(curr); curr = Next(curr)) {
+    if (curr == object) {
+      Next(prev) = Next(curr);
+      Next(curr) = first_free;
+      first_free = curr;
+      return 1;
+    } else
+      prev = curr;
+  }
+  return 0;
+
+}
 
 /** Return a cleaned up object off the free list or NOTHING.
  * \return a garbage object or NOTHING.
@@ -902,12 +924,12 @@ free_get(void)
     if (nrecur++ == 20) {
       first_free = NOTHING;
       report();
-      do_rawlog(LT_ERR, T("ERROR: Removed free list and continued\n"));
+      do_rawlog(LT_ERR, T("ERROR: Removed free list and continued"));
       return (NOTHING);
     }
     report();
-    do_rawlog(LT_TRACE, T("ERROR: Object #%d should not be free\n"), newobj);
-    do_rawlog(LT_TRACE, T("ERROR: Corrupt free list, fixing\n"));
+    do_rawlog(LT_TRACE, T("ERROR: Object #%d should not be free"), newobj);
+    do_rawlog(LT_TRACE, T("ERROR: Corrupt free list, fixing"));
     fix_free_list();
     temp = free_get();
     nrecur--;
@@ -991,6 +1013,7 @@ dbck(void)
   check_connected_rooms();
   check_zones();
   local_dbck();
+  validate_config();
 }
 
 /* Do sanity checks on non-destroyed objects. */
