@@ -1,3 +1,4 @@
+
 /**
  * \file extmail.c
  *
@@ -62,6 +63,9 @@
 
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -70,6 +74,9 @@
 #include <sys/types.h>
 #endif
 #include <string.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 
 #include "conf.h"
 #include "externs.h"
@@ -82,6 +89,7 @@
 #include "attrib.h"
 #include "parse.h"
 #include "mymalloc.h"
+#include "ansi.h"
 #include "pueblo.h"
 #include "flags.h"
 #include "log.h"
@@ -94,8 +102,8 @@
 extern int do_convtime(const char *str, struct tm *ttm);        /* funtime.c */
 
 static void do_mail_flags
-  (dbref player, const char *msglist, mail_flag flag, int negate);
-static char *mail_list_time(const char *the_time, int flag);
+  (dbref player, const char *msglist, mail_flag flag, bool negate);
+static char *mail_list_time(const char *the_time, bool flag);
 static MAIL *mail_fetch(dbref player, int num);
 static MAIL *real_mail_fetch(dbref player, int num, int folder);
 static MAIL *mailfun_fetch(dbref player, int nargs, char *arg1, char *arg2);
@@ -135,6 +143,8 @@ static int was_sender(dbref player, MAIL *mp);
 MAIL *maildb;            /**< The head of the mail list */
 MAIL *tail_ptr;          /**< The end of the mail list */
 
+slab *mail_slab; /**< slab for 'struct mail' allocations */
+
 #define HEAD  maildb     /**< The head of the mail list */
 #define TAIL  tail_ptr   /**< The end of the mail list */
 
@@ -282,7 +292,7 @@ do_mail_change_folder(dbref player, char *fld, char *newname)
       return;
     }
     for (p = newname; p && *p; p++) {
-      if (!isdigit((unsigned char) *p) && !isalpha((unsigned char) *p)) {
+      if (!isalnum((unsigned char) *p)) {
         notify(player, T("MAIL: Illegal folder name"));
         return;
       }
@@ -372,11 +382,12 @@ do_mail_unclear(dbref player, const char *msglist)
  * \param negate if 1, clear the flag; if 0, set the flag.
  */
 static void
-do_mail_flags(dbref player, const char *msglist, mail_flag flag, int negate)
+do_mail_flags(dbref player, const char *msglist, mail_flag flag, bool negate)
 {
   MAIL *mp;
   struct mail_selector ms;
-  int j, folder;
+  int j;
+  mail_flag folder;
   folder_array i;
   int notified = 0;
 
@@ -458,7 +469,8 @@ do_mail_file(dbref player, char *msglist, char *folder)
 {
   MAIL *mp;
   struct mail_selector ms;
-  int j, foldernum, origfold;
+  int j, foldernum;
+  mail_flag origfold;
   folder_array i;
   int notified = 0;
 
@@ -515,7 +527,8 @@ do_mail_read(dbref player, char *msglist)
   char tbuf1[BUFFER_LEN];
   char folderheader[BUFFER_LEN];
   struct mail_selector ms;
-  int j, folder;
+  int j;
+  mail_flag folder;
   folder_array i;
 
   if (!parse_msglist(msglist, &ms, player)) {
@@ -532,15 +545,15 @@ do_mail_read(dbref player, char *msglist)
         /* Read it */
         j++;
         if (SUPPORT_PUEBLO) {
-          notify_noenter(player, tprintf("%cSAMP%c", TAG_START, TAG_END));
-          sprintf(folderheader,
-                  "%cA XCH_HINT=\"List messages in this folder\" XCH_CMD=\"@mail/list %d:1-\"%c%s%c/A%c",
-                  TAG_START, Folder(mp), TAG_END, T("Folder:"), TAG_START,
-                  TAG_END);
+          notify_noenter(player, open_tag("SAMP"));
+          snprintf(folderheader, BUFFER_LEN,
+                   "%c%cA XCH_HINT=\"List messages in this folder\" XCH_CMD=\"@mail/list %d:1-\"%c%s%c%c/A%c",
+                   TAG_START, MARKUP_HTML, Folder(mp), TAG_END, T("Folder:"),
+                   TAG_START, MARKUP_HTML, TAG_END);
         } else
-          strcpy(folderheader, T("Folder:"));
+          mush_strncpy(folderheader, T("Folder:"), BUFFER_LEN);
         notify(player, DASH_LINE);
-        strcpy(tbuf1, get_sender(mp, 1));
+        mush_strncpy(tbuf1, get_sender(mp, 1), BUFFER_LEN);
         notify_format(player,
                       T
                       ("From: %-55s %s\nDate: %-25s   %s %2d   Message: %d\nStatus: %s"),
@@ -553,12 +566,11 @@ do_mail_read(dbref player, char *msglist)
         notify_format(player, T("Subject: %s"), get_subject(mp));
         notify(player, DASH_LINE);
         if (SUPPORT_PUEBLO)
-          notify_noenter(player, tprintf("%c/SAMP%c", TAG_START, TAG_END));
+          notify_noenter(player, close_tag("SAMP"));
         strcpy(tbuf1, get_message(mp));
         notify(player, tbuf1);
         if (SUPPORT_PUEBLO)
-          notify_format(player, "%cSAMP%c%s%c/SAMP%c", TAG_START, TAG_END,
-                        DASH_LINE, TAG_START, TAG_END);
+          notify(player, wrap_tag("SAMP", DASH_LINE));
         else
           notify(player, DASH_LINE);
         if (Unread(mp))
@@ -586,7 +598,8 @@ do_mail_list(dbref player, const char *msglist)
   char sender[30];
   MAIL *mp;
   struct mail_selector ms;
-  int j, folder;
+  int j;
+  mail_flag folder;
   folder_array i;
 
   if (!parse_msglist(msglist, &ms, player)) {
@@ -596,7 +609,7 @@ do_mail_list(dbref player, const char *msglist)
   j = 0;
   folder = AllInFolder(ms) ? player_folder(player) : MSFolder(ms);
   if (SUPPORT_PUEBLO)
-    notify_noenter(player, tprintf("%cSAMP%c", TAG_START, TAG_END));
+    notify_noenter(player, open_tag("SAMP"));
   notify_format(player,
                 T
                 ("---------------------------  MAIL (folder %2d)  ------------------------------"),
@@ -610,8 +623,8 @@ do_mail_list(dbref player, const char *msglist)
         if (SUPPORT_PUEBLO)
           notify_noenter(player,
                          tprintf
-                         ("%cA XCH_CMD=\"@mail/read %d:%d\" XCH_HINT=\"Read message %d in folder %d\"%c",
-                          TAG_START, Folder(mp), i[Folder(mp)],
+                         ("%c%cA XCH_CMD=\"@mail/read %d:%d\" XCH_HINT=\"Read message %d in folder %d\"%c",
+                          TAG_START, MARKUP_HTML, Folder(mp), i[Folder(mp)],
                           i[Folder(mp)], Folder(mp), TAG_END));
         strcpy(subj, chopstr(get_subject(mp), 28));
         strcpy(sender, chopstr(get_sender(mp, 0), 12));
@@ -623,18 +636,19 @@ do_mail_list(dbref player, const char *msglist)
                        ? '*' : ' '), sender, 30, subj,
                       mail_list_time(show_time(mp->time, 0), 1));
         if (SUPPORT_PUEBLO)
-          notify_noenter(player, tprintf("%c/A%c", TAG_START, TAG_END));
+          notify_noenter(player, tprintf("%c%c/A%c", TAG_START,
+                                         MARKUP_HTML, TAG_END));
       }
     }
   }
   notify(player, DASH_LINE);
   if (SUPPORT_PUEBLO)
-    notify_format(player, "%c/SAMP%c", TAG_START, TAG_END);
+    notify(player, close_tag("SAMP"));
   return;
 }
 
 static char *
-mail_list_time(const char *the_time, int flag /* 1 for no year */ )
+mail_list_time(const char *the_time, bool flag /* 1 for no year */ )
 {
   static char newtime[BUFFER_LEN];
   const char *p;
@@ -697,16 +711,12 @@ do_mail_purge(dbref player)
       mdb_top--;
       free(mp->subject);
       chunk_delete(mp->msgid);
-      mush_free((Malloc_t) mp, "mail");
+      slab_free(mail_slab, mp);
     } else {
       nextp = mp->next;
     }
   }
-  /* Clean up the player's mailp */
-  if (Connected(player)) {
-    desc_mail_set(player, NULL);
-    desc_mail_set(player, find_exact_starting_point(player));
-  }
+  set_objdata(player, "MAIL", NULL);
   if (command_check_byname(player, "@MAIL"))
     notify(player, T("MAIL: Mailbox purged."));
   return;
@@ -726,7 +736,8 @@ do_mail_fwd(dbref player, char *msglist, char *tolist)
   MAIL *mp;
   MAIL *last;
   struct mail_selector ms;
-  int j, num, folder;
+  int j, num;
+  mail_flag folder;
   folder_array i;
   const char *head;
   MAIL *temp;
@@ -775,8 +786,8 @@ do_mail_fwd(dbref player, char *msglist, char *tolist)
             } else {
               char tbuf1[BUFFER_LEN];
               unsigned char tbuf2[BUFFER_LEN];
-              strcpy(tbuf1, uncompress(mp->subject));
-              u_strcpy(tbuf2, get_compressed_message(mp));
+              mush_strncpy(tbuf1, uncompress(mp->subject), BUFFER_LEN);
+              u_strncpy(tbuf2, get_compressed_message(mp), BUFFER_LEN);
               send_mail(player, temp->from, tbuf1, (char *) tbuf2,
                         M_FORWARD | M_REPLY, 1, 0);
               num_recpts++;
@@ -795,8 +806,8 @@ do_mail_fwd(dbref player, char *msglist, char *tolist)
             } else {
               char tbuf1[BUFFER_LEN];
               unsigned char tbuf2[BUFFER_LEN];
-              strcpy(tbuf1, uncompress(mp->subject));
-              u_strcpy(tbuf2, get_compressed_message(mp));
+              mush_strncpy(tbuf1, uncompress(mp->subject), BUFFER_LEN);
+              u_strncpy(tbuf2, get_compressed_message(mp), BUFFER_LEN);
               send_mail(player, target, tbuf1, (char *) tbuf2, M_FORWARD, 1, 0);
               num_recpts++;
             }
@@ -824,7 +835,7 @@ do_mail_send(dbref player, char *tolist, char *message, mail_flag flags,
   const char *head;
   int num;
   dbref target;
-  int mail_flags;
+  mail_flag mail_flags;
   char sbuf[SUBJECT_LEN + 1], *sb, *mb;
   int i = 0, subject_given = 0;
   const char **start;
@@ -919,7 +930,8 @@ real_mail_fetch(dbref player, int num, int folder)
   for (mp = find_exact_starting_point(player); mp != NULL; mp = mp->next) {
     if (mp->to > player)
       break;
-    if ((mp->to == player) && ((folder < 0) || (Folder(mp) == folder)))
+    if ((mp->to == player) && ((folder < 0)
+                               || (Folder(mp) == (mail_flag) folder)))
       i++;
     if (i == num)
       return mp;
@@ -940,7 +952,8 @@ count_mail(dbref player, int folder, int *rcount, int *ucount, int *ccount)
   cc = rc = uc = 0;
   for (mp = find_exact_starting_point(player);
        mp && (mp->to == player); mp = mp->next) {
-    if ((mp->to == player) && ((folder == -1) || (Folder(mp) == folder))) {
+    if ((mp->to == player) && ((folder == -1) ||
+                               (Folder(mp) == (mail_flag) folder))) {
       if (Cleared(mp))
         cc++;
       else if (Read(mp))
@@ -1056,8 +1069,14 @@ real_send_mail(dbref player, dbref target, char *subject, char *message,
     return 0;
   }
 
+  /* Where do we insert it? After mp, wherever that is.
+   * This can return NULL if there are no messages or
+   * if we insert at the head of the list 
+   */
+  mp = find_insertion_point(target);
+
   /* initialize the appropriate fields */
-  newp = (MAIL *) mush_malloc(sizeof(struct mail), "mail");
+  newp = slab_malloc(mail_slab, mp);
   newp->to = target;
   newp->from = player;
   newp->from_ctime = CreTime(player);
@@ -1079,19 +1098,19 @@ real_send_mail(dbref player, dbref target, char *subject, char *message,
     newp->subject = compress(sbuf);
   if (flags & M_FORWARD) {
     /* Forwarding passes the message already compressed */
-    u_int_16 len = strlen(message) + 1;
+    size_t len = strlen(message) + 1;
     newp->msgid = chunk_create((unsigned char *) message, len, 1);
   } else {
-    u_int_16 len;
+    uint16_t len;
     unsigned char *text;
-    newmsg = (char *) mush_malloc(BUFFER_LEN, "string");
+    newmsg = mush_malloc(BUFFER_LEN, "string");
     if (!newmsg)
       mush_panic(T("Failed to allocate string in send_mail"));
     nm = newmsg;
     safe_str(message, newmsg, &nm);
     if (!nosig && ((a = atr_get_noparent(player, "MAILSIGNATURE")) != NULL)) {
       /* Append the MAILSIGNATURE to the mail - Cordin@Dune's idea */
-      buff = (char *) mush_malloc(BUFFER_LEN, "string");
+      buff = mush_malloc(BUFFER_LEN, "string");
       if (!buff)
         mush_panic(T("Failed to allocate string in send_mail"));
       ms = mailsig = safe_atr_value(a);
@@ -1101,25 +1120,19 @@ real_send_mail(dbref player, dbref target, char *subject, char *message,
       *bp = '\0';
       free(mailsig);
       safe_str(buff, newmsg, &nm);
-      mush_free((Malloc_t) buff, "string");
+      mush_free(buff, "string");
     }
     *nm = '\0';
     text = compress(newmsg);
     len = u_strlen(text) + 1;
     newp->msgid = chunk_create(text, len, 1);
     free(text);
-    mush_free((Malloc_t) newmsg, "string");
+    mush_free(newmsg, "string");
   }
 
   newp->time = mudtime;
   newp->read = flags & M_FMASK; /* Send to folder 0 */
 
-  /* Where do we insert it? After mp, wherever that is.
-   * This can return NULL if there are no messages or
-   * if we insert at the head of the list 
-   */
-  mp = find_insertion_point(target);
-
   if (mp) {
     newp->prev = mp;
     newp->next = mp->next;
@@ -1144,12 +1157,6 @@ real_send_mail(dbref player, dbref target, char *subject, char *message,
     }
   }
 
-  /* If the target's mailp isn't pointing to their list, we'd better
-   * set it
-   */
-  if (Connected(target))
-    desc_mail_set(target, find_exact_starting_point(target));
-
   mdb_top++;
 
   /* notify people */
@@ -1189,12 +1196,11 @@ do_mail_nuke(dbref player)
     if (mp->subject)
       free(mp->subject);
     chunk_delete(mp->msgid);
-    mush_free((Malloc_t) mp, "mail");
+    slab_free(mail_slab, mp);
   }
 
   HEAD = TAIL = NULL;
   mdb_top = 0;
-  desc_mail_clear();
 
   do_log(LT_ERR, 0, 0, T("** MAIL PURGE ** done by %s(#%d)."),
          Name(player), player);
@@ -1269,7 +1275,7 @@ do_mail_debug(dbref player, const char *action, const char *victim)
         if (mp->subject)
           free(mp->subject);
         chunk_delete(mp->msgid);
-        mush_free((Malloc_t) mp, "mail");
+        slab_free(mail_slab, mp);
       } else if (!GoodObject(mp->from)) {
         /* Oops, it's from a player whose dbref is out of range!
          * We'll make it appear to be from #0 instead because there's 
@@ -1898,7 +1904,7 @@ FUNCTION(fun_mailsubject)
  * \return number of mail messages saved.
  */
 int
-dump_mail(FILE * fp)
+dump_mail(PENNFILE *fp)
 {
   MAIL *mp;
   int count = 0;
@@ -1910,11 +1916,11 @@ dump_mail(FILE * fp)
   mail_flags += MDBF_SENDERCTIME;
 
   if (mail_flags)
-    fprintf(fp, "+%d\n", mail_flags);
+    penn_fprintf(fp, "+%d\n", mail_flags);
 
   save_malias(fp);
 
-  OUTPUT(fprintf(fp, "%d\n", mdb_top));
+  penn_fprintf(fp, "%d\n", mdb_top);
 
   for (mp = HEAD; mp != NULL; mp = mp->next) {
     putref(fp, mp->to);
@@ -1930,15 +1936,14 @@ dump_mail(FILE * fp)
     count++;
   }
 
-  OUTPUT(fputs(EOD, fp));
-  fflush(fp);
+  penn_fputs(EOD, fp);
 
   if (count != mdb_top) {
     do_log(LT_ERR, 0, 0, T("MAIL: Count of messages is %d, mdb_top is %d."),
            count, mdb_top);
     mdb_top = count;            /* Doesn't help if we forked, but oh well */
   }
-  return (count);
+  return count;
 }
 
 
@@ -1957,11 +1962,10 @@ find_exact_starting_point(dbref player)
 
   if (!HEAD)
     return NULL;
-  mp = desc_mail(player);
+
+  mp = get_objdata(player, "MAIL");
   if (!mp) {
-    /* Player is connected and has no mail, or nobody's connected who
-     * has mail - we have to scan the maildb.
-     */
+    /* Player hasn't already done something that looks up their mail. */
     if (HEAD->to > player)
       return NULL;              /* No mail chain */
     for (mp = HEAD; mp && (mp->to < player); mp = mp->next) ;
@@ -1973,9 +1977,11 @@ find_exact_starting_point(dbref player)
     while (mp && (mp->to < player))
       mp = mp->next;
   }
-  if (mp && (mp->to == player))
+  if (mp && (mp->to == player)) {
+    set_objdata(player, "MAIL", mp);
     return mp;
-  return NULL;
+  } else
+    return NULL;
 }
 
 
@@ -1991,11 +1997,8 @@ find_insertion_point(dbref player)
 
   if (!HEAD)
     return NULL;
-  mp = desc_mail(player);
+  mp = get_objdata(player, "MAIL");
   if (!mp) {
-    /* Player is connected and has no mail, or nobody's connected who
-     * has mail - we have to scan the maildb.
-     */
     if (HEAD->to > player)
       return NULL;              /* No mail chain */
     for (mp = TAIL; mp && (mp->to > player); mp = mp->prev) ;
@@ -2015,21 +2018,27 @@ find_insertion_point(dbref player)
 void
 mail_init(void)
 {
-  mdb_top = 0;
-  HEAD = NULL;
-  TAIL = NULL;
+  static bool init_called = 0;
+  if (!init_called) {
+    init_called = 1;
+    mdb_top = 0;
+    mail_slab = slab_create("mail messages", sizeof(struct mail));
+    slab_set_opt(mail_slab, SLAB_HINTLESS_THRESHOLD, 5);
+    HEAD = NULL;
+    TAIL = NULL;
+  }
 }
 
 /** Load mail from disk.
  * \param fp pointer to filehandle from which to load mail.
  */
 int
-load_mail(FILE * fp)
+load_mail(PENNFILE *fp)
 {
   char nbuf1[8];
   unsigned char *tbuf = NULL;
   unsigned char *text;
-  u_int_16 len;
+  size_t len;
   int mail_top = 0;
   int mail_flags = 0;
   int i = 0;
@@ -2039,7 +2048,7 @@ load_mail(FILE * fp)
   struct tm ttm;
 
   /* find out how many messages we should be loading */
-  fgets(nbuf1, sizeof(nbuf1), fp);
+  penn_fgets(nbuf1, sizeof(nbuf1), fp);
   /* If it starts with +, it's telling us the mail db flags */
   if (*nbuf1 == '+') {
     mail_flags = atoi(nbuf1 + 1);
@@ -2047,15 +2056,14 @@ load_mail(FILE * fp)
     if (mail_flags & MDBF_ALIASES) {
       load_malias(fp);
     }
-    fgets(nbuf1, sizeof(nbuf1), fp);
+    penn_fgets(nbuf1, sizeof(nbuf1), fp);
   }
   mail_top = atoi(nbuf1);
   if (!mail_top) {
     /* mail_top could be 0 from an error or actually be 0. */
     if (nbuf1[0] == '0' && nbuf1[1] == '\n') {
       char buff[20];
-      fgets(buff, sizeof buff, fp);
-      if (!buff)
+      if (!penn_fgets(buff, sizeof buff, fp))
         do_rawlog(LT_ERR,
                   T("MAIL: Missing end-of-dump marker in mail database."));
       else if (strcmp(buff, (mail_flags & MDBF_NEW_EOD)
@@ -2067,11 +2075,11 @@ load_mail(FILE * fp)
     return 0;
   }
   /* first one is a special case */
-  mp = (MAIL *) mush_malloc(sizeof(struct mail), "mail");
+  mp = slab_malloc(mail_slab, NULL);
   mp->to = getref(fp);
   mp->from = getref(fp);
   if (mail_flags & MDBF_SENDERCTIME)
-    mp->from_ctime = getref(fp);
+    mp->from_ctime = (time_t) getref(fp);
   else
     mp->from_ctime = 0;         /* No one will have this creation time */
 
@@ -2102,11 +2110,11 @@ load_mail(FILE * fp)
 
   /* now loop through */
   for (; i < mail_top; i++) {
-    mp = (MAIL *) mush_malloc(sizeof(struct mail), "mail");
+    mp = slab_malloc(mail_slab, NULL);
     mp->to = getref(fp);
     mp->from = getref(fp);
     if (mail_flags & MDBF_SENDERCTIME)
-      mp->from_ctime = getref(fp);
+      mp->from_ctime = (time_t) getref(fp);
     else
       mp->from_ctime = 0;       /* No one will have this creation time */
     if (do_convtime(getstring_noalloc(fp), &ttm))
@@ -2124,10 +2132,10 @@ load_mail(FILE * fp)
     if (tbuf)
       mp->subject = tbuf;
     else {
-      strcpy(sbuf, get_message(mp));
+      mush_strncpy(sbuf, get_message(mp), BUFFER_LEN);
       mp->subject = compress(chopstr(sbuf, SUBJECT_LEN));
     }
-    mp->read = getref(fp);
+    mp->read = (uint32_t) getref(fp);
 
     /* We now to a sorted insertion, sorted by recipient db# */
     if (mp->to >= TAIL->to) {
@@ -2170,8 +2178,7 @@ load_mail(FILE * fp)
   }
   {
     char buff[20];
-    fgets(buff, sizeof buff, fp);
-    if (!buff)
+    if (!penn_fgets(buff, sizeof buff, fp))
       do_rawlog(LT_ERR,
                 T("MAIL: Missing end-of-dump marker in mail database."));
     else if (strcmp(buff, (mail_flags & MDBF_NEW_EOD)
@@ -2181,7 +2188,8 @@ load_mail(FILE * fp)
   }
 
   do_mail_debug(GOD, "fix", "");
-  return (mdb_top);
+  slab_set_opt(mail_slab, SLAB_ALLOC_BEST_FIT, 1);
+  return mdb_top;
 }
 
 static int
@@ -2193,8 +2201,8 @@ get_folder_number(dbref player, char *name)
   a = (ATTR *) atr_get_noparent(player, "MAILFOLDERS");
   if (!a)
     return -1;
-  strcpy(str, atr_value(a));
-  sprintf(pat, ":%s:", strupper(name));
+  mush_strncpy(str, atr_value(a), BUFFER_LEN);
+  snprintf(pat, BUFFER_LEN, ":%s:", strupper(name));
   res = strstr(str, pat);
   if (!res)
     return -1;
@@ -2261,7 +2269,7 @@ add_folder_name(dbref player, int fld, const char *name)
     mush_panic(T("Failed to allocate strings in add_folder_name"));
 
   if (name && *name)
-    sprintf(new, "%d:%s:%d ", fld, strupper(name), fld);
+    snprintf(new, BUFFER_LEN, "%d:%s:%d ", fld, strupper(name), fld);
   else
     strcpy(new, " ");
   sprintf(pat, "%d:", fld);
@@ -2273,7 +2281,7 @@ add_folder_name(dbref player, int fld, const char *name)
     old = (char *) string_match(str, pat);
   }
   if (old && *old) {
-    strcpy(tbuf, str);
+    mush_strncpy(tbuf, str, BUFFER_LEN);
     r = old;
     while (!isspace((unsigned char) *r))
       r++;
@@ -2367,7 +2375,7 @@ mail_match(dbref player, MAIL *mp, struct mail_selector ms, int num)
   mpflag = Read(mp) ? mp->read : (mp->read | M_MSUNREAD);
   if (!All(ms) && !(ms.flags & mpflag))
     return 0;
-  if (AllInFolder(ms) && (Folder(mp) == player_folder(player)))
+  if (AllInFolder(ms) && (Folder(mp) == (mail_flag) player_folder(player)))
     return 1;
   if (ms.days != -1) {
     /* Get the time now, subtract mp->time, and compare the results with
@@ -2687,7 +2695,7 @@ parse_message_spec(dbref player, const char *s, int *msglow, int *msghigh,
         /* f: */
         *msglow = 0;
         if (msghigh)
-          *msghigh = HUGE_INT;
+          *msghigh = INT_MAX;
       } else {
         if (!is_integer(p))
           return 0;
