@@ -22,6 +22,9 @@
 
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -44,31 +47,33 @@
 #include "lock.h"
 #include "log.h"
 #include "dbio.h"
+#include "sort.h"
+#include "mymalloc.h"
 #include "oldflags.h"
 #include "confmagic.h"
 
 
 static int can_set_flag(dbref player, dbref thing, FLAG *flagp, int negate);
-static FLAG *letter_to_flagptr(FLAGSPACE * n, char c, int type);
-static void flag_add(FLAGSPACE * n, const char *name, FLAG *f);
-static int has_flag_ns(FLAGSPACE * n, dbref thing, FLAG *f);
+static FLAG *letter_to_flagptr(FLAGSPACE *n, char c, int type);
+static void flag_add(FLAGSPACE *n, const char *name, FLAG *f);
+static int has_flag_ns(FLAGSPACE *n, dbref thing, FLAG *f);
 
-static FLAG *flag_read(FILE * in);
-static FLAG *flag_read_oldstyle(FILE * in);
-static void flag_read_all_oldstyle(FILE * in, const char *ns);
-static void flag_write(FILE * out, FLAG *f, const char *name);
-static FLAG *flag_hash_lookup(FLAGSPACE * n, const char *name, int type);
+static FLAG *flag_read(PENNFILE *in);
+static FLAG *flag_read_oldstyle(PENNFILE *in);
+static void flag_read_all_oldstyle(PENNFILE *in, const char *ns);
+static void flag_write(PENNFILE *out, FLAG *f, const char *name);
+static FLAG *flag_hash_lookup(FLAGSPACE *n, const char *name, int type);
 static FLAG *clone_flag(FLAG *f);
 static FLAG *new_flag(void);
-static void flag_add_additional(FLAGSPACE * n);
-static char *list_aliases(FLAGSPACE * n, FLAG *given);
-static void realloc_object_flag_bitmasks(FLAGSPACE * n);
-static FLAG *match_flag_ns(FLAGSPACE * n, const char *name);
+static void flag_add_additional(FLAGSPACE *n);
+static char *list_aliases(FLAGSPACE *n, FLAG *given);
+static void realloc_object_flag_bitmasks(FLAGSPACE *n);
+static FLAG *match_flag_ns(FLAGSPACE *n, const char *name);
 
 PTAB ptab_flag;                 /**< Table of flags by name, inc. aliases */
 PTAB ptab_power;                /**< Table of powers by name, inc. aliases */
 HASHTAB htab_flagspaces;                /**< Hash of flagspaces */
-
+slab *flag_slab = NULL;
 extern PTAB ptab_command;       /* Uses flag bitmasks */
 
 /** Attempt to find a flagspace from its name */
@@ -303,7 +308,7 @@ match_power(const char *name)
  * \return poiner to flag structure, or NULL.
  */
 static FLAG *
-match_flag_ns(FLAGSPACE * n, const char *name)
+match_flag_ns(FLAGSPACE *n, const char *name)
 {
   return (FLAG *) ptab_find(n->tab, name);
 }
@@ -319,7 +324,7 @@ match_flag_ns(FLAGSPACE * n, const char *name)
  * \return pointer to flag structure, or NULL.
  */
 static FLAG *
-flag_hash_lookup(FLAGSPACE * n, const char *name, int type)
+flag_hash_lookup(FLAGSPACE *n, const char *name, int type)
 {
   FLAG *f;
 
@@ -352,7 +357,11 @@ flag_hash_lookup(FLAGSPACE * n, const char *name, int type)
 static FLAG *
 new_flag(void)
 {
-  FLAG *f = (FLAG *) mush_malloc(sizeof(FLAG), "flag");
+  FLAG *f;
+
+  if (flag_slab == NULL)
+    flag_slab = slab_create("flags", sizeof(FLAG));
+  f = slab_malloc(flag_slab, NULL);
   if (!f)
     mush_panic("Unable to allocate memory for a new flag!\n");
   return f;
@@ -360,7 +369,7 @@ new_flag(void)
 
 /* Deallocate all flag-related memory */
 static void
-clear_all_flags(FLAGSPACE * n)
+clear_all_flags(FLAGSPACE *n)
 {
   FLAG *f;
 
@@ -368,23 +377,25 @@ clear_all_flags(FLAGSPACE * n)
     f->perms = DECR_FLAG_REF(f->perms);
     if (FLAG_REF(f->perms) == 0) {
       mush_free((void *) f->name, "flag.name");
-      mush_free(f, "flag");
+      slab_free(flag_slab, f);
     }
   }
 
   ptab_free(n->tab);
+
   /* Finally, the flags array */
   if (n->flags)
     mush_free(n->flags, "flagspace.flags");
   n->flags = NULL;
   n->flagbits = 0;
+
 }
 
 static FLAG *
 clone_flag(FLAG *f)
 {
   FLAG *clone = new_flag();
-  clone->name = mush_strdup(f->name, "flag name");
+  clone->name = mush_strdup(f->name, "flag.name");
   clone->letter = f->letter;
   clone->type = f->type;
   clone->bitpos = f->bitpos;
@@ -400,7 +411,7 @@ clone_flag(FLAG *f)
  * f->bitpos is used.
  */
 static void
-flag_add(FLAGSPACE * n, const char *name, FLAG *f)
+flag_add(FLAGSPACE *n, const char *name, FLAG *f)
 {
   /* If this flag has no bitpos assigned, assign it the next one.
    * We could improve this algorithm to use the next available
@@ -417,9 +428,7 @@ flag_add(FLAGSPACE * n, const char *name, FLAG *f)
   f->perms = INCR_FLAG_REF(f->perms);
 
   /* Insert the flag in the ptab by the given name (maybe an alias) */
-  ptab_start_inserts(n->tab);
-  ptab_insert(n->tab, name, f);
-  ptab_end_inserts(n->tab);
+  ptab_insert_one(n->tab, name, f);
 
   /* Is this a canonical flag (as opposed to an alias?)
    * If it's an alias, we're done.
@@ -433,13 +442,12 @@ flag_add(FLAGSPACE * n, const char *name, FLAG *f)
     int i;
     if (f->bitpos >= n->flagbits) {
       /* Oops, we need a bigger array */
-      if (n->flagbits == 0)
-        n->flags = mush_malloc(sizeof(FLAG *), "flagspace.flags");
-      else {
-        n->flags = realloc(n->flags, (f->bitpos + 1) * sizeof(FLAG *));
-        if (!n->flags)
-          mush_panic("Unable to reallocate flags array!\n");
-      }
+      n->flags =
+        mush_realloc(n->flags, (f->bitpos + 1) * sizeof(FLAG *),
+                     "flagspace.flags");
+      if (!n->flags)
+        mush_panic("Unable to reallocate flags array!\n");
+
       /* Make sure the new space is full of NULLs */
       for (i = n->flagbits; i <= f->bitpos; i++)
         n->flags[i] = NULL;
@@ -458,7 +466,7 @@ flag_add(FLAGSPACE * n, const char *name, FLAG *f)
 }
 
 static void
-realloc_object_flag_bitmasks(FLAGSPACE * n)
+realloc_object_flag_bitmasks(FLAGSPACE *n)
 {
   dbref it;
   object_flag_type p;
@@ -501,13 +509,13 @@ realloc_object_flag_bitmasks(FLAGSPACE * n)
 
 /* Read in a flag from a file and return it */
 static FLAG *
-flag_read_oldstyle(FILE * in)
+flag_read_oldstyle(PENNFILE *in)
 {
   FLAG *f;
   char *c;
-  c = mush_strdup(getstring_noalloc(in), "flag name");
+  c = mush_strdup(getstring_noalloc(in), "flag.name");
   if (!strcmp(c, "FLAG ALIASES")) {
-    mush_free(c, "flag name");
+    mush_free(c, "flag.name");
     return NULL;                /* We're done */
   }
   f = new_flag();
@@ -522,7 +530,7 @@ flag_read_oldstyle(FILE * in)
 }
 
 static FLAG *
-flag_alias_read_oldstyle(FILE * in, char *alias, FLAGSPACE * n)
+flag_alias_read_oldstyle(PENNFILE *in, char *alias, FLAGSPACE *n)
 {
   FLAG *f;
   char *c;
@@ -561,7 +569,7 @@ flag_alias_read_oldstyle(FILE * in, char *alias, FLAGSPACE * n)
  * \param ns name of namespace to search.
  */
 static void
-flag_read_all_oldstyle(FILE * in, const char *ns)
+flag_read_all_oldstyle(PENNFILE *in, const char *ns)
 {
   FLAG *f;
   FLAGSPACE *n;
@@ -585,14 +593,14 @@ flag_read_all_oldstyle(FILE * in, const char *ns)
 
 /* Read in a flag from a file and return it */
 static FLAG *
-flag_read(FILE * in)
+flag_read(PENNFILE *in)
 {
   FLAG *f;
   char *c;
   char *tmp;
 
   db_read_this_labeled_string(in, "name", &tmp);
-  c = mush_strdup(tmp, "flag name");
+  c = mush_strdup(tmp, "flag.name");
   f = new_flag();
   f->name = c;
   db_read_this_labeled_string(in, "letter", &tmp);
@@ -608,7 +616,7 @@ flag_read(FILE * in)
 }
 
 static FLAG *
-flag_alias_read(FILE * in, char *alias, FLAGSPACE * n)
+flag_alias_read(PENNFILE *in, char *alias, FLAGSPACE *n)
 {
   FLAG *f;
   char *c;
@@ -644,7 +652,7 @@ flag_alias_read(FILE * in, char *alias, FLAGSPACE * n)
  * \param ns name of namespace to search.
  */
 void
-flag_read_all(FILE * in, const char *ns)
+flag_read_all(PENNFILE *in, const char *ns)
 {
   FLAG *f;
   FLAGSPACE *n;
@@ -662,12 +670,12 @@ flag_read_all(FILE * in, const char *ns)
   }
   /* If we are reading flags from the db, they are definitive. */
   clear_all_flags(n);
-  db_read_this_labeled_number(in, "flagcount", &count);
+  db_read_this_labeled_int(in, "flagcount", &count);
   for (;;) {
     int c;
 
-    c = fgetc(in);
-    ungetc(c, in);
+    c = penn_fgetc(in);
+    penn_ungetc(c, in);
 
     if (c != ' ')
       break;
@@ -685,12 +693,12 @@ flag_read_all(FILE * in, const char *ns)
               found, count);
 
   /* Assumes we'll always have at least one alias */
-  db_read_this_labeled_number(in, "flagaliascount", &count);
+  db_read_this_labeled_int(in, "flagaliascount", &count);
   for (found = 0;;) {
     int c;
 
-    c = fgetc(in);
-    ungetc(c, in);
+    c = penn_fgetc(in);
+    penn_ungetc(c, in);
 
     if (c != ' ')
       break;
@@ -712,7 +720,7 @@ flag_read_all(FILE * in, const char *ns)
 
 /* Write a flag out to a file */
 static void
-flag_write(FILE * out, FLAG *f, const char *name)
+flag_write(PENNFILE *out, FLAG *f, const char *name)
 {
   db_write_labeled_string(out, " name", name);
   db_write_labeled_string(out, "  letter", tprintf("%c", f->letter));
@@ -726,7 +734,7 @@ flag_write(FILE * out, FLAG *f, const char *name)
 
 /* Write a flag alias out to a file */
 static void
-flag_alias_write(FILE * out, FLAG *f, const char *name)
+flag_alias_write(PENNFILE *out, FLAG *f, const char *name)
 {
   db_write_labeled_string(out, " name", f->name);
   db_write_labeled_string(out, "  alias", name);
@@ -738,7 +746,7 @@ flag_alias_write(FILE * out, FLAG *f, const char *name)
  * \param out file pointer to write to.
  */
 void
-flag_write_all(FILE * out, const char *ns)
+flag_write_all(PENNFILE *out, const char *ns)
 {
   int i, count;
   FLAG *f;
@@ -755,7 +763,7 @@ flag_write_all(FILE * out, const char *ns)
     if (n->flags[i])
       count++;
   }
-  db_write_labeled_number(out, "flagcount", count);
+  db_write_labeled_int(out, "flagcount", count);
   for (i = 0; i < n->flagbits; i++) {
     if (n->flags[i])
       flag_write(out, n->flags[i], n->flags[i]->name);
@@ -771,7 +779,7 @@ flag_write_all(FILE * out, const char *ns)
       count++;
     f = ptab_nextentry_new(n->tab, flagname);
   }
-  db_write_labeled_number(out, "flagaliascount", count);
+  db_write_labeled_int(out, "flagaliascount", count);
   f = ptab_firstentry_new(n->tab, flagname);
   while (f) {
     if (strcmp(n->flags[f->bitpos]->name, flagname)) {
@@ -789,10 +797,11 @@ init_flagspaces(void)
 {
   FLAGSPACE *flags;
 
-  hashinit(&htab_flagspaces, 4, sizeof(FLAGSPACE));
-  flags = (FLAGSPACE *) mush_malloc(sizeof(FLAGSPACE), "flagspace");
+  hashinit(&htab_flagspaces, 4);
+  flags = mush_malloc(sizeof(FLAGSPACE), "flagspace");
   flags->name = strdup("FLAG");
   flags->tab = &ptab_flag;
+  ptab_init(&ptab_flag);
   flags->flagbits = 0;
   flags->flags = NULL;
   flags->flag_table = flag_table;
@@ -801,6 +810,7 @@ init_flagspaces(void)
   flags = (FLAGSPACE *) mush_malloc(sizeof(FLAGSPACE), "flagspace");
   flags->name = strdup("POWER");
   flags->tab = &ptab_power;
+  ptab_init(&ptab_power);
   flags->flagbits = 0;
   flags->flags = NULL;
   flags->flag_table = power_table;
@@ -826,13 +836,14 @@ init_flag_table(const char *ns)
     return;
   }
 
-  ptab_init(n->tab);
+  ptab_start_inserts(n->tab);
   /* do regular flags first */
   for (f = n->flag_table; f->name; f++) {
     cf = clone_flag(f);
     cf->bitpos = -1;
     flag_add(n, cf->name, cf);
   }
+  ptab_end_inserts(n->tab);
   /* now add in the aliases */
   for (a = n->flag_alias_table; a->alias; a++) {
     if ((f = match_flag_ns(n, a->realname)))
@@ -849,7 +860,7 @@ init_flag_table(const char *ns)
  * and use local_flags() in flaglocal.c instead.
  */
 static void
-flag_add_additional(FLAGSPACE * n)
+flag_add_additional(FLAGSPACE *n)
 {
   FLAG *f;
   FLAGSPACE *flags;
@@ -904,6 +915,7 @@ flag_add_additional(FLAGSPACE * n)
     add_power("Debit", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
     add_power("Pueblo_Send", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
     add_power("Many_Attribs", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
+    add_power("hook", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
     /* Aliases for other servers */
     if ((f = match_power("tport_anything")) && !match_power("tel_anything"))
       flag_add(flags, "tel_anything", f);
@@ -993,7 +1005,7 @@ flags_from_old_flags(const char *ns, long old_flags, long old_toggles, int type)
 
 /* Given a single character, return the matching flag definition */
 static FLAG *
-letter_to_flagptr(FLAGSPACE * n, char c, int type)
+letter_to_flagptr(FLAGSPACE *n, char c, int type)
 {
   FLAG *f;
   int i;
@@ -1031,7 +1043,7 @@ new_flag_bitmask(const char *ns)
   object_flag_type bitmask;
   FLAGSPACE *n;
   Flagspace_Lookup(n, ns);
-  bitmask = (object_flag_type) mush_malloc(FlagBytes(n), "flag_bitmask");
+  bitmask = mush_malloc(FlagBytes(n), "flag_bitmask");
   if (!bitmask)
     mush_panic("Unable to allocate memory for flag bitmask");
   memset(bitmask, 0, FlagBytes(n));
@@ -1282,7 +1294,7 @@ string_to_bits(const char *ns, const char *str)
  * \param thing object to check.
  * \param flag name of flag to check for (a string).
  * \param type allowed types of flags to check for.
- * \retval 1 object has the flag.
+ * \retval >0 object has the flag.
  * \retval 0 object does not have the flag.
  */
 int
@@ -1299,7 +1311,7 @@ has_flag_in_space_by_name(const char *ns, dbref thing, const char *flag,
 }
 
 static int
-has_flag_ns(FLAGSPACE * n, dbref thing, FLAG *f)
+has_flag_ns(FLAGSPACE *n, dbref thing, FLAG *f)
 {
   if (!GoodObject(thing) || IsGarbage(thing))
     return 0;
@@ -1532,6 +1544,7 @@ set_flag(dbref player, dbref thing, const char *flag, int negate,
   char tbuf1[BUFFER_LEN];
   char *tp;
   FLAGSPACE *n;
+  int current;
 
   n = (FLAGSPACE *) hashfind("FLAG", &htab_flagspaces);
   if ((f = flag_hash_lookup(n, flag, Typeof(thing))) == NULL) {
@@ -1549,6 +1562,8 @@ set_flag(dbref player, dbref thing, const char *flag, int negate,
     return;
   }
 
+  current = sees_flag("FLAG", player, thing, f->name);
+
   twiddle_flag(n, thing, f, negate);
   if (negate) {
     /* log if necessary */
@@ -1604,6 +1619,8 @@ set_flag(dbref player, dbref thing, const char *flag, int negate,
       safe_str(Name(thing), tbuf1, &tp);
       safe_str(" - ", tbuf1, &tp);
       safe_str(f->name, tbuf1, &tp);
+      if (!current)
+        safe_str(" (already)", tbuf1, &tp);
       safe_str(T(" reset."), tbuf1, &tp);
       *tp = '\0';
       notify(player, tbuf1);
@@ -1666,6 +1683,8 @@ set_flag(dbref player, dbref thing, const char *flag, int negate,
       safe_str(Name(thing), tbuf1, &tp);
       safe_str(" - ", tbuf1, &tp);
       safe_str(f->name, tbuf1, &tp);
+      if (current)
+        safe_str(" (already)", tbuf1, &tp);
       safe_str(T(" set."), tbuf1, &tp);
       *tp = '\0';
       notify(player, tbuf1);
@@ -1962,7 +1981,7 @@ add_flag_generic(const char *ns, const char *name, const char letter, int type,
       return f;
   }
   f = new_flag();
-  f->name = mush_strdup(strupper(name), "flag name");
+  f->name = mush_strdup(strupper(name), "flag.name");
   f->letter = letter;
   f->type = type;
   f->perms = perms;
@@ -2372,9 +2391,8 @@ alias_flag_generic(const char *ns, const char *name, const char *alias)
 
   f->perms = INCR_FLAG_REF(f->perms);
 
-  ptab_start_inserts(n->tab);
-  ptab_insert(n->tab, strupper(alias), f);
-  ptab_end_inserts(n->tab);
+  ptab_insert_one(n->tab, strupper(alias), f);
+
   return ((f = match_flag_ns(n, alias)) ? 1 : 0);
 }
 
@@ -2522,8 +2540,8 @@ do_flag_delete(const char *ns, dbref player, const char *name)
   ptab_delete(n->tab, f->name);
   notify_format(player, T("%s %s deleted."), strinitial(ns), f->name);
   /* Free the flag. */
-  mush_free((char *) f->name, "flag name");
-  mush_free(f, "flag");
+  mush_free((void *) f->name, "flag.name");
+  slab_free(flag_slab, f);
 }
 
 /** Enable a disabled flag.
@@ -2561,7 +2579,7 @@ do_flag_enable(const char *ns, dbref player, const char *name)
 
 
 static char *
-list_aliases(FLAGSPACE * n, FLAG *given)
+list_aliases(FLAGSPACE *n, FLAG *given)
 {
   FLAG *f;
   static char buf[BUFFER_LEN];
