@@ -25,21 +25,65 @@
 #include "flags.h"
 #include "game.h"
 #include "mymalloc.h"
+#include "sort.h"
 #include "funs.h"
 #include "confmagic.h"
+#include "ansi.h"
 
 static void func_hash_insert(const char *name, FUN *func);
 extern void local_functions(void);
 static int apply_restrictions(unsigned int result, const char *restriction);
+static char *build_function_report(dbref player, FUN *fp);
 
-USERFN_ENTRY *userfn_tab;   /**< Table of user-defined functions */
 HASHTAB htab_function;      /**< Function hash table */
 HASHTAB htab_user_function; /**< User-defined function hash table */
+slab *function_slab; /**< slab for 'struct fun' allocations */
 
 /* -------------------------------------------------------------------------*
  * Utilities.
  */
 
+/* Save and restore regexp data */
+void
+save_regexp_context(struct re_save *save)
+{
+  save->re_code = global_eval_context.re_code;
+  save->re_from = global_eval_context.re_from;
+  save->re_subpatterns = global_eval_context.re_subpatterns;
+  save->re_offsets = global_eval_context.re_offsets;
+}
+
+void
+restore_regexp_context(struct re_save *save)
+{
+  global_eval_context.re_code = save->re_code;
+  global_eval_context.re_from = save->re_from;
+  global_eval_context.re_subpatterns = save->re_subpatterns;
+  global_eval_context.re_offsets = save->re_offsets;
+}
+
+/** Save a single q-register
+ */
+void
+save_partial_global_reg(const char *funcname, char *preserve[], int i)
+{
+  preserve[i] = mush_strdup(global_eval_context.renv[i], funcname);
+}
+
+/** Restore q-registers saved with save_partial_global_reg()
+ */
+void
+restore_partial_global_regs(const char *funcname, char *preserve[])
+{
+  int i;
+  for (i = 0; i < NUMQ; i++) {
+    if (preserve[i]) {
+      mush_strncpy(global_eval_context.renv[i], preserve[i], BUFFER_LEN);
+      mush_free(preserve[i], funcname);
+    }
+  }
+}
+
 /** Save a copy of the q-registers.
  * \param funcname name of function calling (for memory leak testing)
  * \param preserve pointer to array to store the q-registers in.
@@ -52,8 +96,7 @@ save_global_regs(const char *funcname, char *preserve[])
     if (!global_eval_context.renv[i][0])
       preserve[i] = NULL;
     else {
-      preserve[i] = (char *) mush_malloc(BUFFER_LEN, funcname);
-      strcpy(preserve[i], global_eval_context.renv[i]);
+      preserve[i] = mush_strdup(global_eval_context.renv[i], funcname);
     }
   }
 }
@@ -68,7 +111,7 @@ restore_global_regs(const char *funcname, char *preserve[])
   int i;
   for (i = 0; i < NUMQ; i++) {
     if (preserve[i]) {
-      strcpy(global_eval_context.renv[i], preserve[i]);
+      mush_strncpy(global_eval_context.renv[i], preserve[i], BUFFER_LEN);
       mush_free(preserve[i], funcname);
       preserve[i] = NULL;
     } else {
@@ -165,8 +208,7 @@ save_global_nxt(const char *funcname, char *preservew[], char *preserver[],
     if (!global_eval_context.rnxt[i])
       valr[i] = NULL;
     else {
-      valr[i] = (char *) mush_malloc(BUFFER_LEN, funcname);
-      strcpy(valr[i], global_eval_context.rnxt[i]);
+      valr[i] = mush_strdup(global_eval_context.rnxt[i], funcname);
     }
   }
   for (i = 0; i < 10; i++) {
@@ -174,8 +216,7 @@ save_global_nxt(const char *funcname, char *preservew[], char *preserver[],
     if (!global_eval_context.wnxt[i])
       valw[i] = NULL;
     else {
-      valw[i] = (char *) mush_malloc(BUFFER_LEN, funcname);
-      strcpy(valw[i], global_eval_context.wnxt[i]);
+      valw[i] = mush_strdup(global_eval_context.wnxt[i], funcname);
     }
   }
 }
@@ -196,7 +237,7 @@ restore_global_nxt(const char *funcname, char *preservew[], char *preserver[],
     global_eval_context.rnxt[i] = preserver[i];
     if (preserver[i]) {
       /* There was a former address, so we can restore to it */
-      strcpy(global_eval_context.rnxt[i], valr[i]);
+      mush_strncpy(global_eval_context.rnxt[i], valr[i], BUFFER_LEN);
       mush_free(valr[i], funcname);
       valr[i] = NULL;
     }
@@ -205,13 +246,14 @@ restore_global_nxt(const char *funcname, char *preservew[], char *preserver[],
     global_eval_context.wnxt[i] = preservew[i];
     if (preservew[i]) {
       /* There was a former address, so we can restore to it */
-      strcpy(global_eval_context.wnxt[i], valw[i]);
+      mush_strncpy(global_eval_context.wnxt[i], valw[i], BUFFER_LEN);
       mush_free(valw[i], funcname);
       valw[i] = NULL;
     }
   }
 }
 
+
 /** Check for a delimiter in an argument of a function call.
  * This function checks a given argument of a function call and sees
  * if it could be used as a delimiter. A delimiter must be a single
@@ -281,8 +323,10 @@ FUNTAB flist[] = {
   {"ANDFLAGS", fun_andflags, 2, 2, FN_REG},
   {"ANDLFLAGS", fun_andlflags, 2, 2, FN_REG},
   {"ANDLPOWERS", fun_andlflags, 2, 2, FN_REG},
-  {"ANDPOWERS", fun_andflags, 2, 2, FN_REG},
-  {"ANSI", fun_ansi, 2, -2, FN_NOPARSE},
+  {"ANSI", fun_ansi, 2, -2, FN_REG},
+#ifdef ANSI_DEBUG
+  {"ANSIINSPECT", fun_ansiinspect, 1, 2, FN_REG},
+#endif
   {"APOSS", fun_aposs, 1, 1, FN_REG},
   {"ART", fun_art, 1, 1, FN_REG},
   {"ATRLOCK", fun_atrlock, 1, 2, FN_REG},
@@ -303,6 +347,7 @@ FUNTAB flist[] = {
   {"CASEALL", fun_switch, 3, INT_MAX, FN_NOPARSE},
   {"CAT", fun_cat, 1, INT_MAX, FN_REG},
   {"CBUFFER", fun_cinfo, 1, 1, FN_REG},
+  {"CBUFFERADD", fun_cbufferadd, 2, 3, FN_REG},
   {"CDESC", fun_cinfo, 1, 1, FN_REG},
   {"CEMIT", fun_cemit, 2, 3, FN_REG},
   {"CFLAGS", fun_cflags, 1, 2, FN_REG},
@@ -324,6 +369,8 @@ FUNTAB flist[] = {
   {"CMDS", fun_cmds, 1, 1, FN_REG},
   {"COMP", fun_comp, 2, 3, FN_REG},
   {"CON", fun_con, 1, 1, FN_REG},
+  {"COND", fun_if, 2, INT_MAX, FN_NOPARSE},
+  {"CONDALL", fun_if, 2, INT_MAX, FN_NOPARSE},
   {"CONFIG", fun_config, 1, 1, FN_REG},
   {"CONN", fun_conn, 1, 1, FN_REG},
   {"CONTROLS", fun_controls, 2, 2, FN_REG},
@@ -331,11 +378,13 @@ FUNTAB flist[] = {
   {"CONVUTCSECS", fun_convsecs, 1, 1, FN_REG},
   {"CONVTIME", fun_convtime, 1, 1, FN_REG},
   {"COR", fun_cor, 2, INT_MAX, FN_NOPARSE},
-  {"CREATE", fun_create, 1, 2, FN_REG},
-  {"CTIME", fun_ctime, 1, 1, FN_REG},
+  {"CREATE", fun_create, 1, 3, FN_REG},
+  {"CSECS", fun_csecs, 1, 1, FN_REG},
+  {"CTIME", fun_ctime, 1, 2, FN_REG},
   {"DEC", fun_dec, 1, 1, FN_REG},
+  {"DECODE64", fun_decode64, 1, -1, FN_REG},
   {"DECOMPOSE", fun_decompose, 1, -1, FN_REG},
-  {"DECRYPT", fun_decrypt, 2, 2, FN_REG},
+  {"DECRYPT", fun_decrypt, 2, 3, FN_REG},
   {"DEFAULT", fun_default, 2, INT_MAX, FN_NOPARSE},
   {"DELETE", fun_delete, 3, 3, FN_REG},
   {"DIE", fun_die, 2, 3, FN_REG},
@@ -352,7 +401,8 @@ FUNTAB flist[] = {
   {"ELIST", fun_itemize, 1, 5, FN_REG},
   {"ELOCK", fun_elock, 2, 2, FN_REG},
   {"EMIT", fun_emit, 1, -1, FN_REG},
-  {"ENCRYPT", fun_encrypt, 2, 2, FN_REG},
+  {"ENCODE64", fun_encode64, 1, -1, FN_REG},
+  {"ENCRYPT", fun_encrypt, 2, 3, FN_REG},
   {"ENTRANCES", fun_entrances, 0, 4, FN_REG},
   {"ETIMEFMT", fun_etimefmt, 2, 2, FN_REG},
   {"EQ", fun_eq, 2, 2, FN_REG},
@@ -368,13 +418,14 @@ FUNTAB flist[] = {
   {"FLAGS", fun_flags, 0, 1, FN_REG},
   {"FLIP", fun_flip, 1, -1, FN_REG},
   {"FLOORDIV", fun_floordiv, 2, 2, FN_REG},
+  {"FN", fun_fn, 1, INT_MAX, FN_NOPARSE},
   {"FOLD", fun_fold, 2, 4, FN_REG},
   {"FOLDERSTATS", fun_folderstats, 0, 2, FN_REG},
   {"FOLLOWERS", fun_followers, 1, 1, FN_REG},
   {"FOLLOWING", fun_following, 1, 1, FN_REG},
   {"FOREACH", fun_foreach, 2, 4, FN_REG},
   {"FRACTION", fun_fraction, 1, 1, FN_REG},
-  {"FUNCTIONS", fun_functions, 0, 0, FN_REG},
+  {"FUNCTIONS", fun_functions, 0, 1, FN_REG},
   {"FULLALIAS", fun_fullalias, 1, 1, FN_REG},
   {"FULLNAME", fun_fullname, 1, 1, FN_REG},
   {"GET", fun_get, 1, 1, FN_REG},
@@ -385,10 +436,10 @@ FUNTAB flist[] = {
   {"GREPI", fun_grep, 3, 3, FN_REG},
   {"GT", fun_gt, 2, 2, FN_REG},
   {"GTE", fun_gte, 2, 2, FN_REG},
-  {"HASATTR", fun_hasattr, 2, 2, FN_REG},
-  {"HASATTRP", fun_hasattr, 2, 2, FN_REG},
-  {"HASATTRPVAL", fun_hasattr, 2, 2, FN_REG},
-  {"HASATTRVAL", fun_hasattr, 2, 2, FN_REG},
+  {"HASATTR", fun_hasattr, 1, 2, FN_REG},
+  {"HASATTRP", fun_hasattr, 1, 2, FN_REG},
+  {"HASATTRPVAL", fun_hasattr, 1, 2, FN_REG},
+  {"HASATTRVAL", fun_hasattr, 1, 2, FN_REG},
   {"HASFLAG", fun_hasflag, 2, 2, FN_REG},
   {"HASPOWER", fun_haspower, 2, 2, FN_REG},
   {"HASTYPE", fun_hastype, 2, 2, FN_REG},
@@ -412,19 +463,21 @@ FUNTAB flist[] = {
   {"ISDBREF", fun_isdbref, 1, 1, FN_REG},
   {"ISINT", fun_isint, 1, 1, FN_REG},
   {"ISNUM", fun_isnum, 1, 1, FN_REG},
+  {"ISOBJID", fun_isobjid, 1, 1, FN_REG},
   {"ISWORD", fun_isword, 1, 1, FN_REG},
   {"ITER", fun_iter, 2, 4, FN_NOPARSE},
   {"ITEMS", fun_items, 2, 2, FN_REG},
   {"ITEMIZE", fun_itemize, 1, 4, FN_REG},
   {"ITEXT", fun_itext, 1, 1, FN_REG},
   {"LAST", fun_last, 1, 2, FN_REG},
-  {"LATTR", fun_lattr, 1, 1, FN_REG},
-  {"LATTRP", fun_lattr, 1, 1, FN_REG},
+  {"LATTR", fun_lattr, 1, 2, FN_REG},
+  {"LATTRP", fun_lattr, 1, 2, FN_REG},
   {"LCON", fun_dbwalker, 1, 1, FN_REG},
   {"LCSTR", fun_lcstr, 1, -1, FN_REG},
   {"LDELETE", fun_ldelete, 2, 3, FN_REG},
   {"LEFT", fun_left, 2, 2, FN_REG},
   {"LEMIT", fun_lemit, 1, -1, FN_REG},
+  {"LETQ", fun_letq, 1, INT_MAX, FN_NOPARSE},
   {"LEXITS", fun_dbwalker, 1, 1, FN_REG},
   {"LFLAGS", fun_lflags, 0, 1, FN_REG},
   {"LINK", fun_link, 2, 3, FN_REG},
@@ -443,7 +496,7 @@ FUNTAB flist[] = {
   {"LOCKS", fun_locks, 1, 1, FN_REG},
   {"LPARENT", fun_lparent, 1, 1, FN_REG},
   {"LPLAYERS", fun_dbwalker, 1, 1, FN_REG},
-  {"LPORTS", fun_lports, 0, 0, FN_REG},
+  {"LPORTS", fun_lports, 0, 1, FN_REG},
   {"LPOS", fun_lpos, 2, 2, FN_REG},
   {"LSEARCH", fun_lsearch, 1, INT_MAX, FN_REG},
   {"LSEARCHR", fun_lsearch, 1, INT_MAX, FN_REG},
@@ -477,12 +530,14 @@ FUNTAB flist[] = {
   {"MEDIAN", fun_median, 1, INT_MAX, FN_REG},
   {"MEMBER", fun_member, 2, 3, FN_REG},
   {"MERGE", fun_merge, 3, 3, FN_REG},
+  {"MESSAGE", fun_message, 3, 13, FN_REG},
   {"MID", fun_mid, 3, 3, FN_REG},
   {"MIN", fun_min, 1, INT_MAX, FN_REG},
   {"MIX", fun_mix, 3, 12, FN_REG},
   {"MODULO", fun_modulo, 2, 2, FN_REG},
   {"MONEY", fun_money, 1, 1, FN_REG},
-  {"MTIME", fun_mtime, 1, 1, FN_REG},
+  {"MSECS", fun_msecs, 1, 1, FN_REG},
+  {"MTIME", fun_mtime, 1, 2, FN_REG},
   {"MUDNAME", fun_mudname, 0, 0, FN_REG},
   {"MUL", fun_mul, 2, INT_MAX, FN_REG},
   {"MUNGE", fun_munge, 3, 5, FN_REG},
@@ -497,6 +552,8 @@ FUNTAB flist[] = {
   {"NATTRP", fun_nattr, 1, 1, FN_REG},
   {"NCHILDREN", fun_lsearch, 1, 1, FN_REG},
   {"NCON", fun_dbwalker, 1, 1, FN_REG},
+  {"NCOND", fun_if, 2, INT_MAX, FN_NOPARSE},
+  {"NCONDALL", fun_if, 2, INT_MAX, FN_NOPARSE},
   {"NEXITS", fun_dbwalker, 1, 1, FN_REG},
   {"NPLAYERS", fun_dbwalker, 1, 1, FN_REG},
   {"NEARBY", fun_nearby, 2, 2, FN_REG},
@@ -537,12 +594,12 @@ FUNTAB flist[] = {
   {"ORFLAGS", fun_orflags, 2, 2, FN_REG},
   {"ORLFLAGS", fun_orlflags, 2, 2, FN_REG},
   {"ORLPOWERS", fun_orlflags, 2, 2, FN_REG},
-  {"ORPOWERS", fun_orflags, 2, 2, FN_REG},
   {"OWNER", fun_owner, 1, 1, FN_REG},
   {"PARENT", fun_parent, 1, 2, FN_REG},
-  {"PCREATE", fun_pcreate, 2, 2, FN_REG},
+  {"PCREATE", fun_pcreate, 2, 3, FN_REG},
   {"PEMIT", fun_pemit, 2, -2, FN_REG},
   {"PLAYERMEM", fun_playermem, 1, 1, FN_REG},
+  {"PLAYER", fun_player, 1, 1, FN_REG},
   {"PMATCH", fun_pmatch, 1, 1, FN_REG},
   {"POLL", fun_poll, 0, 0, FN_REG},
   {"PORTS", fun_ports, 1, 1, FN_REG},
@@ -612,6 +669,7 @@ FUNTAB flist[] = {
   {"SOUNDSLIKE", fun_soundlike, 2, 2, FN_REG},
   {"SPACE", fun_space, 1, 1, FN_REG},
   {"SPEAK", fun_speak, 2, 7, FN_REG},
+  {"SPEAKPENN", fun_speak, 2, 7, FN_REG},
   {"SPELLNUM", fun_spellnum, 1, 1, FN_REG},
   {"SPLICE", fun_splice, 3, 4, FN_REG},
   {"SQL", fun_sql, 1, 3, FN_REG},
@@ -626,7 +684,7 @@ FUNTAB flist[] = {
   {"STRIPACCENTS", fun_stripaccents, 1, 1, FN_REG},
   {"STRIPANSI", fun_stripansi, 1, -1, FN_REG},
   {"STRLEN", fun_strlen, 1, -1, FN_REG},
-  {"STRMATCH", fun_strmatch, 2, 2, FN_REG},
+  {"STRMATCH", fun_strmatch, 2, 3, FN_REG},
   {"STRREPLACE", fun_strreplace, 4, 4, FN_REG},
   {"SUB", fun_sub, 2, 2, FN_REG},
   {"SUBJ", fun_subj, 1, 1, FN_REG},
@@ -668,8 +726,8 @@ FUNTAB flist[] = {
   {"WORDPOS", fun_wordpos, 2, 3, FN_REG},
   {"WORDS", fun_words, 1, 2, FN_REG},
   {"WRAP", fun_wrap, 2, 4, FN_REG},
-  {"XATTR", fun_lattr, 3, 3, FN_REG},
-  {"XATTRP", fun_lattr, 3, 3, FN_REG},
+  {"XATTR", fun_lattr, 3, 4, FN_REG},
+  {"XATTRP", fun_lattr, 3, 4, FN_REG},
   {"XCON", fun_dbwalker, 3, 3, FN_REG},
   {"XEXITS", fun_dbwalker, 3, 3, FN_REG},
   {"XMWHO", fun_xwho, 2, 2, FN_REG},
@@ -738,41 +796,65 @@ void
 do_list_functions(dbref player, int lc)
 {
   /* lists all built-in functions. */
-  char *b = list_functions();
-  notify_format(player, "Functions: %s", lc ? strlower(b) : b);
+  char *b = list_functions(NULL);
+  notify_format(player, T("Functions: %s"), lc ? strlower(b) : b);
 }
 
 /** Return a list of function names.
  * This function returns the list of function names as a string.
+ * \param type if "local", returns @functions only.  If "builtin",
+ *   hardcoded functions. If omitted, both.
  * \return list of function names as a static string.
  */
 char *
-list_functions(void)
+list_functions(const char *type)
 {
   FUN *fp;
   const char *ptrs[BUFFER_LEN / 2];
   static char buff[BUFFER_LEN];
   char *bp;
   int nptrs = 0, i;
-  for (fp = (FUN *) hash_firstentry(&htab_function);
-       fp; fp = (FUN *) hash_nextentry(&htab_function)) {
-    if (fp->flags & FN_OVERRIDE)
-      continue;
-    ptrs[nptrs++] = fp->name;
+  int which = 0;
+  /* 0x1 for builtin, 0x2 for @function */
+
+  if (!type)
+    which = 0x3;
+  else if (strcmp(type, "all") == 0)
+    which = 0x3;
+  else if (strcmp(type, "builtin") == 0)
+    which = 0x1;
+  else if (strcmp(type, "local") == 0)
+    which = 0x2;
+  else {
+    mush_strncpy(buff, T("#-1 INVALID ARGUMENT"), BUFFER_LEN);
+    return buff;
   }
-  fp = (FUN *) hash_firstentry(&htab_user_function);
-  while (fp) {
-    ptrs[nptrs++] = fp->name;
-    fp = (FUN *) hash_nextentry(&htab_user_function);
+
+  if (which & 0x1) {
+    for (fp = hash_firstentry(&htab_function);
+         fp; fp = hash_nextentry(&htab_function)) {
+      if (fp->flags & FN_OVERRIDE)
+        continue;
+      ptrs[nptrs++] = fp->name;
+    }
   }
+
+  if (which & 0x2) {
+    for (fp = hash_firstentry(&htab_user_function);
+         fp; fp = hash_nextentry(&htab_user_function))
+      ptrs[nptrs++] = fp->name;
+  }
+
   /* do_gensort needs a dbref now, but only for sort types that aren't
    * used here anyway */
-  do_gensort((dbref) 0, (char **) ptrs, NULL, nptrs, ALPHANUM_LIST);
+  do_gensort(0, (char **) ptrs, NULL, nptrs, ALPHANUM_LIST);
   bp = buff;
-  safe_str(ptrs[0], buff, &bp);
-  for (i = 1; i < nptrs; i++) {
-    safe_chr(' ', buff, &bp);
-    safe_str(ptrs[i], buff, &bp);
+  if (nptrs > 0) {
+    safe_str(ptrs[0], buff, &bp);
+    for (i = 1; i < nptrs; i++) {
+      safe_chr(' ', buff, &bp);
+      safe_str(ptrs[i], buff, &bp);
+    }
   }
   *bp = '\0';
   return buff;
@@ -797,6 +879,18 @@ func_hash_lookup(const char *name)
   return f;
 }
 
+/** Look up a function by name, builtins only.
+ * \param name name of function to look up.
+ * \return pointer to function data, or NULL.
+ */
+FUN *
+builtin_func_hash_lookup(const char *name)
+{
+  FUN *f;
+  f = (FUN *) hashfind(strupper(name), &htab_function);
+  return f;
+}
+
 static void
 func_hash_insert(const char *name, FUN *func)
 {
@@ -812,8 +906,9 @@ init_func_hashtab(void)
 {
   FUNTAB *ftp;
 
-  hashinit(&htab_function, 512, sizeof(FUN));
-  hash_init(&htab_user_function, 32, sizeof(FUN), delete_function);
+  hashinit(&htab_function, 512);
+  hash_init(&htab_user_function, 32, delete_function);
+  function_slab = slab_create("functions", sizeof(FUN));
   for (ftp = flist; ftp->name; ftp++) {
     function_add(ftp->name, ftp->fun, ftp->minargs, ftp->maxargs, ftp->flags);
   }
@@ -828,9 +923,6 @@ init_func_hashtab(void)
 void
 function_init_postconfig(void)
 {
-  userfn_tab =
-    (USERFN_ENTRY *) mush_malloc(MAX_GLOBAL_FNS * sizeof(USERFN_ENTRY),
-                                 "userfn_tab");
 }
 
 /** Check permissions to run a function.
@@ -905,7 +997,10 @@ function_add(const char *name, function_func fun, int minargs, int maxargs,
              int ftype)
 {
   FUN *fp;
-  fp = (FUN *) mush_malloc(sizeof(FUN), "function");
+
+  if (!name || name[0] == '\0')
+    return;
+  fp = slab_malloc(function_slab, NULL);
   memset(fp, 0, sizeof(FUN));
   fp->name = name;
   fp->where.fun = fun;
@@ -955,14 +1050,18 @@ strip_braces(const char *str)
  * User-defined global function handlers 
  */
 
-
-static Size_t userfn_count = 0;
-
 static int
-apply_restrictions(unsigned int result, const char *restriction)
+apply_restrictions(unsigned int result, const char *xres)
 {
   int flag, clear = 0;
   char *tp;
+  char *restriction, *rsave;
+
+  if (!xres || !*xres)
+    return 0;
+
+  rsave = restriction = mush_strdup(xres, "ar.string");
+
   while (restriction && *restriction) {
     if ((tp = strchr(restriction, ' ')))
       *tp++ = '\0';
@@ -995,6 +1094,8 @@ apply_restrictions(unsigned int result, const char *restriction)
       flag = FN_NOPARSE;
     } else if (!strcasecmp(restriction, "localize")) {
       flag = FN_LOCALIZE;
+    } else if (!strcasecmp(restriction, "userfn")) {
+      flag = FN_USERFN;
     }
     if (clear)
       result &= ~flag;
@@ -1002,6 +1103,7 @@ apply_restrictions(unsigned int result, const char *restriction)
       result |= flag;
     restriction = tp;
   }
+  mush_free(rsave, "ar.string");
   return result;
 }
 
@@ -1021,6 +1123,7 @@ apply_restrictions(unsigned int result, const char *restriction)
  *   noplayer   can't be used by players, just objects/rooms/exits
  *   nosidefx   can't be used to do side-effect thingies
  *   localize   localize q-registers
+ *   userfn     can only be used inside @functions.
  * \endverbatim
  * \param name name of function to restrict.
  * \param restriction name of restriction to apply to function.
@@ -1048,12 +1151,16 @@ restrict_function(const char *name, const char *restriction)
  * \param player the enactor.
  * \param name name of function to restrict.
  * \param restriction name of restriction to add.
+ * \param builtin operate on the builtin version, whether overridden or not
  */
 void
-do_function_restrict(dbref player, const char *name, const char *restriction)
+do_function_restrict(dbref player, const char *name, const char *restriction,
+                     int builtin)
 {
   FUN *fp;
   unsigned int flags;
+  char tbuf1[BUFFER_LEN];
+  char *bp = tbuf1;
 
   if (!Wizard(player)) {
     notify(player, T("Permission denied."));
@@ -1067,20 +1174,48 @@ do_function_restrict(dbref player, const char *name, const char *restriction)
     notify(player, T("Do what with the function?"));
     return;
   }
-  fp = func_hash_lookup(name);
+  fp = builtin ? builtin_func_hash_lookup(name) : func_hash_lookup(name);
   if (!fp) {
     notify(player, T("No such function."));
     return;
   }
   flags = fp->flags;
   fp->flags = apply_restrictions(flags, restriction);
-  if (fp->flags == flags)
-    notify(player, T("Restrictions unchanged."));
+  if (fp->flags & FN_BUILTIN)
+    safe_format(tbuf1, &bp, "%s %s - ", T("Builtin function"), fp->name);
   else
-    notify(player, T("Restrictions modified."));
+    safe_format(tbuf1, &bp, "%s #%d/%s - ", T("@function"),
+                fp->where.ufun->thing, fp->where.ufun->name);
+  if (fp->flags == flags)
+    safe_str(T("Restrictions unchanged."), tbuf1, &bp);
+  else
+    safe_str(T("Restrictions modified."), tbuf1, &bp);
+  *bp = '\0';
+  notify(player, tbuf1);
 }
 
 
+/* Sort FUN*s by dbref and then function name */
+static int
+func_comp(const void *s1, const void *s2)
+{
+  const FUN *a, *b;
+  dbref da, db;
+
+  a = *(const FUN **) s1;
+  b = *(const FUN **) s2;
+
+  da = a->where.ufun->thing;
+  db = b->where.ufun->thing;
+
+  if (da == db)
+    return strcmp(a->name, b->name);
+  else if (da < db)
+    return -1;
+  else
+    return 1;
+}
+
 /** Add a user-defined function.
  * \verbatim
  * This is the implementation of the @function command. If no arguments
@@ -1100,6 +1235,7 @@ do_function(dbref player, char *name, char *argv[], int preserve)
   char *bp = tbuf1;
   dbref thing;
   FUN *fp;
+  size_t userfn_count = htab_user_function.entries;
 
   /* if no arguments, just give the list of user functions, by walking
    * the function hash table, and looking up all functions marked
@@ -1115,22 +1251,41 @@ do_function(dbref player, char *name, char *argv[], int preserve)
       /* if the player is privileged, display user-def'ed functions
        * with corresponding dbref number of thing and attribute name.
        */
+      FUN **funclist;
+      int n = 0;
+
+      funclist = mush_calloc(userfn_count, sizeof(FUN *), "function.fp.list");
       notify(player, T("Function Name                   Dbref #    Attrib"));
       for (fp = (FUN *) hash_firstentry(&htab_user_function);
            fp; fp = (FUN *) hash_nextentry(&htab_user_function)) {
+        funclist[n] = fp;
+        n++;
+      }
+      qsort(funclist, userfn_count, sizeof(FUN *), func_comp);
+      for (n = 0; n < (int) userfn_count; n++) {
+        fp = funclist[n];
         notify_format(player,
                       "%-32s %6d    %s", fp->name,
-                      userfn_tab[fp->where.offset].thing,
-                      userfn_tab[fp->where.offset].name);
+                      fp->where.ufun->thing, fp->where.ufun->name);
       }
+      mush_free(funclist, "function.fp.list");
     } else {
+      const char **funcnames;
+      int n = 0;
       /* just print out the list of available functions */
       safe_str(T("User functions:"), tbuf1, &bp);
+      funcnames = mush_calloc(userfn_count, sizeof(char *), "function.list");
       for (fp = (FUN *) hash_firstentry(&htab_user_function);
            fp; fp = (FUN *) hash_nextentry(&htab_user_function)) {
-        safe_chr(' ', tbuf1, &bp);
-        safe_str(fp->name, tbuf1, &bp);
+        funcnames[n] = fp->name;
+        n++;
       }
+      qsort(funcnames, userfn_count, sizeof(char *), str_comp);
+      for (n = 0; n < (int) userfn_count; n++) {
+        safe_chr(' ', tbuf1, &bp);
+        safe_str(funcnames[n], tbuf1, &bp);
+      }
+      mush_free(funcnames, "function.list");
       *bp = '\0';
       notify(player, tbuf1);
     }
@@ -1180,18 +1335,13 @@ do_function(dbref player, char *name, char *argv[], int preserve)
 
   fp = func_hash_lookup(upcasestr(name));
   if (!fp) {
-    if (userfn_count >= (Size_t) MAX_GLOBAL_FNS) {
-      notify(player, T("Function table full."));
-      return;
-    }
     if (argv[6] && *argv[6]) {
       notify(player, T("Expected between 1 and 5 arguments."));
       return;
     }
     /* a completely new entry. First, insert it into general hash table */
-    fp = (FUN *) mush_malloc(sizeof(FUN), "func_hash.FUN");
+    fp = slab_malloc(function_slab, NULL);
     fp->name = mush_strdup(name, "func_hash.name");
-    fp->where.offset = userfn_count;
     if (argv[3] && *argv[3]) {
       fp->minargs = parse_integer(argv[3]);
       if (fp->minargs < 0)
@@ -1218,11 +1368,9 @@ do_function(dbref player, char *name, char *argv[], int preserve)
     hashadd(name, fp, &htab_user_function);
 
     /* now add it to the user function table */
-    userfn_tab[userfn_count].thing = thing;
-    userfn_tab[userfn_count].name =
-      mush_strdup(upcasestr(argv[2]), "userfn_tab.name");
-    userfn_tab[userfn_count].fn = mush_strdup(name, "usrfn_tab.fn");
-    userfn_count++;
+    fp->where.ufun = mush_malloc(sizeof(USERFN_ENTRY), "userfn");
+    fp->where.ufun->thing = thing;
+    fp->where.ufun->name = mush_strdup(upcasestr(argv[2]), "userfn.name");
 
     notify(player, T("Function added."));
     return;
@@ -1234,23 +1382,16 @@ do_function(dbref player, char *name, char *argv[], int preserve)
       return;
     }
     if (fp->flags & FN_BUILTIN) {       /* Overriding a built in function */
-      if (userfn_count >= (Size_t) MAX_GLOBAL_FNS) {
-        notify(player, T("Function table full."));
-        return;
-      }
-      fp = (FUN *) mush_malloc(sizeof(FUN), "func_hash.FUN");
+      fp = slab_malloc(function_slab, NULL);
       fp->name = mush_strdup(name, "func_hash.name");
-      fp->where.offset = userfn_count;
+      fp->where.ufun = mush_malloc(sizeof(USERFN_ENTRY), "userfn");
       fp->flags = 0;
-      userfn_count++;
       hashadd(name, fp, &htab_user_function);
     }
-    userfn_tab[fp->where.offset].thing = thing;
-    if (userfn_tab[fp->where.offset].name)
-      mush_free((Malloc_t) userfn_tab[fp->where.offset].name,
-                "userfn_tab.name");
-    userfn_tab[fp->where.offset].name =
-      mush_strdup(upcasestr(argv[2]), "userfn_tab.name");
+    fp->where.ufun->thing = thing;
+    if (fp->where.ufun->name)
+      mush_free(fp->where.ufun->name, "userfn.name");
+    fp->where.ufun->name = mush_strdup(upcasestr(argv[2]), "userfn.name");
     if (argv[3] && *argv[3]) {
       fp->minargs = parse_integer(argv[3]);
       if (fp->minargs < 0)
@@ -1285,26 +1426,12 @@ do_function(dbref player, char *name, char *argv[], int preserve)
 static void
 delete_function(void *data)
 {
-  size_t table_index, i;
   FUN *fp = data;
 
-  table_index = fp->where.offset;
   mush_free((void *) fp->name, "func_hash.name");
-  mush_free(fp, "func_hash.FUN");
-  /* Fix up the user function table. Expensive, but how often will
-   * we need to delete an @function anyway?
-   */
-  mush_free((Malloc_t) userfn_tab[table_index].name, "userfn_tab.name");
-  mush_free((Malloc_t) userfn_tab[table_index].fn, "usrfn_tab.fn");
-  userfn_count--;
-  for (i = table_index; i < userfn_count; i++) {
-    fp = (FUN *) hashfind(userfn_tab[i + 1].fn, &htab_user_function);
-    fp->where.offset = i;
-    userfn_tab[i].thing = userfn_tab[i + 1].thing;
-    userfn_tab[i].name = userfn_tab[i + 1].name;
-    userfn_tab[i].fn = userfn_tab[i + 1].fn;
-  }
-
+  mush_free(fp->where.ufun->name, "userfn.name");
+  mush_free(fp->where.ufun, "userfn");
+  slab_free(function_slab, fp);
 }
 
 /** Restore an overridden built-in function.
@@ -1387,7 +1514,7 @@ do_function_delete(dbref player, char *name)
     return;
   }
 
-  if (!controls(player, userfn_tab[fp->where.offset].thing)) {
+  if (!controls(player, fp->where.ufun->thing)) {
     notify(player, T("You can't delete that @function."));
     return;
   }
@@ -1440,32 +1567,46 @@ do_function_toggle(dbref player, char *name, int toggle)
 void
 do_function_report(dbref player, char *name)
 {
-  FUN *fp;
-  char tbuf[BUFFER_LEN];
-  char *tp;
-  const char *state, *state2;
-  int first = 1;
-  int maxargs;
+  FUN *fp, *bfp;
 
   fp = func_hash_lookup(name);
   if (!fp) {
     notify(player, T("No such function."));
     return;
   }
+  notify(player, build_function_report(player, fp));
+
+  bfp = builtin_func_hash_lookup(name);
+  if (bfp && (fp != bfp))
+    notify(player, build_function_report(player, bfp));
+}
+
+static char *
+build_function_report(dbref player, FUN *fp)
+{
+  char tbuf[BUFFER_LEN];
+  char *tp = tbuf;
+  static char buff[BUFFER_LEN];
+  char *bp = buff;
+  const char *state, *state2;
+  int first = 1;
+  int maxargs;
 
   if (fp->flags & FN_BUILTIN)
-    state2 = "";
+    state2 = " builtin";
   else
     state2 = " @function";
 
   if (fp->flags & FN_DISABLED)
     state = "Disabled";
+  else if (fp->flags & FN_OVERRIDE)
+    state = "Overridden";
   else
     state = "Enabled";
 
-  notify_format(player, T("Name      : %s() (%s%s)"), fp->name, state, state2);
+  safe_format(buff, &bp, T("Name      : %s() (%s%s)"), fp->name, state, state2);
+  safe_chr('\n', buff, &bp);
 
-  tp = tbuf;
   tbuf[0] = '\0';
   if (fp->flags & FN_NOPARSE) {
     safe_str("Noparse", tbuf, &tp);
@@ -1480,6 +1621,13 @@ do_function_report(dbref player, char *name)
     first = 0;
   }
 
+  if (fp->flags & FN_LOCALIZE) {
+    if (first == 0)
+      safe_strl(", ", 2, tbuf, &tp);
+    safe_str("Userfn", tbuf, &tp);
+    first = 0;
+  }
+
   if (fp->flags & FN_LITERAL) {
     if (first == 0)
       safe_strl(", ", 2, tbuf, &tp);
@@ -1550,13 +1698,12 @@ do_function_report(dbref player, char *name)
   }
 
   *tp = '\0';
-  notify_format(player, T("Flags     : %s"), tbuf);
+  safe_format(buff, &bp, T("Flags     : %s"), tbuf);
+  safe_chr('\n', buff, &bp);
 
-  if (!(fp->flags & FN_BUILTIN) && Global_Funcs(player)) {
-    notify_format(player, T("Location  : #%d/%s"),
-                  userfn_tab[fp->where.offset].thing,
-                  userfn_tab[fp->where.offset].name);
-  }
+  if (!(fp->flags & FN_BUILTIN) && Global_Funcs(player))
+    safe_format(buff, &bp, T("Location  : #%d/%s\n"),
+                fp->where.ufun->thing, fp->where.ufun->name);
 
   maxargs = abs(fp->maxargs);
 
@@ -1569,10 +1716,12 @@ do_function_report(dbref player, char *name)
     tbuf[0] = '\0';
 
   if (fp->minargs == maxargs)
-    notify_format(player, T("Arguments : %d %s"), fp->minargs, tbuf);
+    safe_format(buff, &bp, T("Arguments : %d %s"), fp->minargs, tbuf);
   else if (fp->maxargs == INT_MAX)
-    notify_format(player, T("Arguments : At least %d %s"), fp->minargs, tbuf);
+    safe_format(buff, &bp, T("Arguments : At least %d %s"), fp->minargs, tbuf);
   else
-    notify_format(player,
-                  T("Arguments : %d to %d %s"), fp->minargs, maxargs, tbuf);
+    safe_format(buff, &bp,
+                T("Arguments : %d to %d %s"), fp->minargs, maxargs, tbuf);
+  *bp = '\0';
+  return buff;
 }
