@@ -37,9 +37,15 @@ extern struct db_stat_info *get_stats(dbref owner);
 static int lattr_helper(dbref player, dbref thing, dbref parent,
                         char const *pattern, ATTR *atr, void *args);
 static dbref
- dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
-        int type, dbref loc, dbref after, int skipdark,
-        int start, int count, int *retcount);
+
+
+
+
+
+
+dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
+       int type, dbref loc, dbref after, int skipdark,
+       int start, int count, int *retcount);
 
 
 const char *
@@ -77,6 +83,7 @@ struct lh_args {
   int count;            /**< How many do we count? */
   char *buff;           /**< Buffer to store output */
   char **bp;            /**< Pointer to address of insertion point in buff */
+  char delim;           /**< Delimiter */
 };
 
 /* this function produces a space-separated list of attributes that are
@@ -97,7 +104,7 @@ lattr_helper(dbref player __attribute__ ((__unused__)),
     if (lh->first)
       lh->first = 0;
     else
-      safe_chr(' ', lh->buff, lh->bp);
+      safe_chr(lh->delim, lh->buff, lh->bp);
     safe_str(AL_NAME(atr), lh->buff, lh->bp);
     return 1;
   }
@@ -140,6 +147,7 @@ FUNCTION(fun_lattr)
   dbref thing;
   char *pattern;
   struct lh_args lh;
+  char delim;
 
   lh.nattr = lh.start = lh.count = 0;
 
@@ -156,6 +164,13 @@ FUNCTION(fun_lattr)
       safe_str(T("#-1 ARGUMENT OUT OF RANGE"), buff, bp);
       return;
     }
+
+    if (!delim_check(buff, bp, nargs, args, 4, &delim))
+      return;
+  } else {
+    /* lattr()/lattrp*( */
+    if (!delim_check(buff, bp, nargs, args, 2, &delim))
+      return;
   }
 
   pattern = strchr(args[0], '/');
@@ -172,6 +187,8 @@ FUNCTION(fun_lattr)
   lh.first = 1;
   lh.buff = buff;
   lh.bp = bp;
+  lh.delim = delim;
+
   if (strchr(called_as, 'P')) {
     (void) atr_iter_get_parent(executor, thing, pattern,
                                !Can_Examine(executor, thing), lattr_helper,
@@ -186,17 +203,29 @@ FUNCTION(fun_lattr)
 FUNCTION(fun_hasattr)
 {
   dbref thing;
+  char *attr;
   ATTR *a;
 
+  if (nargs == 1) {
+    attr = strchr(args[0], '/');
+    if (!attr) {
+      safe_format(buff, bp, T("#-1 BAD ARGUMENT FORMAT TO %s"), called_as);
+      return;
+    }
+    *attr++ = '\0';
+  } else {
+    attr = args[1];
+  }
+
   thing = match_thing(executor, args[0]);
   if (!GoodObject(thing)) {
     safe_str(T(e_notvis), buff, bp);
     return;
   }
   if (strchr(called_as, 'P'))
-    a = atr_get(thing, upcasestr(args[1]));
+    a = atr_get(thing, upcasestr(attr));
   else
-    a = atr_get_noparent(thing, upcasestr(args[1]));
+    a = atr_get_noparent(thing, upcasestr(attr));
   if (a && Can_Read_Attr(executor, thing, a)) {
     if (strchr(called_as, 'V'))
       safe_chr(*AL_STR(a) ? '1' : '0', buff, bp);
@@ -814,6 +843,7 @@ FUNCTION(fun_entrances)
   int found;
   int exd, td, pd, rd;          /* what we're looking for */
   char *p;
+  int controlswhere = 0;
 
   if (!command_check_byname(executor, "@entrances")) {
     safe_str(T(e_perm), buff, bp);
@@ -890,20 +920,15 @@ FUNCTION(fun_entrances)
   if (!GoodObject(high))
     high = db_top - 1;
 
-  if (!controls(executor, where) && !Search_All(executor)) {
+  controlswhere = controls(executor, where);
+  if (!controlswhere && !Search_All(executor)) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
-  if (!payfor(executor, FIND_COST)) {
-    notify_format(executor, T("You don't have %d %s to do that."),
-                  FIND_COST, ((FIND_COST == 1) ? MONEY : MONIES));
-    safe_str("#-1", buff, bp);
-    return;
-  }
   /* Ok, do the real work */
   found = 0;
   for (counter = low; counter <= high; counter++) {
-    if (controls(executor, where) || controls(executor, counter)) {
+    if (GoodObject(counter)) {
       if ((exd && IsExit(counter)) ||
           (td && IsThing(counter)) ||
           (pd && IsPlayer(counter)) || (rd && IsRoom(counter))) {
@@ -912,11 +937,13 @@ FUNCTION(fun_entrances)
         else
           entrance = Location(counter);
         if (entrance == where) {
-          if (!found)
-            found = 1;
-          else
-            safe_chr(' ', buff, bp);
-          safe_dbref(counter, buff, bp);
+          if (controlswhere || controls(executor, counter)) {
+            if (!found)
+              found = 1;
+            else
+              safe_chr(' ', buff, bp);
+            safe_dbref(counter, buff, bp);
+          }
         }
       }
     }
@@ -947,16 +974,31 @@ FUNCTION(fun_nearby)
 FUNCTION(fun_controls)
 {
   dbref it = match_thing(executor, args[0]);
-  dbref thing = match_thing(executor, args[1]);
+  dbref thing;
+  char *attrname;
 
-  if (!GoodObject(it))
+  if (!GoodObject(it)) {
     safe_str(T("#-1 ARG1 NOT FOUND"), buff, bp);
-  else if (!GoodObject(thing))
+    return;
+  }
+
+  if ((attrname = strchr(args[1], '/')) != NULL)
+    *attrname++ = '\0';
+  thing = match_thing(executor, args[1]);
+  if (!GoodObject(thing)) {
     safe_str(T("#-1 ARG2 NOT FOUND"), buff, bp);
-  else if (!(controls(executor, it) || controls(executor, thing)
-             || See_All(executor)))
+    return;
+  }
+  if (!(controls(executor, it) || controls(executor, thing)
+        || See_All(executor)))
     safe_str(T(e_perm), buff, bp);
-  else
+  else if (attrname) {
+    if (!good_atr_name(upcasestr(attrname))) {
+      safe_str(T("#-1 BAD ATTR NAME"), buff, bp);
+      return;
+    }
+    safe_chr(can_edit_attr(it, thing, attrname) ? '1' : '0', buff, bp);
+  } else
     safe_chr(controls(it, thing) ? '1' : '0', buff, bp);
 }
 
@@ -1031,13 +1073,13 @@ FUNCTION(fun_hasflag)
 {
   dbref thing;
   ATTR *attrib;
-  int f;
+  privbits f = 0;
 
   if (strchr(args[0], '/')) {
     parse_attrib(executor, args[0], &thing, &attrib);
     if (!attrib)
       safe_str("#-1", buff, bp);
-    else if ((f = string_to_atrflag(executor, args[1])) < 0)
+    else if (string_to_atrflag(executor, args[1], &f) < 0)
       safe_str("#-1", buff, bp);
     else
       safe_boolean(AL_FLAGS(attrib) & f, buff, bp);
@@ -1212,8 +1254,8 @@ FUNCTION(fun_lockflags)
   }
   ltype = get_locktype(p);
 
-  if (GoodObject(it) && (ltype != NULL)
-      && Can_Read_Lock(executor, it, ltype)) {
+  if (GoodObject(it) && (ltype !=NULL)
+      &&Can_Read_Lock(executor, it, ltype)) {
     ll = getlockstruct(it, ltype);
     if (ll) {
       if (fullname)
@@ -1251,7 +1293,7 @@ FUNCTION(fun_lock)
   lock_type real_ltype;
 
   if ((ltype = strchr(args[0], '/'))) {
-    *ltype++ = '\0';
+    *ltype ++ = '\0';
     upcasestr(ltype);
   }
 
@@ -1295,7 +1337,8 @@ FUNCTION(fun_elock)
   it = match_thing(executor, args[0]);
   ltype = get_locktype(p);
 
-  if (!GoodObject(it) || (ltype == NULL) || !Can_Read_Lock(executor, it, ltype)) {
+  if (!GoodObject(it) || !GoodObject(victim)
+      || (ltype == NULL) ||!Can_Read_Lock(executor, it, ltype)) {
     safe_str("#-1", buff, bp);
     return;
   }
@@ -1349,16 +1392,51 @@ FUNCTION(fun_objid)
 /* ARGSUSED */
 FUNCTION(fun_ctime)
 {
+  bool utc = 0;
   dbref it = match_thing(executor, args[0]);
 
-  if (GoodObject(it) && !IsGarbage(it))
-    safe_str(show_time(CreTime(it), 0), buff, bp);
-  else
+  if (nargs == 2)
+    utc = parse_boolean(args[1]);
+
+  if (!GoodObject(it) || IsGarbage(it))
     safe_str(T(e_notvis), buff, bp);
+  else if (!Can_Examine(executor, it))
+    safe_str(T(e_perm), buff, bp);
+  else
+    safe_str(show_time(CreTime(it), utc), buff, bp);
+}
+
+FUNCTION(fun_csecs)
+{
+  dbref it = match_thing(executor, args[0]);
+
+  if (!GoodObject(it) || IsGarbage(it))
+    safe_str(T(e_notvis), buff, bp);
+  else if (!Can_Examine(executor, it))
+    safe_str(T(e_perm), buff, bp);
+  else
+    safe_integer((intmax_t) CreTime(it), buff, bp);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_mtime)
+{
+  bool utc = 0;
+  dbref it = match_thing(executor, args[0]);
+
+  if (nargs == 2)
+    utc = parse_boolean(args[1]);
+
+  if (!GoodObject(it) || IsGarbage(it))
+    safe_str(T(e_notvis), buff, bp);
+  else if (!Can_Examine(executor, it) || IsPlayer(it))
+    safe_str(T(e_perm), buff, bp);
+  else
+    safe_str(show_time(ModTime(it), utc), buff, bp);
+}
+
+/* ARGSUSED */
+FUNCTION(fun_msecs)
 {
   dbref it = match_thing(executor, args[0]);
   if (!GoodObject(it) || IsGarbage(it))
@@ -1366,7 +1444,7 @@ FUNCTION(fun_mtime)
   else if (!Can_Examine(executor, it) || IsPlayer(it))
     safe_str(T(e_perm), buff, bp);
   else
-    safe_str(show_time(ModTime(it), 0), buff, bp);
+    safe_integer((intmax_t) ModTime(it), buff, bp);
 }
 
 /* ARGSUSED */
@@ -1581,8 +1659,10 @@ FUNCTION(fun_alias)
   dbref it;
 
   it = match_thing(executor, args[0]);
-  if (!GoodObject(it))
+  if (!GoodObject(it)) {
     safe_str(T(e_notvis), buff, bp);
+    return;
+  }
 
   /* Support changing alias via function if side-effects are enabled */
   if (nargs == 2) {
@@ -1909,7 +1989,7 @@ FUNCTION(fun_create)
     cost = parse_integer(args[1]);
   else
     cost = OBJECT_COST;
-  safe_dbref(do_create(executor, args[0], cost), buff, bp);
+  safe_dbref(do_create(executor, args[0], cost, args[2]), buff, bp);
 }
 
 /* ARGSUSED */
@@ -1923,7 +2003,7 @@ FUNCTION(fun_pcreate)
     safe_str(T(e_perm), buff, bp);
     return;
   }
-  safe_dbref(do_pcreate(executor, args[0], args[1]), buff, bp);
+  safe_dbref(do_pcreate(executor, args[0], args[1], args[2]), buff, bp);
 }
 
 /* ARGSUSED */
@@ -2083,7 +2163,13 @@ FUNCTION(fun_tel)
 /* ARGSUSED */
 FUNCTION(fun_isdbref)
 {
-  safe_boolean(parse_dbref(args[0]) != NOTHING, buff, bp);
+  safe_boolean(parse_objid(args[0]) != NOTHING, buff, bp);
+}
+
+/* ARGSUSED */
+FUNCTION(fun_isobjid)
+{
+  safe_boolean(is_objid(args[0]), buff, bp);
 }
 
 /* ARGSUSED */
