@@ -27,6 +27,7 @@
 #include "game.h"
 #include "attrib.h"
 #include "confmagic.h"
+#include "ansi.h"
 
 #ifdef WIN32
 #pragma warning( disable : 4761)        /* NJG: disable warning re conversion */
@@ -81,6 +82,17 @@ FUNCTION(fun_pemit)
   orator = saved_orator;
 }
 
+FUNCTION(fun_message)
+{
+  int i;
+  char *argv[10];
+
+  for (i = 0; (i + 3) < nargs; i++) {
+    argv[i] = args[i + 3];
+  }
+
+  do_message_list(executor, executor, args[0], args[2], args[1], 0, i, argv);
+}
 
 /* ARGSUSED */
 FUNCTION(fun_oemit)
@@ -196,6 +208,87 @@ FUNCTION(fun_setq)
   }
 }
 
+FUNCTION(fun_letq)
+{
+  char **values = NULL;
+  int *regs = NULL;
+  int npairs;
+  int n;
+  char tbuf[BUFFER_LEN], *tbp;
+  char *preserve[NUMQ];
+  const char *p;
+
+  if ((nargs % 2) != 1) {
+    safe_str(T("#-1 FUNCTION (letq) EXPECTS AN ODD NUMBER OF ARGUMENTS"),
+             buff, bp);
+    return;
+  }
+
+  npairs = (nargs - 1) / 2;
+
+  for (n = 0; n < NUMQ; n++)
+    preserve[n] = NULL;
+
+  if (npairs) {
+    values = mush_calloc(npairs, sizeof(char *), "letq.values");
+    if (!values) {
+      safe_str(T("#-1 UNABLE TO ALLOCATE MEMORY"), buff, bp);
+      return;
+    }
+
+    regs = mush_calloc(npairs, sizeof(int), "letq.registers");
+    if (!regs) {
+      safe_str(T("#-1 UNABLE TO ALLOCATE MEMORY"), buff, bp);
+      mush_free(values, "letq.values");
+      return;
+    }
+
+    for (n = 0; n < npairs; n++) {
+      int i = n * 2;
+
+      tbp = tbuf;
+      p = args[i];
+      process_expression(tbuf, &tbp, &p, executor, caller, enactor, PE_DEFAULT,
+                         PT_DEFAULT, pe_info);
+      *tbp = '\0';
+      regs[n] = qreg_indexes[(unsigned char) tbuf[0]];
+      if (regs[n] < 0) {
+        safe_str(T("#-1 REGISTER OUT OF RANGE"), buff, bp);
+        goto cleanup;
+      }
+
+      tbp = tbuf;
+      p = args[i + 1];
+      process_expression(tbuf, &tbp, &p, executor, caller, enactor, PE_DEFAULT,
+                         PT_DEFAULT, pe_info);
+      *tbp = '\0';
+      values[n] = mush_strdup(tbuf, "letq.value");
+      if (!values[n]) {
+        safe_str(T("#-1 UNABLE TO ALLOCATE MEMORY"), buff, bp);
+        goto cleanup;
+      }
+    }
+
+    for (n = 0; n < npairs; n++) {
+      save_partial_global_reg("letq", preserve, regs[n]);
+      mush_strncpy(global_eval_context.renv[regs[n]], values[n], BUFFER_LEN);
+    }
+  }
+  p = args[nargs - 1];
+  process_expression(buff, bp, &p, executor, caller, enactor, PE_DEFAULT,
+                     PT_DEFAULT, pe_info);
+
+cleanup:
+  if (regs)
+    mush_free(regs, "letq.registers");
+  if (values) {
+    restore_partial_global_regs("letq", preserve);
+    for (n = 0; n < npairs; n++)
+      mush_free(values[n], "letq.value");
+    mush_free(values, "letq.values");
+  }
+}
+
 /* ARGSUSED */
 FUNCTION(fun_r)
 {
@@ -244,7 +337,7 @@ FUNCTION(fun_rand)
     return;
   }
 
-  safe_integer(get_random_long(low, high), buff, bp);
+  safe_integer(get_random32(low, high), buff, bp);
 }
 
 /* ARGSUSED */
@@ -275,11 +368,11 @@ FUNCTION(fun_die)
         first = 0;
       else
         safe_chr(' ', buff, bp);
-      safe_uinteger(get_random_long(1, die), buff, bp);
+      safe_uinteger(get_random32(1, die), buff, bp);
     }
   } else {
     for (count = 0; count < n; count++)
-      total += get_random_long(1, die);
+      total += get_random32(1, die);
 
     safe_uinteger(total, buff, bp);
   }
@@ -425,18 +518,37 @@ FUNCTION(fun_if)
 {
   char tbuf[BUFFER_LEN], *tp;
   char const *sp;
+  /* Since this may be used as COND(), NCOND(), CONDALL() or NCONDALL,
+   * we check for that. */
+  int findtrue = 1;
+  int findall = 0;
+  int found = 0;
+  int i;
 
-  tp = tbuf;
-  sp = args[0];
-  process_expression(tbuf, &tp, &sp, executor, caller, enactor,
-                     PE_DEFAULT, PT_DEFAULT, pe_info);
-  *tp = '\0';
-  if (parse_boolean(tbuf)) {
-    sp = args[1];
-    process_expression(buff, bp, &sp, executor, caller, enactor,
+  if (called_as[0] == 'N')
+    findtrue = 0;
+
+  if (strstr(called_as, "ALL"))
+    findall = 1;
+
+  for (i = 0; i < nargs - 1; i += 2) {
+    tp = tbuf;
+    sp = args[i];
+    process_expression(tbuf, &tp, &sp, executor, caller, enactor,
                        PE_DEFAULT, PT_DEFAULT, pe_info);
-  } else if (nargs > 2) {
-    sp = args[2];
+    *tp = '\0';
+    if (parse_boolean(tbuf) == findtrue) {
+      sp = args[i + 1];
+      process_expression(buff, bp, &sp, executor, caller, enactor,
+                         PE_DEFAULT, PT_DEFAULT, pe_info);
+      if (!findall)
+        return;
+      found = 1;
+    }
+  }
+  /* If we've found no true case, run the default if it exists. */
+  if (!found && (nargs & 1)) {
+    sp = args[nargs - 1];
     process_expression(buff, bp, &sp, executor, caller, enactor,
                        PE_DEFAULT, PT_DEFAULT, pe_info);
   }
@@ -479,12 +591,7 @@ FUNCTION(fun_restarts)
   safe_integer(globals.reboot_count, buff, bp);
 }
 
-/* Data for soundex functions */
-static char soundex_val[26] = {
-  0, 1, 2, 3, 0, 1, 2, 0, 0,
-  2, 2, 4, 5, 5, 0, 1, 2, 6,
-  2, 3, 0, 1, 0, 2, 0, 2
-};
+extern char soundex_val[UCHAR_MAX + 1];
 
 /* The actual soundex routine */
 static char *
@@ -493,28 +600,27 @@ soundex(char *str)
   static char tbuf1[BUFFER_LEN];
   char *p, *q;
 
-  tbuf1[0] = '\0';
-  tbuf1[1] = '\0';
-  tbuf1[2] = '\0';
-  tbuf1[3] = '\0';
+  q = remove_markup(str, NULL);
+  memset(tbuf1, '\0', 4);
 
   p = tbuf1;
-  q = upcasestr(remove_markup(str, NULL));
+
   /* First character is just copied */
-  *p = *q++;
+  *p = UPCASE(*q);
+  q++;
   /* Special case for PH->F */
-  if ((*p == 'P') && *q && (*q == 'H')) {
+  if ((UPCASE(*p) == 'P') && *q && (UPCASE(*q) == 'H')) {
     *p = 'F';
     q++;
   }
   p++;
-  /* Convert letters to soundex values, squash duplicates */
+  /* Convert letters to soundex values, squash duplicates, skip accents and other non-ascii characters */
   while (*q) {
-    if (!isalpha((unsigned char) *q) || !isascii((unsigned char) *q)) {
+    if (!isalpha((unsigned char) *q) || (unsigned char) *q > 127) {
       q++;
       continue;
     }
-    *p = soundex_val[*q++ - 'A'] + '0';
+    *p = soundex_val[(unsigned char) *q++];
     if (*p != *(p - 1))
       p++;
   }
@@ -582,7 +688,7 @@ FUNCTION(fun_soundlike)
 /* ARGSUSED */
 FUNCTION(fun_functions)
 {
-  safe_str(list_functions(), buff, bp);
+  safe_str(list_functions(nargs == 1 ? args[0] : NULL), buff, bp);
 }
 
 /* ARGSUSED */
@@ -611,7 +717,9 @@ FUNCTION(fun_list)
   else if (string_prefix("fullmotd", args[0]) && Hasprivs(executor))
     safe_str(cf_fullmotd_msg, buff, bp);
   else if (string_prefix("functions", args[0]))
-    safe_str(list_functions(), buff, bp);
+    safe_str(list_functions(NULL), buff, bp);
+  else if (string_prefix("@functions", args[0]))
+    safe_str(list_functions("local"), buff, bp);
   else if (string_prefix("commands", args[0]))
     safe_str(list_commands(), buff, bp);
   else if (string_prefix("attribs", args[0]))
@@ -661,7 +769,7 @@ enum whichof_t { DO_FIRSTOF, DO_ALLOF };
 static void
 do_whichof(char *args[], int nargs, enum whichof_t flag,
            char *buff, char **bp, dbref executor,
-           dbref caller, dbref enactor, PE_Info * pe_info)
+           dbref caller, dbref enactor, PE_Info *pe_info)
 {
   int j;
   char tbuf[BUFFER_LEN], *tp;
