@@ -13,11 +13,11 @@
 #include <fcntl.h>
 #include <string.h>
 #include <signal.h>
-#ifdef I_SYS_WAIT
-#include <sys/wait.h>
-#endif
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -30,7 +30,7 @@ void Win32MUSH_setup(void);
 #ifdef I_SYS_TYPES
 #include <sys/types.h>
 #endif
-#ifdef HAS_GETRUSAGE
+#ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
 #include <stdlib.h>
@@ -60,6 +60,7 @@ void Win32MUSH_setup(void);
 #include "command.h"
 #include "htab.h"
 #include "ptab.h"
+#include "intmap.h"
 #include "log.h"
 #include "lock.h"
 #include "dbdefs.h"
@@ -67,7 +68,11 @@ void Win32MUSH_setup(void);
 #include "function.h"
 #include "help.h"
 #include "dbio.h"
-#include "pcre.h"
+#include "mypcre.h"
+#ifndef WIN32
+#include "wait.h"
+#endif
+#include "ansi.h"
 
 #ifdef hpux
 #include <sys/syscall.h>
@@ -75,39 +80,30 @@ void Win32MUSH_setup(void);
 #endif                          /* fix to HP-UX getrusage() braindamage */
 
 #include "confmagic.h"
-#ifdef HAS_WAITPID
-/** Return value of wait* functions */
-#define WAIT_TYPE int
-#else
-#ifdef UNION_WAIT
-#define WAIT_TYPE union wait
-#else
-#define WAIT_TYPE int
-#endif
-#endif
 
 /* declarations */
 GLOBALTAB globals = { 0, "", 0, 0, 0, 0, 0, 0, 0, 0 };
+
 static int epoch = 0;
 static int reserved;                    /**< Reserved file descriptor */
 static dbref *errdblist = NULL; /**< List of dbrefs to return errors from */
-static dbref *errdbtail;        /**< Pointer to end of errdblist */
-static int errdbsize = 4;       /**< Current size of errdblist array */
+static dbref *errdbtail = NULL;        /**< Pointer to end of errdblist */
+#define ERRDB_INITIAL_SIZE 5
+static int errdbsize = ERRDB_INITIAL_SIZE; /**< Current size of errdblist array */
 static void errdb_grow(void);
 
 
 extern void initialize_mt(void);
 extern const unsigned char *tables;
 extern void conf_default_set(void);
-static int dump_database_internal(void);
-static FILE *db_open(const char *filename);
-static FILE *db_open_write(const char *filename);
-static void db_close(FILE * f);
+static bool dump_database_internal(void);
+static PENNFILE *db_open(const char *);
+static PENNFILE *db_open_write(const char *);
 static int fail_commands(dbref player);
 void do_readcache(dbref player);
 int check_alias(const char *command, const char *list);
-int list_check(dbref thing, dbref player, char type,
-               char end, char *str, int just_match);
+static int list_check(dbref thing, dbref player, char type,
+                      char end, char *str, int just_match);
 int alias_list_check(dbref thing, const char *command, const char *type);
 int loc_alias_check(dbref loc, const char *command, const char *type);
 void do_poor(dbref player, char *arg1);
@@ -125,6 +121,7 @@ void rusage_stats(void);
 #endif
 
 void do_list_memstats(dbref player);
+void do_list_allocations(dbref player);
 void st_stats_header(dbref player);
 void st_stats(dbref player, StrTree *root, const char *name);
 void do_timestring(char *buff, char **bp, const char *format,
@@ -141,7 +138,7 @@ extern void compress_stats(long *entries,
 #endif
 
 
-Pid_t forked_dump_pid = -1;
+pid_t forked_dump_pid = -1;
 
 /** Open /dev/null to reserve a file descriptor that can be reused later. */
 void
@@ -295,11 +292,6 @@ do_shutdown(dbref player, enum shutdown_type flag)
     do_log(LT_ERR, player, NOTHING, T("SHUTDOWN by %s(%s)\n"),
            Name(player), unparse_dbref(player));
 
-    /* This will create a file used to check if a restart should occur */
-#ifdef AUTORESTART
-    system("touch NORESTART");
-#endif
-
     if (flag == SHUT_PANIC) {
       mush_panic("@shutdown/panic");
     } else {
@@ -318,20 +310,20 @@ do_shutdown(dbref player, enum shutdown_type flag)
 
 jmp_buf db_err;
 
-static int
+static bool
 dump_database_internal(void)
 {
   char realdumpfile[2048];
   char realtmpfl[2048];
   char tmpfl[2048];
-  FILE *f = NULL;
+  PENNFILE *f = NULL;
 
 #ifndef PROFILING
 #ifndef WIN32
   ignore_signal(SIGPROF);
 #endif
 #endif
-#ifdef I_SETJMP
+
   if (setjmp(db_err)) {
     /* The dump failed. Disk might be full or something went bad with the
        compression slave. Boo! */
@@ -339,13 +331,13 @@ dump_database_internal(void)
     flag_broadcast("WIZARD ROYALTY", 0,
                    T("GAME: ERROR! Database save failed!"));
     if (f)
-      db_close(f);
+      penn_fclose(f);
 #ifndef PROFILING
 #ifdef HAS_ITIMER
     install_sig_handler(SIGPROF, signal_cpu_limit);
 #endif
 #endif
-    return 1;
+    return false;
   } else {
     local_dump_database();
 
@@ -375,13 +367,13 @@ dump_database_internal(void)
         db_paranoid_write(f, 1);
         break;
       }
-      db_close(f);
+      penn_fclose(f);
       if (rename_file(realtmpfl, realdumpfile) < 0) {
-        perror(realtmpfl);
+        penn_perror(realtmpfl);
         longjmp(db_err, 1);
       }
     } else {
-      perror(realtmpfl);
+      penn_perror(realtmpfl);
       longjmp(db_err, 1);
     }
     sprintf(realdumpfile, "%s%s", options.mail_db, options.compresssuff);
@@ -390,13 +382,13 @@ dump_database_internal(void)
     if (mdb_top >= 0) {
       if ((f = db_open_write(tmpfl)) != NULL) {
         dump_mail(f);
-        db_close(f);
+        penn_fclose(f);
         if (rename_file(realtmpfl, realdumpfile) < 0) {
-          perror(realtmpfl);
+          penn_perror(realtmpfl);
           longjmp(db_err, 1);
         }
       } else {
-        perror(realtmpfl);
+        penn_perror(realtmpfl);
         longjmp(db_err, 1);
       }
     }
@@ -405,26 +397,25 @@ dump_database_internal(void)
     sprintf(realtmpfl, "%s%s", tmpfl, options.compresssuff);
     if ((f = db_open_write(tmpfl)) != NULL) {
       save_chatdb(f);
-      db_close(f);
+      penn_fclose(f);
       if (rename_file(realtmpfl, realdumpfile) < 0) {
-        perror(realtmpfl);
+        penn_perror(realtmpfl);
         longjmp(db_err, 1);
       }
     } else {
-      perror(realtmpfl);
+      penn_perror(realtmpfl);
       longjmp(db_err, 1);
     }
     time(&globals.last_dump_time);
   }
 
-#endif
 #ifndef PROFILING
 #ifdef HAS_ITIMER
   install_sig_handler(SIGPROF, signal_cpu_limit);
 #endif
 #endif
 
-  return 0;
+  return true;
 }
 
 /** Crash gracefully.
@@ -438,7 +429,7 @@ void
 mush_panic(const char *message)
 {
   const char *panicfile = options.crash_db;
-  FILE *f = NULL;
+  PENNFILE *f = NULL;
   static int already_panicking = 0;
 
   if (already_panicking) {
@@ -467,7 +458,7 @@ mush_panic(const char *message)
       do_rawlog(LT_ERR, T("CANNOT DUMP PANIC DB. OOPS."));
       _exit(134);
     } else {
-      if ((f = fopen(panicfile, FOPEN_WRITE)) == NULL) {
+      if ((f = penn_fopen(panicfile, FOPEN_WRITE)) == NULL) {
         do_rawlog(LT_ERR, T("CANNOT OPEN PANIC FILE, YOU LOSE"));
         _exit(135);
       } else {
@@ -475,7 +466,7 @@ mush_panic(const char *message)
         db_write(f, DBF_PANIC);
         dump_mail(f);
         save_chatdb(f);
-        fclose(f);
+        penn_fclose(f);
         do_rawlog(LT_ERR, T("DUMPING: %s (done)"), panicfile);
       }
     }
@@ -523,7 +514,7 @@ dump_database(void)
   epoch++;
 
   do_rawlog(LT_ERR, "DUMPING: %s.#%d#", globals.dumpfile, epoch);
-  if (!dump_database_internal())
+  if (dump_database_internal())
     do_rawlog(LT_ERR, "DUMPING: %s.#%d# (done)", globals.dumpfile, epoch);
 }
 
@@ -539,19 +530,20 @@ dump_database(void)
 void
 fork_and_dump(int forking)
 {
-  int child, nofork, status, split;
+  pid_t child;
+  bool nofork, status, split;
   epoch++;
 
 #ifdef LOG_CHUNK_STATS
   chunk_stats(NOTHING, 0);
   chunk_stats(NOTHING, 1);
 #endif
-  do_rawlog(LT_CHECK, "CHECKPOINTING: %s.#%d#\n", globals.dumpfile, epoch);
+  do_rawlog(LT_CHECK, "CHECKPOINTING: %s.#%d#", globals.dumpfile, epoch);
   if (NO_FORK)
     nofork = 1;
   else
     nofork = !forking || (globals.paranoid_dump == 2);  /* Don't fork for dump/debug */
-#ifdef WIN32
+#if defined(WIN32) || !defined(HAVE_FORK)
   nofork = 1;
 #endif
   split = 0;
@@ -572,7 +564,13 @@ fork_and_dump(int forking)
   }
   if (!nofork) {
 #ifndef WIN32
+#ifdef HAVE_FORK
     child = fork();
+#else
+    /* Never actually reached, since nofork is set to 1 if HAVE_FORK is
+     *  not defined. */
+    child = -1;
+#endif
     if (child < 0) {
       /* Oops, fork failed. Let's do a nofork dump */
       do_log(LT_ERR, 0, 0,
@@ -587,17 +585,10 @@ fork_and_dump(int forking)
       }
     } else if (child > 0) {
       forked_dump_pid = child;
+      lower_priority_by(child, 8);
       chunk_fork_parent();
     } else {
       chunk_fork_child();
-#ifdef HAS_SETPRIORITY
-      /* Lower the priority of the child to make parent more responsive */
-#ifdef HAS_GETPRIORITY
-      setpriority(PRIO_PROCESS, child, getpriority(PRIO_PROCESS, child) + 4);
-#else                           /* HAS_GETPRIORITY */
-      setpriority(PRIO_PROCESS, child, 8);
-#endif                          /* HAS_GETPRIORITY */
-#endif                          /* HAS_SETPRIORITY */
     }
 #endif                          /* WIN32 */
   } else {
@@ -614,10 +605,10 @@ fork_and_dump(int forking)
       chunk_fork_done();
 #endif
     if (!nofork) {
-      _exit(status);            /* !!! */
+      _exit(status ? 0 : 1);    /* d_d_i() returns true on success but exit code should be 0 on success */
     } else {
       reserve_fd();
-      if (!status && DUMP_NOFORK_COMPLETE && *DUMP_NOFORK_COMPLETE)
+      if (status && DUMP_NOFORK_COMPLETE && *DUMP_NOFORK_COMPLETE)
         flag_broadcast(0, 0, "%s", DUMP_NOFORK_COMPLETE);
     }
   }
@@ -685,8 +676,9 @@ do_restart(void)
   }
 }
 
-extern void init_names(void);
+void init_names(void);
 extern struct db_stat_info current_state;
+void init_queue(void);
 
 /** Initialize game structures and read the most of the configuration file.
  * This function runs before we read in the databases. It is responsible
@@ -699,6 +691,12 @@ void
 init_game_config(const char *conf)
 {
   int a;
+  pid_t mypid = -1;
+
+  /* initialize random number generator */
+  initialize_mt();
+
+  init_queue();
 
   global_eval_context.process_command_port = 0;
   global_eval_context.break_called = 0;
@@ -724,8 +722,7 @@ init_game_config(const char *conf)
   init_flag_table("FLAG");
   init_flag_table("POWER");
   init_func_hashtab();
-  init_math_hashtab();
-  init_tag_hashtab();
+  init_ansi_codes();
   init_aname_table();
   init_atr_name_tree();
   init_locks();
@@ -744,9 +741,13 @@ init_game_config(const char *conf)
   /* Initialize the attribute chunk storage */
   chunk_init();
 
+#ifdef HAVE_GETPID
+  mypid = getpid();
+#endif
+
   do_rawlog(LT_ERR, "%s", VERSION);
   do_rawlog(LT_ERR, T("MUSH restarted, PID %d, at %s"),
-            (int) getpid(), show_time(globals.start_time, 0));
+            (int) mypid, show_time(globals.start_time, 0));
 }
 
 /** Post-db-load configuration.
@@ -761,8 +762,6 @@ init_game_postdb(const char *conf)
 {
   /* access file stuff */
   read_access_file();
-  /* initialize random number generator */
-  initialize_mt();
   /* set up signal handlers for the timer */
   init_timer();
   /* Commands and functions require the flag table for restrictions */
@@ -772,6 +771,7 @@ init_game_postdb(const char *conf)
   attr_init_postconfig();
   /* Load further restrictions from config file */
   config_file_startup(conf, 1);
+  validate_config();
   /* Call Local Startup */
   local_startup();
   /* everything else ok. Restart all objects. */
@@ -797,7 +797,7 @@ extern int dbline;
 int
 init_game_dbs(void)
 {
-  FILE *f;
+  PENNFILE *f;
   int c;
   const char *infile, *outfile;
   const char *mailfile;
@@ -815,25 +815,24 @@ init_game_dbs(void)
   /* read small text files into cache */
   fcache_init();
 
-  f = db_open(infile);
-
-  if (!f) {
+  if (setjmp(db_err) == 1) {
     do_rawlog(LT_ERR, "Couldn't open %s! Creating minimal world.", infile);
     init_compress(NULL);
     create_minimal_db();
     return 0;
+  } else {
+    f = db_open(infile);
+    c = penn_fgetc(f);
+    if (c == EOF) {
+      do_rawlog(LT_ERR, "Couldn't read %s! Creating minimal world.", infile);
+      init_compress(NULL);
+      create_minimal_db();
+      return 0;
+    }
+    
+    penn_ungetc(c, f);
   }
 
-  c = getc(f);
-  if (c == EOF) {
-    do_rawlog(LT_ERR, "Couldn't read %s! Creating minimal world.", infile);
-    init_compress(NULL);
-    create_minimal_db();
-    return 0;
-  }
-
-  ungetc(c, f);
-
   if (setjmp(db_err) == 0) {
     /* ok, read it in */
     do_rawlog(LT_ERR, "ANALYZING: %s", infile);
@@ -844,7 +843,7 @@ init_game_dbs(void)
     do_rawlog(LT_ERR, "ANALYZING: %s (done)", infile);
 
     /* everything ok */
-    db_close(f);
+    penn_fclose(f);
 
     f = db_open(infile);
     if (!f)
@@ -855,7 +854,7 @@ init_game_dbs(void)
     dbline = 0;
     if (db_read(f) < 0) {
       do_rawlog(LT_ERR, "ERROR LOADING %s", infile);
-      db_close(f);
+      penn_fclose(f);
       return -1;
     }
     do_rawlog(LT_ERR, "LOADING: %s (done)", infile);
@@ -864,10 +863,10 @@ init_game_dbs(void)
      * format db, with everything shoved together. In that case,
      * don't close the file
      */
-    panicdb = ((globals.indb_flags & DBF_PANIC) && !feof(f));
+    panicdb = ((globals.indb_flags & DBF_PANIC) && !penn_feof(f));
 
     if (!panicdb)
-      db_close(f);
+      penn_fclose(f);
 
     /* complain about bad config options */
     if (!GoodObject(PLAYER_START) || (!IsRoom(PLAYER_START)))
@@ -892,7 +891,7 @@ init_game_dbs(void)
       do_rawlog(LT_ERR, T("LOADING: Trying to get mail from %s"), infile);
       if (load_mail(f) <= 0) {
         do_rawlog(LT_ERR, T("FAILED: Reverting to normal maildb"));
-        db_close(f);
+        penn_fclose(f);
         panicdb = 0;
       }
     }
@@ -905,7 +904,7 @@ init_game_dbs(void)
         dbline = 0;
         load_mail(f);
         do_rawlog(LT_ERR, "LOADING: %s (done)", mailfile);
-        db_close(f);
+        penn_fclose(f);
       }
     }
 
@@ -915,7 +914,7 @@ init_game_dbs(void)
       do_rawlog(LT_ERR, T("LOADING: Trying to get chat from %s"), infile);
       if (load_chatdb(f) <= 0) {
         do_rawlog(LT_ERR, T("FAILED: Reverting to normal chatdb"));
-        db_close(f);
+        penn_fclose(f);
         panicdb = 0;
       }
     }
@@ -931,9 +930,10 @@ init_game_dbs(void)
           do_rawlog(LT_ERR, "ERROR LOADING %s", options.chatdb);
           return -1;
         }
-        db_close(f);
+        penn_fclose(f);
       }
-    }
+    } else                      /* Close the panicdb file handle */
+      penn_fclose(f);
 
   } else {
     do_rawlog(LT_ERR, "ERROR READING DATABASE");
@@ -963,10 +963,10 @@ do_readcache(dbref player)
 /** Check each attribute on each object in x for a $command matching cptr */
 #define list_match(x)        list_check(x, player, '$', ':', cptr, 0)
 /** Check each attribute on x for a $command matching cptr */
-#define cmd_match(x)         atr_comm_match(x, player, '$', ':', cptr, 0, NULL, NULL, &errdb)
+#define cmd_match(x)         atr_comm_match(x, player, '$', ':', cptr, 0, 1, NULL, NULL, &errdb)
 #define MAYBE_ADD_ERRDB(errdb)  \
         do { \
-          if (GoodObject(errdb)) { \
+          if (GoodObject(errdb) && errdblist) { \
             if ((errdbtail - errdblist) >= errdbsize) \
               errdb_grow(); \
             if ((errdbtail - errdblist) < errdbsize) { \
@@ -1064,7 +1064,9 @@ process_command(dbref player, char *command, dbref cause, int from_port)
   dbref check_loc;
 
   if (!errdblist)
-    errdblist = mush_malloc(errdbsize * sizeof(dbref), "errdblist");
+    if (!(errdblist = mush_calloc(errdbsize, sizeof(dbref), "errdblist")))
+      mush_panic("Unable to allocate memory in process_command()!");
+
   errdbtail = errdblist;
   errdb = NOTHING;
   if (!command) {
@@ -1238,11 +1240,14 @@ process_command(dbref player, char *command, dbref cause, int from_port)
   /* command has been executed. Free up memory. */
 
 done:
-  ;
+  mush_free(errdblist, "errdblist");
+  errdblist = errdbtail = NULL;
+  errdbsize = ERRDB_INITIAL_SIZE;
 }
 
 
-COMMAND (cmd_with) {
+COMMAND(cmd_with)
+{
   dbref what;
   char *cptr = arg_right;
   dbref errdb;
@@ -1318,7 +1323,7 @@ check_alias(const char *command, const char *list)
  * \retval 1 a match was made.
  * \retval 0 no match was made.
  */
-int
+static int
 list_check(dbref thing, dbref player, char type, char end, char *str,
            int just_match)
 {
@@ -1327,7 +1332,7 @@ list_check(dbref thing, dbref player, char type, char end, char *str,
 
   while (thing != NOTHING) {
     if (atr_comm_match
-        (thing, player, type, end, str, just_match, NULL, NULL, &errdb))
+        (thing, player, type, end, str, just_match, 1, NULL, NULL, &errdb))
       match = 1;
     else {
       MAYBE_ADD_ERRDB(errdb);
@@ -1531,7 +1536,7 @@ bind_and_queue(dbref player, dbref cause, char *action,
 /** Would the scan command find an matching attribute on x for player p? */
 #define ScanFind(p,x)  \
   (Can_Examine(p,x) && \
-      ((num = atr_comm_match(x, p, '$', ':', command, 1, atrname, &ptr, NULL)) != 0))
+      ((num = atr_comm_match(x, p, '$', ':', command, 1, 1, atrname, &ptr, NULL)) != 0))
 
 /** Scan for matches of $commands.
  * This function scans for possible matches of user-def'd commands from the
@@ -1891,7 +1896,7 @@ linux_uptime(dbref player __attribute__ ((__unused__)))
   FILE *fp;
   char line[128];               /* Overkill */
   char *nl;
-  Pid_t pid;
+  pid_t pid;
   int psize;
 #ifdef HAS_GETRUSAGE
   struct rusage usage;
@@ -1996,7 +2001,8 @@ linux_uptime(dbref player __attribute__ ((__unused__)))
 static void
 unix_uptime(dbref player __attribute__ ((__unused__)))
 {
-#ifdef HAS_UPTIME
+#ifndef WIN32
+#ifdef HAVE_UPTIME
   FILE *fp;
   char c;
   int i;
@@ -2004,18 +2010,16 @@ unix_uptime(dbref player __attribute__ ((__unused__)))
 #ifdef HAS_GETRUSAGE
   struct rusage usage;
 #endif
-#ifndef WIN32
   char tbuf1[BUFFER_LEN];
-#endif
-  Pid_t pid;
+  pid_t pid;
   int psize;
 
-#ifdef HAS_UPTIME
+#ifdef HAVE_UPTIME
   fp =
 #ifdef __LCC__
     (FILE *)
 #endif
-    popen(UPTIME_PATH, "r");
+    popen(UPTIME, "r");
 
   /* just in case the system is screwy */
   if (fp == NULL) {
@@ -2030,7 +2034,7 @@ unix_uptime(dbref player __attribute__ ((__unused__)))
   pclose(fp);
 
   notify(player, tbuf1);
-#endif                          /* HAS_UPTIME */
+#endif                          /* HAVE_UPTIME */
 
   /* do process stats */
 
@@ -2062,7 +2066,7 @@ unix_uptime(dbref player __attribute__ ((__unused__)))
                 usage.ru_nvcsw, usage.ru_nivcsw);
   notify_format(player, "Signals:     %10ld", usage.ru_nsignals);
 #endif                          /* HAS_GETRUSAGE */
-
+#endif
 }
 
 static void
@@ -2099,6 +2103,7 @@ do_uptime(dbref player, int mortal)
 {
   char tbuf1[BUFFER_LEN];
   struct tm *when;
+  ldiv_t secs;
 
   when = localtime(&globals.first_start_time);
   strftime(tbuf1, sizeof tbuf1, T("     Up since %a %b %d %X %Z %Y"), when);
@@ -2124,49 +2129,54 @@ do_uptime(dbref player, int mortal)
   /* calculate times until various events */
   when = localtime(&options.dump_counter);
   strftime(tbuf1, sizeof tbuf1, "%X", when);
+  secs = ldiv((long) difftime(options.dump_counter, mudtime), 60);
   notify_format(player,
                 T
                 ("Time until next database save: %ld minutes %ld seconds, at %s"),
-                ((long) difftime(options.dump_counter, mudtime)) / 60,
-                ((long) difftime(options.dump_counter, mudtime)) % 60, tbuf1);
+                secs.quot, secs.rem, tbuf1);
 
   when = localtime(&options.dbck_counter);
   strftime(tbuf1, sizeof tbuf1, "%X", when);
+  secs = ldiv((long) difftime(options.dbck_counter, mudtime), 60);
   notify_format(player,
                 T
                 ("   Time until next dbck check: %ld minutes %ld seconds, at %s."),
-                ((long) difftime(options.dbck_counter, mudtime)) / 60,
-                ((long) difftime(options.dbck_counter, mudtime)) % 60, tbuf1);
+                secs.quot, secs.rem, tbuf1);
 
   when = localtime(&options.purge_counter);
   strftime(tbuf1, sizeof tbuf1, "%X", when);
+  secs = ldiv((long) difftime(options.purge_counter, mudtime), 60);
   notify_format(player,
                 T
                 ("        Time until next purge: %ld minutes %ld seconds, at %s."),
-                ((long) difftime(options.purge_counter, mudtime)) / 60,
-                ((long) difftime(options.purge_counter, mudtime)) % 60, tbuf1);
+                secs.quot, secs.rem, tbuf1);
 
   if (options.warn_interval) {
     when = localtime(&options.warn_counter);
     strftime(tbuf1, sizeof tbuf1, "%X", when);
+    secs = ldiv((long) difftime(options.warn_counter, mudtime), 60);
     notify_format(player,
                   T
                   ("    Time until next @warnings: %ld minutes %ld seconds, at %s."),
-                  ((long) difftime(options.warn_counter, mudtime)) / 60,
-                  ((long) difftime(options.warn_counter, mudtime)) % 60, tbuf1);
+                  secs.quot, secs.rem, tbuf1);
   }
 
-  notify_format(player,
-                T
-                ("PennMUSH Uptime: %ld days %ld hours %ld minutes %ld seconds"),
-                ((long) difftime(mudtime, globals.first_start_time)) / 86400,
-                ((long) difftime(mudtime, globals.first_start_time) % 86400) /
-                3600,
-                (((long) difftime(mudtime, globals.first_start_time) % 86400) %
-                 3600) / 60,
-                (((long) difftime(mudtime, globals.first_start_time) % 86400) %
-                 3600) % 60);
+  {
+    /* 86400 == seconds in 1 day. 3600 == seconds in 1 hour */
+    long days;
+    ldiv_t hours, mins;
 
+    secs = ldiv((long) difftime(mudtime, globals.first_start_time), 86400);
+    days = secs.quot;
+    hours = ldiv(secs.rem, 3600);
+    mins = ldiv(hours.rem, 60);
+
+
+    notify_format(player,
+                  T
+                  ("PennMUSH Uptime: %ld days %ld hours %ld minutes %ld seconds"),
+                  days, hours.quot, mins.quot, mins.rem);
+  }
   /* Mortals, go no further! */
   if (!Wizard(player) || mortal)
     return;
@@ -2187,44 +2197,83 @@ do_uptime(dbref player, int mortal)
 }
 
 
-/* Open a db file, which may be compressed, and return a file pointer */
-static FILE *
-db_open(const char *filename)
+/* Open a db file, which may be compressed, and return a file pointer. These probably should be moved into db.c or 
+ a new dbio.c */
+static PENNFILE *
+db_open(const char *fname)
 {
-  FILE *f;
+  PENNFILE *pf;
+  char filename[BUFFER_LEN];
+
+  snprintf(filename, sizeof filename, "%s%s", fname, options.compresssuff);
+
+  pf = mush_malloc(sizeof *pf, "pennfile");
+
+#ifdef HAVE_LIBZ
+  if (*options.uncompressprog && strcmp(options.uncompressprog, "gunzip") == 0) {
+    pf->type = PFT_GZFILE;
+    pf->handle.g = gzopen(filename, "rb");
+    if (!pf->handle.g) {
+      do_rawlog(LT_ERR, "Unable to open %s with libz: %s\n", filename, strerror(errno));
+      mush_free(pf, "pennfile");
+      longjmp(db_err, 1);
+    }
+    return pf;
+  }
+#endif
+
 #ifndef WIN32
-  if (options.uncompressprog && *options.uncompressprog) {
+  if (*options.uncompressprog) {
+    pf->type = PFT_PIPE;
     /* We do this because on some machines (SGI Irix, for example),
      * the popen will not return NULL if the mailfile isn't there.
      */
-    f = fopen(tprintf("%s%s", filename, options.compresssuff), "r");
-    if (f) {
-      fclose(f);
-      f =
+
+    if (access(filename, R_OK) == 0) {
+      pf->handle.f =
 #ifdef __LCC__
         (FILE *)
 #endif
-        popen(tprintf
-              ("%s < %s%s", options.uncompressprog, filename,
-               options.compresssuff), "r");
+        popen(tprintf("%s < '%s'", options.uncompressprog, filename), "r");
       /* Force the pipe to be fully buffered */
-      if (f)
-        setvbuf(f, NULL, _IOFBF, BUFSIZ);
+      if (pf->handle.f) {
+        setvbuf(pf->handle.f, NULL, _IOFBF, BUFSIZ);
+      } else
+        do_rawlog(LT_ERR, "Unable to run '%s < %s': %s",
+                  options.uncompressprog, filename, strerror(errno));
+    } else {
+      mush_free(pf, "pennfile");
+      longjmp(db_err, 1);
     }
   } else
 #endif                          /* WIN32 */
   {
-    f = fopen(filename, FOPEN_READ);
+    pf->type = PFT_FILE;
+    pf->handle.f = fopen(filename, FOPEN_READ);
+    if (!pf->handle.f)
+      do_rawlog(LT_ERR, "Unable to open %s: %s\n", filename, strerror(errno));
+#ifdef HAVE_POSIX_FADVISE
+    else if (pf->handle.f)
+      posix_fadvise(fileno(pf->handle.f), 0, 0, POSIX_FADV_SEQUENTIAL);
+#endif
+
   }
-  return f;
+  if (!pf->handle.f) {
+    mush_free(pf, "pennfile");
+    longjmp(db_err, 1);
+  }
+  return pf;
 }
 
 /* Open a file or pipe (if compressing) for writing */
-static FILE *
-db_open_write(const char *filename)
+static PENNFILE *
+db_open_write(const char *fname)
 {
-  FILE *f;
+  PENNFILE *pf;
   char workdir[BUFFER_LEN];
+  char filename[BUFFER_LEN];
+
+  snprintf(filename, sizeof filename, "%s%s", fname, options.compresssuff);
 
   /* Be safe in case our game directory was removed and restored,
    * in which case our inode is screwy
@@ -2245,43 +2294,53 @@ db_open_write(const char *filename)
             "getcwd failed during db_open_write, errno %d (%s)\n",
             errno, strerror(errno));
   }
+
+  pf = mush_malloc(sizeof *pf, "pennfile");
+
+#ifdef HAVE_LIBZ
+  if (*options.compressprog && strcmp(options.compressprog, "gzip") == 0) {
+    pf->type = PFT_GZFILE;
+    pf->handle.g = gzopen(filename, "wb");
+    if (!pf->handle.g) {
+      do_rawlog(LT_ERR, "Unable to open %s with libz: %s\n", filename, strerror(errno));
+      mush_free(pf, "pennfile");
+      longjmp(db_err, 1);
+    }
+    return pf;
+  }
+#endif
+
 #ifndef WIN32
-  if (options.compressprog && *options.compressprog) {
-    f =
+  if (*options.compressprog) {
+    pf->type = PFT_PIPE;
+    pf->handle.f =
 #ifdef __LCC__
       (FILE *)
 #endif
-      popen(tprintf
-            ("%s >%s%s", options.compressprog, filename,
-             options.compresssuff), "w");
+      popen(tprintf("%s > '%s'", options.compressprog, filename), "w");
     /* Force the pipe to be fully buffered */
-    if (f)
-      setvbuf(f, NULL, _IOFBF, BUFSIZ);
+    if (pf->handle.f) {
+      setvbuf(pf->handle.f, NULL, _IOFBF, BUFSIZ);
+    } else
+      do_rawlog(LT_ERR, "Unable to run '%s > %s': %s",
+                options.compressprog, filename, strerror(errno));
+
   } else
 #endif                          /* WIN32 */
   {
-    f = fopen(filename, FOPEN_WRITE);
+    pf->type = PFT_FILE;
+    pf->handle.f = fopen(filename, "wb");
+    if (!pf->handle.f)
+      do_rawlog(LT_ERR, "Unable to open %s: %s\n", filename, strerror(errno));
   }
-  if (!f)
+  if (!pf->handle.f) {
+    mush_free(pf, "pennfile");
     longjmp(db_err, 1);
-  return f;
-}
-
-
-/* Close a db file, which may really be a pipe */
-static void
-db_close(FILE * f)
-{
-#ifndef WIN32
-  if (options.compressprog && *options.compressprog) {
-    pclose(f);
-  } else
-#endif                          /* WIN32 */
-  {
-    fclose(f);
   }
+  return pf;
 }
 
+
 /** List various goodies.
  * \verbatim
  * This function implements @list.
@@ -2307,24 +2366,30 @@ do_list(dbref player, char *arg, int lc)
     do_list_flags("FLAG", player, "", lc, T("Flags"));
   else if (string_prefix("powers", arg))
     do_list_flags("POWER", player, "", lc, T("Powers"));
+  else if (string_prefix("locks", arg))
+    do_list_locks(player, NULL, lc, T("Locks"));
+  else if (string_prefix("allocations", arg))
+    do_list_allocations(player);
   else
     notify(player, T("I don't understand what you want to @list."));
 }
 
 extern HASHTAB htab_function;
 extern HASHTAB htab_user_function;
-extern HASHTAB htab_math;
-extern HASHTAB htab_tag;
 extern HASHTAB htab_player_list;
 extern HASHTAB htab_reserved_aliases;
 extern HASHTAB help_files;
 extern HASHTAB htab_objdata;
+extern HASHTAB htab_objdata_keys;
+extern HASHTAB htab_locks;
+extern HASHTAB local_options;
 extern StrTree atr_names;
 extern StrTree lock_names;
 extern StrTree object_names;
 extern PTAB ptab_command;
 extern PTAB ptab_attrib;
 extern PTAB ptab_flag;
+extern intmap *queue_map, *descs_by_fd;
 
 /** Reports stats on various in-memory data structures.
  * \param player the enactor.
@@ -2336,12 +2401,13 @@ do_list_memstats(dbref player)
   hash_stats_header(player);
   hash_stats(player, &htab_function, "Functions");
   hash_stats(player, &htab_user_function, "@Functions");
-  hash_stats(player, &htab_math, "Math funs");
-  hash_stats(player, &htab_tag, "HTML tags");
   hash_stats(player, &htab_player_list, "Players");
   hash_stats(player, &htab_reserved_aliases, "Aliases");
   hash_stats(player, &help_files, "HelpFiles");
   hash_stats(player, &htab_objdata, "ObjData");
+  hash_stats(player, &htab_objdata_keys, "ObjDataKeys");
+  hash_stats(player, &htab_locks, "@locks");
+  hash_stats(player, &local_options, "ConfigOpts");
   notify(player, "Prefix Trees:");
   ptab_stats_header(player);
   ptab_stats(player, &ptab_attrib, "AttrPerms");
@@ -2352,6 +2418,11 @@ do_list_memstats(dbref player)
   st_stats(player, &atr_names, "AttrNames");
   st_stats(player, &object_names, "ObjNames");
   st_stats(player, &lock_names, "LockNames");
+  notify(player, "Integer Maps:");
+  im_stats_header(player);
+  im_stats(player, queue_map, "Queue IDs");
+  im_stats(player, descs_by_fd, "Connections");
+
 #if (COMPRESSION_TYPE >= 3) && defined(COMP_STATS)
   if (Wizard(player)) {
     long items, used, total_comp, total_uncomp;
@@ -2384,9 +2455,6 @@ do_list_memstats(dbref player)
              "          " "(Compression ratio improves with larger database)");
   }
 #endif
-
-  if (God(player))
-    list_mem_check(player);
 }
 
 static char *
@@ -2423,9 +2491,14 @@ fail_commands(dbref player)
 static void
 errdb_grow(void)
 {
+  dbref *newerrdb;
   if (errdbsize >= 50)
     return;                     /* That's it, no more, forget it */
-  errdbsize++;
-  errdblist = realloc(errdblist, errdbsize * sizeof(dbref));
-  errdbtail = errdblist + errdbsize - 1;
+  newerrdb = mush_realloc(errdblist, (errdbsize + 1) * sizeof(dbref),
+                          "errdblist");
+  if (newerrdb) {
+    errdblist = newerrdb;
+    errdbtail = errdblist + errdbsize;
+    errdbsize += 1;
+  }
 }
