@@ -43,46 +43,45 @@
 #include "match.h"
 #include "log.h"
 #include "flags.h"
-#include "dbdefs.h"
 #include "mymalloc.h"
 #include "strtree.h"
 #include "privtab.h"
 #include "parse.h"
+#include "htab.h"
 #include "confmagic.h"
 
 
 /* If any lock_type ever contains the character '|', reading in locks
  * from the db will break.
  */
-const lock_type Basic_Lock = "Basic";     /**< Name of basic lock */
-const lock_type Enter_Lock = "Enter";     /**< Name of enter lock */
-const lock_type Use_Lock = "Use";         /**< Name of use lock */
-const lock_type Zone_Lock = "Zone";       /**< Name of zone lock */
-const lock_type Page_Lock = "Page";       /**< Name of page lock */
-const lock_type Tport_Lock = "Teleport";  /**< Name of teleport lock */
-const lock_type Speech_Lock = "Speech";   /**< Name of speech lock */
-const lock_type Listen_Lock = "Listen";   /**< Name of listen lock */
-const lock_type Command_Lock = "Command"; /**< Name of command lock */
-const lock_type Parent_Lock = "Parent";   /**< Name of parent lock */
-const lock_type Link_Lock = "Link";       /**< Name of link lock */
-const lock_type Leave_Lock = "Leave";     /**< Name of leave lock */
-const lock_type Drop_Lock = "Drop";       /**< Name of drop lock */
-const lock_type Give_Lock = "Give";       /**< Name of give lock */
-const lock_type Mail_Lock = "Mail";       /**< Name of mail lock */
-const lock_type Follow_Lock = "Follow";   /**< Name of follow lock */
-const lock_type Examine_Lock = "Examine"; /**< Name of examine lock */
-const lock_type Chzone_Lock = "Chzone";   /**< Name of chzone lock */
-const lock_type Forward_Lock = "Forward"; /**< Name of forward lock */
-const lock_type Control_Lock = "Control"; /**< Name of control lock */
-const lock_type Dropto_Lock = "Dropto";   /**< Name of dropto lock */
-const lock_type Destroy_Lock = "Destroy"; /**< Name of destroy lock */
-const lock_type Interact_Lock = "Interact"; /**< Name of interaction lock */
-const lock_type MailForward_Lock = "MailForward"; /**< Name of mailforward lock */
-const lock_type Take_Lock = "Take"; /**< Name of take lock */
-/* Define new lock types here. */
+lock_type Basic_Lock = "Basic";     /**< Name of basic lock */
+lock_type Enter_Lock = "Enter";     /**< Name of enter lock */
+lock_type Use_Lock = "Use";         /**< Name of use lock */
+lock_type Zone_Lock = "Zone";       /**< Name of zone lock */
+lock_type Page_Lock = "Page";       /**< Name of page lock */
+lock_type Tport_Lock = "Teleport";  /**< Name of teleport lock */
+lock_type Speech_Lock = "Speech";   /**< Name of speech lock */
+lock_type Listen_Lock = "Listen";   /**< Name of listen lock */
+lock_type Command_Lock = "Command"; /**< Name of command lock */
+lock_type Parent_Lock = "Parent";   /**< Name of parent lock */
+lock_type Link_Lock = "Link";       /**< Name of link lock */
+lock_type Leave_Lock = "Leave";     /**< Name of leave lock */
+lock_type Drop_Lock = "Drop";       /**< Name of drop lock */
+lock_type Give_Lock = "Give";       /**< Name of give lock */
+lock_type Mail_Lock = "Mail";       /**< Name of mail lock */
+lock_type Follow_Lock = "Follow";   /**< Name of follow lock */
+lock_type Examine_Lock = "Examine"; /**< Name of examine lock */
+lock_type Chzone_Lock = "Chzone";   /**< Name of chzone lock */
+lock_type Forward_Lock = "Forward"; /**< Name of forward lock */
+lock_type Control_Lock = "Control"; /**< Name of control lock */
+lock_type Dropto_Lock = "Dropto";   /**< Name of dropto lock */
+lock_type Destroy_Lock = "Destroy"; /**< Name of destroy lock */
+lock_type Interact_Lock = "Interact"; /**< Name of interaction lock */
+lock_type MailForward_Lock = "MailForward"; /**< Name of mailforward lock */
+lock_type Take_Lock = "Take"; /**< Name of take lock */
 
-/** Table of lock names and permissions */
-const lock_list lock_types[] = {
+ /** Table of lock names and permissions */
+lock_list lock_types[] = {
   {"Basic", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
   {"Enter", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
   {"Use", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
@@ -108,13 +107,13 @@ const lock_list lock_types[] = {
   {"Interact", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
   {"MailForward", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
   {"Take", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
-  /* Add new lock types just before this line. */
   {NULL, TRUE_BOOLEXP, GOD, 0, NULL}
 };
 
+HASHTAB htab_locks;
 
-/** Table of base attributes associated with success and failure of
- * locks. These are the historical ones; we automatically generate
+ /** Table of base attributes associated with success and failure of
+  * locks. These are the historical ones; we automatically generate
  * such attribute names for those that aren't in this table using
  * <lock>_LOCK`<message>
  */
@@ -145,12 +144,8 @@ static int delete_lock(dbref player, dbref thing, lock_type type);
 static int can_write_lock(dbref player, dbref thing, lock_list *lock);
 static lock_list *getlockstruct_noparent(dbref thing, lock_type type);
 
-/** Number of locks to store in a page, assuming 4096 byte pages */
-#define LOCKS_PER_PAGE 200
-
-static lock_list *free_list = NULL;
-
-static lock_list *next_free_lock(void);
+slab *lock_slab = NULL;
+static lock_list *next_free_lock(const void *hint);
 static void free_lock(lock_list *ll);
 
 extern int unparsing_boolexp;
@@ -167,7 +162,8 @@ list_locks(char *buff, char **bp, const char *name)
   int first = 1;
   const lock_list *ptr;
   rp = rbuff;
-  for (ptr = lock_types; ptr->type != NULL; ptr++) {
+  for (ptr = hash_firstentry(&htab_locks);
+       ptr; ptr = hash_nextentry(&htab_locks)) {
     /* Skip those that don't match */
     if (name && !string_prefix(ptr->type, name))
       continue;
@@ -259,22 +255,50 @@ lock_flags_long(lock_list *ll)
 
 
 static int
-string_to_lockflag(dbref player, char const *p)
+string_to_lockflag(dbref player, char const *p, privbits *flag)
 {
-  int f;
+  privbits f;
   f = string_to_privs(lock_privs, p, 0);
   if (!f)
     return -1;
   if (!See_All(player) && (f & LF_WIZARD))
     return -1;
-  return f;
+  *flag = f;
+  return 0;
 }
 
 /** Initialize the lock strtree. */
 void
 init_locks(void)
 {
+  lock_list *ll;
   st_init(&lock_names);
+
+  hashinit(&htab_locks, 25);
+
+  for (ll = lock_types; ll->type && *ll->type; ll++)
+    hashadd(strupper(ll->type), ll, &htab_locks);
+
+  local_locks();
+}
+
+
+/** Add a new lock to the table.
+ * \param name The name of the lock
+ * \param flags The default flags.
+ */
+void
+define_lock(lock_type name, privbits flags)
+{
+  lock_list *newlock;
+
+  newlock = mush_malloc(sizeof *newlock, "lock");
+  newlock->type = mush_strdup(strupper(name), "lock.name");
+  newlock->flags = flags;
+  newlock->creator = GOD;
+  newlock->key = TRUE_BOOLEXP;
+  newlock->next = NULL;
+  hashadd((char *) newlock->type, newlock, &htab_locks);
 }
 
 static int
@@ -297,53 +321,19 @@ can_write_lock(dbref player, dbref thing, lock_list *lock)
 
 
 static lock_list *
-next_free_lock(void)
+next_free_lock(const void *hint)
 {
-  lock_list *ll;
-
-  if (!free_list) {
-    size_t n;
-
-    ll = mush_malloc(sizeof(lock_list) * LOCKS_PER_PAGE, "lock_page");
-
-    if (!ll)
-      mush_panic("Unable to allocate memory for locks!");
-
-    for (n = 0; n < LOCKS_PER_PAGE - 1; n++) {
-      ll[n].type = NULL;
-      ll[n].key = TRUE_BOOLEXP;
-      ll[n].creator = NOTHING;
-      ll[n].flags = 0;
-      ll[n].next = &ll[n + 1];
-    }
-
-    ll[n].next = NULL;
-    ll[n].type = NULL;
-    ll[n].key = TRUE_BOOLEXP;
-    ll[n].creator = NOTHING;
-    ll[n].flags = 0;
-
-    free_list = &ll[0];
+  if (lock_slab == NULL) {
+    lock_slab = slab_create("locks", sizeof(lock_list));
+    slab_set_opt(lock_slab, SLAB_ALLOC_BEST_FIT, 1);
   }
-
-  ll = free_list;
-  free_list = ll->next;
-
-  ll->type = NULL;
-  ll->key = TRUE_BOOLEXP;
-
-  return ll;
+  return slab_malloc(lock_slab, hint);
 }
 
 static void
 free_lock(lock_list *ll)
 {
-  ll->type = NULL;
-  ll->key = TRUE_BOOLEXP;
-  ll->creator = NOTHING;
-  ll->flags = 0;
-  ll->next = free_list;
-  free_list = ll;
+  slab_free(lock_slab, ll);
 }
 
 /** Given a lock type, find a lock, possibly checking parents.
@@ -431,13 +421,12 @@ getlockstruct_noparent(dbref thing, lock_type type)
 lock_type
 match_lock(lock_type type)
 {
-  int i;
-  for (i = 0; lock_types[i].type != NULL; i++) {
-    if (strcasecmp(lock_types[i].type, type) == 0) {
-      return lock_types[i].type;
-    }
-  }
-  return NULL;
+  lock_list *ll;
+  ll = hashfind(strupper(type), &htab_locks);
+  if (ll)
+    return ll->type;
+  else
+    return NULL;
 }
 
 /** Return the proper entry from lock_types, or NULL.
@@ -447,14 +436,7 @@ match_lock(lock_type type)
 const lock_list *
 get_lockproto(lock_type type)
 {
-  const lock_list *ll;
-
-  for (ll = lock_types; ll->type; ll++)
-    if (strcasecmp(type, ll->type) == 0)
-      return ll;
-
-  return NULL;
-
+  return hashfind(type, &htab_locks);
 }
 
 /** Add a lock to an object (primitive).
@@ -472,7 +454,7 @@ get_lockproto(lock_type type)
  * \retval 1 success.
  */
 int
-add_lock(dbref player, dbref thing, lock_type type, boolexp key, int flags)
+add_lock(dbref player, dbref thing, lock_type type, boolexp key, privbits flags)
 {
   lock_list *ll, **t;
   lock_type real_type = type;
@@ -492,10 +474,10 @@ add_lock(dbref player, dbref thing, lock_type type, boolexp key, int flags)
     free_boolexp(ll->key);
     ll->key = key;
     ll->creator = player;
-    if (flags != -1)
+    if (flags != LF_DEFAULT)
       ll->flags = flags;
   } else {
-    ll = next_free_lock();
+    ll = next_free_lock(Locks(thing));
     if (!ll) {
       /* Oh, this sucks */
       do_log(LT_ERR, 0, 0, "Unable to malloc memory for lock_list!");
@@ -504,7 +486,7 @@ add_lock(dbref player, dbref thing, lock_type type, boolexp key, int flags)
       ll->type = real_type;
       ll->key = key;
       ll->creator = player;
-      if (flags == -1) {
+      if (flags == LF_DEFAULT) {
         const lock_list *l2 = get_lockproto(real_type);
         if (l2)
           ll->flags = l2->flags;
@@ -544,7 +526,8 @@ add_lock(dbref player, dbref thing, lock_type type, boolexp key, int flags)
  * \retval 0 failure.
  */
 int
-add_lock_raw(dbref player, dbref thing, lock_type type, boolexp key, int flags)
+add_lock_raw(dbref player, dbref thing, lock_type type, boolexp key,
+             privbits flags)
 {
   lock_list *ll, **t;
   lock_type real_type = type;
@@ -553,7 +536,7 @@ add_lock_raw(dbref player, dbref thing, lock_type type, boolexp key, int flags)
     return 0;
   }
 
-  ll = next_free_lock();
+  ll = next_free_lock(Locks(thing));
   if (!ll) {
     /* Oh, this sucks */
     do_log(LT_ERR, 0, 0, "Unable to malloc memory for lock_list!");
@@ -562,7 +545,7 @@ add_lock_raw(dbref player, dbref thing, lock_type type, boolexp key, int flags)
     ll->type = real_type;
     ll->key = key;
     ll->creator = player;
-    if (flags == -1) {
+    if (flags == LF_DEFAULT) {
       const lock_list *l2 = get_lockproto(real_type);
       if (l2)
         ll->flags = l2->flags;
@@ -643,7 +626,6 @@ free_locks(lock_list *ll)
  * As an extra check, we don't allow '|' in lock names because it
  * will confuse our db-reading routines.
  *
- * Might destructively modify name.
  *
  * \param player the enactor, for notification.
  * \param thing object on which to check the lock.
@@ -654,7 +636,9 @@ static lock_type
 check_lock_type(dbref player, dbref thing, lock_type name)
 {
   lock_type ll;
-  char *sp;
+  char user_name[BUFFER_LEN];
+  char *colon;
+
   /* Special-case for basic locks. */
   if (!name || !*name)
     return Basic_Lock;
@@ -668,27 +652,26 @@ check_lock_type(dbref player, dbref thing, lock_type name)
   if (getlock(thing, name) != TRUE_BOOLEXP)
     return name;
   /* Check to see if it's a well-formed user-defined lock. */
-  sp = strchr(name, ':');
-  if (!sp) {
-    notify(player, T("Unknown lock type."));
-    return NULL;
-  }
-  *sp++ = '\0';
 
-  if (!string_prefix("User", name)) {
+  if (!string_prefix(name, "User:")) {
     notify(player, T("Unknown lock type."));
     return NULL;
   }
-  if (strchr(sp, '|')) {
+  if (strchr(name, '|')) {
     notify(player, T("The character \'|\' may not be used in lock names."));
     return NULL;
   }
-  if (!good_atr_name(sp)) {
+  mush_strncpy(user_name, name, BUFFER_LEN);
+  colon = strchr(user_name, ':');
+  if (colon)                    /* Should always be true */
+    *colon = '\0';
+
+  if (!good_atr_name(user_name)) {
     notify(player, T("That is not a valid lock name."));
     return NULL;
   }
 
-  return sp;
+  return strchr(name, ':') + 1;
 }
 
 /** Unlock a lock (user interface).
@@ -703,12 +686,10 @@ void
 do_unlock(dbref player, const char *name, lock_type type)
 {
   dbref thing;
-  char *sp;
   lock_type real_type;
 
   /* check for '@unlock <object>/<atr>'  */
-  sp = strchr(name, '/');
-  if (sp) {
+  if (strchr(name, '/')) {
     do_atrlock(player, name, "off");
     return;
   }
@@ -745,11 +726,9 @@ do_lock(dbref player, const char *name, const char *keyname, lock_type type)
   lock_type real_type;
   dbref thing;
   boolexp key;
-  char *sp;
 
   /* check for '@lock <object>/<atr>'  */
-  sp = strchr(name, '/');
-  if (sp) {
+  if (strchr(name, '/')) {
     do_atrlock(player, name, "on");
     return;
   }
@@ -784,7 +763,7 @@ do_lock(dbref player, const char *name, const char *keyname, lock_type type)
   } else {
     if ((real_type = check_lock_type(player, thing, type)) != NULL) {
       /* everything ok, do it */
-      if (add_lock(player, thing, real_type, key, -1)) {
+      if (add_lock(player, thing, real_type, key, LF_DEFAULT)) {
         if (!AreQuiet(player, thing))
           notify_format(player, T("%s(%s) - %s locked."), Name(thing),
                         unparse_dbref(thing), real_type);
@@ -897,7 +876,7 @@ fail_lock(dbref player, dbref thing, lock_type ltype, const char *def,
   /* Find the lock's failure attribute, if it's there */
   for (lm = lock_msgs; lm->type; lm++) {
     if (!strcmp(lm->type, ltype))
-      break;
+       break;
   }
   if (lm->type) {
     strcpy(atr, lm->failbase);
@@ -934,7 +913,7 @@ fail_lock(dbref player, dbref thing, lock_type ltype, const char *def,
  * \retval (non-zero) lock is visual.
  * \retval 0 lock is not visual.
  */
-int
+bool
 lock_visual(dbref thing, lock_type ltype)
 {
   lock_list *l = getlockstruct(thing, ltype);
@@ -958,8 +937,8 @@ do_lset(dbref player, char *what, char *flags)
   dbref thing;
   lock_list *l;
   char *lname;
-  int flag;
-  int unset = 0;
+  privbits flag;
+  bool unset = 0;
 
   if ((lname = strchr(what, '/')) == NULL) {
     notify(player, T("No lock name given."));
@@ -975,7 +954,7 @@ do_lset(dbref player, char *what, char *flags)
     flags++;
   }
 
-  if ((flag = string_to_lockflag(player, flags)) < 0) {
+  if (string_to_lockflag(player, flags, &flag) < 0) {
     notify(player, T("Unrecognized lock flag."));
     return;
   }
@@ -1014,7 +993,8 @@ check_zone_lock(dbref player, dbref zone, int noisy)
 {
   boolexp key = getlock(zone, Zone_Lock);
   if (key == TRUE_BOOLEXP) {
-    add_lock(GOD, zone, Zone_Lock, parse_boolexp(zone, "=me", Zone_Lock), -1);
+    add_lock(GOD, zone, Zone_Lock, parse_boolexp(zone, "=me", Zone_Lock),
+             LF_DEFAULT);
     if (noisy)
       notify_format(player,
                     T
