@@ -8,7 +8,7 @@
 
 #include "config.h"
 #include "copyrite.h"
-
+#define _GNU_SOURCE
 #include <string.h>
 #include <ctype.h>
 
@@ -27,10 +27,8 @@
 #include "command.h"
 #include "parse.h"
 #include "privtab.h"
-#include "log.h"
-#include "case.h"
 #include "confmagic.h"
-
+#include "log.h"
 
 static void look_exits(dbref player, dbref loc, const char *exit_name);
 static void look_contents(dbref player, dbref loc, const char *contents_name);
@@ -53,9 +51,8 @@ void decompile_locks(dbref player, dbref thing, const char *name,
                      int skipdef, const char *prefix);
 static char *parent_chain(dbref player, dbref thing);
 
-static void insert_spaces(int count, int dospace, char *buff, char **bp);
-
 extern PRIV attr_privs_view[];
+extern int real_decompose_str(char *str, char *buff, char **bp);
 
 static void
 look_exits(dbref player, dbref loc, const char *exit_name)
@@ -181,7 +178,7 @@ look_exits(dbref player, dbref loc, const char *exit_name)
       if (Transparented(loc) && !(Opaque(thing))) {
         if (SUPPORT_PUEBLO && !texits) {
           texits = 1;
-          notify_noenter_by(loc, player, tprintf("%cUL%c", TAG_START, TAG_END));
+          notify_noenter_by(loc, player, open_tag("UL"));
         }
         s1 = tbuf1;
         safe_tag("LI", tbuf1, &s1);
@@ -341,11 +338,11 @@ look_helper_veiled(dbref player, dbref thing __attribute__ ((__unused__)),
       if (GoodObject(parent))
         notify_format(player,
                       "%s#%d/%s [#%d%s]%s is veiled", ANSI_HILITE, parent,
-                      AL_NAME(atr), Owner(AL_CREATOR(atr)), fbuf, ANSI_NORMAL);
+                      AL_NAME(atr), Owner(AL_CREATOR(atr)), fbuf, ANSI_END);
       else
         notify_format(player,
                       "%s%s [#%d%s]%s is veiled", ANSI_HILITE, AL_NAME(atr),
-                      Owner(AL_CREATOR(atr)), fbuf, ANSI_NORMAL);
+                      Owner(AL_CREATOR(atr)), fbuf, ANSI_END);
     } else {
       if (GoodObject(parent))
         notify_format(player,
@@ -362,12 +359,11 @@ look_helper_veiled(dbref player, dbref thing __attribute__ ((__unused__)),
       if (GoodObject(parent))
         notify_format(player,
                       "%s#%d/%s [#%d%s]:%s %s", ANSI_HILITE, parent,
-                      AL_NAME(atr), Owner(AL_CREATOR(atr)), fbuf, ANSI_NORMAL,
-                      r);
+                      AL_NAME(atr), Owner(AL_CREATOR(atr)), fbuf, ANSI_END, r);
       else
         notify_format(player,
                       "%s%s [#%d%s]:%s %s", ANSI_HILITE, AL_NAME(atr),
-                      Owner(AL_CREATOR(atr)), fbuf, ANSI_NORMAL, r);
+                      Owner(AL_CREATOR(atr)), fbuf, ANSI_END, r);
     } else {
       if (GoodObject(parent))
         notify_format(player, "#%d/%s [#%d%s]: %s", parent, AL_NAME(atr),
@@ -401,11 +397,11 @@ look_helper(dbref player, dbref thing __attribute__ ((__unused__)),
     if (GoodObject(parent))
       notify_format(player,
                     "%s#%d/%s [#%d%s]:%s %s", ANSI_HILITE, parent,
-                    AL_NAME(atr), Owner(AL_CREATOR(atr)), fbuf, ANSI_NORMAL, r);
+                    AL_NAME(atr), Owner(AL_CREATOR(atr)), fbuf, ANSI_END, r);
     else
       notify_format(player,
                     "%s%s [#%d%s]:%s %s", ANSI_HILITE, AL_NAME(atr),
-                    Owner(AL_CREATOR(atr)), fbuf, ANSI_NORMAL, r);
+                    Owner(AL_CREATOR(atr)), fbuf, ANSI_END, r);
   } else {
     if (GoodObject(parent))
       notify_format(player, "#%d/%s [#%d%s]: %s", parent, AL_NAME(atr),
@@ -673,8 +669,12 @@ do_look_at(dbref player, const char *name, int key)
     /* look at a thing in location */
     if ((thing = match_result(player, name, NOTYPE, MAT_NEARBY)) == NOTHING) {
       dbref box;
-      const char *boxname = name;
-      box = parse_match_possessor(player, &name);
+      const char *boxname;
+      char objnamebuf[BUFFER_LEN], *objname;
+      boxname = name;
+      strcpy(objnamebuf, name);
+      objname = objnamebuf;
+      box = parse_match_possessor(player, &objname);
       if (box == NOTHING) {
         notify(player, T("I don't see that here."));
         return;
@@ -682,7 +682,7 @@ do_look_at(dbref player, const char *name, int key)
         notify_format(player, T("I can't tell which %s."), boxname);
         return;
       }
-      thing = match_result(box, name, NOTYPE, MAT_POSSESSION);
+      thing = match_result(box, objname, NOTYPE, MAT_POSSESSION);
       if (thing == NOTHING) {
         notify(player, T("I don't see that here."));
         return;
@@ -721,7 +721,7 @@ do_look_at(dbref player, const char *name, int key)
   case TYPE_PLAYER:
     look_simple(player, thing);
     if (!(Opaque(thing)))
-      look_contents(player, thing, "Carrying:");
+      look_contents(player, thing, T("Carrying:"));
     break;
   default:
     look_simple(player, thing);
@@ -738,7 +738,7 @@ do_look_at(dbref player, const char *name, int key)
  * \param parent if 1, include parent attributes
  */
 void
-do_examine(dbref player, const char *name, enum exam_type flag, int all,
+do_examine(dbref player, const char *xname, enum exam_type flag, int all,
            int parent)
 {
   dbref thing;
@@ -747,6 +747,7 @@ do_examine(dbref player, const char *name, enum exam_type flag, int all,
   dbref content;
   dbref exit_dbref;
   const char *real_name = NULL;
+  char *name = NULL;
   char *attrib_name = NULL;
   char *tp;
   char *tbuf;
@@ -754,12 +755,11 @@ do_examine(dbref player, const char *name, enum exam_type flag, int all,
   int listed = 0;
   PUEBLOBUFF;
 
-  if (*name == '\0') {
+  if (*xname == '\0') {
     if ((thing = Location(player)) == NOTHING)
       return;
-    attrib_name = NULL;
   } else {
-
+    name = mush_strdup(xname, "de.string");
     if ((attrib_name = strchr(name, '/')) != NULL) {
       *attrib_name = '\0';
       attrib_name++;
@@ -768,17 +768,23 @@ do_examine(dbref player, const char *name, enum exam_type flag, int all,
     /* look it up */
     if ((thing =
          noisy_match_result(player, real_name, NOTYPE,
-                            MAT_EVERYTHING)) == NOTHING)
+                            MAT_EVERYTHING)) == NOTHING) {
+      mush_free(name, "de.string");
       return;
+    }
   }
   /*  can't examine destructed objects  */
   if (IsGarbage(thing)) {
     notify(player, T("Garbage is garbage."));
+    if (name)
+      mush_free(name, "de.string");
     return;
   }
   /*  only look at some of the attributes */
   if (attrib_name && *attrib_name) {
     look_atrs(player, thing, attrib_name, all, 0, parent);
+    if (name)
+      mush_free(name, "de.string");
     return;
   }
   if (flag == EXAM_MORTAL) {
@@ -798,7 +804,9 @@ do_examine(dbref player, const char *name, enum exam_type flag, int all,
     safe_str(object_header(player, Owner(thing)), tbuf, &tp);
     *tp = '\0';
     notify(player, tbuf);
-    mush_free((Malloc_t) tbuf, "string");
+    mush_free(tbuf, "string");
+    if (name)
+      mush_free(name, "de.string");
     return;
   }
   if (ok) {
@@ -884,7 +892,9 @@ do_examine(dbref player, const char *name, enum exam_type flag, int all,
     safe_str(object_header(player, Owner(thing)), tbuf, &tp);
     *tp = '\0';
     notify(player, tbuf);
-    mush_free((Malloc_t) tbuf, "string");
+    mush_free(tbuf, "string");
+    if (name)
+      mush_free(name, "de.string");
     return;
   }
   switch (Typeof(thing)) {
@@ -955,7 +965,9 @@ do_examine(dbref player, const char *name, enum exam_type flag, int all,
     /* do nothing */
     break;
   }
-  mush_free((Malloc_t) tbuf, "string");
+  mush_free(tbuf, "string");
+  if (name)
+    mush_free(name, "de.string");
 }
 
 /** The score command: check a player's money.
@@ -1173,12 +1185,12 @@ do_sweep(dbref player, const char *arg1)
       } else {
         if (Hearer(here) || Listener(here)) {
           if (Connected(here))
-            notify_format(player, "%s [speech]. (connected)", Name(here));
+            notify_format(player, T("%s [speech]. (connected)"), Name(here));
           else
-            notify_format(player, "%s [speech].", Name(here));
+            notify_format(player, T("%s [speech]."), Name(here));
         }
         if (Commer(here))
-          notify_format(player, "%s [commands].", Name(here));
+          notify_format(player, T("%s [commands]."), Name(here));
       }
     }
   }
@@ -1189,9 +1201,9 @@ do_sweep(dbref player, const char *arg1)
       for (here = Exits(Location(player)); here != NOTHING; here = Next(here)) {
         if (Audible(here)) {
           strcpy(tbuf1, Name(here));
-          for (p = tbuf1; *p && (*p != ';'); p++) ;
+          p = seek_char(tbuf1, ';');
           *p = '\0';
-          notify_format(player, "%s [broadcasting].", tbuf1);
+          notify_format(player, T("%s [broadcasting]."), tbuf1);
         }
       }
     }
@@ -1213,12 +1225,12 @@ do_sweep(dbref player, const char *arg1)
       } else {
         if (Hearer(here) || Listener(here)) {
           if (Connected(here))
-            notify_format(player, "%s [speech]. (connected)", Name(here));
+            notify_format(player, T("%s [speech]. (connected)"), Name(here));
           else
-            notify_format(player, "%s [speech].", Name(here));
+            notify_format(player, T("%s [speech]."), Name(here));
         }
         if (Commer(here))
-          notify_format(player, "%s [commands].", Name(here));
+          notify_format(player, T("%s [commands]."), Name(here));
       }
     }
   }
@@ -1336,7 +1348,7 @@ do_entrances(dbref player, const char *where, char *argv[], enum ent_type val)
         if (exd) {
           if (Location(counter) == place) {
             notify_format(player,
-                          "%s(#%d) [from: %s(#%d)]", Name(counter),
+                          T("%s(#%d) [from: %s(#%d)]"), Name(counter),
                           counter, Name(Source(counter)), Source(counter));
             exc++;
           }
@@ -1345,7 +1357,8 @@ do_entrances(dbref player, const char *where, char *argv[], enum ent_type val)
       case TYPE_ROOM:
         if (rd) {
           if (Location(counter) == place) {
-            notify_format(player, "%s(#%d) [dropto]", Name(counter), counter);
+            notify_format(player, T("%s(#%d) [dropto]"), Name(counter),
+                          counter);
             rc++;
           }
         }
@@ -1353,7 +1366,7 @@ do_entrances(dbref player, const char *where, char *argv[], enum ent_type val)
       case TYPE_THING:
         if (td) {
           if (Home(counter) == place) {
-            notify_format(player, "%s(#%d) [home]", Name(counter), counter);
+            notify_format(player, T("%s(#%d) [home]"), Name(counter), counter);
             tc++;
           }
         }
@@ -1361,7 +1374,7 @@ do_entrances(dbref player, const char *where, char *argv[], enum ent_type val)
       case TYPE_PLAYER:
         if (pd) {
           if (Home(counter) == place) {
-            notify_format(player, "%s(#%d) [home]", Name(counter), counter);
+            notify_format(player, T("%s(#%d) [home]"), Name(counter), counter);
             pc++;
           }
         }
@@ -1376,7 +1389,8 @@ do_entrances(dbref player, const char *where, char *argv[], enum ent_type val)
   } else {
     notify(player, T("----------  Entrances Done  ----------"));
     notify_format(player,
-                  "Totals: Rooms...%d  Exits...%d  Objects...%d  Players...%d",
+                  T
+                  ("Totals: Rooms...%d  Exits...%d  Objects...%d  Players...%d"),
                   rc, exc, tc, pc);
     return;
   }
@@ -1389,258 +1403,15 @@ struct dh_args {
   int skipdef;          /**< Skip default flags on attributes if true */
 };
 
-extern char escaped_chars[UCHAR_MAX + 1];
-
-static void
-insert_spaces(int count, int dospace, char *buff, char **bp)
-{
-  if (count) {
-    if (count >= 5) {
-      safe_str("[space(", buff, bp);
-      safe_number(count, buff, bp);
-      safe_str(")]", buff, bp);
-    } else if (count == 1) {
-      if (dospace) {
-        safe_str("%b", buff, bp);
-      } else {
-        safe_chr(' ', buff, bp);
-      }
-    } else {
-      /* Take care of the final %b */
-      count--;
-      if (dospace) {
-        safe_str("%b", buff, bp);
-        count--;
-      }
-      while (count) {
-        safe_chr(' ', buff, bp);
-        count--;
-        if (count) {
-          count--;
-          safe_str("%b", buff, bp);
-        }
-      }
-      safe_str("%b", buff, bp);
-    }
-  }
-}
-
 char *
 decompose_str(char *what)
 {
   static char value[BUFFER_LEN];
-  char *ptr, *s, *codestart;
-  char ansi_letter = '\0';
-  int len;
-  int dospace;
-  int spaces;
-  int flag_depth, ansi_depth;
-  int digits;
+  char *vp = value;
 
-  len = strlen(what);
-  flag_depth = 0;
-  ansi_depth = 0;
+  real_decompose_str(what, value, &vp);
+  *vp = '\0';
 
-  /* Go through the string, escaping characters and
-   * turning every other space into %b. */
-
-  s = value;
-  ptr = what;
-#ifdef NEVER
-  /* Put a \ at the beginning if it won't already be put there,
-   * unless it's a space, which would require %b, %r, or %t anyway */
-  if (!escaped_chars[(unsigned int) *what] && !isspace((unsigned char) *what)) {
-    safe_chr('\\', value, &s);
-  }
-#endif
-
-  dospace = 1;
-  spaces = 0;
-  for (; *ptr; ptr++) {
-    switch (*ptr) {
-    case ' ':
-      spaces++;
-      break;
-    case '\n':
-      insert_spaces(spaces, dospace, value, &s);
-      spaces = 0;
-      dospace = 0;
-      safe_str("%r", value, &s);
-      break;
-    case '\t':
-      insert_spaces(spaces, dospace, value, &s);
-      spaces = 0;
-      dospace = 0;
-      safe_str("%t", value, &s);
-      break;
-    case ESC_CHAR:
-      insert_spaces(spaces, dospace, value, &s);
-      spaces = 0;
-      ptr++;
-      if (!*ptr) {
-        ptr--;
-        break;
-      }
-      /* Check if this is any sort of useful code. */
-      if (*ptr == '[' && *(ptr + 1) && *(ptr + 2)) {
-        codestart = ptr;        /* Save the address of the escape code. */
-        ptr++;
-        digits = 0;             /* Digit count is zero. */
-        /* The following code works in this way:
-         * 1) If a character is a ;, we are allowed to count 2 more digits
-         * 2) If the digits count is 3, break out of the "ansi" code.
-         * 3) If the character is not a number or ;, break out.
-         * 4) If an 'm' is encountered, the for-loop exits.
-         * The only non-breaking exit occurs when the code ends with "m".
-         * Otherwise, we break out with the ptr pointing to the end of
-         * the invalid code, causing decompose() to ignore it entirely.
-         */
-        for (; *ptr && (*ptr != 'm'); ptr++) {
-          if (*ptr == ';') {
-            if (digits == 0) {  /* No double-;s are allowed. */
-              digits = 3;
-              break;
-            }
-            digits = 0;
-          } else if (digits >= 2) {
-            digits = 3;         /* If we encounter a 3-number code, break out. */
-            break;
-          } else if (isdigit((unsigned char) *ptr)) {
-            digits++;           /* Count the numbers we encounter. */
-          } else {
-            digits = 3;
-            break;
-          }
-        }
-
-        /* 3 is the break-code. 0 means there's no ANSI at all! */
-        if (!*ptr || digits == 3 || digits == 0) {
-          break;
-        }
-
-        /* It IS an ansi code! It ends in "m" anyway.
-         * Set ptr to point to the first digit in the code. We are
-         * promised at this point that ptr+1 is not NUL. 
-         */
-        ptr = codestart + 1;
-
-        /* Check if the first part of the code is two-digit (color) */
-        if (*(ptr + 1) >= '0' && *(ptr + 1) <= '7') {
-          if (flag_depth < ansi_depth) {
-            safe_str(")]", value, &s);
-            ansi_depth--;
-          }
-        } else {                /* ansi "flag", inverse, flash, underline, hilight */
-          flag_depth = ansi_depth + 1;
-        }
-        /* Check to see if this is an 'ansi-reset' code. */
-        if (*ptr == '0' && *(ptr + 1) == 'm') {
-          for (; ansi_depth > 0; ansi_depth--) {
-            safe_str(")]", value, &s);
-          }
-          flag_depth = 0;
-          ptr++;
-          break;
-        }
-
-        ansi_depth++;
-        safe_str("[ansi(", value, &s);
-        dospace = 1;
-
-        /* code for decompiling ansi */
-        for (; isdigit((unsigned char) *ptr) || *ptr == ';'; ptr++) {
-          if (*ptr == ';')      /* Yes, it is necessary to do it this way. */
-            ptr++;
-          /* Break if there is an 'm' here. */
-          if (!*ptr || !isdigit((unsigned char) *ptr))
-            break;
-          /* Check to see if the code is one character long. */
-          if (*(ptr + 1) == ';' || *(ptr + 1) == 'm') {
-            /* ANSI flag */
-            switch (*ptr) {
-            case '1':
-              safe_chr('h', value, &s);
-              break;
-            case '4':
-              safe_chr('u', value, &s);
-              break;
-            case '5':
-              safe_chr('f', value, &s);
-              break;
-            case '7':
-              safe_chr('i', value, &s);
-              break;
-            default:           /* Not a valid code. */
-              break;
-            }
-          } else {
-            if (!*(ptr + 1))
-              break;            /* Sudden string end or lack of real color code. */
-            ptr++;
-
-            /* Check if this could be a real color (starts with 3 or 4) */
-            if (*(ptr - 1) == '3' || *(ptr - 1) == '4') {
-              switch (*ptr) {
-              case '0':
-                ansi_letter = 'x';
-                break;
-              case '1':
-                ansi_letter = 'r';
-                break;
-              case '2':
-                ansi_letter = 'g';
-                break;
-              case '3':
-                ansi_letter = 'y';
-                break;
-              case '4':
-                ansi_letter = 'b';
-                break;
-              case '5':
-                ansi_letter = 'm';
-                break;
-              case '6':
-                ansi_letter = 'c';
-                break;
-              case '7':
-                ansi_letter = 'w';
-                break;
-              default:
-                break;          /* Not a valid color. */
-              }
-              /* If background color, change the letter to a capital. */
-              if (*(ptr - 1) == '4')
-                ansi_letter = UPCASE(ansi_letter);
-              safe_chr(ansi_letter, value, &s);
-            }
-            /* No "else" here: If a two-digit code
-             * doesn't start with 3 or 4, is isn't ANSI. */
-          }
-        }
-        safe_chr(',', value, &s);
-      } else {
-        ptr--;
-        /* This shouldn't happen if we only have ansi codes
-         * So if more dirty things must be added later... */
-      }
-      break;
-    default:
-      insert_spaces(spaces, dospace, value, &s);
-      spaces = 0;
-      if (escaped_chars[(unsigned int) *ptr]) {
-        safe_chr('\\', value, &s);
-      }
-      safe_chr(*ptr, value, &s);
-      dospace = 0;
-    }
-  }
-  /* Now check the last space. */
-  insert_spaces(spaces, dospace, value, &s);
-  spaces = 0;
-  for (; ansi_depth > 0; ansi_depth--) {
-    safe_str(")]", value, &s);
-  }
-  *s = '\0';
   return value;
 }
 
@@ -1666,10 +1437,10 @@ decompile_helper(dbref player, dbref thing __attribute__ ((__unused__)),
 
   avalue = atr_value(atr);
   avlen = strlen(avalue);
-  /* If avalue includes a %r, a %t, or begins or ends with a %b,
+  /* If avalue includes a %r, a %t, begins or ends with a %b, or has markup,
    * then use @set on the decompose_str'd value instead of &atrname */
-  if (strchr(avalue, '\n') ||
-      strchr(avalue, '\t') || *avalue == ' ' || avalue[avlen - 1] == ' ') {
+  if (strchr(avalue, '\n') || strchr(avalue, '\t') ||
+      strchr(avalue, TAG_START) || *avalue == ' ' || avalue[avlen - 1] == ' ') {
     safe_str("@set ", msg, &bp);
     safe_str(dh->name, msg, &bp);
     safe_chr('=', msg, &bp);
@@ -1698,7 +1469,7 @@ decompile_helper(dbref player, dbref thing __attribute__ ((__unused__)),
     if (dh->skipdef && ptr) {
       /* Standard attribute. Get the default perms, if any. */
       /* Are we different? If so, do as usual */
-      unsigned int npmflags = AL_FLAGS(ptr) & (~AF_PREFIXMATCH);
+      uint32_t npmflags = AL_FLAGS(ptr) & (~AF_PREFIXMATCH);
       if (AL_FLAGS(atr) != AL_FLAGS(ptr) && AL_FLAGS(atr) != npmflags)
         privs = privs_to_string(attr_privs_view, AL_FLAGS(atr));
     } else {
@@ -1779,30 +1550,34 @@ decompile_locks(dbref player, dbref thing, const char *name,
  * \param skipdef if true, skip showing default flags on attributes/locks.
  */
 void
-do_decompile(dbref player, const char *name, const char *prefix,
+do_decompile(dbref player, const char *xname, const char *prefix,
              enum dec_type dbflag, int skipdef)
 {
   dbref thing;
   const char *object = NULL;
-  char *attrib;
+  char *attrib, *name;
   char dbnum[40];
 
   /* @decompile must always have an argument */
-  if (!name || !*name) {
+  if (!xname || !*xname) {
     notify(player, T("What do you want to @decompile?"));
     return;
   }
+  name = mush_strdup(xname, "dd.string");
   attrib = strchr(name, '/');
   if (attrib)
     *attrib++ = '\0';
 
   /* find object */
   if ((thing = noisy_match_result(player, name, NOTYPE, MAT_EVERYTHING)) ==
-      NOTHING)
+      NOTHING) {
+    mush_free(name, "dd.string");
     return;
+  }
 
   if (IsGarbage(thing)) {
     notify(player, T("Garbage is garbage."));
+    mush_free(name, "dd.string");
     return;
   }
   sprintf(dbnum, "#%d", thing);
@@ -1820,11 +1595,13 @@ do_decompile(dbref player, const char *name, const char *prefix,
         decompile_atrs(player, thing, Name(thing), attrib, prefix, skipdef);
       break;
     }
+    mush_free(name, "dd.string");
     return;
   }
   /* else we have a full decompile */
   if (!Can_Examine(player, thing)) {
     notify(player, T("Permission denied."));
+    mush_free(name, "dd.string");
     return;
   }
   /* determine creation and what we call the object */
@@ -1894,6 +1671,7 @@ do_decompile(dbref player, const char *name, const char *prefix,
   if (dbflag != DEC_FLAG) {
     decompile_atrs(player, thing, object, "**", prefix, skipdef);
   }
+  mush_free(name, "dd.string");
 }
 
 static char *
