@@ -39,6 +39,9 @@
 
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -151,7 +154,7 @@ do_malias_create(dbref player, char *alias, char *tolist)
   good = "`$_-.'";
   /* Make sure that the name contains legal characters only */
   for (scan = alias + 1; scan && *scan; scan++) {
-    if (isalpha((unsigned char) *scan) || isdigit((unsigned char) *scan))
+    if (isalnum((unsigned char) *scan))
       continue;
     if (!strchr(good, *scan)) {
       notify(player, T("MAIL: Invalid character in mail alias."));
@@ -165,16 +168,12 @@ do_malias_create(dbref player, char *alias, char *tolist)
   }
   if (!ma_size) {
     ma_size = MA_INC;
-    malias =
-      (struct mail_alias *) mush_malloc(sizeof(struct mail_alias) *
-                                        ma_size, "malias_list");
+    malias = mush_calloc(ma_size, sizeof(struct mail_alias), "malias_list");
   } else if (ma_top >= ma_size) {
     ma_size += MA_INC;
-    m =
-      (struct mail_alias *) mush_malloc(sizeof(struct mail_alias) *
-                                        (ma_size), "malias_list");
+    m = mush_calloc(ma_size, sizeof(struct mail_alias), "malias_list");
     memcpy(m, malias, sizeof(struct mail_alias) * ma_top);
-    mush_free((Malloc_t) malias, "malias_list");
+    mush_free(malias, "malias_list");
     malias = m;
   }
   i = 0;
@@ -238,7 +237,7 @@ do_malias_create(dbref player, char *alias, char *tolist)
     return;
   }
   m = &malias[ma_top];
-  m->members = (dbref *) mush_malloc(sizeof(dbref) * i, "malias_members");
+  m->members = mush_calloc(i, sizeof(dbref), "malias_members");
   memcpy(m->members, alist, sizeof(dbref) * i);
 
   na = alias + 1;
@@ -615,8 +614,8 @@ do_malias_set(dbref player, char *alias, char *tolist)
     return;
   }
   if (m->members)
-    mush_free((Malloc_t) m->members, "malias_members");
-  m->members = (dbref *) mush_malloc(sizeof(dbref) * i, "malias_members");
+    mush_free(m->members, "malias_members");
+  m->members = mush_calloc(i, sizeof(dbref), "malias_members");
   memcpy(m->members, alist, sizeof(dbref) * i);
   m->size = i;
   notify(player, T("MAIL: Alias list set."));
@@ -830,12 +829,11 @@ do_malias_add(dbref player, char *alias, char *tolist)
     notify(player, T("MAIL: No valid recipients for alias-list!"));
     return;
   }
-  members =
-    (dbref *) mush_malloc(sizeof(dbref) * (i + m->size), "malias_members");
+  members = mush_calloc(i + m->size, sizeof(dbref), "malias_members");
 
   memcpy(members, m->members, sizeof(dbref) * m->size);
   memcpy(&members[m->size], alist, sizeof(dbref) * i);
-  mush_free((Malloc_t) m->members, "malias_members");
+  mush_free(m->members, "malias_members");
   m->members = members;
 
   m->size += i;
@@ -1047,7 +1045,7 @@ get_malias(dbref player, char *alias)
  * \param fp file pointer to read from.
  */
 void
-load_malias(FILE * fp)
+load_malias(PENNFILE *fp)
 {
   int i, j;
   char buffer[BUFFER_LEN];
@@ -1059,9 +1057,7 @@ load_malias(FILE * fp)
   ma_size = ma_top;
 
   if (ma_top > 0)
-    malias =
-      (struct mail_alias *) mush_malloc(sizeof(struct mail_alias) *
-                                        ma_size, "malias_list");
+    malias = mush_calloc(ma_size, sizeof(struct mail_alias), "malias_list");
   else
     malias = NULL;
 
@@ -1078,8 +1074,7 @@ load_malias(FILE * fp)
     m->size = getref(fp);
 
     if (m->size > 0) {
-      m->members =
-        (dbref *) mush_malloc(m->size * sizeof(dbref), "malias_members");
+      m->members = mush_calloc(m->size, sizeof(dbref), "malias_members");
       for (j = 0; j < m->size; j++) {
         m->members[j] = getref(fp);
       }
@@ -1087,7 +1082,7 @@ load_malias(FILE * fp)
       m->members = NULL;
     }
   }
-  s = fgets(buffer, sizeof(buffer), fp);
+  s = penn_fgets(buffer, sizeof(buffer), fp);
 
   if (!s || strcmp(buffer, "\"*** End of MALIAS ***\"\n") != 0) {
     do_rawlog(LT_ERR, T("MAIL: Error reading MALIAS list"));
@@ -1098,7 +1093,7 @@ load_malias(FILE * fp)
  * \param fp file pointer to write to.
  */
 void
-save_malias(FILE * fp)
+save_malias(PENNFILE *fp)
 {
   int i, j;
   struct mail_alias *m;
