@@ -58,7 +58,7 @@
 static dbref match_result_internal
   (dbref who, const char *name, int type, long flags);
 static dbref simple_matches(dbref who, const char *name, long flags);
-static int parse_english(const char **name, long *flags);
+static int parse_english(char **name, long *flags);
 static dbref match_me(const dbref who, const char *name);
 static dbref match_here(const dbref who, const char *name);
 /** Convenience alias for parse_objid */
@@ -227,20 +227,25 @@ MATCH_FUNC_PROTO(match_container);
 MATCH_FUNC_PROTO(match_list);
 
 static dbref
-match_result_internal(dbref who, const char *name, int type, long flags)
+match_result_internal(dbref who, const char *xname, int type, long flags)
 {
   dbref match = NOTHING, last_match = NOTHING, exact_match = NOTHING;
   int exact_matches_to_go, matches_to_go;
   int matchnum = 0;
   int result;
+  char *name, *sname;
 
   /* The quick ones that can never be ambiguous */
-  match = simple_matches(who, name, flags);
+  match = simple_matches(who, xname, flags);
   if (GoodObject(match))
     return match;
 
-  /* Check for adjective phrases */
-  matchnum = parse_english(&name, &flags);
+  sname = name = mush_strdup(xname, "mri.string");
+
+  if (flags & MAT_ENGLISH) {
+    /* Check for adjective phrases */
+    matchnum = parse_english(&name, &flags);
+  }
 
   /* Perform matching. We've already had flags restricted by any
    * adjective phrases. If matchnum is set, collect the matchnum'th
@@ -296,6 +301,7 @@ finished:
 
   /* Handle noisy_match_result */
   if (flags & MAT_NOISY) {
+    mush_free(sname, "mri.string");
     switch (match) {
     case NOTHING:
       notify(who, T("I can't see that here."));
@@ -307,6 +313,7 @@ finished:
       return match;
     }
   }
+  mush_free(sname, "mri.string");
   return match;
 }
 
@@ -376,10 +383,10 @@ simple_matches(dbref who, const char *name, long flags)
  * are restrictions
  */
 static int
-parse_english(const char **name, long *flags)
+parse_english(char **name, long *flags)
 {
   int saveflags = *flags;
-  const char *savename = *name;
+  char *savename = *name;
   char *mname;
   char *e;
   int count = 0;
