@@ -100,8 +100,9 @@ moveit(dbref what, dbref where, int nomovemsgs)
         /* If the player is leaving a zone, do zone messages */
         /* The tricky bit here is that we only care about the zone of
          * the outermost contents */
-        if (GoodObject(absold) && GoodObject(Zone(absold))
-            && (Zone(absloc) != Zone(absold)))
+        if (GoodObject(absold) && GoodObject(Zone(absold)) &&
+            (!GoodObject(absloc) || !GoodObject(Zone(absloc)) ||
+             (Zone(absloc) != Zone(absold))))
           did_it_interact(what, Zone(absold), "ZLEAVE", NULL, "OZLEAVE", NULL,
                           "AZLEAVE", old, NA_INTER_SEE);
         if (GoodObject(old) && !IsRoom(old))
@@ -111,27 +112,30 @@ moveit(dbref what, dbref where, int nomovemsgs)
           did_it_interact(what, where, NULL, NULL, "OXENTER", NULL, NULL, old,
                           NA_INTER_SEE);
         /* If the player is entering a new zone, do zone messages */
-        if (!GoodObject(absold)
-            || (GoodObject(Zone(absloc)) && (Zone(absloc) != Zone(absold))))
+        if (GoodObject(absloc) && GoodObject(Zone(absloc)) &&
+            (!GoodObject(absold) || !GoodObject(Zone(absold)) ||
+             (Zone(absloc) != Zone(absold))))
           did_it_interact(what, Zone(absloc), "ZENTER", NULL, "OZENTER", NULL,
                           "AZENTER", where, NA_INTER_SEE);
-        did_it_interact(what, where, "ENTER", NULL, "OENTER", T("has arrived."),
-                        "AENTER", where, NA_INTER_PRESENCE);
+        did_it_interact(what, where, "ENTER", NULL, "OENTER",
+                        T("has arrived."), "AENTER", where, NA_INTER_PRESENCE);
       } else {
         /* non-listeners only trigger the actions not the messages */
         did_it(what, old, NULL, NULL, NULL, NULL, "ALEAVE", old);
-        if (GoodObject(absold) && GoodObject(Zone(absold))
-            && (Zone(absloc) != Zone(absold)))
+        if (GoodObject(absold) && GoodObject(Zone(absold)) &&
+            (!GoodObject(absloc) || !GoodObject(Zone(absloc)) ||
+             (Zone(absloc) != Zone(absold))))
           did_it(what, Zone(absold), NULL, NULL, NULL, NULL, "AZLEAVE", old);
-        if (!GoodObject(absold)
-            || (GoodObject(Zone(absloc)) && (Zone(absloc) != Zone(absold))))
+        if (GoodObject(absloc) && GoodObject(Zone(absloc)) &&
+            (!GoodObject(absold) || !GoodObject(Zone(absold)) ||
+             (Zone(absloc) != Zone(absold))))
           did_it(what, Zone(absloc), NULL, NULL, NULL, NULL, "AZENTER", where);
         did_it(what, where, NULL, NULL, NULL, NULL, "AENTER", where);
       }
     }
   if (!nomovemsgs)
-    did_it_interact(what, what, "MOVE", NULL, "OMOVE", NULL, "AMOVE", where,
-                    NA_INTER_SEE);
+    did_it_interact(what, what, "MOVE", NULL, "OMOVE", NULL,
+                    "AMOVE", where, NA_INTER_SEE);
 }
 
 /** A dropper is an object that can hear and has a connected owner */
@@ -531,9 +535,13 @@ do_get(dbref player, const char *what)
   if (match_result(player, what, TYPE_THING, match_flags) == NOTHING) {
     if (POSSESSIVE_GET) {
       dbref box;
-      const char *boxname = what;
+      const char *boxname;
+      char objnamebuf[BUFFER_LEN], *objname;
+      boxname = what;
+      strcpy(objnamebuf, what);
+      objname = objnamebuf;
       /* take care of possessive get (stealing) */
-      box = parse_match_possessor(player, &what);
+      box = parse_match_possessor(player, &objname);
       if (box == NOTHING) {
         notify(player, T("I don't see that here."));
         return;
@@ -541,7 +549,7 @@ do_get(dbref player, const char *what)
         notify_format(player, T("I can't tell which %s."), boxname);
         return;
       }
-      thing = match_result(box, what, NOTYPE, MAT_POSSESSION);
+      thing = match_result(box, objname, NOTYPE, MAT_POSSESSION);
       if (thing == NOTHING) {
         notify(player, T("I don't see that here."));
         return;
