@@ -14,6 +14,11 @@
 #include <ctype.h>
 #include <string.h>
 
+#ifdef WIN32
+#include <windows.h>
+#include <winsock.h>
+#endif
+
 #ifdef I_SYS_TYPES
 #include <sys/types.h>
 #endif
@@ -65,11 +70,9 @@
 #include <netdb.h>
 #endif
 
-#ifndef h_errno
-#ifndef __CYGWIN__
+#if !defined(HAVE_H_ERRNO) && !defined(WIN32)
 extern int h_errno;
 #endif
-#endif
 
 #include <errno.h>
 
@@ -81,10 +84,21 @@ extern int h_errno;
 #include <sys/file.h>
 #endif
 
-#include <time.h>
-
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
+#else
+#include <time.h>
+#endif
+
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#endif
+
+#ifdef I_SYS_SELECT
+#include <sys/select.h>
 #endif
 
 #include "conf.h"
@@ -94,13 +108,8 @@ extern int h_errno;
 #include "ident.h"
 #include "confmagic.h"
 
-#ifndef __ptr_t
-#if defined (__cplusplus) || (defined (__STDC__) && __STDC__)
-#define __ptr_t void *
-#else
-#define __ptr_t char *
-#endif
-#endif
+static int connect_nonb
+  (int sockfd, const struct sockaddr *saptr, socklen_t salen, bool nonb);
 
 
 #ifndef INFOSLAVE
@@ -149,18 +158,19 @@ ip_convert(struct sockaddr *host, int len)
 }
 #endif                          /* INFOSLAVE */
 
-/** Open a tcp connection to a given host and port. Basically
+/** Open a connection to a given host and port. Basically
  * tcp_connect from UNPv1
  * \param host hostname or ip address to connect to, as a string.
+ * \param socktype The type of socket - SOCK_STREAM or SOCK_DGRAM
  * \param myiterface pointer to sockaddr structure for specific interface.
  * \param myilen length of myiterface
  * \param port port to connect to.
- * \param timeout pointer to timeout for connection.
+ * \param nonb true to do a nonblocking connect in the background.
  * \return file descriptor for connected socket, or -1 for failure.
  */
 int
-make_socket_conn(const char *host, struct sockaddr *myiterface,
-                 socklen_t myilen, Port_t port, int *timeout)
+make_socket_conn(const char *host, int socktype, struct sockaddr *myiterface,
+                 socklen_t myilen, Port_t port, bool nonb)
 {
   struct addrinfo hints, *server, *save;
   char cport[NI_MAXSERV];
@@ -169,22 +179,25 @@ make_socket_conn(const char *host, struct sockaddr *myiterface,
   int err;
 
   memset(&hints, 0, sizeof(struct addrinfo));
-  hints.ai_flags = AI_PASSIVE;
   hints.ai_family = AF_UNSPEC;  /* Try to use IPv6 if available */
-  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_socktype = socktype;
 
   sprintf(cport, "%hu", port);
 
   if ((res = getaddrinfo(host, cport, &hints, &server)) != 0) {
+    lock_file(stderr);
     fprintf(stderr, "In getaddrinfo: %s\n", gai_strerror(res));
     fprintf(stderr, "Host: %s Port %hu\n", host, port);
     fflush(stderr);
+    unlock_file(stderr);
     return -1;
   }
 
   if (!server) {
+    lock_file(stderr);
     fprintf(stderr, "Couldn't get address for host %s port %hu\n", host, port);
     fflush(stderr);
+    unlock_file(stderr);
     return -1;
   }
 
@@ -195,18 +208,19 @@ make_socket_conn(const char *host, struct sockaddr *myiterface,
     if (s < 0)
       continue;
 
-    if (myiterface && myilen > 0) {
-      /* Bind to a specific interface. Needed for ident lookups */
+    if (myiterface && myilen > 0 && myiterface->sa_family == server->ai_family) {
+      /* Bind to a specific interface. Don't even try for the case of
+       * an IPv4 socket and an IPv6 interface. Happens with ident, which
+       * seems to work okay without the bind(). */
       if (bind(s, myiterface, myilen) < 0)
-        perror("bind failed (Possibly harmless)");
+        penn_perror("bind failed (Possibly harmless)");
     }
 
-    if ((err = connect_nonb(s, server->ai_addr, server->ai_addrlen, timeout)) ==
-        0)
+    if ((err = connect_nonb(s, server->ai_addr, server->ai_addrlen, nonb)) == 0)
       break;
 
 #ifdef DEBUG
-    perror("connect failed (Probably harmless)");
+    penn_perror("connect failed (Probably harmless)");
 #endif
 
     closesocket(s);
@@ -223,8 +237,10 @@ make_socket_conn(const char *host, struct sockaddr *myiterface,
 
     if (port != IDPORT) {
 #endif
+      lock_file(stderr);
       fprintf(stderr, "Couldn't connect to %s on port %hu\n", host, port);
       fflush(stderr);
+      unlock_file(stderr);
 #ifndef DEBUG
     }
 #endif
@@ -233,16 +249,18 @@ make_socket_conn(const char *host, struct sockaddr *myiterface,
   return s;
 }
 
-/** Start listening to TCP on a given port. Basically tcp_listen
+
+/** Start listening on a given port. Basically tcp_listen
  * from UNPv1
  * \param port port to listen on.
+ * \param socktype the type of socket - SOCK_STREAM or SOCK_DGRAM
  * \param addr pointer to sockaddr to copy address data to.
  * \param len length of addr.
  * \param host hostname or address to listen on, as a string.
  * \return file descriptor of listening socket, or -1 for failure.
  */
 int
-make_socket(Port_t port, union sockaddr_u *addr, socklen_t *len,
+make_socket(Port_t port, int socktype, union sockaddr_u *addr, socklen_t * len,
             const char *host)
 {
   int s, opt, ipv = 4;
@@ -252,7 +270,7 @@ make_socket(Port_t port, union sockaddr_u *addr, socklen_t *len,
    * Unix Network Programming, vol 1.  If getaddrinfo() isn't
    * present on the system, we'll use our own version, also from UNPv1. */
   struct addrinfo *server, *save, hints;
-  char cport[NI_MAXSERV];
+  char portbuf[NI_MAXSERV], *cport;
   int res;
 
   memset(&hints, 0, sizeof(struct addrinfo));
@@ -263,9 +281,13 @@ make_socket(Port_t port, union sockaddr_u *addr, socklen_t *len,
 #else
   hints.ai_family = AF_UNSPEC;  /* Try to use IPv6 if available */
 #endif
-  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_socktype = socktype;
 
-  sprintf(cport, "%hu", port);
+  if (port > 0) {
+    sprintf(portbuf, "%hu", port);
+    cport = portbuf;
+  } else
+    cport = NULL;
 
   if (strlen(host) == 0)
     host = NULL;
@@ -290,14 +312,21 @@ make_socket(Port_t port, union sockaddr_u *addr, socklen_t *len,
 
     opt = 1;
     if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &opt, sizeof(opt)) < 0) {
-      perror("setsockopt (Possibly ignorable)");
+      penn_perror("setsockopt (Possibly ignorable)");
       continue;
     }
 
     if (bind(s, server->ai_addr, server->ai_addrlen) == 0)
       break;                    /* Success */
 
-    perror("binding stream socket (Possibly ignorable)");
+    if (errno == EADDRINUSE) {
+      fprintf(stderr,
+              "Another process (Possibly another copy of this mush?) appears to be using port %hu. Aborting.\n",
+              port);
+      exit(1);
+    }
+
+    penn_perror("binding stream socket (Possibly ignorable)");
     closesocket(s);
   } while ((server = server->ai_next) != NULL);
 
@@ -330,29 +359,69 @@ make_nonblocking(int s)
 #ifdef WIN32
   unsigned long arg = 1;
   if (ioctlsocket(s, FIONBIO, &arg) == -1) {
+    penn_perror("make_nonblocking: ioctlsocket");
+    mush_panic(T("Fatal network error!"));
+  }
 #else
   int flags;
 
   if ((flags = fcntl(s, F_GETFL, 0)) == -1) {
-    perror("make_nonblocking: fcntl");
+    penn_perror("make_nonblocking: fcntl");
 #ifndef INFOSLAVE
-    mush_panic("F_GETFL fcntl failed");
+    mush_panic(T("Fatal network error!"));
 #else
     exit(1);
 #endif
   }
 
-  if (fcntl(s, F_SETFL, flags | O_NDELAY) == -1) {
-#endif                          /* WIN32 */
-    perror("make_nonblocking: fcntl");
+  flags |= O_NDELAY;
+
+  if (fcntl(s, F_SETFL, flags) == -1) {
+    penn_perror("make_nonblocking: fcntl");
 #ifndef INFOSLAVE
-    mush_panic("O_NDELAY fcntl failed");
+    mush_panic(T("Fatal network error!"));
 #else
     exit(1);
 #endif
   }
+#endif
 }
 
+/** Make a socket do blocking i/o.
+ * \param s file descriptor of socket.
+ */
+void
+make_blocking(int s)
+{
+#ifdef WIN32
+  unsigned long arg = 0;
+  if (ioctlsocket(s, FIONBIO, &arg) == -1) {
+    penn_perror("make_blocking: ioctlsocket");
+    mush_panic(T("Fatal network error"));
+  }
+#else
+  int flags;
+
+  if ((flags = fcntl(s, F_GETFL, 0)) == -1) {
+    penn_perror("make_blocking: fcntl");
+#ifndef INFOSLAVE
+    mush_panic(T("Fatal network error!"));
+#else
+    exit(1);
+#endif
+  }
+
+  flags &= ~O_NDELAY;
+  if (fcntl(s, F_SETFL, flags) == -1) {
+    penn_perror("make_nonblocking: fcntl");
+#ifndef INFOSLAVE
+    mush_panic(T("Fatal network error!"));
+#else
+    exit(1);
+#endif
+  }
+#endif
+}
 
 #ifndef INFOSLAVE
 /** Enable TCP keepalive on the given socket if we can.
@@ -374,7 +443,7 @@ set_keepalive(int s __attribute__ ((__unused__)))
     fprintf(stderr, "[%d] could not set SO_KEEPALIVE: errno %d", s, errno);
 
   /* And set the ping time to something reasonable instead of the
-     default 2 hours. Linux, NetBSD and o thers use TCP_KEEPIDLE to do
+     default 2 hours. Linux and possibly others use TCP_KEEPIDLE to do
      this. OS X and possibly others use TCP_KEEPALIVE. */
 #if defined(TCP_KEEPIDLE)
   if (setsockopt(s, IPPROTO_TCP, TCP_KEEPIDLE,
@@ -399,24 +468,19 @@ set_keepalive(int s __attribute__ ((__unused__)))
  * \param sockfd file descriptor of socket.
  * \param saptr pointer to sockaddr structure with connection data.
  * \param salen length of saptr.
- * \param timeout pointer to timeout value. If given, nonblocking connect.
+ * \param nonb true for a nonblocking connect in the background
  * \retval 0 success.
  * \retval -1 failure.
  */
 int
 connect_nonb(int sockfd, const struct sockaddr *saptr, socklen_t salen,
-             int *timeout)
+             bool nonb)
 {
-  int n, error;
-  time_t start, end;
-  socklen_t len;
-  fd_set rset, wset;
-  struct timeval tval;
+  int n;
 
-  if (timeout && *timeout)
+  if (nonb)
     make_nonblocking(sockfd);
 
-  error = 0;
   if ((n = connect(sockfd, (const struct sockaddr *) saptr, salen)) < 0)
 #ifdef WIN32
     if (n == SOCKET_ERROR && WSAGetLastError() != WSAEWOULDBLOCK)
@@ -425,44 +489,51 @@ connect_nonb(int sockfd, const struct sockaddr *saptr, socklen_t salen,
 #endif
       return -1;
 
-  /* connect completed immediately or we did a blocking connect  */
-  if (!timeout || !*timeout || n == 0)
-    goto done;
-
-  FD_ZERO(&rset);
-  FD_SET(sockfd, &rset);
-  wset = rset;
-  tval.tv_sec = *timeout;
-  tval.tv_usec = 0;
-  time(&start);
-
-  if ((n = select(sockfd + 1, &rset, &wset, NULL, &tval)) == 0) {
-    closesocket(sockfd);        /* timeout */
-    errno = ETIMEDOUT;
-    return -1;
-  }
-
-  time(&end);
-  *timeout -= end - start;
-  *timeout = *timeout < 0 ? 0 : *timeout;
-
-  if (FD_ISSET(sockfd, &rset) || FD_ISSET(sockfd, &wset)) {
-    len = sizeof(error);
-    if (getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (__ptr_t) &error, &len) < 0)
-      return -1;                /* Solaris pending error */
-  } else
-    return -1;
-
-done:
-
-  if (error) {
-    closesocket(sockfd);        /* just in case */
-    errno = error;
-    return -1;
-  }
   return 0;
 }
 
+/** Wait up to N seconds for a non-blocking connect to establish.
+ * \param s the socket
+ * \param secs timeout value
+ * \return -1 on error, 0 if the socket is not yet connected, >0 on success
+ */
+int
+wait_for_connect(int s, int secs)
+{
+  int res;
+#ifdef HAVE_POLL
+  struct pollfd ev;
+
+  ev.fd = s;
+  ev.events = POLLOUT;
+  if ((res = poll(&ev, 1, secs)) <= 0) {
+    if (res == 0)
+      errno = EINPROGRESS;
+    return res;
+  } else {
+    errno = ENOTCONN;
+    return ev.revents & POLLOUT;
+  }
+#else
+  fd_set wrs;
+  struct timeval timeout, *to;
+
+  FD_ZERO(&wrs);
+  FD_SET(s, &wrs);
+  timeout.tv_sec = secs;
+  timeout.tv_usec = 0;
+  if (secs >= 0)
+    to = &timeout;
+  if ((res = select(s + 1, NULL, &wrs, NULL, to)) <= 0) {
+#ifndef WIN32
+    if (res == 0)
+      errno = EINPROGRESS;
+#endif
+    return res;
+  } else
+    return FD_ISSET(s, &wrs);
+#endif
+}
 
 /* The following functions are from W. Ridhard Steven's libgai,
  * modified by Shawn Wagner for PennMUSH. These arn't full
@@ -938,7 +1009,7 @@ getnameinfo(const struct sockaddr *sa, socklen_t salen,
     }
 #endif
 
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
   case AF_INET6:{
       struct sockaddr_in6 *sain = (struct sockaddr_in6 *) sa;
 
@@ -1029,7 +1100,7 @@ getaddrinfo(const char *hostname, const char *servname,
     }
 #endif
 
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
     /* 4check for an IPv6 hex string */
     if ((isxdigit((unsigned char) sptr->host[0]) || sptr->host[0] == ':') &&
         (strchr(sptr->host, ':') != NULL)) {
@@ -1050,18 +1121,18 @@ getaddrinfo(const char *hostname, const char *servname,
 /* end ga3 */
 /* include ga4 */
     /* 4remainder of for() to look up hostname */
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
     if ((_res.options & RES_INIT) == 0)
       res_init();               /* need this to set _res.options */
 #endif
 
     if (nsearch == 2) {
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
       _res.options &= ~RES_USE_INET6;
 #endif
       hptr = gethostbyname2(sptr->host, sptr->family);
     } else {
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
       if (sptr->family == AF_INET6)
         _res.options |= RES_USE_INET6;
       else
@@ -1164,7 +1235,7 @@ ga_echeck(const char *hostname, const char *servname,
       return (EAI_SOCKTYPE);    /* invalid socket type */
     break;
 #endif
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
   case AF_INET6:
     if (socktype != 0 &&
         (socktype != SOCK_STREAM &&
@@ -1204,7 +1275,7 @@ ga_nsearch(const char *hostname, const struct addrinfo *hintsp,
         nsearch++;
         break;
 #endif
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
       case AF_INET6:
         search[nsearch].host = "0::0";
         search[nsearch].family = AF_INET6;
@@ -1212,7 +1283,7 @@ ga_nsearch(const char *hostname, const struct addrinfo *hintsp,
         break;
 #endif
       case AF_UNSPEC:
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
         search[nsearch].host = "0::0";  /* IPv6 first, then IPv4 */
         search[nsearch].family = AF_INET6;
         nsearch++;
@@ -1236,7 +1307,7 @@ ga_nsearch(const char *hostname, const struct addrinfo *hintsp,
         nsearch++;
         break;
 #endif
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
       case AF_INET6:
         search[nsearch].host = "0::1";
         search[nsearch].family = AF_INET6;
@@ -1244,7 +1315,7 @@ ga_nsearch(const char *hostname, const struct addrinfo *hintsp,
         break;
 #endif
       case AF_UNSPEC:
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
         search[nsearch].host = "0::1";  /* IPv6 first, then IPv4 */
         search[nsearch].family = AF_INET6;
         nsearch++;
@@ -1268,7 +1339,7 @@ ga_nsearch(const char *hostname, const struct addrinfo *hintsp,
       nsearch++;
       break;
 #endif
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
     case AF_INET6:
       search[nsearch].host = hostname;
       search[nsearch].family = AF_INET6;
@@ -1276,7 +1347,7 @@ ga_nsearch(const char *hostname, const struct addrinfo *hintsp,
       break;
 #endif
     case AF_UNSPEC:
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
       search[nsearch].host = hostname;
       search[nsearch].family = AF_INET6;        /* IPv6 first */
       nsearch++;
@@ -1342,7 +1413,7 @@ ga_aistruct(struct addrinfo ***paipnext, const struct addrinfo *hintsp,
       break;
     }
 #endif                          /* IPV4 */
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
   case AF_INET6:{
       struct sockaddr_in6 *sin6ptr;
 
@@ -1461,7 +1532,7 @@ ga_port(struct addrinfo *aihead, int port, int socktype)
       nfound++;
       break;
 #endif
-#ifdef  HAS_IPV6
+#ifdef  HAVE_SOCKADDR_IN6
     case AF_INET6:
       ((struct sockaddr_in6 *) ai->ai_addr)->sin6_port = port;
       nfound++;
