@@ -34,6 +34,11 @@ void shutdown_checkpoint(void);
 #endif
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
+#else
+#include <time.h>
 #endif
 #include <sys/ioctl.h>
 #include <errno.h>
@@ -53,10 +58,6 @@ void shutdown_checkpoint(void);
 #include <sys/stat.h>
 #endif
 #endif                          /* !WIN32 */
-#include <time.h>
-#ifdef I_SYS_WAIT
-#include <sys/wait.h>
-#endif
 #include <fcntl.h>
 #include <ctype.h>
 #include <signal.h>
@@ -81,6 +82,7 @@ void shutdown_checkpoint(void);
 #include "myssl.h"
 #include "log.h"
 #include "parse.h"
+#include "SFMT.h"
 #include "confmagic.h"
 
 #define MYSSL_RB        0x1     /**< Read blocked (on read) */
@@ -106,8 +108,6 @@ static DH *get_dh1024(void);
 static BIO *bio_err = NULL;
 static SSL_CTX *ctx = NULL;
 
-unsigned int genrand_int32(void);
-
 /** Initialize the SSL context.
  * \return pointer to SSL context object.
  */
@@ -132,18 +132,20 @@ ssl_init(char *private_key_file, char *ca_file, int req_client_cert)
     /* At this point, a system with /dev/urandom or a EGD file in the usual
        places will have enough entropy. Otherwise, be lazy and use random numbers
        until it's satisfied. */
-    unsigned int gibberish[4];
+    uint32_t gibberish[4];
     int n;
 
     for (n = 0; n < 4; n++)
-      gibberish[n] = genrand_int32();
+      gibberish[n] = gen_rand32();
 
     RAND_seed(gibberish, sizeof gibberish);
 
     reps += 1;
   }
+
   do_rawlog(LT_ERR, "Seeded after %u %s.", reps, reps > 1 ? "cycles" : "cycle");
 
+
   /* Set up SIGPIPE handler here? */
 
   /* Create context */
@@ -186,7 +188,7 @@ ssl_init(char *private_key_file, char *ca_file, int req_client_cert)
 
   /* Set up DH callback */
   dh = get_dh1024();
-  SSL_CTX_set_tmp_dh(ctx, get_dh1024());
+  SSL_CTX_set_tmp_dh(ctx, dh);
   /* The above function makes a private copy of this */
   DH_free(dh);
 
@@ -253,16 +255,32 @@ get_dh1024(void)
   };
   DH *dh;
   if ((dh = DH_new()) == NULL)
-    return (NULL);
-  dh->p = BN_bin2bn(dh1024_p, sizeof(dh1024_p), NULL);
-  dh->g = BN_bin2bn(dh1024_g, sizeof(dh1024_g), NULL);
-  if ((dh->p == NULL) || (dh->g == NULL)) {
-    DH_free(dh);
-    return (NULL);
+    return NULL;
+  if (dh->p) {
+    BN_free(dh->p);
+    dh->p = NULL;
+  }
+  if (dh->g) {
+    BN_free(dh->g);
+    dh->g = NULL;
   }
-  return (dh);
-}
 
+  dh->p = BN_bin2bn(dh1024_p, sizeof(dh1024_p), NULL);
+  if (!dh->p) {
+    do_rawlog(LT_ERR, "Error in BN_bin2bn 1");
+    DH_free(dh);
+    return NULL;
+  }
+
+  dh->g = BN_bin2bn(dh1024_g, sizeof(dh1024_g), NULL);
+  if (!dh->g) {
+    do_rawlog(LT_ERR, "ERror in BN_bin2bn 2");
+    DH_free(dh);
+    return NULL;
+  }
+
+  return dh;
+}
 
 /** Associate an SSL object with a socket and return it.
  * \param sock socket descriptor to associate with an SSL object.
