@@ -26,6 +26,11 @@
 #endif
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
+#else
+#include <time.h>
 #endif
 #include <sys/ioctl.h>
 #ifdef I_SYS_SOCKET
@@ -44,7 +49,6 @@
 #include <sys/stat.h>
 #endif
 #endif                          /* !WIN32 */
-#include <time.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <signal.h>
@@ -76,8 +80,6 @@
 #include "ident.h"
 #include "strtree.h"
 #include "log.h"
-
-
 #include "mymalloc.h"
 
 #include "extchat.h"
@@ -87,7 +89,6 @@ extern CHAN *channels;
 #include "game.h"
 #include "confmagic.h"
 
-
 static int under_limit = 1;
 
 
@@ -160,6 +161,8 @@ enum na_type {
 /** Number of possible message text renderings */
 #define MESSAGE_TYPES 12
 
+/* These should be removed. I can't imagine anyone uses them in hacks,
+ * but I'm leaving them here just in case... */
 #define TA_BGC 0        /**< Text attribute background color */
 #define TA_FGC 1        /**< Text attribute foreground color */
 #define TA_BOLD 2       /**< Text attribute bold/hilite */
@@ -177,119 +180,15 @@ struct notify_strings {
   int made;                     /**< True if message has been rendered. */
 };
 
-static void fillstate(int state[], const unsigned char **f);
-static void ansi_change_state(char *t, char **o, int color, int *state,
-                              int *newstate);
 static enum na_type notify_type(DESC *d);
 static void free_strings(struct notify_strings messages[]);
 static void zero_strings(struct notify_strings messages[]);
+static int output_ansichange(ansi_data *states, int *ansi_ptr,
+                             const unsigned char **ptr, char *buff, char **bp);
 static unsigned char *notify_makestring(const char *message,
                                         struct notify_strings messages[],
                                         enum na_type type, int flags);
 
-
-static void
-fillstate(int state[6], const unsigned char **f)
-{
-  const unsigned char *p;
-  int i;
-  int n;
-  p = *f;
-  p++;
-  if (*p != '[') {
-    while (*p && *p != 'm')
-      p++;
-  } else {
-    p++;
-    while (*p && *p != 'm') {
-      if ((*p > '9') || (*p < '0')) {
-        /* Nada */
-      } else if (!(*(p + 1)) || (*(p + 1) == 'm') || (*(p + 1) == ';')) {
-        /* ShortCode */ ;
-        switch (*p) {
-        case '0':
-          for (i = 0; i < 6; i++)
-            state[i] = 0;
-          break;
-        case '1':
-          state[TA_BOLD] = 1;
-          break;
-        case '7':
-          state[TA_REV] = 1;
-          break;
-        case '5':
-          state[TA_BLINK] = 1;
-          break;
-        case '4':
-          state[TA_ULINE] = 1;
-          break;
-        }
-      } else {
-        n = (*p - '0') * 10;
-        p++;
-        n += (*p - '0');
-        if ((n >= 30) && (n <= 37))
-          state[TA_FGC] = n - 29;
-        else if ((n >= 40) && (n <= 47))
-          state[TA_BGC] = n - 39;
-      }
-      p++;
-    }
-  }
-  if ((p != *f) && (*p != 'm'))
-    p--;
-  *f = p;
-}
-
-/** Add an ansi tag if it's needed here */
-#define add_ansi_if(x,c) \
-  do { \
-    if (newstate[(x)] && (newstate[(x)]!=state[(x)])) { \
-      if (i) safe_chr(';',t,o); \
-      safe_str((c),t,o); \
-      i=1; \
-    } \
-  } while (0)
-
-static void
-ansi_change_state(char *t, char **o, int color, int *state, int *newstate)
-{
-  int i, n;
-  if ((state[TA_BOLD] && !newstate[TA_BOLD]) ||
-      (state[TA_REV] && !newstate[TA_REV]) ||
-      (state[TA_BLINK] && !newstate[TA_BLINK]) ||
-      (state[TA_ULINE] && !newstate[TA_ULINE]) ||
-      (color && state[TA_FGC] && !newstate[TA_FGC]) ||
-      (color && state[TA_BGC] && !newstate[TA_BGC])) {
-    for (n = 0; n < 6; n++)
-      state[n] = 0;
-    safe_str(ANSI_NORMAL, t, o);
-  }
-  if ((newstate[TA_BOLD] && (newstate[TA_BOLD] != state[TA_BOLD])) ||
-      (newstate[TA_REV] && (newstate[TA_REV] != state[TA_REV])) ||
-      (newstate[TA_BLINK] && (newstate[TA_BLINK] != state[TA_BLINK])) ||
-      (newstate[TA_ULINE] && (newstate[TA_ULINE] != state[TA_ULINE])) ||
-      (color && newstate[TA_FGC] && (newstate[TA_FGC] != state[TA_FGC])) ||
-      (color && newstate[TA_BGC] && (newstate[TA_BGC] != state[TA_BGC]))) {
-    safe_chr(ESC_CHAR, t, o);
-    safe_chr('[', t, o);
-    i = 0;
-    add_ansi_if(TA_BOLD, "1");
-    add_ansi_if(TA_REV, "7");
-    add_ansi_if(TA_BLINK, "5");
-    add_ansi_if(TA_ULINE, "4");
-    if (color) {
-      add_ansi_if(TA_FGC, unparse_integer(newstate[TA_FGC] + 29));
-      add_ansi_if(TA_BGC, unparse_integer(newstate[TA_BGC] + 39));
-    }
-    safe_chr('m', t, o);
-  }
-  for (n = 0; n < 6; n++)
-    state[n] = newstate[n];
-}
-
-#undef add_ansi_if
-
 static void
 zero_strings(struct notify_strings messages[])
 {
@@ -310,6 +209,58 @@ free_strings(struct notify_strings messages[])
       mush_free(messages[n].message, "string");
 }
 
+static int
+output_ansichange(ansi_data *states, int *ansi_ptr,
+                  const unsigned char **ptr, char *buff, char **bp)
+{
+  const unsigned char *p = *ptr;
+  int newaptr = *ansi_ptr;
+  int retval = 0;
+  ansi_data cur = states[*ansi_ptr];
+
+  /* This is color */
+  while (*p &&
+         ((*p == TAG_START && *(p + 1) == MARKUP_COLOR) || (*p == ESC_CHAR))) {
+    switch (*p) {
+    case TAG_START:
+      p += 2;
+      if (*p != '/') {
+        newaptr++;
+        define_ansi_data(&(states[newaptr]), (const char *) p);
+      } else {
+        if (*(p + 1) == 'a') {
+          newaptr = 0;
+        } else {
+          if (newaptr > 0)
+            newaptr--;
+        }
+      }
+      while (*p && *p != TAG_END)
+        p++;
+      break;
+    case ESC_CHAR:
+      newaptr++;
+      read_raw_ansi_data(&states[newaptr], (const char *) p);
+      while (*p && *p != 'm')
+        p++;
+      break;
+    }
+    if (newaptr > 0)
+      nest_ansi_data(&(states[newaptr - 1]), &(states[newaptr]));
+    /* Advance past the tag ending, if there's more. */
+    if (*p && ((*(p + 1) == TAG_START && *(p + 2) == MARKUP_COLOR) ||
+               (*(p + 1) == ESC_CHAR)))
+      p++;
+  }
+  /* Do we print anything? */
+  if (*p && *ptr != p) {
+    retval = write_raw_ansi_data(&cur, &(states[newaptr]), buff, bp);
+    *(ansi_ptr) = newaptr;
+  }
+  *ptr = p;
+  return retval;
+}
+
 static unsigned char *
 notify_makestring(const char *message, struct notify_strings messages[],
                   enum na_type type, int flags)
@@ -317,15 +268,23 @@ notify_makestring(const char *message, struct notify_strings messages[],
   char *o;
   const unsigned char *p;
   char *t;
-  int state[6] = { 0, 0, 0, 0, 0, 0 };
-  int newstate[6] = { 0, 0, 0, 0, 0, 0 };
-  int changed = 0;
   int color = 0;
   int strip = 0;
   int pueblo = 0;
   static char tbuf[BUFFER_LEN];
   char *bp;
 
+  static ansi_data states[BUFFER_LEN];
+  int ansi_ptr, ansifix;
+  ansi_ptr = 0;
+  ansifix = 0;
+
+  /* Everything is explicitly off by default */
+  states[0].bits = 0;
+  states[0].offbits = 0;
+  states[0].fore = 0;
+  states[0].back = 0;
+
   if (messages[type].made) {
     if (flags & NA_PROMPT) {
       bp = tbuf;
@@ -444,28 +403,26 @@ notify_makestring(const char *message, struct notify_strings messages[],
     while (*p) {
       switch ((unsigned char) *p) {
       case IAC:
-        if (changed) {
-          changed = 0;
-          ansi_change_state(t, &o, color, state, newstate);
-        }
         if (type == NA_TANSI || type == NA_TCOLOR)
-          safe_str("\xFF\xFF", t, &o);
+          safe_strl("\xFF\xFF", 2, t, &o);
         else if (strip && accent_table[IAC].base)
           safe_str(accent_table[IAC].base, t, &o);
         else
           safe_chr((char) IAC, t, &o);
         break;
       case TAG_START:
-        if (pueblo) {
+        if (*(p + 1) == MARKUP_COLOR) {
+          ansifix += output_ansichange(states, &ansi_ptr, &p, t, &o);
+        } else if (pueblo && (*(p + 1) == MARKUP_HTML)) {
           safe_chr('<', t, &o);
-          p++;
+          /* Skip over the 'p' for Pueblo */
+          p += 2;
           while ((*p) && (*p != TAG_END)) {
             safe_chr(*p, t, &o);
             p++;
           }
           safe_chr('>', t, &o);
         } else {
-          /* Non-pueblo */
           while (*p && *p != TAG_END)
             p++;
         }
@@ -476,14 +433,10 @@ notify_makestring(const char *message, struct notify_strings messages[],
       case '\r':
         break;
       case ESC_CHAR:
-        fillstate(newstate, &p);
-        changed = 1;
+        /* After the ansi changes, I really hope we don't encounter this. */
+        ansifix += output_ansichange(states, &ansi_ptr, &p, t, &o);
         break;
       default:
-        if (changed) {
-          changed = 0;
-          ansi_change_state(t, &o, color, state, newstate);
-        }
         if (pueblo) {
           if (strip) {
             /* Even if we're NOACCENTS, we must still translate a few things */
@@ -516,12 +469,32 @@ notify_makestring(const char *message, struct notify_strings messages[],
             safe_chr(*p, t, &o);
         }
       }
-      p++;
+      if (*p)
+        p++;
+    }
+    /* We possibly have some unclosed ansi. Force an
+     * ANSI_NORMAL for now. */
+    if (ansifix || (ansi_ptr && safe_str(ANSI_RAW_NORMAL, t, &o))) {
+      int sub = 7;
+      char *ptr;
+      int q;
+      if (flags & NA_PROMPT)
+        sub += 2;
+      ptr = t + BUFFER_LEN - sub;
+      for (q = 20; q > 0 && *ptr != ESC_CHAR; q--, ptr--) ;
+      if (pueblo) {
+        for (q = 20; q > 0 && *ptr != ESC_CHAR && *ptr != '<'; q--, ptr--) ;
+      } else {
+        for (q = 20; q > 0 && *ptr != ESC_CHAR; q--, ptr--) ;
+      }
+      if (q > 0) {
+        o = ptr;
+      } else {
+        o = t + BUFFER_LEN - sub;
+      }
+      safe_str(ANSI_RAW_NORMAL, t, &o);
+      *o = '\0';
     }
-    if (state[TA_BOLD] || state[TA_REV] ||
-        state[TA_BLINK] || state[TA_ULINE] ||
-        (color && (state[TA_FGC] || state[TA_BGC])))
-      safe_str(ANSI_NORMAL, t, &o);
 
     break;
   }
@@ -848,8 +821,7 @@ notify_anything_loc(dbref speaker, na_lookup func,
             queue_newwrite(d, pstring, plen);
 
           if (!((flags & NA_NOENTER) || ((flags & NA_PROMPT)
-                                         && !(d->
-                                              conn_flags &
+                                         && !(d->conn_flags &
                                               CONN_PROMPT_NEWLINES)))) {
             if ((poutput == NA_PUEBLO) || (poutput == NA_NPUEBLO)) {
               if (flags & NA_NOPENTER)
@@ -915,7 +887,8 @@ notify_anything_loc(dbref speaker, na_lookup func,
     a = atr_get_noparent(target, "LISTEN");
     if (a) {
       char match_space[BUFFER_LEN * 2];
-      int match_space_len = BUFFER_LEN * 2;
+      ssize_t match_space_len = BUFFER_LEN * 2;
+
       if (!tbuf1)
         tbuf1 = (char *) mush_malloc(BUFFER_LEN, "string");
       strcpy(tbuf1, atr_value(a));
@@ -982,7 +955,7 @@ notify_anything_loc(dbref speaker, na_lookup func,
       )
       atr_comm_match(target, speaker, '^', ':',
                      (char *) notify_makestring(msgbuf, messages, NA_ASCII, 0),
-                     0, NULL, NULL, NULL);
+                     0, 1, NULL, NULL, NULL);
 
     /* If object is flagged AUDIBLE and has a @FORWARDLIST, send
      *  stuff on */
@@ -1164,100 +1137,6 @@ notify_list(dbref speaker, dbref thing, const char *atr, const char *msg,
   free((Malloc_t) orig);
 }
 
-/** Safely add a tag into a buffer.
- * If we support pueblo, this function adds the tag start token,
- * the tag, and the tag end token. If not, it does nothing.
- * If we can't fit the tag in, we don't put any of it in.
- * \param a_tag the html tag to add.
- * \param buf the buffer to append to.
- * \param bp pointer to address in buf to insert.
- * \retval 0, successfully added.
- * \retval 1, tag wouldn't fit in buffer.
- */
-int
-safe_tag(char const *a_tag, char *buf, char **bp)
-{
-  int result = 0;
-  char *save = buf;
-
-  if (SUPPORT_PUEBLO) {
-    result = safe_chr(TAG_START, buf, bp);
-    result = safe_str(a_tag, buf, bp);
-    result = safe_chr(TAG_END, buf, bp);
-  }
-  /* If it didn't all fit, rewind. */
-  if (result)
-    *bp = save;
-
-  return result;
-}
-
-/** Safely add a closing tag into a buffer.
- * If we support pueblo, this function adds the tag start token,
- * a slash, the tag, and the tag end token. If not, it does nothing.
- * If we can't fit the tag in, we don't put any of it in.
- * \param a_tag the html tag to add.
- * \param buf the buffer to append to.
- * \param bp pointer to address in buf to insert.
- * \retval 0, successfully added.
- * \retval 1, tag wouldn't fit in buffer.
- */
-int
-safe_tag_cancel(char const *a_tag, char *buf, char **bp)
-{
-  int result = 0;
-  char *save = buf;
-
-  if (SUPPORT_PUEBLO) {
-    result = safe_chr(TAG_START, buf, bp);
-    result = safe_chr('/', buf, bp);
-    result = safe_str(a_tag, buf, bp);
-    result = safe_chr(TAG_END, buf, bp);
-  }
-  /* If it didn't all fit, rewind. */
-  if (result)
-    *bp = save;
-
-  return result;
-}
-
-/** Safely add a tag, some text, and a matching closing tag into a buffer.
- * If we can't fit the stuff, we don't put any of it in.
- * \param a_tag the html tag to add.
- * \param params tag parameters.
- * \param data the text to wrap the tag around.
- * \param buf the buffer to append to.
- * \param bp pointer to address in buf to insert.
- * \param player the player involved in all this, or NOTHING if internal.
- * \retval 0, successfully added.
- * \retval 1, tagged text wouldn't fit in buffer.
- */
-int
-safe_tag_wrap(char const *a_tag, char const *params, char const *data,
-              char *buf, char **bp, dbref player)
-{
-  int result = 0;
-  char *save = buf;
-
-  if (SUPPORT_PUEBLO) {
-    result = safe_chr(TAG_START, buf, bp);
-    result = safe_str(a_tag, buf, bp);
-    if (params && *params && ok_tag_attribute(player, params)) {
-      result = safe_chr(' ', buf, bp);
-      result = safe_str(params, buf, bp);
-    }
-    result = safe_chr(TAG_END, buf, bp);
-  }
-  result = safe_str(data, buf, bp);
-  if (SUPPORT_PUEBLO) {
-    result = safe_tag_cancel(a_tag, buf, bp);
-  }
-  /* If it didn't all fit, rewind. */
-  if (result)
-    *bp = save;
-  return result;
-}
-
 /** Wrapper to notify a single player with a message, unconditionally.
  * \param player player to notify.
  * \param msg message to send.
@@ -1307,18 +1186,22 @@ flag_broadcast(const char *flag1, const char *flag2, const char *fmt, ...)
   }
 }
 
-
+slab *text_block_slab = NULL; /**< Slab for 'struct text_block' allocations */
 
 static struct text_block *
 make_text_block(const unsigned char *s, int n)
 {
   struct text_block *p;
-  p =
-    (struct text_block *) mush_malloc(sizeof(struct text_block), "text_block");
+  if (text_block_slab == NULL) {
+    text_block_slab = slab_create("output lines", sizeof(struct text_block));
+    /* See what stats are like on M*U*S*H, maybe change */
+    slab_set_opt(text_block_slab, SLAB_ALLOC_FIRST_FIT, 1);
+    slab_set_opt(text_block_slab, SLAB_ALWAYS_KEEP_A_PAGE, 1);
+  }
+  p = slab_malloc(text_block_slab, NULL);
   if (!p)
     mush_panic("Out of memory");
-  p->buf =
-    (unsigned char *) mush_malloc(sizeof(unsigned char) * n, "text_block_buff");
+  p->buf = mush_malloc(n, "text_block_buff");
   if (!p->buf)
     mush_panic("Out of memory");
 
@@ -1338,7 +1221,7 @@ free_text_block(struct text_block *t)
   if (t) {
     if (t->buf)
       mush_free((Malloc_t) t->buf, "text_block_buff");
-    mush_free((Malloc_t) t, "text_block");
+    slab_free(text_block_slab, t);
   }
 }
 
