@@ -16,6 +16,10 @@
 #include <ctype.h>
 #include <signal.h>
 #include <errno.h>
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#include <stdio.h>
 
 #include "conf.h"
 #include "externs.h"
@@ -27,7 +31,7 @@
 #include "mushdb.h"
 #include "parse.h"
 #include "attrib.h"
-#include "pcre.h"
+#include "mypcre.h"
 #include "flags.h"
 #include "log.h"
 #include "mymalloc.h"
@@ -40,7 +44,7 @@ int global_fun_invocations;
 int global_fun_recursions;
 /* extern int re_subpatterns; */
 /* extern int *re_offsets; */
-/* extern char *re_from; */
+/* extern ansi_string *re_from; */
 extern sig_atomic_t cpu_time_limit_hit;
 extern int cpu_limit_warning_sent;
 
@@ -145,13 +149,12 @@ qparse_dbref(const char *s)
 dbref
 parse_objid(char const *str)
 {
-  char *p;
+  const char *p;
   if ((p = strchr(str, ':'))) {
     char tbuf1[BUFFER_LEN];
     dbref it;
     /* A unique id, probably */
-    strncpy(tbuf1, str, (p - str));
-    tbuf1[p - str] = '\0';
+    mush_strncpy(tbuf1, str, (p - str) + 1);
     it = parse_dbref(tbuf1);
     if (GoodObject(it)) {
       time_t matchtime;
@@ -177,7 +180,7 @@ parse_objid(char const *str)
  * \retval 1 str represents a true value.
  * \retval 0 str represents a false value.
  */
-int
+bool
 parse_boolean(char const *str)
 {
   if (TINY_BOOLEANS) {
@@ -213,7 +216,7 @@ parse_boolean(char const *str)
  * \retval 1 string is a valid boolean.
  * \retval 0 string is not a valid boolean.
  */
-int
+bool
 is_boolean(char const *str)
 {
   if (TINY_BOOLEANS)
@@ -229,7 +232,7 @@ is_boolean(char const *str)
  * \retval 1 string is a dbref.
  * \retval 0 string is not a dbref.
  */
-int
+bool
 is_dbref(char const *str)
 {
   if (!str || (*str != NUMBER_TOKEN) || !*(str + 1))
@@ -251,29 +254,32 @@ is_dbref(char const *str)
  * \retval 1 string is an objid
  * \retval 0 string is not an objid.
  */
-int
+bool
 is_objid(char const *str)
 {
   static pcre *re = NULL;
   const char *errptr;
   int erroffset;
+  char *val;
+  size_t vlen;
 
   if (!str)
     return 0;
   if (!re)
     re = pcre_compile("^#-?\\d+(?::\\d+)?$", 0, &errptr, &erroffset, NULL);
-  return (pcre_exec(re, NULL, str, strlen(str), 0, 0, NULL, 0) >= 0);
+  val = remove_markup((const char *) str, &vlen);
+  return pcre_exec(re, NULL, val, vlen - 1, 0, 0, NULL, 0) >= 0;
 }
 
 /** Is string an integer?
  * To TinyMUSH, any string is an integer. To PennMUSH, a string that
- * passes strtol is an integer, and a blank string is an integer
+ * passes parse_int is an integer, and a blank string is an integer
  * if NULL_EQ_ZERO is turned on.
  * \param str string to check.
  * \retval 1 string is an integer.
  * \retval 0 string is not an integer.
  */
-int
+bool
 is_integer(char const *str)
 {
   char *end;
@@ -288,21 +294,21 @@ is_integer(char const *str)
   if (*str == '\0')
     return NULL_EQ_ZERO;
   errno = 0;
-  strtol(str, &end, 10);
+  parse_int(str, &end, 10);
   if (errno == ERANGE || *end != '\0')
     return 0;
   return 1;
 }
 
-/** Is string an uinteger?
+/** Is string an unsigned integer?
  * To TinyMUSH, any string is an uinteger. To PennMUSH, a string that
- * passes strtoul is an uinteger, and a blank string is an uinteger
+ * passes parse_uint is an uinteger, and a blank string is an uinteger
  * if NULL_EQ_ZERO is turned on.
  * \param str string to check.
  * \retval 1 string is an uinteger.
  * \retval 0 string is not an uinteger.
  */
-int
+bool
 is_uinteger(char const *str)
 {
   char *end;
@@ -320,7 +326,7 @@ is_uinteger(char const *str)
   if (!(isdigit((unsigned char) *str) || *str == '+'))
     return 0;
   errno = 0;
-  strtoul(str, &end, 10);
+  parse_uint(str, &end, 10);
   if (errno == ERANGE || *end != '\0')
     return 0;
   return 1;
@@ -332,7 +338,7 @@ is_uinteger(char const *str)
  * \retval 1 string is a strict number.
  * \retval 0 string is not a strict number.
  */
-int
+bool
 is_strict_number(char const *str)
 {
   char *end;
@@ -346,12 +352,14 @@ is_strict_number(char const *str)
   return end > str;
 }
 
+#ifndef HAVE_ISNORMAL
 /** Is string a number that isn't inf or nan?
+ * Only needed for systems without isnormal()
  * \param num NVAL
  * \retval 1 num is a good number.
  * \retval 0 num is not a good number.
  */
-int
+bool
 is_good_number(NVAL val)
 {
   char numbuff[128];
@@ -366,14 +374,15 @@ is_good_number(NVAL val)
     return 0;
   return 1;
 }
+#endif
 
 /** Is string an integer by the strict definition?
- * A strict integer is a non-null string that passes strtol.
+ * A strict integer is a non-null string that passes parse_int.
  * \param str string to check.
  * \retval 1 string is a strict integer.
  * \retval 0 string is not a strict integer.
  */
-int
+bool
 is_strict_integer(char const *str)
 {
   char *end;
@@ -381,7 +390,7 @@ is_strict_integer(char const *str)
   if (!str)
     return 0;
   errno = 0;
-  val = strtol(str, &end, 10);
+  val = parse_int(str, &end, 10);
   if (errno == ERANGE || *end != '\0')
     return 0;
   return end > str;
@@ -394,7 +403,7 @@ is_strict_integer(char const *str)
  * \retval 1 string is a number.
  * \retval 0 string is not a number.
  */
-int
+bool
 is_number(char const *str)
 {
   /* If we're emulating Tiny, anything is a number */
@@ -407,6 +416,137 @@ is_number(char const *str)
   return is_strict_number(str);
 }
 
+/** Convert a string containing a signed integer into an int.
+ * Does not do any format checking. Invalid strings will return 0.
+ * Use this instead of strtol() when storing to an int to avoid problems
+ * where sizeof(int) < sizeof(long).
+ * \param s The string to convert
+ * \param end pointer to store the end of the parsed part of the string in
+ * if not NULL.
+ * \param base the base to convert from.
+ * \return the number, or INT_MIN on underflow, INT_MAX on overflow,
+ * with errno set to ERANGE.
+ */
+int
+parse_int(const char *s, char **end, int base)
+{
+  long x;
+
+  x = strtol(s, end, base);
+
+#if SIZEOF_INT == SIZEOF_LONG
+  return x;
+#else
+  /* These checks are only meaningful on 64-bit systems */
+  if (x < INT_MIN) {
+    errno = ERANGE;
+    return INT_MIN;
+  } else if (x > INT_MAX) {
+    errno = ERANGE;
+    return INT_MAX;
+  } else
+    return x;
+#endif
+}
+
+/** Convert a string containing a signed integer into an int32_t.
+ * Does not do any format checking. Invalid strings will return 0.
+ * \param s The string to convert
+ * \param end pointer to store the end of the parsed part of the string in
+ * if not NULL.
+ * \param base the base to convert from.
+ * \return the number, or INT32_MIN on underflow, INT32_MAX on overflow,
+ * with errno set to ERANGE.
+ */
+int32_t
+parse_int32(const char *s, char **end, int base)
+{
+#if SIZEOF_INT == 4
+  return parse_int(s, end, base);
+#elif defined(SCNd32)
+  /* This won't do overflow checking, which is why it isn't first */
+  const char *fmt;
+  int32_t val;
+
+  if (base == 10)
+    fmt = "%" SCNd32;
+  else if (base == 8)
+    fmt = "%" SCNo32;
+  else if (base == 16)
+    fmt = "%" SCNx32;
+  else
+    /* Unsupported base in this mode */
+    return 0;
+
+  if (sscanf(s, fmt, &val) != 1)
+    return 0;
+  else
+    return val;
+#else
+#error "No way to parse a 32-bit integer string"
+#endif
+}
+
+
+/** Convert a string containing an unsigned integer into an int.
+ * Does not  do any format checking. Invalid strings will return 0.
+ * Use this instead of strtoul() when storing to an int to avoid problems
+ * where  sizeof(int) < sizeof(long).
+ * \param s The string to convert
+ * \return the number, or UINT_MAX on overflow
+ * with errno set to ERANGE.
+ */
+unsigned int
+parse_uint(const char *s, char **end, int base)
+{
+  unsigned long x;
+
+  x = strtoul(s, end, base);
+
+#if SIZEOF_INT == SIZEOF_LONG
+  return x;
+#else
+  /* These checks are only meaningful on 64-bit systems */
+  if (x > UINT_MAX) {
+    errno = ERANGE;
+    return UINT_MAX;
+  } else
+    return x;
+#endif
+}
+
+/** Convert a string containing an unsigned integer into an uint32_t.
+ * Does not do any format checking. Invalid strings will return 0.
+ * \param s The string to convert
+ * \param end pointer to store the end of the parsed part of the string in
+ * if not NULL.
+ * \param base the base to convert from.
+ * \return the number, or UINT32_MIN on underflow, UINT32_MAX on overflow,
+ * with errno set to ERANGE.
+ */
+uint32_t
+parse_uint32(const char *s, char **end, int base)
+{
+#if SIZEOF_INT == 4
+  return parse_uint(s, end, base);
+#elif defined(SCNu32)
+  /* This won't do overflow checking, which is why it isn't first */
+  const char *fmt;
+  uint32_t val;
+
+  if (base != 10)
+    return 0;
+
+  if (sscanf(s, SCNu32, &val) != 1)
+    return 0;
+  else
+    return val;
+#else
+#error "No way to parse a 32-bit integer string"
+#endif
+}
+
+
 /* Table of interesting characters for process_expression() */
 extern char active_table[UCHAR_MAX + 1];
 /* Indexes of valid q-regs into the renv array. -1 is error. */
@@ -470,7 +610,7 @@ extern signed char qreg_indexes[UCHAR_MAX + 1];
 int
 process_expression(char *buff, char **bp, char const **str,
                    dbref executor, dbref caller, dbref enactor,
-                   int eflags, int tflags, PE_Info * pe_info)
+                   int eflags, int tflags, PE_Info *pe_info)
 {
   int debugging = 0, made_info = 0;
   char *debugstr = NULL, *sourcestr = NULL;
@@ -483,9 +623,7 @@ process_expression(char *buff, char **bp, char const **str,
   int temp_eflags;
   int old_iter_limit;
   int qindex;
-  int e_len;
   int retval = 0;
-  const char *e_msg;
 
   if (!buff || !bp || !str || !*str)
     return 0;
@@ -549,10 +687,12 @@ process_expression(char *buff, char **bp, char const **str,
   }
 
   if (CALL_LIMIT && (pe_info->call_depth++ > CALL_LIMIT)) {
+    const char *e_msg;
+    size_t e_len;
     e_msg = T(e_call);
     e_len = strlen(e_msg);
     if ((buff + e_len > *bp) || strcmp(e_msg, *bp - e_len))
-      safe_str(e_msg, buff, bp);
+      safe_strl(e_msg, e_len, buff, bp);
     goto exit_sequence;
   }
 
@@ -655,7 +795,16 @@ process_expression(char *buff, char **bp, char const **str,
     }
 
     switch (**str) {
-    case 0x1B:                 /* ANSI escapes. */
+    case TAG_START:
+      /* Skip over until TAG_END. */
+      for (; *str && **str && **str != TAG_END; (*str)++)
+        safe_chr(**str, buff, bp);
+      if (*str && **str) {
+        safe_chr(**str, buff, bp);
+        (*str)++;
+      }
+      break;
+    case ESC_CHAR:             /* ANSI escapes. */
       /* Skip over until the 'm' that matches the end. */
       for (; *str && **str && **str != 'm'; (*str)++)
         safe_chr(**str, buff, bp);
@@ -669,12 +818,10 @@ process_expression(char *buff, char **bp, char const **str,
           global_eval_context.re_subpatterns >= 0 &&
           global_eval_context.re_offsets != NULL &&
           global_eval_context.re_from != NULL) {
-        char obuf[BUFFER_LEN];
         int p = -1;
         char subspace[BUFFER_LEN];
         char *named_substring = NULL;
 
-        obuf[0] = '\0';
         (*str)++;
         /* Check the first character after the $ for a number */
         if (isdigit((unsigned char) **str)) {
@@ -698,19 +845,18 @@ process_expression(char *buff, char **bp, char const **str,
           break;
         }
 
-        if (named_substring) {
-          pcre_copy_named_substring(global_eval_context.re_code,
-                                    global_eval_context.re_from,
-                                    global_eval_context.re_offsets,
-                                    global_eval_context.re_subpatterns,
-                                    named_substring, obuf, BUFFER_LEN);
+        if (named_substring != NULL) {
+          ansi_pcre_copy_named_substring(global_eval_context.re_code,
+                                         global_eval_context.re_from,
+                                         global_eval_context.re_offsets,
+                                         global_eval_context.re_subpatterns,
+                                         named_substring, 0, buff, bp);
         } else {
-          pcre_copy_substring(global_eval_context.re_from,
-                              global_eval_context.re_offsets,
-                              global_eval_context.re_subpatterns,
-                              p, obuf, BUFFER_LEN);
+          ansi_pcre_copy_substring(global_eval_context.re_from,
+                                   global_eval_context.re_offsets,
+                                   global_eval_context.re_subpatterns,
+                                   p, 0, buff, bp);
         }
-        safe_str(obuf, buff, bp);
       } else {
         safe_chr('$', buff, bp);
         (*str)++;
@@ -1032,7 +1178,9 @@ process_expression(char *buff, char **bp, char const **str,
         for (sp = startpos, tp = name; sp < *bp; sp++)
           safe_chr(UPCASE(*sp), name, &tp);
         *tp = '\0';
-        fp = func_hash_lookup(name);
+        fp = (eflags & PE_BUILTINONLY) ? builtin_func_hash_lookup(name)
+          : func_hash_lookup(name);
+        eflags &= ~PE_BUILTINONLY;      /* Only applies to the outermost call */
         if (!fp) {
           if (eflags & PE_FUNCTION_MANDATORY) {
             *bp = startpos;
@@ -1070,10 +1218,12 @@ process_expression(char *buff, char **bp, char const **str,
         /* Check for the invocation limit */
         if ((pe_info->fun_invocations >= FUNCTION_LIMIT) ||
             (global_fun_invocations >= FUNCTION_LIMIT * 5)) {
+          const char *e_msg;
+          size_t e_len;
           e_msg = T(e_invoke);
           e_len = strlen(e_msg);
           if ((buff + e_len > *bp) || strcmp(e_msg, *bp - e_len))
-            safe_str(e_msg, buff, bp);
+            safe_strl(e_msg, e_len, buff, bp);
           if (process_expression(name, &tp, str,
                                  executor, caller, enactor,
                                  PE_NOTHING, PT_PAREN, pe_info))
@@ -1107,6 +1257,7 @@ process_expression(char *buff, char **bp, char const **str,
           break;
         }
         denied = !check_func(executor, fp);
+        denied = denied || ((fp->flags & FN_USERFN) && !(eflags & PE_USERFN));
         if (denied)
           temp_eflags &=
             ~(PE_COMPRESS_SPACES | PE_EVALUATE | PE_FUNCTION_CHECK);
@@ -1119,20 +1270,18 @@ process_expression(char *buff, char **bp, char const **str,
           if (nfargs >= args_alloced) {
             char **nargs;
             int *narglens;
-            nargs = (char **) mush_malloc((nfargs + 10) * sizeof(char *),
-                                          "process_expression.function_arglist");
-            narglens = (int *) mush_malloc((nfargs + 10) * sizeof(int),
-                                           "process_expression.function_arglens");
+            nargs = mush_calloc(nfargs + 10, sizeof(char *),
+                                "process_expression.function_arglist");
+            narglens = mush_calloc(nfargs + 10, sizeof(int),
+                                   "process_expression.function_arglens");
             for (j = 0; j < nfargs; j++) {
               nargs[j] = fargs[j];
               narglens[j] = arglens[j];
             }
             if (fargs != sargs)
-              mush_free((Malloc_t) fargs,
-                        "process_expression.function_arglist");
+              mush_free(fargs, "process_expression.function_arglist");
             if (arglens != sarglens)
-              mush_free((Malloc_t) arglens,
-                        "process_expression.function_arglens");
+              mush_free(arglens, "process_expression.function_arglens");
             fargs = nargs;
             arglens = narglens;
             args_alloced += 10;
@@ -1176,25 +1325,23 @@ process_expression(char *buff, char **bp, char const **str,
             nfargs = 0;
           }
           if ((nfargs < fp->minargs) || (nfargs > abs(fp->maxargs))) {
-            safe_str(T("#-1 FUNCTION ("), buff, bp);
-            safe_str(fp->name, buff, bp);
-            safe_str(") EXPECTS ", buff, bp);
+            safe_format(buff, bp, T("#-1 FUNCTION (%s) EXPECTS "), fp->name);
             if (fp->minargs == abs(fp->maxargs)) {
               safe_integer(fp->minargs, buff, bp);
             } else if ((fp->minargs + 1) == abs(fp->maxargs)) {
               safe_integer(fp->minargs, buff, bp);
-              safe_str(" OR ", buff, bp);
+              safe_str(T(" OR "), buff, bp);
               safe_integer(abs(fp->maxargs), buff, bp);
             } else if (fp->maxargs == INT_MAX) {
-              safe_str("AT LEAST ", buff, bp);
+              safe_str(T("AT LEAST "), buff, bp);
               safe_integer(fp->minargs, buff, bp);
             } else {
-              safe_str("BETWEEN ", buff, bp);
+              safe_str(T("BETWEEN "), buff, bp);
               safe_integer(fp->minargs, buff, bp);
-              safe_str(" AND ", buff, bp);
+              safe_str(T(" AND "), buff, bp);
               safe_integer(abs(fp->maxargs), buff, bp);
             }
-            safe_str(" ARGUMENTS BUT GOT ", buff, bp);
+            safe_str(T(" ARGUMENTS BUT GOT "), buff, bp);
             safe_integer(nfargs, buff, bp);
           } else {
             global_fun_recursions++;
@@ -1226,25 +1373,25 @@ process_expression(char *buff, char **bp, char const **str,
               ATTR *attrib;
               global_fun_invocations++;
               pe_info->fun_invocations++;
-              thing = userfn_tab[fp->where.offset].thing;
-              attrib = atr_get(thing, userfn_tab[fp->where.offset].name);
+              thing = fp->where.ufun->thing;
+              attrib = atr_get(thing, fp->where.ufun->name);
               if (!attrib) {
                 do_rawlog(LT_ERR,
                           T("ERROR: @function (%s) without attribute (#%d/%s)"),
-                          fp->name, thing, userfn_tab[fp->where.offset].name);
+                          fp->name, thing, fp->where.ufun->name);
                 safe_str("#-1 @FUNCTION (", buff, bp);
                 safe_str(fp->name, buff, bp);
                 safe_str(") MISSING ATTRIBUTE (", buff, bp);
                 safe_dbref(thing, buff, bp);
                 safe_chr('/', buff, bp);
-                safe_str(userfn_tab[fp->where.offset].name, buff, bp);
+                safe_str(fp->where.ufun->name, buff, bp);
                 safe_chr(')', buff, bp);
               } else {
                 char *preserve[NUMQ];
                 if (fp->flags & FN_LOCALIZE)
                   save_global_regs("@function.save", preserve);
                 do_userfn(buff, bp, thing, attrib, nfargs, fargs,
-                          executor, caller, enactor, pe_info);
+                          executor, caller, enactor, pe_info, PE_USERFN);
                 if (fp->flags & FN_LOCALIZE)
                   restore_global_regs("@function.save", preserve);
               }
