@@ -18,6 +18,9 @@
 #endif
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -128,7 +131,7 @@ charge_action(dbref player, dbref thing, const char *awhat)
       return res;
     } else {
       /* no charges left, try to execute runout */
-      return queue_attribute(thing, "RUNOUT", 0);
+      return queue_attribute(thing, "RUNOUT", player);
     }
   }
 }
@@ -633,7 +636,7 @@ void
 change_quota(dbref who, int payment)
 {
   (void) atr_add(Owner(who), "RQUOTA",
-                 tprintf("%d", get_current_quota(who) + payment), GOD, NOTHING);
+                 tprintf("%d", get_current_quota(who) + payment), GOD, 0);
 }
 
 /** Debit a player's quota, if they can afford it.
@@ -1056,7 +1059,7 @@ do_switch(dbref player, char *expression, char **argv, dbref cause,
  * \return matching dbref or NOTHING or AMBIGUOUS.
  */
 dbref
-parse_match_possessor(dbref player, const char **str)
+parse_match_possessor(dbref player, char **str)
 {
   const char *box;              /* name of container */
   char *obj;                    /* name of object */
@@ -1364,7 +1367,7 @@ grep_helper(dbref player, dbref thing __attribute__ ((__unused__)),
       strncpy(buf, s, gh->len);
       buf[gh->len] = '\0';
       s += gh->len;
-      safe_format(tbuf1, &tbp, "%s%s%s", ANSI_HILITE, buf, ANSI_NORMAL);
+      safe_format(tbuf1, &tbp, "%s%s%s", ANSI_HILITE, buf, ANSI_END);
     } else {
       safe_chr(*s, tbuf1, &tbp);
       s++;
@@ -1378,7 +1381,7 @@ grep_helper(dbref player, dbref thing __attribute__ ((__unused__)),
                   ANSI_HILITE, AL_NAME(atr),
                   Owner(AL_CREATOR(atr)),
                   privs_to_letters(attr_privs_view, AL_FLAGS(atr)),
-                  ANSI_NORMAL, tbuf1);
+                  ANSI_END, tbuf1);
   return found;
 }
 
