@@ -30,7 +30,7 @@
 #define THIS_SERVICE_DISPLAY "PennMUSH for Win32"
 
 int WIN32_CDECL main(int argc, char **argv);
-void mainthread(int argc, char **argv);
+int mainthread(int argc, char **argv);
 
 
 SERVICE_STATUS ssStatus;        /*  current status of the service */
@@ -68,9 +68,7 @@ int WIN32_CDECL service_error(DWORD error_code, char *themessage, ...);
 
 
 int WIN32_CDECL
-main(argc, argv)
-    int argc;
-    char **argv;
+main(int argc, char **argv)
 {
 
   SERVICE_TABLE_ENTRY dispatchTable[] = {
@@ -433,7 +431,7 @@ worker_thread(VOID * notused)
 
 /*  start up the main MUSH code */
 
-  mainthread(argc, argv);
+  exit(mainthread(argc, argv));
 
 }                               /*  end of worker_thread */
 
