@@ -13,6 +13,9 @@
 #include <string.h>
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -60,7 +63,7 @@ void
 do_name(dbref player, const char *name, char *newname)
 {
   dbref thing;
-  char *password;
+  char *eon;                    /* End Of Name */
   char *myenv[10];
   int i;
 
@@ -74,45 +77,26 @@ do_name(dbref player, const char *name, char *newname)
     }
     /* check for renaming a player */
     if (IsPlayer(thing)) {
-      if (PLAYER_NAME_SPACES) {
-        if (*newname == '\"') {
-          for (; *newname && ((*newname == '\"')
-                              || isspace((unsigned char) *newname));
-               newname++) ;
-          password = newname;
-          while (*password && (*password != '\"')) {
-            while (*password && (*password != '\"'))
-              password++;
-            if (*password == '\"') {
-              *password++ = '\0';
-              while (*password && isspace((unsigned char) *password))
-                password++;
-              break;
-            }
-          }
-        } else {
-          password = newname;
-          while (*password && !isspace((unsigned char) *password))
-            password++;
-          if (*password) {
-            *password++ = '\0';
-            while (*password && isspace((unsigned char) *password))
-              password++;
+      if (*newname == '"') {
+        for (; *newname && ((*newname == '"')
+                            || isspace((unsigned char) *newname)); newname++) ;
+        eon = newname;
+        while (*eon && (*eon != '"')) {
+          while (*eon && (*eon != '"'))
+            eon++;
+          if (*eon == '"') {
+            *eon++ = '\0';
+            while (*eon && isspace((unsigned char) *eon))
+              eon++;
+            break;
           }
         }
       } else {
-
-        /* split off password */
-        for (password = newname + strlen(newname) - 1;
-             *password && !isspace((unsigned char) *password); password--) ;
-        for (; *password && isspace((unsigned char) *password); password--) ;
-        /* eat whitespace */
-        if (*password) {
-          *++password = '\0';   /* terminate name */
-          password++;
-          while (*password && isspace((unsigned char) *password))
-            password++;
-        }
+        eon = newname;
+        while (*eon && !isspace((unsigned char) *eon))
+          eon++;
+        if (*eon)
+          *eon++ = '\0';
       }
       if (!ok_player_name(newname, player, thing)) {
         notify(player, T("You can't give a player that name."));
@@ -136,8 +120,7 @@ do_name(dbref player, const char *name, char *newname)
     /* everything ok, change the name */
     myenv[0] = (char *) mush_malloc(BUFFER_LEN, "string");
     myenv[1] = (char *) mush_malloc(BUFFER_LEN, "string");
-    strncpy(myenv[0], Name(thing), BUFFER_LEN - 1);
-    myenv[0][BUFFER_LEN - 1] = '\0';
+    mush_strncpy(myenv[0], Name(thing), BUFFER_LEN);
     strcpy(myenv[1], newname);
     for (i = 2; i < 10; i++)
       myenv[i] = NULL;
@@ -169,13 +152,11 @@ do_chown(dbref player, const char *name, const char *newobj, int preserve)
 {
   dbref thing;
   dbref newowner = NOTHING;
-  char *sp;
   long match_flags = MAT_POSSESSION | MAT_HERE | MAT_EXIT | MAT_ABSOLUTE;
 
 
   /* check for '@chown <object>/<atr>=<player>'  */
-  sp = strchr(name, '/');
-  if (sp) {
+  if (strchr(name, '/')) {
     do_atrchown(player, name, newobj);
     return;
   }
@@ -471,8 +452,8 @@ do_chzone(dbref player, char const *name, char const *newobj, int noisy)
 
 /** Structure for af_helper() data. */
 struct af_args {
-  int setf;             /**< flag bits to set */
-  int clrf;             /**< flag bits to clear */
+  privbits setf;        /**< flag bits to set */
+  privbits clrf;        /**< flag bits to clear */
   char *setflags;       /**< list of names of flags to set */
   char *clrflags;       /**< list of names of flags to clear */
 };
@@ -580,22 +561,39 @@ do_attrib_flags(dbref player, const char *obj, const char *atrname,
  * \retval 0 failure to set.
  */
 int
-do_set(dbref player, const char *name, char *flag)
+do_set(dbref player, const char *xname, char *flag)
 {
   dbref thing;
   int her, listener, negate;
-  char *p, *f;
+  char *p, *f, *name;
   char flagbuff[BUFFER_LEN];
 
+  if (!xname || !*xname) {
+    notify(player, T("I can't see that here."));
+    return 0;
+  }
+  if (!flag || !*flag) {
+    notify(player, T("What do you want to set?"));
+    return 0;
+  }
+
+  name = mush_strdup(xname, "ds.string");
+
   /* check for attribute flag set first */
   if ((p = strchr(name, '/')) != NULL) {
     *p++ = '\0';
     do_attrib_flags(player, name, p, flag);
+    mush_free(name, "ds.string");
     return 1;
   }
   /* find thing */
-  if ((thing = match_controlled(player, name)) == NOTHING)
+  if ((thing = match_controlled(player, name)) == NOTHING) {
+    mush_free(name, "ds.string");
     return 0;
+  }
+
+  mush_free(name, "ds.string");
+
   if (God(thing) && !God(player)) {
     notify(player, T("Only God can set himself!"));
     return 0;
@@ -714,12 +712,12 @@ do_cpattr(dbref player, char *oldpair, char **newpair, int move, int noflagcopy)
   free((Malloc_t) text);        /* safe_uncompress malloc()s memory */
   if (copies) {
     notify_format(player, T("Attribute %s (%d copies)"),
-                  (move ? "moved" : "copied"), copies);
+                  (move ? T("moved") : T("copied")), copies);
     if (move)
       do_set_atr(oldobj, AL_NAME(a), NULL, player, 1);
   } else {
     notify_format(player, T("Unable to %s attribute."),
-                  (move ? "move" : "copy"));
+                  (move ? T("move") : T("copy")));
   }
   return;
 }
@@ -772,14 +770,14 @@ gedit_helper(dbref player, dbref thing,
     safe_str(r, tbuf1, &tbufp);
 
     if (safe_format(tbuf_ansi, &tbufap, "%s%s%s%s", s, ANSI_HILITE, r,
-                    ANSI_NORMAL))
+                    ANSI_END))
       ansi_long_flag = 1;
   } else if (vlen == 1 && *val == '^') {
     /* prepend */
     safe_str(r, tbuf1, &tbufp);
     safe_str(s, tbuf1, &tbufp);
 
-    if (safe_format(tbuf_ansi, &tbufap, "%s%s%s%s", ANSI_HILITE, r, ANSI_NORMAL,
+    if (safe_format(tbuf_ansi, &tbufap, "%s%s%s%s", ANSI_HILITE, r, ANSI_END,
                     s))
       ansi_long_flag = 1;
   } else if (!*val) {
@@ -793,7 +791,7 @@ gedit_helper(dbref player, dbref thing,
     /* Add one at the start */
     if (!safe_strl(r, rlen, tbuf1, &tbufp)) {
       if (gargs->target != EDIT_FIRST) {
-        for (last = 0; last < haystack->len; last++) {
+        for (last = 0; last < (size_t) haystack->len; last++) {
           /* Add the next character */
           if (safe_ansi_string(haystack, last, 1, tbuf1, &tbufp)) {
             too_long = 1;
@@ -804,13 +802,13 @@ gedit_helper(dbref player, dbref thing,
               ansi_long_flag = 1;
           }
           /* Copy in r */
-          if (safe_strl(r, rlen, tbuf1, &tbufp)) {
+          if (safe_str(r, tbuf1, &tbufp)) {
             too_long = 1;
             break;
           }
           if (!ansi_long_flag) {
             if (safe_format(tbuf_ansi, &tbufap, "%s%s%s", ANSI_HILITE, r,
-                            ANSI_NORMAL))
+                            ANSI_END))
               ansi_long_flag = 1;
           }
         }
@@ -826,7 +824,7 @@ gedit_helper(dbref player, dbref thing,
 
     haystack = parse_ansi_string(s);
 
-    while (last < haystack->len
+    while (last < (size_t) haystack->len
            && (p = strstr(haystack->text + last, val)) != NULL) {
       if (safe_ansi_string(haystack, last, p - (haystack->text + last),
                            tbuf1, &tbufp)) {
@@ -840,20 +838,19 @@ gedit_helper(dbref player, dbref thing,
       }
 
       /* Copy in r */
-      if (safe_strl(r, rlen, tbuf1, &tbufp)) {
+      if (safe_str(r, tbuf1, &tbufp)) {
         too_long = 1;
         break;
       }
       if (!ansi_long_flag) {
-        if (safe_format(tbuf_ansi, &tbufap, "%s%s%s", ANSI_HILITE, r,
-                        ANSI_NORMAL))
+        if (safe_format(tbuf_ansi, &tbufap, "%s%s%s", ANSI_HILITE, r, ANSI_END))
           ansi_long_flag = 1;
       }
       last = p - haystack->text + vlen;
       if (gargs->target == EDIT_FIRST)
         break;
     }
-    if (last < haystack->len && !too_long) {
+    if (last < (size_t) haystack->len && !too_long) {
       safe_ansi_string(haystack, last, haystack->len, tbuf1, &tbufp);
       if (!ansi_long_flag) {
         if (safe_ansi_string(haystack, last, haystack->len, tbuf_ansi, &tbufap))
@@ -870,9 +867,9 @@ gedit_helper(dbref player, dbref thing,
     if (do_set_atr(thing, AL_NAME(a), tbuf1, player, 0) &&
         !AreQuiet(player, thing)) {
       if (!ansi_long_flag && ShowAnsi(player))
-        notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf_ansi);
+        notify_format(player, T("%s - Set: %s"), AL_NAME(a), tbuf_ansi);
       else
-        notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf1);
+        notify_format(player, T("%s - Set: %s"), AL_NAME(a), tbuf1);
     }
   } else {
     /* We don't do it - we just pemit it. */
@@ -1002,7 +999,7 @@ do_use(dbref player, const char *what)
       fail_lock(player, thing, Use_Lock, T("Permission denied."), NOTHING);
       return;
     } else
-      did_it(player, thing, "USE", "Used.", "OUSE", NULL, "AUSE", NOTHING);
+      did_it(player, thing, "USE", T("Used."), "OUSE", NULL, "AUSE", NOTHING);
   }
 }
 
@@ -1087,6 +1084,7 @@ wipe_helper(dbref player, dbref thing,
 
   if (wildcard(pattern) && AF_Wizard(atr) && !God(player))
     return 0;
+
   switch (wipe_atr(thing, AL_NAME(atr), player)) {
   case AE_SAFE:
     notify_format(player, T("Attribute %s is SAFE. Set it !SAFE to modify it."),
@@ -1106,7 +1104,6 @@ wipe_helper(dbref player, dbref thing,
   }
 }
 
-
 /** Clear an attribute.
  * \verbatim
  * This implements @wipe.
@@ -1140,7 +1137,8 @@ do_wipe(dbref player, char *name)
     return;
   }
 
-  switch ((wiped = atr_iter_get(player, thing, pattern, 0, wipe_helper, NULL))) {
+  wiped = atr_iter_get(player, thing, pattern, 0, wipe_helper, NULL);
+  switch (wiped) {
   case 0:
     notify(player, T("No attributes wiped."));
     break;
