@@ -12,9 +12,10 @@
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include "conf.h"
-#include "ansi.h"
 #include "externs.h"
+#include "ansi.h"
 #include "mushdb.h"
 #include "dbdefs.h"
 #include "lock.h"
@@ -24,7 +25,8 @@
 #include "attrib.h"
 #include "parse.h"
 #include "game.h"
-#include "pcre.h"
+#include "mypcre.h"
+#include "sort.h"
 #include "confmagic.h"
 
 int okay_pemit(dbref player, dbref target);
@@ -130,7 +132,7 @@ do_teach(dbref player, dbref cause, const char *tbuf1)
   recurse = 1;                  /* Protect use from recursive teach */
   notify_except(Contents(loc), NOTHING,
                 tprintf(T("%s types --> %s%s%s"), spname(player),
-                        ANSI_HILITE, tbuf1, ANSI_NORMAL), NA_INTER_HEAR);
+                        ANSI_HILITE, tbuf1, ANSI_END), NA_INTER_HEAR);
   command = mush_strdup(tbuf1, "string");       /* process_command is destructive */
   process_command(player, command, cause, 1);
   mush_free(command, "string");
@@ -165,19 +167,6 @@ do_say(dbref player, const char *tbuf1)
                 NA_INTER_HEAR);
 }
 
-/** A comparator for raw dbrefs.
- * This exists because the i_comp function in funlist.c got hairy some time ago.
- * \param a the first element to compare.
- * \param b the second element to compare.
- */
-static int
-dbref_comp(const void *a, const void *b)
-{
-  const dbref *x = (const dbref *) a;
-  const dbref *y = (const dbref *) b;
-  return (int) *x - (int) *y;
-}
-
 /** The oemit(/list) command.
  * \verbatim
  * This implements @oemit and @oemit/list.
@@ -357,7 +346,7 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
     notify_format(player, T("Unable to whisper to:%s"), tbuf);
 
   if (!gcount) {
-    mush_free((Malloc_t) tbuf, "string");
+    mush_free(tbuf, "string");
     return;
   }
 
@@ -367,9 +356,9 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
 
   /* Set up list of good names */
   tp = tbuf;
-  safe_str(" to ", tbuf, &tp);
+  safe_str(T(" to "), tbuf, &tp);
   for (who = 0; who < gcount; who++) {
-    if (noisy && (get_random_long(0, 100) < WHISPER_LOUDNESS))
+    if (noisy && (get_random32(0, 100) < (uint32_t) WHISPER_LOUDNESS))
       overheard = 1;
     safe_itemizer(who + 1, (who == gcount - 1), ",", T("and"), " ", tbuf, &tp);
     safe_str(Name(good[who]), tbuf, &tp);
@@ -382,7 +371,7 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
     p = tprintf("You sense: %s%s%s", Name(player), gap, arg2);
   } else {
     notify_format(player, T("You whisper, \"%s\"%s."), arg2, tbuf);
-    p = tprintf("%s whispers%s: %s", Name(player),
+    p = tprintf(T("%s whispers%s: %s"), Name(player),
                 gcount > 1 ? tbuf : "", arg2);
   }
 
@@ -395,7 +384,7 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
     dbref first = Contents(Location(player));
     if (!GoodObject(first))
       return;
-    p = tprintf("%s whispers%s.", Name(player), tbuf);
+    p = tprintf(T("%s whispers%s."), Name(player), tbuf);
     DOLIST(first, first) {
       overheard = 1;
       for (who = 0; who < gcount; who++) {
@@ -408,7 +397,64 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
         notify_noecho(first, p);
     }
   }
-  mush_free((Malloc_t) tbuf, "string");
+  mush_free(tbuf, "string");
+}
+
+/** Send an @message to a list of dbrefs, using <attr> to format it
+ * if present.
+ * The list is destructively modified.
+ * \param player the enactor.
+ * \param list the list of players to pemit to, destructively modified.
+ * \param attrib the ufun attribute to use to format the message.
+ * \param message the default message.
+ * \param flags PEMIT_* flags
+ * \param numargs The number of arguments for the ufun.
+ * \param ... The arguments for the ufun.
+ */
+void
+do_message_list(dbref player, dbref enactor, char *list, char *attrname,
+                char *message, int flags, int numargs, char *argv[])
+{
+  const char *start;
+  char *current;
+  char plist[BUFFER_LEN], *pp;
+  dbref victim;
+  int first = 0;
+  ATTR *attrib;
+
+  start = list;
+
+  pp = plist;
+  *pp = '\0';
+
+  while (start && *start) {
+    current = next_in_list(&start);
+    if (*current == '*')
+      current = current + 1;
+    victim = noisy_match_result(player, current, NOTYPE, MAT_EVERYTHING);
+    if (GoodObject(victim) && !IsGarbage(victim)) {
+      /* Can we evaluate its <attribute> ? */
+
+      attrib = atr_get(victim, upcasestr(attrname));
+      if (attrib && CanEvalAttr(player, victim, attrib)) {
+        if (flags & PEMIT_SPOOF) {
+          messageformat(victim, attrname, enactor, NA_SPOOF, numargs, argv);
+        } else {
+          messageformat(victim, attrname, enactor, 0, numargs, argv);
+        }
+      } else {
+        if (!first) {
+          safe_chr(' ', plist, &pp);
+        }
+        first = 0;
+        safe_dbref(victim, plist, &pp);
+      }
+    }
+  }
+  if (plist[0]) {
+    *pp = '\0';
+    do_pemit_list(enactor, plist, message, flags);
+  }
 }
 
 /** Send a message to a list of dbrefs. To avoid repeated generation
@@ -627,7 +673,76 @@ do_wall(dbref player, const char *message, enum wall_type target, int emit)
   else
     flag_broadcast(mask, 0,
                    "%s %s %s, \"%s\"", prefix, Name(player),
-                   target == WALL_ALL ? "shouts" : "says", message);
+                   target == WALL_ALL ? T("shouts") : T("says"), message);
+}
+
+/** messageformat. This is the wrapper that makes calling PAGEFORMAT,
+ *  CHATFORMAT, etc easy.
+ *
+ * \param player The victim to call it on.
+ * \param attribute The attribute on the player to call.
+ * \param enactor The enactor who caused the message.
+ * \param flags NA_INTER_HEAR and NA_SPOOF
+ * \param arg0 First argument
+ * \param arg4 Last argument.
+ * \retval 1 The player had the fooformat attribute.
+ * \retval 0 The default message was sent.
+ */
+int
+vmessageformat(dbref player, const char *attribute, dbref enactor, int flags,
+               int numargs, ...)
+{
+  va_list ap;
+  char *s;
+  int i;
+  char *argv[10];
+
+  va_start(ap, numargs);
+
+  for (i = 0; i < 10; i++) {
+    if (i < numargs) {
+      /* Pop another char * off the stack. */
+      s = va_arg(ap, char *);
+      argv[i] = s;
+    } else {
+      argv[i] = NULL;
+    }
+  }
+  va_end(ap);
+
+  return messageformat(player, attribute, enactor, flags, numargs, argv);
+}
+
+/** messageformat. This is the wrapper that makes calling PAGEFORMAT,
+ *  CHATFORMAT, etc easy.
+ *
+ * \param player The victim to call it on.
+ * \param attribute The attribute on the player to call.
+ * \param enactor The enactor who caused the message.
+ * \param flags NA_INTER_HEAR and NA_SPOOF
+ * \param arg0 First argument
+ * \param arg4 Last argument.
+ * \retval 1 The player had the fooformat attribute.
+ * \retval 0 The default message was sent.
+ */
+int
+messageformat(dbref player, const char *attribute, dbref enactor, int flags,
+              int numargs, char *argv[])
+{
+  /* It's only static because I expect this thing to get
+   * called a LOT, so it may or may not save time. */
+  static char messbuff[BUFFER_LEN];
+
+  *messbuff = '\0';
+  if (call_attrib(player, attribute, (const char **) argv, numargs,
+                  messbuff, enactor, NULL)) {
+    /* We have a returned value. Notify the player. */
+    if (*messbuff)
+      notify_anything(enactor, na_one, &player, ns_esnotify, flags, messbuff);
+    return 1;
+  } else {
+    return 0;
+  }
 }
 
 /** The page command.
@@ -664,7 +779,7 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
   ATTR *a;
   char *alias;
 
-  tp2 = tbuf2 = (char *) mush_malloc(BUFFER_LEN, "string");
+  tp2 = tbuf2 = (char *) mush_malloc(BUFFER_LEN, "page_buff");
   if (!tbuf2)
     mush_panic("Unable to allocate memory in do_page");
 
@@ -688,19 +803,19 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
     a = atr_get_noparent(player, "LASTPAGED");
     if (!a || !*((hp = head = safe_atr_value(a)))) {
       notify(player, T("You haven't paged anyone since connecting."));
-      mush_free((Malloc_t) tbuf2, "string");
+      mush_free((Malloc_t) tbuf2, "page_buff");
       return;
     }
     if (!message || !*message) {
       notify_format(player, T("You last paged %s."), head);
-      mush_free((Malloc_t) tbuf2, "string");
+      mush_free((Malloc_t) tbuf2, "page_buff");
       if (hp)
         free((Malloc_t) hp);
       return;
     }
   }
 
-  tp = tbuf = (char *) mush_malloc(BUFFER_LEN, "string");
+  tp = tbuf = (char *) mush_malloc(BUFFER_LEN, "page_buff");
   if (!tbuf)
     mush_panic("Unable to allocate memory in do_page");
 
@@ -769,8 +884,8 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
    * anyone, this looks like a spam attack. */
   if (gcount == 99) {
     notify(player, T("You're trying to page too many people at once."));
-    mush_free((Malloc_t) tbuf, "string");
-    mush_free((Malloc_t) tbuf2, "string");
+    mush_free((Malloc_t) tbuf, "page_buff");
+    mush_free((Malloc_t) tbuf2, "page_buff");
     if (hp)
       free((Malloc_t) hp);
     return;
@@ -786,8 +901,8 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
 
   if (!gcount) {
     /* Well, that was a total waste of time. */
-    mush_free((Malloc_t) tbuf, "string");
-    mush_free((Malloc_t) tbuf2, "string");
+    mush_free((Malloc_t) tbuf, "page_buff");
+    mush_free((Malloc_t) tbuf2, "page_buff");
     if (hp)
       free((Malloc_t) hp);
     return;
@@ -796,8 +911,8 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
   /* Can the player afford to pay for this thing? */
   if (!payfor(player, PAGE_COST * gcount)) {
     notify_format(player, T("You don't have enough %s."), MONIES);
-    mush_free((Malloc_t) tbuf, "string");
-    mush_free((Malloc_t) tbuf2, "string");
+    mush_free((Malloc_t) tbuf, "page_buff");
+    mush_free((Malloc_t) tbuf2, "page_buff");
     if (hp)
       free((Malloc_t) hp);
     return;
@@ -807,10 +922,10 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
    * actually going to someone. We're in this for keeps now. */
 
   /* Evaluate the message if we need to. */
-  if (noeval)
+  if (noeval) {
     msgbuf = NULL;
-  else {
-    mb = msgbuf = (char *) mush_malloc(BUFFER_LEN, "string");
+  } else {
+    mb = msgbuf = (char *) mush_malloc(BUFFER_LEN, "page_buff");
     if (!msgbuf)
       mush_panic("Unable to allocate memory in do_page");
 
@@ -872,7 +987,7 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
                   message);
 
   /* Figure out the 'name' of the player */
-  if (PAGE_ALIASES && (alias = shortalias(player)) && *alias)
+  if ((alias = shortalias(player)) && *alias && PAGE_ALIASES)
     current = tprintf("%s (%s)", Name(player), alias);
   else
     current = (char *) Name(player);
@@ -904,20 +1019,40 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
   safe_str(message, tbuf, &tp);
   *tp = '\0';
 
-  /* Tell each page recipient with tbuf */
+  tp2 = tbuf2;
+  if (multipage) {
+    for (i = 0; i < gcount; i++) {
+      if (i)
+        safe_chr(' ', tbuf2, &tp2);
+      safe_dbref(good[i], tbuf2, &tp2);
+    }
+  }
+  *tp2 = '\0';
   for (i = 0; i < gcount; i++) {
-    if (!IsPlayer(player) && Nospoof(good[i]))
-      notify_format(good[i], "[#%d] %s", player, tbuf);
-    else
+    if (!IsPlayer(player) && Nospoof(good[i])) {
+      if (msgbuf == NULL) {
+        msgbuf = mush_malloc(BUFFER_LEN, "page buffer");
+      }
+      snprintf(msgbuf, BUFFER_LEN, "[#%d] %s", player, tbuf);
+      /* Swap tbuf and msgbuf */
+      tp = tbuf;
+      tbuf = msgbuf;
+      msgbuf = tp;
+    }
+    if (!vmessageformat(good[i], "PAGEFORMAT", player, 0, 5, message,
+                        (key == 1) ? (*gap ? ":" : ";") : "\"",
+                        (alias && *alias) ? alias : "", tbuf2, tbuf)) {
+      /* Player doesn't have Pageformat, or it eval'd to 0 */
       notify(good[i], tbuf);
+    }
 
     page_return(player, good[i], "Idle", "IDLE", NULL);
   }
 
-  mush_free((Malloc_t) tbuf, "string");
-  mush_free((Malloc_t) tbuf2, "string");
+  mush_free((Malloc_t) tbuf, "page_buff");
+  mush_free((Malloc_t) tbuf2, "page_buff");
   if (msgbuf)
-    mush_free((Malloc_t) msgbuf, "string");
+    mush_free((Malloc_t) msgbuf, "page_buff");
   if (hp)
     free((Malloc_t) hp);
 }
