@@ -530,6 +530,25 @@ st_delete(char const *s, StrTree *root)
   root->count--;
 }
 
+static void
+st_node_walk(StrNode *node, STFunc callback, void *data)
+{
+  if (node->left)
+    st_node_walk(node->left, callback, data);
+  callback(node->string, node->info >> ST_COLOR, data);
+  if (node->right)
+    st_node_walk(node->right, callback, data);
+}
+
+/** Call a function for each node in the tree, in-order */
+void
+st_walk(StrTree *tree, STFunc callback, void *data)
+{
+  if (!tree || !tree->root)
+    return;
+  st_node_walk(tree->root, callback, data);
+}
+
 /* Print the tree, for debugging purposes. */
 static void
 st_print_tree(StrNode *node, int tree_depth, int lead)
@@ -588,7 +607,7 @@ st_depth_helper(StrNode *node, int *maxdepth, int *mindepth,
   if (node->info >= ST_USE_LIMIT)
     (*perms)++;
   else
-    (*nperms) += node->info;
+    (*nperms) += node->info >> ST_COLOR;
 
   if (count > *maxdepth)
     *maxdepth = count;
