@@ -8,30 +8,35 @@
 
 #include "config.h"
 
+#include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <limits.h>
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
 #include "copyrite.h"
 #include "conf.h"
 #include "case.h"
-#include "ansi.h"
 #include "pueblo.h"
 #include "parse.h"
 #include "externs.h"
+#include "ansi.h"
 #include "mymalloc.h"
 #include "log.h"
 #include "confmagic.h"
 
 char *next_token(char *str, char sep);
-int format_long(long val, char *buff, char **bp, int maxlen, int base);
-static char *
+int format_long(intmax_t val, char *buff, char **bp, int maxlen, int base);
+
+char *
 mush_strndup(const char *src, size_t len, const char *check)
   __attribute_malloc__;
 
 /* Duplicate the first len characters of s */
-    static char *mush_strndup(const char *src, size_t len, const char *check)
+    char *mush_strndup(const char *src, size_t len, const char *check)
 {
   char *copy;
   size_t rlen = strlen(src);
@@ -48,6 +53,7 @@ mush_strndup(const char *src, size_t len, const char *check)
   return copy;
 }
 
+
 /** Our version of strdup, with memory leak checking.
  * This should be used in preference to strdup, and in assocation
  * with mush_free().
@@ -60,7 +66,7 @@ mush_strdup(const char *s, const char *check __attribute__ ((__unused__)))
 {
   char *x;
 
-#ifdef HAS_STRDUP
+#ifdef HAVE_STRDUP
   x = strdup(s);
   if (x)
     add_check(check);
@@ -74,7 +80,22 @@ mush_strdup(const char *s, const char *check __attribute__ ((__unused__)))
   return x;
 }
 
+#ifdef HAVE__VSNPRINTF_S
+/** Wrapper for the Win32 _snprintf_s() function */
+int
+sane_snprintf_s(char *str, size_t len, const char *fmt, ...)
+{
+  va_list args;
+  int ret;
+  va_start(args, fmt);
+  ret = _vsnprintf_s(str, len, _TRUNCATE, fmt, args);
+  va_end(args);
+  return ret;
+}
+#endif
+
 /** Return the string chopped at lim characters.
+ * lim must be <= BUFFER_LEN
  * \param str string to chop.
  * \param lim character at which to chop the string.
  * \return statically allocated buffer with chopped string.
@@ -85,14 +106,14 @@ chopstr(const char *str, size_t lim)
   static char tbuf1[BUFFER_LEN];
   if (strlen(str) <= lim)
     return (char *) str;
-  strncpy(tbuf1, str, lim);
-  tbuf1[lim] = '\0';
+  if (lim >= BUFFER_LEN)
+    lim = BUFFER_LEN;
+  mush_strncpy(tbuf1, str, lim);
   return tbuf1;
 }
 
 
-#ifndef HAS_STRCASECMP
-#ifndef WIN32
+#if !defined(HAVE_STRCASECMP) && !defined(HAVE__STRICMP)
 /** strcasecmp for systems without it.
  * \param s1 one string to compare.
  * \param s2 another string to compare.
@@ -108,7 +129,9 @@ strcasecmp(const char *s1, const char *s2)
 
   return (DOWNCASE(*s1) - DOWNCASE(*s2));
 }
+#endif
 
+#if !defined(HAVE_STRNCASECMP) && !defined(HAVE__STRNICMP)
 /** strncasecmp for systems without it.
  * \param s1 one string to compare.
  * \param s2 another string to compare.
@@ -128,8 +151,7 @@ strncasecmp(const char *s1, const char *s2, size_t n)
   return 0;
 
 }
-#endif                          /* !WIN32 */
-#endif                          /* !HAS_STRCASECMP */
+#endif
 
 /** Does string begin with prefix? 
  * This comparison is case-insensitive.
@@ -164,11 +186,9 @@ string_match(const char *src, const char *sub)
       if (string_prefix(src, sub))
         return src;
       /* else scan to beginning of next word */
-      while (*src && (isalpha((unsigned char) *src)
-                      || isdigit((unsigned char) *src)))
+      while (*src && isalnum((unsigned char) *src))
         src++;
-      while (*src && !isalpha((unsigned char) *src)
-             && !isdigit((unsigned char) *src))
+      while (*src && !isalnum((unsigned char) *src))
         src++;
     }
   }
@@ -210,7 +230,7 @@ strupper(const char *s)
     buf1[0] = '\0';
     return buf1;
   }
-  strcpy(buf1, s);
+  mush_strncpy(buf1, s, BUFFER_LEN);
   for (p = buf1; *p; p++)
     *p = UPCASE(*p);
   return buf1;
@@ -230,7 +250,7 @@ strlower(const char *s)
     buf1[0] = '\0';
     return buf1;
   }
-  strcpy(buf1, s);
+  mush_strncpy(buf1, s, BUFFER_LEN);
   for (p = buf1; *p; p++)
     *p = DOWNCASE(*p);
   return buf1;
@@ -579,7 +599,7 @@ safe_accent(const char *RESTRICT base, const char *RESTRICT tmplate, size_t len,
 
 
 /** Define the args used in APPEND_TO_BUF */
-#define APPEND_ARGS int len, blen, clen
+#define APPEND_ARGS size_t len, blen, clen
 /** Append a string c to the end of buff, starting at *bp.
  * This macro is used by the safe_XXX functions.
  */
@@ -641,7 +661,7 @@ safe_format(char *buff, char **bp, const char *RESTRICT fmt, ...)
  * \return 0 on success, non-zero on failure.
  */
 int
-safe_integer(int i, char *buff, char **bp)
+safe_integer(intmax_t i, char *buff, char **bp)
 {
   return format_long(i, buff, bp, BUFFER_LEN, 10);
 }
@@ -653,9 +673,9 @@ safe_integer(int i, char *buff, char **bp)
  * \return 0 on success, non-zero on failure.
  */
 int
-safe_uinteger(unsigned int i, char *buff, char **bp)
+safe_uinteger(uintmax_t i, char *buff, char **bp)
 {
-  return safe_str(unparse_uinteger(i), buff, bp);
+  return safe_str(unparse_integer(i), buff, bp);
 }
 
 /** Safely store an unsigned integer into a short buffer.
@@ -665,7 +685,7 @@ safe_uinteger(unsigned int i, char *buff, char **bp)
  * \return 0 on success, non-zero on failure.
  */
 int
-safe_integer_sbuf(int i, char *buff, char **bp)
+safe_integer_sbuf(intmax_t i, char *buff, char **bp)
 {
   return format_long(i, buff, bp, SBUF_LEN, 10);
 }
@@ -765,13 +785,15 @@ safe_str_space(const char *c, char *buff, char **bp)
  * \retval 1 failure.
  */
 int
-safe_strl(const char *s, int len, char *buff, char **bp)
+safe_strl(const char *s, size_t len, char *buff, char **bp)
 {
-  int blen, clen;
+  size_t blen, clen;
 
   if (!s || !*s)
     return 0;
-  if (len == 1)
+  if (len == 0)
+    return 0;
+  else if (len == 1)
     return safe_chr(*s, buff, bp);
 
   blen = *bp - buff;
@@ -839,8 +861,9 @@ skip_space(const char *s)
   return c;
 }
 
+#ifndef HAVE_STRCHRNUL
 /** Return a pointer to next char in s which matches c, or to the terminating
- * null at the end of s.
+ * nul at the end of s.
  * \param s string to search.
  * \param c character to search for.
  * \return pointer to next occurence of c or to the end of s.
@@ -849,30 +872,36 @@ char *
 seek_char(const char *s, char c)
 {
   char *p = (char *) s;
-  while (p && *p && (*p != c))
+  if (!p)
+    return NULL;
+  while (*p && (*p != c))
     p++;
   return p;
 }
+#endif
 
 /** Unsigned char version of strlen.
  * \param s string.
  * \return length of s.
  */
-int
+size_t
 u_strlen(const unsigned char *s)
 {
   return strlen((const char *) s);
 }
 
-/** Unsigned char version of strcpy. Equally dangerous.
+/** Unsigned char version of mush_strncpy(). Destination string
+ * is nul-terminated.
  * \param target destination for copy.
  * \param source string to copy.
+ * \param len maximum number of bytes to copy.
  * \return pointer to copy.
  */
 unsigned char *
-u_strcpy(unsigned char *target, const unsigned char *source)
+u_strncpy(unsigned char *target, const unsigned char *source, size_t len)
 {
-  return (unsigned char *) strcpy((char *) target, (const char *) source);
+  return (unsigned char *) mush_strncpy((char *) target, (const char *) source,
+                                        len);
 }
 
 /** Search for all copies of old in string, and replace each with newbit.
@@ -883,8 +912,8 @@ u_strcpy(unsigned char *target, const unsigned char *source)
  * \return allocated string with replacements performed.
  */
 char *
-replace_string(const char *RESTRICT old, const char *RESTRICT newbit,
-               const char *RESTRICT string)
+replace_string(const char *restrict old, const char *restrict newbit,
+               const char *restrict string)
 {
   char *result, *r;
   size_t len, newlen;
@@ -927,7 +956,7 @@ const char *standard_tokens[2] = { "##", "#@" };
  */
 char *
 replace_string2(const char *old[2], const char *newbits[2],
-                const char *RESTRICT string)
+                const char *restrict string)
 {
   char *result, *rp;
   char firsts[3] = { '\0', '\0', '\0' };
@@ -992,7 +1021,7 @@ trim_space_sep(char *str, char sep)
   str += strspn(str, " ");
   for (p = str; *p; p++) ;
   /* And trailing */
-  for (p--; (*p == ' ') && (p > str); p--) ;
+  for (p--; p > str && *p == ' '; p--) ;
   p++;
   *p = '\0';
   return str;
@@ -1009,8 +1038,22 @@ next_token(char *str, char sep)
 {
   /* move pointer to start of the next token */
 
-  while (*str && (*str != sep))
+  while (*str) {
+    if (*str == sep) {
+      break;
+    }
+    switch (*str) {
+    case TAG_START:
+      while (*str && *str != TAG_END)
+        str++;
+      break;
+    case ESC_CHAR:
+      while (*str && *str != 'm')
+        str++;
+      break;
+    }
     str++;
+  }
   if (!*str)
     return NULL;
   str++;
@@ -1070,66 +1113,6 @@ do_wordcount(char *str, char sep)
   return n;
 }
 
-/** A version of strlen that ignores ansi and HTML sequences.
- * \param p string to get length of.
- * \return length of string p, not including ansi/html sequences.
- */
-int
-ansi_strlen(const char *p)
-{
-  int i = 0;
-
-  if (!p)
-    return 0;
-
-  while (*p) {
-    if (*p == ESC_CHAR) {
-      while ((*p) && (*p != 'm'))
-        p++;
-    } else if (*p == TAG_START) {
-      while ((*p) && (*p != TAG_END))
-        p++;
-    } else {
-      i++;
-    }
-    p++;
-  }
-  return i;
-}
-
-/** Returns the apparent length of a string, up to numchars visible 
- * characters. The apparent length skips over nonprinting ansi and
- * tags.
- * \param p string.
- * \param numchars maximum size to report.
- * \return apparent length of string.
- */
-int
-ansi_strnlen(const char *p, size_t numchars)
-{
-  int i = 0;
-
-  if (!p)
-    return 0;
-  while (*p && numchars > 0) {
-    if (*p == ESC_CHAR) {
-      while ((*p) && (*p != 'm')) {
-        p++;
-        i++;
-      }
-    } else if (*p == TAG_START) {
-      while ((*p) && (*p != TAG_END)) {
-        p++;
-        i++;
-      }
-    } else
-      numchars--;
-    i++;
-    p++;
-  }
-  return i;
-}
-
 /** Given a string, a word, and a separator, remove first occurence
  * of the word from the string. Destructive.
  * \param list a string containing a separated list.
@@ -1203,78 +1186,25 @@ next_in_list(const char **head)
 
 }
 
-/** Strip all ansi and html markup from a string. As a side effect,
- * stores the length of the stripped string in a provided address.
- * NOTE! Length returned is length *including* the terminating NULL,
- * because we usually memcpy the result.
- * \param orig string to strip.
- * \param s_len address to store length of stripped string, if provided.
- * \return pointer to static buffer containing stripped string.
- */
-char *
-remove_markup(const char *orig, size_t * s_len)
+#ifndef HAVE_IMAXDIV
+typedef struct imaxdiv_t {
+  intmax_t rem;
+  intmax_t quot;
+} imaxdiv_t;
+
+imaxdiv_t
+imaxdiv(intmax_t num, intmax_t denom)
 {
-  static char buff[BUFFER_LEN];
-  char *bp = buff;
-  const char *q;
-  size_t len = 0;
-
-  if (!orig) {
-    if (s_len)
-      *s_len = 0;
-    return NULL;
+  imaxdiv_t r;
+  r.quot = num / denom;
+  r.rem = num % denom;
+  if (num >= 0 && r.rem < 0) {
+    r.quot++;
+    r.rem -= denom;
   }
-
-  for (q = orig; *q;) {
-    switch (*q) {
-    case ESC_CHAR:
-      /* Skip over ansi */
-      while (*q && *q++ != 'm') ;
-      break;
-    case TAG_START:
-      /* Skip over HTML */
-      while (*q && *q++ != TAG_END) ;
-      break;
-    default:
-      safe_chr(*q++, buff, &bp);
-      len++;
-    }
-  }
-  *bp = '\0';
-  if (s_len)
-    *s_len = len + 1;
-  return buff;
+  return r;
 }
-
-/** Safe version of strncpy() that always nul-terminates the
- * destination string. The only reason it's not called
- * safe_strncpy() is to avoid confusion with the unrelated
- * safe_*() pennstr functions.
- * \param dst the destination string to copy to
- * \param src the source string to copy from
- * \param len the maximum number of bytes to copy
- * return dst
- */
-char *
-mush_strncpy(char *RESTRICT dst, const char *RESTRICT src, size_t len)
-{
-  size_t n = 0;
-  char *start = dst;
-
-  if (!src || !dst || len == 0)
-    return dst;
-
-  len--;
-
-  while (*src && n < len) {
-    *dst++ = *src++;
-    n++;
-  }
-
-  *dst = '\0';
-  return start;
-}
-
+#endif                          /* !HAVE_IMAXDIV */
 
 /** Safely append an int to a string. Returns a true value on failure.
  * This will someday take extra arguments for use with our version 
@@ -1289,14 +1219,14 @@ mush_strncpy(char *RESTRICT dst, const char *RESTRICT src, size_t len)
  * \param base the base to render the number in.
  */
 int
-format_long(long val, char *buff, char **bp, int maxlen, int base)
+format_long(intmax_t val, char *buff, char **bp, int maxlen, int base)
 {
   char stack[128];              /* Even a negative 64 bit number will only be 21
                                    digits or so max. This should be plenty of
                                    buffer room. */
   char *current;
   int size = 0, neg = 0;
-  ldiv_t r;
+  imaxdiv_t r;
   const char *digits = "0123456789abcdefghijklmnopqrstuvwxyz";
 
   /* Sanity checks */
@@ -1316,13 +1246,25 @@ format_long(long val, char *buff, char **bp, int maxlen, int base)
     if (val < 0) {
       /* -LONG_MIN == LONG_MIN on 2's complement systems. Take the
          easy way out since this value is rarely encountered. */
+
+      /* Most of these defaults are probably wrong on Win32. I hope it
+         has at least the headers from C99. */
+#ifndef PRIdMAX
+#define PRIdMAX "lld"
+#endif
+#ifndef PRIxMAX
+#define PRIxMAX "llx"
+#endif
+#ifndef PRIoMAX
+#define PRIoMAX "llo"
+#endif
       switch (base) {
       case 10:
-        return safe_format(buff, bp, "%ld", val);
+        return safe_format(buff, bp, "%" PRIdMAX, val);
       case 16:
-        return safe_format(buff, bp, "%lx", val);
+        return safe_format(buff, bp, "%" PRIxMAX, val);
       case 8:
-        return safe_format(buff, bp, "%lo", val);
+        return safe_format(buff, bp, "%" PRIoMAX, val);
       default:
         /* Weird base /and/ LONG_MIN. Fix someday. */
         return 0;
@@ -1340,7 +1282,7 @@ format_long(long val, char *buff, char **bp, int maxlen, int base)
     /* ldiv(x, y) does x/y and x%y at the same time (both of
      * which we need).
      */
-    r = ldiv(r.quot, base);
+    r = imaxdiv(r.quot, base);
     *(--current) = digits[r.rem];
   } while (r.quot);
 
@@ -1355,7 +1297,6 @@ format_long(long val, char *buff, char **bp, int maxlen, int base)
 
   size = stack + sizeof(stack) - current;
 
-  /* if (size < (int) ((buff + maxlen - 1) - *bp)) { */
   if (((int) (*bp - buff)) + size < maxlen - 2) {
     switch (size % 8) {
     case 0:
@@ -1389,7 +1330,7 @@ format_long(long val, char *buff, char **bp, int maxlen, int base)
   return 0;
 }
 
-#if defined(HAS_STRXFRM) && !defined(WIN32)
+#ifndef HAVE__STRNCOLL
 /** A locale-sensitive strncmp.
  * \param s1 first string to compare.
  * \param s2 second string to compare.
@@ -1401,6 +1342,7 @@ format_long(long val, char *buff, char **bp, int maxlen, int base)
 int
 strncoll(const char *s1, const char *s2, size_t t)
 {
+#ifdef HAVE_STRXFRM
   char *d1, *d2, *ns1, *ns2;
   int result;
   size_t s1_len, s2_len;
@@ -1424,8 +1366,13 @@ strncoll(const char *s1, const char *s2, size_t t)
   mush_free(d1, "string");
   mush_free(d2, "string");
   return result;
+#else
+  return strncmp(s1, s2, t);
+#endif
 }
+#endif
 
+#ifndef HAVE__STRICOLL
 /** A locale-sensitive strcasecmp.
  * \param s1 first string to compare.
  * \param s2 second string to compare.
@@ -1436,6 +1383,7 @@ strncoll(const char *s1, const char *s2, size_t t)
 int
 strcasecoll(const char *s1, const char *s2)
 {
+#ifdef HAVE_STRXFRM
   char *d1, *d2;
   int result;
   size_t s1_len, s2_len;
@@ -1451,8 +1399,13 @@ strcasecoll(const char *s1, const char *s2)
   mush_free(d1, "string");
   mush_free(d2, "string");
   return result;
+#else
+  return strcasecmp(s1, s2);
+#endif
 }
+#endif
 
+#ifndef HAVE__STRNICOLL
 /** A locale-sensitive strncasecmp.
  * \param s1 first string to compare.
  * \param s2 second string to compare.
@@ -1464,6 +1417,7 @@ strcasecoll(const char *s1, const char *s2)
 int
 strncasecoll(const char *s1, const char *s2, size_t t)
 {
+#ifdef HAVE_STRXFRM
   char *d1, *d2, *ns1, *ns2;
   int result;
   size_t s1_len, s2_len;
@@ -1487,417 +1441,41 @@ strncasecoll(const char *s1, const char *s2, size_t t)
   mush_free(d1, "string");
   mush_free(d2, "string");
   return result;
+#else
+  return strncasecmp(s1, s2, t);
+#endif
 }
-#endif                          /* HAS_STRXFRM && !WIN32 */
+#endif
 
-/** Return a string pointer past any ansi/html markup at the start.
- * \param p a string.
- * \return pointer to string after any initial ansi/html markup.
+/** Safe version of strncpy() that always nul-terminates the
+ * destination string. The only reason it's not called
+ * safe_strncpy() is to avoid confusion with the unrelated
+ * safe_*() pennstr functions.
+ * \param dst the destination string to copy to
+ * \param src the source string to copy from
+ * \param len the maximum number of bytes to copy
+ * \return dst
  */
 char *
-skip_leading_ansi(const char *p)
+mush_strncpy(char *restrict dst, const char *restrict src, size_t len)
 {
-  if (!p)
-    return NULL;
-  while (*p == ESC_CHAR || *p == TAG_START) {
-    if (*p == ESC_CHAR) {
-      while (*p && *p != 'm')
-        p++;
-    } else {                    /* TAG_START */
-      while (*p && *p != TAG_END)
-        p++;
-    }
-    if (*p)
-      p++;
-  }
-  return (char *) p;
+  size_t n = 0;
+  char *start = dst;
 
+  if (!src || !dst || len == 0)
+    return dst;
+
+  len--;
+
+  while (*src && n < len) {
+    *dst++ = *src++;
+    n++;
+  }
+
+  *dst = '\0';
+  return start;
 }
 
-/** Convert a string into an ansi_string.
- * This takes a string that may contain ansi/html markup codes and
- * converts it to an ansi_string structure that separately stores
- * the plain string and the markup codes for each character.
- * \param src string to parse.
- * \return pointer to an ansi_string structure representing the src string.
- */
-ansi_string *
-parse_ansi_string(const char *src)
-{
-  ansi_string *data;
-  char *y, *current = NULL;
-  size_t p = 0;
-
-  if (!src)
-    return NULL;
-
-  data = mush_malloc(sizeof *data, "ansi_string");
-  if (!data)
-    return NULL;
-
-  data->len = ansi_strlen(src);
-
-  while (*src) {
-    y = skip_leading_ansi(src);
-    if (y != src) {
-      if (current)
-        mush_free(current, "markup_codes");
-      current = mush_strndup(src, y - src, "markup_codes");
-      src = y;
-    }
-    if (current)
-      data->codes[p] = mush_strdup(current, "markup_codes");
-    else
-      data->codes[p] = NULL;
-    data->text[p] = *src;
-    if (*src)
-      src++;
-    p++;
-  }
-  data->text[p] = '\0';
-
-  while (p <= data->len) {
-    data->codes[p] = NULL;
-    p++;
-  }
-
-  if (current)
-    mush_free(current, "markup_codes");
-
-  return data;
-}
-
-
-/** Fill up an ansi_string with codes so that when a code starts it
- * applies to all the following characters until there's a new code.
- * \param as pointer to an ansi_string to populate codes in.
- */
-void
-populate_codes(ansi_string *as)
-{
-  size_t p;
-  char *current = NULL;
-
-  if (!as)
-    return;
-
-  for (p = 0; p < as->len; p++)
-    if (as->codes[p]) {
-      if (current)
-        mush_free(current, "markup_codes");
-      current = mush_strdup(as->codes[p], "markup_codes");
-    } else {
-      if (!current)
-        current = mush_strdup(ANSI_NORMAL, "markup_codes");
-      as->codes[p] = mush_strdup(current, "markup_codes");
-    }
-  if (current)
-    mush_free(current, "markup_codes");
-}
-
-/** Strip out codes from an ansi_string, leaving in only the codes where
- * they change.
- * \param as pointer to an ansi_string.
- */
-void
-depopulate_codes(ansi_string *as)
-{
-  size_t p, m;
-  int normal = 1;
-
-  if (!as)
-    return;
-
-  for (p = 0; p <= as->len; p++) {
-    if (as->codes[p]) {
-      if (normal) {
-        if (strcmp(as->codes[p], ANSI_NORMAL) == 0) {
-          mush_free(as->codes[p], "markup_codes");
-          as->codes[p] = NULL;
-          continue;
-        } else {
-          normal = 0;
-        }
-      }
-
-      m = p;
-      for (p++; p < as->len; p++) {
-        if (as->codes[p] && strcmp(as->codes[p], as->codes[m]) == 0) {
-          mush_free(as->codes[p], "markup_codes");
-          as->codes[p] = NULL;
-        } else {
-          p--;
-          break;
-        }
-      }
-    }
-  }
-}
-
-/** Reverse an ansi string, preserving its ansification.
- * This function destructively modifies the ansi_string passed.
- * \param as pointer to an ansi string.
- */
-void
-flip_ansi_string(ansi_string *as)
-{
-  int p, n;
-
-  populate_codes(as);
-
-  for (p = 0, n = as->len - 1; p < n; p++, n--) {
-    char *tcode;
-    char t;
-
-    tcode = as->codes[p];
-    t = as->text[p];
-    as->codes[p] = as->codes[n];
-    as->text[p] = as->text[n];
-    as->codes[n] = tcode;
-    as->text[n] = t;
-  }
-}
-
-
-static int is_ansi_code(const char *s);
-static int is_start_html_code(const char *s) __attribute__ ((__unused__));
-static int is_end_html_code(const char *s);
-/** Is s a string that signifies the end of ANSI codes? */
-#define is_end_ansi_code(s) (!strcmp((s),ANSI_NORMAL))
-
-
-static int
-is_ansi_code(const char *s)
-{
-  return s && *s == ESC_CHAR;
-}
-
-static int
-is_start_html_code(const char *s)
-{
-  return s && *s == TAG_START && *(s + 1) != '/';
-}
-
-static int
-is_end_html_code(const char *s)
-{
-  return s && *s == TAG_START && *(s + 1) == '/';
-}
-
-/** Free an ansi_string.
- * \param as pointer to ansi_string to free.
- */
-void
-free_ansi_string(ansi_string *as)
-{
-  int p;
-
-  if (!as)
-    return;
-  for (p = as->len; p >= 0; p--) {
-    if (as->codes[p])
-      mush_free(as->codes[p], "markup_codes");
-  }
-  mush_free(as, "ansi_string");
-}
-
-/** Safely append an ansi_string into a buffer as a real string.
- * \param as pointer to ansi_string to append.
- * \param start position in as to start copying from.
- * \param len length in characters to copy from as.
- * \param buff buffer to insert into.
- * \param bp pointer to pointer to insertion point of buff.
- * \retval 0 success.
- * \retval 1 failure.
- */
-int
-safe_ansi_string(ansi_string *as, size_t start, size_t len, char *buff,
-                 char **bp)
-{
-  int p, q;
-  int in_ansi = 0;
-  int in_html = 0;
-
-  if (!as)
-    return 1;
-
-  depopulate_codes(as);
-
-  if (start > as->len || len == 0 || as->len == 0)
-    return safe_str("", buff, bp);
-
-  /* Find the starting codes by working our way backward until we
-   * reach some opening codes, and then working our way back from there
-   * until we hit a non-opening code or non-code 
-   */
-  p = start;
-  while ((p >= 0) && (as->codes[p] == NULL))
-    p--;
-  /* p is now either <0 or pointing to a code */
-  if ((p >= 0) && !is_end_html_code(as->codes[p]) &&
-      !is_end_ansi_code(as->codes[p])) {
-    /* p is now pointing to a starting code */
-    q = p;
-    while ((q >= 0) && as->codes[q] && !is_end_html_code(as->codes[q]) &&
-           !is_end_ansi_code(as->codes[q])) {
-      if (is_ansi_code(as->codes[q]))
-        in_ansi = 1;
-      else if (is_start_html_code(as->codes[q]))
-        in_html++;
-      q--;
-    }
-    /* p is now pointing to the first starting code, and we know if we're
-     * in ansi, html, or both. We also know how many html tags have been
-     * opened.
-     */
-  }
-
-  /* Copy the text. The right thing to do now would be to have a stack
-   * of open html tags and clear in_html once all of the tags have
-   * been closed. We don't quite do that, alas.
-   */
-  for (p = (int) start; p < (int) (start + len) && p < (int) as->len; p++) {
-    if (as->codes[p]) {
-      if (safe_str(as->codes[p], buff, bp))
-        return 1;
-      if (is_end_ansi_code(as->codes[p]))
-        in_ansi = 0;
-      else if (is_ansi_code(as->codes[p]))
-        in_ansi = 1;
-      if (is_end_html_code(as->codes[p]))
-        in_html--;
-      else if (is_start_html_code(as->codes[p]))
-        in_html++;
-    }
-    if (safe_chr(as->text[p], buff, bp))
-      return 1;
-  }
-
-  /* Output (only) closing codes if needed. */
-  while (p <= (int) as->len) {
-    if (!in_ansi && !in_html)
-      break;
-    if (as->codes[p]) {
-      if (is_end_ansi_code(as->codes[p])) {
-        in_ansi = 0;
-        if (safe_str(as->codes[p], buff, bp))
-          return 1;
-      } else if (is_end_html_code(as->codes[p])) {
-        in_html--;
-        if (safe_str(as->codes[p], buff, bp))
-          return 1;
-      }
-    }
-    p++;
-  }
-  if (in_ansi)
-    safe_str(ANSI_NORMAL, buff, bp);
-  return 0;
-}
-
-/** Safely append an ansi_string into a buffer as a real string,
- * with extra copying of starting tags (for wrap()/align()).
- * \param as pointer to ansi_string to append.
- * \param start position in as to start copying from.
- * \param len length in characters to copy from as.
- * \param buff buffer to insert into.
- * \param bp pointer to pointer to insertion point of buff.
- * \retval 0 success.
- * \retval 1 failure.
- */
-int
-safe_ansi_string2(ansi_string *as, size_t start, size_t len, char *buff,
-                  char **bp)
-{
-  int p, q;
-  int in_ansi = 0;
-  int in_html = 0;
-
-  if (!as)
-    return 1;
-
-  depopulate_codes(as);
-
-  if (start > as->len || len == 0 || as->len == 0)
-    return safe_str("", buff, bp);
-
-  /* Find the starting codes by working our way backward until we
-   * reach some opening codes, and then working our way back from there
-   * until we hit a non-opening code or non-code 
-   */
-  p = start;
-  while ((p >= 0) && (as->codes[p] == NULL))
-    p--;
-  /* p is now either <0 or pointing to a code */
-  if ((p >= 0) && !is_end_html_code(as->codes[p]) &&
-      !is_end_ansi_code(as->codes[p])) {
-    /* p is now pointing to a starting code */
-    q = p;
-    while ((q >= 0) && as->codes[q] && !is_end_html_code(as->codes[q]) &&
-           !is_end_ansi_code(as->codes[q])) {
-      if (is_ansi_code(as->codes[q]))
-        in_ansi = 1;
-      else if (is_start_html_code(as->codes[q]))
-        in_html++;
-      q--;
-    }
-    /* p is now pointing to the first starting code, and we know if we're
-     * in ansi, html, or both. We also know how many html tags have been
-     * opened.
-     */
-    /* Except there's this one problem - Now we know it, we weren't
-     * doing anything with it.
-     */
-    for (q = q + 1; q <= p; q++) {
-      if (safe_str(as->codes[q], buff, bp))
-        return 1;
-    }
-  }
-
-  /* Copy the text. The right thing to do now would be to have a stack
-   * of open html tags and clear in_html once all of the tags have
-   * been closed. We don't quite do that, alas.
-   */
-  for (p = (int) start; p < (int) (start + len) && p < (int) as->len; p++) {
-    if (as->codes[p]) {
-      if (safe_str(as->codes[p], buff, bp))
-        return 1;
-      if (is_end_ansi_code(as->codes[p]))
-        in_ansi = 0;
-      else if (is_ansi_code(as->codes[p]))
-        in_ansi = 1;
-      if (is_end_html_code(as->codes[p]))
-        in_html--;
-      else if (is_start_html_code(as->codes[p]))
-        in_html++;
-    }
-    if (safe_chr(as->text[p], buff, bp))
-      return 1;
-  }
-
-  /* Output (only) closing codes if needed. */
-  while (p <= (int) as->len) {
-    if (!in_ansi && !in_html)
-      break;
-    if (as->codes[p]) {
-      if (is_end_ansi_code(as->codes[p])) {
-        in_ansi = 0;
-        if (safe_str(as->codes[p], buff, bp))
-          return 1;
-      } else if (is_end_html_code(as->codes[p])) {
-        in_html--;
-        if (safe_str(as->codes[p], buff, bp))
-          return 1;
-      }
-    }
-    p++;
-  }
-  if (in_ansi)
-    safe_str(ANSI_NORMAL, buff, bp);
-  return 0;
-}
-
-
 /** Safely append a list item to a buffer, possibly with punctuation
  * and conjunctions.
  * Given the current item number in a list, whether it's the last item
@@ -1942,7 +1520,7 @@ safe_itemizer(int cur_num, int done, const char *delim, const char *conjoin,
  * \return a pointer to a static buffer with the stringified time.
  */
 char *
-show_time(time_t t, int utc)
+show_time(time_t t, bool utc)
 {
   struct tm *when;
 
