@@ -32,10 +32,14 @@
 #include <wtypes.h>
 #include <winbase.h>            /* For GetCurrentProcessId() */
 #endif
-#include "conf.h"
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 
+#include "conf.h"
 #include "match.h"
 #include "externs.h"
+#include "ansi.h"
 #include "mushdb.h"
 #include "mymalloc.h"
 #include "log.h"
@@ -44,47 +48,11 @@
 #include "attrib.h"
 #include "parse.h"
 #include "lock.h"
+#include "SFMT.h"
 #include "confmagic.h"
 
 dbref find_entrance(dbref door);
 void initialize_mt(void);
-unsigned int genrand_int32(void);
-static void init_genrand(unsigned long);
-static void init_by_array(unsigned long *, int);
-
-/** A malloc wrapper that tracks type of allocation.
- * This should be used in preference to malloc() when possible,
- * to enable memory leak tracing with MEM_CHECK.
- * \param size bytes to allocate.
- * \param check string to label allocation with.
- * \return allocated block of memory or NULL.
- */
-Malloc_t
-mush_malloc(size_t size, const char *check)
-{
-  Malloc_t ptr;
-  add_check(check);
-  ptr = malloc(size);
-  if (ptr == NULL)
-    do_log(LT_ERR, 0, 0, "mush_malloc failed to malloc %lu bytes for %s",
-           (unsigned long) size, check);
-  return ptr;
-}
-
-/** A free wrapper that tracks type of allocation.
- * If mush_malloc() gets the memory, mush_free() should free it
- * to enable memory leak tracing with MEM_CHECK.
- * \param ptr pointer to block of member to free.
- * \param check string to label allocation with.
- */
-void
-mush_free(Malloc_t RESTRICT ptr, const char *RESTRICT check
-          __attribute__ ((__unused__)))
-{
-  del_check(check);
-  free(ptr);
-  return;
-}
 
 
 /** Parse object/attribute strings into components.
@@ -142,7 +110,7 @@ parse_anon_attrib(dbref player, char *str, dbref *thing, ATTR **attrib)
       AL_CREATOR(*attrib) = player;
       AL_NAME(*attrib) = mush_strdup("#lambda", "anon_attr.lambda");
       t = compress(str);
-      (*attrib)->data = chunk_create(t, (u_int_16) u_strlen(t), 0);
+      (*attrib)->data = chunk_create(t, u_strlen(t), 0);
       free(t);
       AL_FLAGS(*attrib) = AF_ANON;
       AL_NEXT(*attrib) = NULL;
@@ -169,10 +137,15 @@ free_anon_attrib(ATTR *attrib)
 /** Given an attribute [<object>/]<name> pair (which may include #lambda),
  * fetch its value, owner (thing), and pe_flags, and store in the struct
  * pointed to by ufun
+ * \param attrname The obj/name of attribute.
+ * \param executor Dbref of the executing object.
+ * \param ufun Pointer to an allocated ufun_attrib struct to fill in.
+ * \param accept_lambda true if #lambda can be used.
+ * \return 0 on failure, true on success.
  */
-int
+bool
 fetch_ufun_attrib(char *attrname, dbref executor, ufun_attrib * ufun,
-                  int accept_lambda)
+                  bool accept_lambda)
 {
   ATTR *attrib;
   dbref thing;
@@ -218,7 +191,7 @@ fetch_ufun_attrib(char *attrname, dbref executor, ufun_attrib * ufun,
     pe_flags |= PE_DEBUG;
 
   /* Populate the ufun object */
-  strncpy(ufun->contents, atr_value(attrib), BUFFER_LEN);
+  mush_strncpy(ufun->contents, atr_value(attrib), BUFFER_LEN);
   ufun->thing = thing;
   ufun->pe_flags = pe_flags;
 
@@ -244,9 +217,9 @@ fetch_ufun_attrib(char *attrname, dbref executor, ufun_attrib * ufun,
  * \retval 0 success
  * \retval 1 process_expression failed. (CPU time limit)
  */
-int
+bool
 call_ufun(ufun_attrib * ufun, char **wenv_args, int wenv_argc, char *ret,
-          dbref executor, dbref enactor, PE_Info * pe_info)
+          dbref executor, dbref enactor, PE_Info *pe_info)
 {
   char rbuff[BUFFER_LEN];
   char *rp;
@@ -256,13 +229,9 @@ call_ufun(ufun_attrib * ufun, char **wenv_args, int wenv_argc, char *ret,
   int pe_ret;
   char const *ap;
 
-  int old_re_subpatterns;
-  int *old_re_offsets;
-  char *old_re_from;
+  struct re_save rsave;
 
-  old_re_subpatterns = global_eval_context.re_subpatterns;
-  old_re_offsets = global_eval_context.re_offsets;
-  old_re_from = global_eval_context.re_from;
+  save_regexp_context(&rsave);
 
   /* Make sure we have a ufun first */
   if (!ufun)
@@ -286,6 +255,7 @@ call_ufun(ufun_attrib * ufun, char **wenv_args, int wenv_argc, char *ret,
 
   /* Set all the regexp patterns to NULL so they are not
    * propogated */
+  global_eval_context.re_code = NULL;
   global_eval_context.re_subpatterns = -1;
   global_eval_context.re_offsets = NULL;
   global_eval_context.re_from = NULL;
@@ -311,13 +281,123 @@ call_ufun(ufun_attrib * ufun, char **wenv_args, int wenv_argc, char *ret,
   }
 
   /* Restore regexp patterns */
-  global_eval_context.re_offsets = old_re_offsets;
-  global_eval_context.re_subpatterns = old_re_subpatterns;
-  global_eval_context.re_from = old_re_from;
+  restore_regexp_context(&rsave);
 
   return pe_ret;
 }
 
+/** Given a thing, attribute, enactor and arguments for %0-%9,
+ * call the ufun with appropriate permissions on values given for
+ * wenv_args. The value returned is stored in the buffer pointed to
+ * by ret, if given.
+ * \param thing The thing that has the attribute to be called
+ * \param attrname The name of the attribute to call.
+ * \param wenv_args An array of string values for global_eval_context.wenv
+ * \param wenv_argc The number of wenv args to use.
+ * \param ret If desired, a pointer to a buffer in which the results
+ * of the process_expression are stored in.
+ * \param enactor The enactor.
+ * \param pe_info The pe_info passed to the FUNCTION
+ * \retval 1 success
+ * \retval 0 No such attribute, or failed.
+ */
+bool
+call_attrib(dbref thing, const char *attrname, const char *wenv_args[],
+            int wenv_argc, char *ret, dbref enactor, PE_Info *pe_info)
+{
+  char atrbuf[BUFFER_LEN];
+  char rbuff[BUFFER_LEN];
+  char *rp;
+  char *old_wenv[10];
+  int old_args = 0;
+  int i;
+  int pe_ret;
+  char const *ap;
+  ATTR *attrib;
+  char *saver[NUMQ];
+
+  struct re_save rsave;
+
+  /* Make sure we have a valid object to call first */
+  if (!GoodObject(thing) || IsGarbage(thing))
+    return 0;
+
+  if (attrname == NULL || !*attrname)
+    return 0;
+
+  /* Fetch the attrib contents */
+  attrib = (ATTR *) atr_get(thing, attrname);
+  if (attrib == NULL)
+    return 0;
+
+  mush_strncpy(atrbuf, atr_value(attrib), BUFFER_LEN);
+
+  if (!*atrbuf) {
+    if (ret)
+      *ret = '\0';
+    return 1;
+  }
+
+  save_global_regs("localize", saver);
+
+  /* Store regepx info */
+  save_regexp_context(&rsave);
+
+  /* If the user doesn't care about the return of the expression,
+   * then use our own rbuff.
+   */
+  if (!ret)
+    ret = rbuff;
+  rp = ret;
+
+  /* Set up %0-%9 */
+  for (i = 0; i < wenv_argc; i++) {
+    old_wenv[i] = global_eval_context.wenv[i];
+    global_eval_context.wenv[i] = (char *) wenv_args[i];
+  }
+  for (; i < 10; i++) {
+    old_wenv[i] = global_eval_context.wenv[i];
+    global_eval_context.wenv[i] = NULL;
+  }
+  /* Clear all q-regs */
+  for (i = 0; i < NUMQ; i++) {
+    global_eval_context.renv[i][0] = '\0';
+  }
+
+  /* Set all the regexp patterns to NULL so they are not
+   * propogated */
+  global_eval_context.re_code = NULL;
+  global_eval_context.re_subpatterns = -1;
+  global_eval_context.re_offsets = NULL;
+  global_eval_context.re_from = NULL;
+
+  /* And now, make the call! =) */
+  if (pe_info) {
+    old_args = pe_info->arg_count;
+    pe_info->arg_count = wenv_argc;
+  }
+
+  ap = atrbuf;
+  pe_ret = process_expression(ret, &rp, &ap, thing, thing,
+                              enactor, PE_DEFAULT, PT_DEFAULT, pe_info);
+  *rp = '\0';
+
+  /* Restore the old wenv */
+  for (i = 0; i < 10; i++) {
+    global_eval_context.wenv[i] = old_wenv[i];
+  }
+
+  if (pe_info) {
+    pe_info->arg_count = old_args;
+  }
+
+  /* Restore regexp patterns */
+  restore_regexp_context(&rsave);
+  restore_global_regs("localize", saver);
+
+  return !pe_ret;
+}
+
 /** Given an exit, find the room that is its source through brute force.
  * This is used in pathological cases where the exit's own source
  * element is invalid.
@@ -372,7 +452,7 @@ remove_first(dbref first, dbref what)
  * \retval 1 found thing on list.
  * \retval 0 did not find thing on list.
  */
-int
+bool
 member(dbref thing, dbref list)
 {
   DOLIST(list, list) {
@@ -395,7 +475,7 @@ member(dbref thing, dbref list)
  * \retval 1 disallow is inside of from.
  * \retval 0 disallow is not inside of from.
  */
-int
+bool
 recursive_member(dbref disallow, dbref from, int count)
 {
   do {
@@ -418,7 +498,7 @@ recursive_member(dbref disallow, dbref from, int count)
  * \retval 1 object or location is unfindable.
  * \retval 0 neither object nor location is unfindable.
  */
-int
+bool
 unfindable(dbref thing)
 {
   int count = 0;
@@ -455,29 +535,18 @@ reverse(dbref list)
 }
 
 
-#define N 624 /**< PRNG constant */
 
-/* We use the Mersenne Twister PRNG. It's quite good as PRNGS go,
- * much better than the typical ones provided in system libc's.
- *
- * The following two functions are based on the reference implementation,
- * with changes in the seeding function to use /dev/urandom as a seed
- * if possible.
- *
- * The Mersenne Twister homepage is:
- *  http://www.math.keio.ac.jp/~matumoto/emt.html
- *
- * You can get the reference code there.
- */
-
-
-/** Wrapper to choose a seed and initialize the Mersenne Twister PRNG. */
+/** Wrapper to choose a seed and initialize the Mersenne Twister PRNG. 
+ * The actual MT code lives in SFMT.c and hdrs/SFMT*.h */
 void
 initialize_mt(void)
 {
 #ifdef HAS_DEV_URANDOM
   int fd;
-  unsigned long buf[N];
+  uint32_t buf[4];              /* The linux manpage for /dev/urandom
+                                   advises against reading large amounts of
+                                   data from it; we used to read 624*4 (Or *8 on 64-bit systems)
+                                   bytes. The new figure is much more reasonable. */
 
   fd = open("/dev/urandom", O_RDONLY);
   if (fd >= 0) {
@@ -498,134 +567,13 @@ initialize_mt(void)
 #endif
   /* Default seeder. Pick a seed that's fairly random */
 #ifdef WIN32
-  init_genrand(GetCurrentProcessId() | (time(NULL) << 16));
+  init_gen_rand(GetCurrentProcessId() | (time(NULL) << 16));
 #else
-  init_genrand(getpid() | (time(NULL) << 16));
+  init_gen_rand(getpid() | (time(NULL) << 16));
 #endif
 }
 
 
-/* A C-program for MT19937, with initialization improved 2002/1/26.*/
-/* Coded by Takuji Nishimura and Makoto Matsumoto.                 */
-
-/* Before using, initialize the state by using init_genrand(seed)  */
-/* or init_by_array(init_key, key_length).                         */
-
-/* This library is free software.                                  */
-/* This library is distributed in the hope that it will be useful, */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of  */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            */
-
-/* Copyright (C) 1997, 2002 Makoto Matsumoto and Takuji Nishimura. */
-/* Any feedback is very welcome.                                   */
-/* http://www.math.keio.ac.jp/matumoto/emt.html                    */
-/* email: matumoto@math.keio.ac.jp                                 */
-
-/* Period parameters */
-#define M 397  /**< PRNG constant */
-#define MATRIX_A 0x9908b0dfUL   /**< PRNG constant vector a */
-#define UPPER_MASK 0x80000000UL /**< PRNG most significant w-r bits */
-#define LOWER_MASK 0x7fffffffUL /**< PRNG least significant r bits */
-
-static unsigned long mt[N];     /* the array for the state vector  */
-static int mti = N + 1;         /* mti==N+1 means mt[N] is not initialized */
-
-/** initializes mt[N] with a seed.
- * \param a seed value.
- */
-static void
-init_genrand(unsigned long s)
-{
-  mt[0] = s & 0xffffffffUL;
-  for (mti = 1; mti < N; mti++) {
-    mt[mti] = (1812433253UL * (mt[mti - 1] ^ (mt[mti - 1] >> 30)) + mti);
-    /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
-    /* In the previous versions, MSBs of the seed affect   */
-    /* only MSBs of the array mt[].                        */
-    /* 2002/01/09 modified by Makoto Matsumoto             */
-    mt[mti] &= 0xffffffffUL;
-    /* for >32 bit machines */
-  }
-}
-
-/** initialize by an array with array-length
- * \param init_key the array for initializing keys 
- * \param key_length the array's length 
- */
-static void
-init_by_array(unsigned long init_key[], int key_length)
-{
-  int i, j, k;
-  init_genrand(19650218UL);
-  i = 1;
-  j = 0;
-  k = (N > key_length ? N : key_length);
-  for (; k; k--) {
-    mt[i] = (mt[i] ^ ((mt[i - 1] ^ (mt[i - 1] >> 30)) * 1664525UL))
-      + init_key[j] + j;        /* non linear */
-    mt[i] &= 0xffffffffUL;      /* for WORDSIZE > 32 machines */
-    i++;
-    j++;
-    if (i >= N) {
-      mt[0] = mt[N - 1];
-      i = 1;
-    }
-    if (j >= key_length)
-      j = 0;
-  }
-  for (k = N - 1; k; k--) {
-    mt[i] = (mt[i] ^ ((mt[i - 1] ^ (mt[i - 1] >> 30)) * 1566083941UL))
-      - i;                      /* non linear */
-    mt[i] &= 0xffffffffUL;      /* for WORDSIZE > 32 machines */
-    i++;
-    if (i >= N) {
-      mt[0] = mt[N - 1];
-      i = 1;
-    }
-  }
-
-  mt[0] = 0x80000000UL;         /* MSB is 1; assuring non-zero initial array */
-}
-
-/* generates a random number on [0,0xffffffff]-interval */
-unsigned int
-genrand_int32(void)
-{
-  unsigned long y;
-  static unsigned long mag01[2] = { 0x0UL, MATRIX_A };
-  /* mag01[x] = x * MATRIX_A  for x=0,1 */
-
-  if (mti >= N) {               /* generate N words at one time */
-    int kk;
-
-    if (mti == N + 1)           /* if init_genrand() has not been called, */
-      init_genrand(5489UL);     /* a default initial seed is used */
-
-    for (kk = 0; kk < N - M; kk++) {
-      y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK);
-      mt[kk] = mt[kk + M] ^ (y >> 1) ^ mag01[y & 0x1UL];
-    }
-    for (; kk < N - 1; kk++) {
-      y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK);
-      mt[kk] = mt[kk + (M - N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
-    }
-    y = (mt[N - 1] & UPPER_MASK) | (mt[0] & LOWER_MASK);
-    mt[N - 1] = mt[M - 1] ^ (y >> 1) ^ mag01[y & 0x1UL];
-
-    mti = 0;
-  }
-
-  y = mt[mti++];
-
-  /* Tempering */
-  y ^= (y >> 11);
-  y ^= (y << 7) & 0x9d2c5680UL;
-  y ^= (y << 15) & 0xefc60000UL;
-  y ^= (y >> 18);
-
-  return y;
-}
-
 
 /** Get a uniform random long between low and high values, inclusive.
  * Based on MUX's RandomINT32()
@@ -633,10 +581,10 @@ genrand_int32(void)
  * \param high upper bound for random number.
  * \return random number between low and high, or 0 or -1 for error.
  */
-long
-get_random_long(long low, long high)
+uint32_t
+get_random32(uint32_t low, uint32_t high)
 {
-  unsigned long x, n, n_limit;
+  uint32_t x, n, n_limit;
 
   /* Validate parameters */
   if (high < low) {
@@ -645,11 +593,7 @@ get_random_long(long low, long high)
     return low;
   }
 
-  x = high - low;
-  if (LONG_MAX < x) {
-    return -1;
-  }
-  x++;
+  x = high - low + 1;
 
   /* We can now look for an random number on the interval [0,x-1].
      //
@@ -668,10 +612,10 @@ get_random_long(long low, long high)
      // we will call getrand() is less than 2.
    */
 
-  n_limit = ULONG_MAX - (ULONG_MAX % x);
+  n_limit = UINT32_MAX - (UINT32_MAX % x);
 
   do {
-    n = genrand_int32();
+    n = gen_rand32();
   } while (n >= n_limit);
 
   return low + (n % x);
@@ -692,8 +636,7 @@ fullalias(dbref it)
     return n;
   }
 
-  strncpy(n, atr_value(a), BUFFER_LEN - 1);
-  n[BUFFER_LEN - 1] = '\0';
+  mush_strncpy(n, atr_value(a), BUFFER_LEN);
 
   return n;
 }
@@ -715,8 +658,7 @@ shortalias(dbref it)
     return n;
   }
 
-  strncpy(n, s, BUFFER_LEN - 1);
-  n[BUFFER_LEN - 1] = '\0';
+  mush_strncpy(n, s, BUFFER_LEN);
   if ((s = strchr(n, ';')))
     *s = '\0';
 
@@ -734,8 +676,8 @@ shortname(dbref it)
   static char n[BUFFER_LEN];    /* STATIC */
   char *s;
 
-  strncpy(n, Name(it), BUFFER_LEN - 1);
-  n[BUFFER_LEN - 1] = '\0';
+  mush_strncpy(n, Name(it), BUFFER_LEN);
+
   if (IsExit(it)) {
     if ((s = strchr(n, ';')))
       *s = '\0';
@@ -760,8 +702,6 @@ absolute_room(dbref it)
   if (!GoodObject(room))
     return NOTHING;
   while (!IsRoom(room)) {
-    if (!GoodObject(Location(room)))
-      return room;
     room = Location(room);
     rec++;
     if (rec > 20)
