@@ -33,7 +33,7 @@
 #include "ansi.h"
 #include "mymalloc.h"
 #include "parse.h"
-#include "pcre.h"
+#include "mypcre.h"
 #include "confmagic.h"
 
 /** Force a char to be lowercase */
@@ -45,15 +45,26 @@
 /** Maximum number of wildcarded arguments */
 #define NUMARGS (10)
 
+/** Skip across all ansi and pueblo markup. */
+#define SKIP_ANSI(x) do { \
+      if (*x == TAG_START) { \
+        while (*x && (*x != TAG_END)) x++; \
+        if (*x) x++; \
+      } else if (*x == ESC_CHAR) { \
+        while (*x && (*x != 'm')) x++; \
+        if (*x) x++; \
+      } \
+  } while ((*x == TAG_START) || (*x == ESC_CHAR))
+
 const unsigned char *tables = NULL;  /** Pointer to character tables */
 
-static int wild1
-  (const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
-   char **wbuf, int *len, int cs, char **ary, int max);
-static int wild(const char *RESTRICT s, const char *RESTRICT d, int p, int cs,
-                char **ary, int max, char *buffer, int len);
-static int check_literals(const char *RESTRICT tstr, const char *RESTRICT dstr,
-                          int cs);
+static bool wild1
+  (const char *restrict tstr, const char *restrict dstr, int arg,
+   char **wbuf, ssize_t * len, bool cs, char **ary, size_t max);
+static bool wild(const char *restrict s, const char *restrict d, int p, bool cs,
+                 char **ary, size_t max, char *buffer, ssize_t len);
+static bool check_literals(const char *restrict tstr, const char *restrict dstr,
+                           bool cs);
 static char *strip_backslashes(const char *str);
 
 /** Do a wildcard match, without remembering the wild data.
@@ -65,8 +76,8 @@ static char *strip_backslashes(const char *str);
  * \retval 1 dstr matches the tstr pattern.
  * \retval 0 dstr does not match the tstr pattern.
  */
-int
-quick_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
+bool
+quick_wild(const char *restrict tstr, const char *restrict dstr)
 {
   /* quick_wild_new does the real work, but before we call it,
    * we do some sanity checking.
@@ -86,10 +97,11 @@ quick_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
  * \retval 1 dstr matches the tstr pattern.
  * \retval 0 dstr does not match the tstr pattern.
  */
-int
-quick_wild_new(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
+bool
+quick_wild_new(const char *restrict tstr, const char *restrict dstr, bool cs)
 {
   while (*tstr != '*') {
+    SKIP_ANSI(dstr);
     switch (*tstr) {
     case '?':
       /* Single character match.  Return false if at
@@ -130,6 +142,7 @@ quick_wild_new(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
       if (!*dstr)
         return 0;
       dstr++;
+      SKIP_ANSI(dstr);
     }
     tstr++;
   }
@@ -147,6 +160,7 @@ quick_wild_new(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
     if (EQUAL(cs, *dstr, *tstr) && quick_wild_new(tstr + 1, dstr + 1, cs))
       return 1;
     dstr++;
+    SKIP_ANSI(dstr);
   }
   return 0;
 }
@@ -164,8 +178,8 @@ quick_wild_new(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
  * \retval 1 dstr matches the tstr pattern.
  * \retval 0 dstr does not match the tstr pattern.
  */
-int
-atr_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
+bool
+atr_wild(const char *restrict tstr, const char *restrict dstr)
 {
   int starcount;
 
@@ -173,6 +187,7 @@ atr_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
     return !strchr(dstr, '`');
 
   while (*tstr != '*') {
+    SKIP_ANSI(dstr);
     switch (*tstr) {
     case '?':
       /* Single character match.  Return false if at
@@ -217,6 +232,7 @@ atr_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
       if (!*dstr || *dstr == '`')
         return 0;
       dstr++;
+      SKIP_ANSI(dstr);
       starcount = 0;
     } else
       starcount++;
@@ -237,6 +253,7 @@ atr_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
       if (*dstr != '`' && atr_wild(tstr + 1, dstr + 1))
         return 1;
       dstr++;
+      SKIP_ANSI(dstr);
     }
   } else {
     /* Skip over a backslash in the pattern string if it is there. */
@@ -254,6 +271,7 @@ atr_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
       if (starcount < 2 && *dstr == '`')
         return 0;
       dstr++;
+      SKIP_ANSI(dstr);
     }
   }
   return 0;
@@ -268,14 +286,15 @@ atr_wild(const char *RESTRICT tstr, const char *RESTRICT dstr)
  * Side Effect: this routine modifies the 'wnxt' global variable,
  * and what it points to.
  */
-static int
-wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
-      char **wbuf, int *len, int cs, char **ary, int max)
+static bool
+wild1(const char *restrict tstr, const char *restrict dstr, int arg,
+      char **wbuf, ssize_t * len, bool cs, char **ary, size_t max)
 {
   const char *datapos;
   int argpos, numextra;
 
   while (*tstr != '*') {
+    SKIP_ANSI(dstr);
     switch (*tstr) {
     case '?':
       /* Single character match.  Return false if at
@@ -317,7 +336,7 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
 
   /* If at end of pattern, slurp the rest, and leave. */
   if (!tstr[1]) {
-    int tlen;
+    ssize_t tlen;
     tlen = strlen(dstr);
     if (tlen < *len) {
       ary[arg] = *wbuf;
@@ -355,6 +374,7 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
           *(*wbuf)++ = '\0';
           *len -= 2;
         }
+        SKIP_ANSI(datapos);
 
         /* Jump to the fast routine if we can. */
         if (argpos >= (int) max || *len < 1)
@@ -372,6 +392,7 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
         return 0;
       tstr++;
       dstr++;
+      SKIP_ANSI(dstr);
       arg++;
       numextra++;
     }
@@ -384,10 +405,12 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
   /* Check for possible matches.  This loop terminates either at
    * end of data (resulting in failure), or at a successful match.
    */
-  if (!*tstr)
-    while (*dstr)
+  if (!*tstr) {
+    while (*dstr) {
       dstr++;
-  else {
+      SKIP_ANSI(dstr);
+    }
+  } else {
     while (1) {
       if (EQUAL(cs, *dstr, *tstr) &&
           ((arg < (int) max) ? wild1(tstr, dstr, arg, wbuf, len, cs, ary, max)
@@ -396,6 +419,7 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
       if (!*dstr)
         return 0;
       dstr++;
+      SKIP_ANSI(dstr);
     }
   }
 
@@ -403,7 +427,7 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
    * First do the '*'...
    */
   {
-    int datalen;
+    ssize_t datalen;
     datalen = (dstr - datapos) - numextra;
     if (datalen + 1 <= *len) {
       ary[argpos++] = *wbuf;
@@ -412,6 +436,7 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
       *(*wbuf)++ = '\0';
       *len -= datalen + 1;
       datapos = dstr - numextra;
+      SKIP_ANSI(datapos);
     }
   }
 
@@ -426,6 +451,7 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
       *len -= 2;
       numextra--;
     }
+    SKIP_ANSI(datapos);
   }
 
   /* It's done! */
@@ -441,13 +467,14 @@ wild1(const char *RESTRICT tstr, const char *RESTRICT dstr, int arg,
  *
  * Side Effect: this routine modifies the 'wnxt' global variable.
  */
-static int
-wild(const char *RESTRICT s, const char *RESTRICT d, int p, int cs,
-     char **ary, int max, char *buffer, int len)
+static bool
+wild(const char *restrict s, const char *restrict d, int p, bool cs,
+     char **ary, size_t max, char *buffer, ssize_t len)
 {
   /* Do fast match to see if pattern matches. If yes, do it again,
      remembering this time.. */
   while ((*s != '*') && (*s != '?')) {
+    SKIP_ANSI(d);
     if (*s == '\\')
       s++;
     if (NOTEQUAL(cs, *d, *s))
@@ -467,15 +494,15 @@ wild(const char *RESTRICT s, const char *RESTRICT d, int p, int cs,
 }
 
 /** Wildcard match, possibly case-sensitive, and remember the wild data
- *  in matches, storing them in data.
+ *  in matches, using the data buffer to store them.
  *
  * This routine will cause crashes if fed NULLs instead of strings.
  *
  * \param s pattern to match against.
  * \param d string to check.
  * \param cs if 1, case-sensitive; if 0, case-insensitive.
- * \param ary An array to store the grabs in
- * \param max Number of elements ary can hold
+ * \param matches An array to store the grabs in
+ * \param nmatches Number of elements ary can hold
  * \param data Buffer used to hold the matches. The elements of ary
  *    are set to pointers into this  buffer.
  * \param len The number of bytes in data. Twice the length of d should
@@ -483,11 +510,11 @@ wild(const char *RESTRICT s, const char *RESTRICT d, int p, int cs,
  * \retval 1 d matches s.
  * \retval 0 d doesn't match s.
  */
-int
-wild_match_case_r(const char *RESTRICT s, const char *RESTRICT d, int cs,
-                  char **matches, int nmatches, char *data, int len)
+bool
+wild_match_case_r(const char *restrict s, const char *restrict d, bool cs,
+                  char **matches, size_t nmatches, char *data, ssize_t len)
 {
-  int n;
+  size_t n;
 
   for (n = 0; n < nmatches; n++)
     matches[n] = NULL;
@@ -510,12 +537,13 @@ wild_match_case_r(const char *RESTRICT s, const char *RESTRICT d, int cs,
  * \retval 1 d matches s
  * \retval 0 d doesn't match s
  */
-int
-regexp_match_case_r(const char *RESTRICT s, const char *RESTRICT val, int cs,
-                    char **matches, int nmatches, char *data, int len)
+bool
+regexp_match_case_r(const char *restrict s, const char *restrict val, bool cs,
+                    char **matches, size_t nmatches, char *data, ssize_t len)
 {
   pcre *re;
-  int i;
+  pcre_extra *extra;
+  size_t i;
   const char *errptr;
   const char *d;
   size_t delenn;
@@ -537,11 +565,12 @@ regexp_match_case_r(const char *RESTRICT s, const char *RESTRICT val, int cs,
   }
   add_check("pcre");
   d = remove_markup(val, &delenn);
+  extra = default_match_limit();
   /*
    * Now we try to match the pattern. The relevant fields will
    * automatically be filled in by this.
    */
-  if ((subpatterns = pcre_exec(re, NULL, d, delenn - 1, 0, 0, offsets, 99))
+  if ((subpatterns = pcre_exec(re, extra, d, delenn - 1, 0, 0, offsets, 99))
       < 0) {
     mush_free(re, "pcre");
     return 0;
@@ -559,8 +588,8 @@ regexp_match_case_r(const char *RESTRICT s, const char *RESTRICT val, int cs,
    * with other languages.
    */
 
-  for (i = 0; i < nmatches && i < subpatterns && len > 1; i++) {
-    int sublen;
+  for (i = 0; i < nmatches && (int) i < subpatterns && (size_t) len > i; i++) {
+    ssize_t sublen;
 
     sublen = pcre_copy_substring(d, offsets, subpatterns, (int) i, data, len);
 
@@ -587,10 +616,11 @@ regexp_match_case_r(const char *RESTRICT s, const char *RESTRICT val, int cs,
  * \retval 1 d matches s.
  * \retval 0 d doesn't match s.
  */
-int
-quick_regexp_match(const char *RESTRICT s, const char *RESTRICT d, int cs)
+bool
+quick_regexp_match(const char *restrict s, const char *restrict d, bool cs)
 {
   pcre *re;
+  pcre_extra *extra;
   const char *sptr;
   size_t slen;
   const char *errptr;
@@ -615,17 +645,38 @@ quick_regexp_match(const char *RESTRICT s, const char *RESTRICT d, int cs)
   }
   add_check("pcre");
   sptr = remove_markup(d, &slen);
+  extra = default_match_limit();
   /*
    * Now we try to match the pattern. The relevant fields will
    * automatically be filled in by this.
    */
-  r = pcre_exec(re, NULL, sptr, slen - 1, 0, 0, offsets, 99);
+  r = pcre_exec(re, extra, sptr, slen - 1, 0, 0, offsets, 99);
 
   mush_free(re, "pcre");
 
   return r >= 0;
 }
 
+/** Regexp match of a pre-compiled regexp, with no memory.
+ * \param re the regular expression
+ * \param subj the string to match against.
+ * \return true or false
+ */
+bool
+qcomp_regexp_match(const pcre * re, const char *subj)
+{
+  int len;
+  int offsets[99];
+  pcre_extra *extra;
+
+  if (!re || !subj)
+    return false;
+
+  len = strlen(subj);
+  extra = default_match_limit();
+  return pcre_exec(re, extra, subj, len, 0, 0, offsets, 99) >= 0;
+}
+
 
 /** Either an order comparison or a wildcard match with no memory.
  *
@@ -636,8 +687,8 @@ quick_regexp_match(const char *RESTRICT s, const char *RESTRICT d, int cs)
  * \retval 1 d matches s.
  * \retval 0 d doesn't match s.
  */
-int
-local_wild_match_case(const char *RESTRICT s, const char *RESTRICT d, int cs)
+bool
+local_wild_match_case(const char *restrict s, const char *restrict d, bool cs)
 {
   if (s && *s) {
     switch (*s) {
@@ -666,7 +717,7 @@ local_wild_match_case(const char *RESTRICT s, const char *RESTRICT d, int cs)
  * \retval 1 s contains a * or ?
  * \retval 0 s does not contain a * or ?
  */
-int
+bool
 wildcard(const char *s)
 {
   if (strchr(s, '*') || strchr(s, '?'))
@@ -674,8 +725,8 @@ wildcard(const char *s)
   return 0;
 }
 
-static int
-check_literals(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
+static bool
+check_literals(const char *restrict tstr, const char *restrict dstr, bool cs)
 {
   /* Every literal string in tstr must appear, in order, in dstr,
    * or no match can happen. That is, tstr is the pattern and dstr
@@ -685,7 +736,10 @@ check_literals(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
   char dbuf1[BUFFER_LEN];
   const char delims[] = "?*";
   char *sp, *dp;
-  mush_strncpy(dbuf1, dstr, BUFFER_LEN);
+  size_t len = 0;
+
+  dp = remove_markup(dstr, &len);
+  memcpy(dbuf1, dp, len);
   mush_strncpy(tbuf1, strip_backslashes(tstr), BUFFER_LEN);
   if (!cs) {
     upcasestr(tbuf1);
@@ -694,7 +748,7 @@ check_literals(const char *RESTRICT tstr, const char *RESTRICT dstr, int cs)
   dp = dbuf1;
   sp = strtok(tbuf1, delims);
   while (sp) {
-    if (!dp)
+    if (!dp || !*dp)
       return 0;
     if (!(dp = strstr(dp, sp)))
       return 0;
