@@ -16,6 +16,9 @@
 #include <math.h>
 #ifdef I_SYS_TIME
 #include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <time.h>
+#endif
 #else
 #include <time.h>
 #endif
@@ -74,13 +77,16 @@ struct search_spec {
   int start;  /**< Limited results: start at this one. */
   int count;  /**< Limited results: return this many */
   int end;    /**< Limited results: return until this one.*/
+  boolexp lock;  /**< Boolexp to check against the objects. */
+  char cmdstring[BUFFER_LEN];
+  char listenstring[BUFFER_LEN];
 };
 
 static int tport_dest_ok(dbref player, dbref victim, dbref dest);
 static int tport_control_ok(dbref player, dbref victim, dbref loc);
 static int mem_usage(dbref thing);
 static int raw_search(dbref player, const char *owner, int nargs,
-                      const char **args, dbref **result, PE_Info * pe_info);
+                      const char **args, dbref **result, PE_Info *pe_info);
 static int fill_search_spec(dbref player, const char *owner, int nargs,
                             const char **args, struct search_spec *spec);
 
@@ -98,10 +104,12 @@ extern char errlog[BUFFER_LEN];
  * \param creator the enactor.
  * \param player_name name of player to create.
  * \param player_password password for player.
+ * \param try_dbref if non-empty, the garbage object to use for the new player.
  * \return dbref of created player object, or NOTHING if failure.
  */
 dbref
-do_pcreate(dbref creator, const char *player_name, const char *player_password)
+do_pcreate(dbref creator, const char *player_name, const char *player_password,
+           const char *try_dbref)
 {
   dbref player;
 
@@ -111,7 +119,13 @@ do_pcreate(dbref creator, const char *player_name, const char *player_password)
   }
   if (!can_pay_fees(creator, 0))
     return NOTHING;
-  player = create_player(player_name, player_password, "None", "None");
+
+  if (try_dbref && *try_dbref)
+    player = parse_dbref(try_dbref);
+  else
+    player = NOTHING;
+
+  player = create_player(player_name, player_password, "None", "None", player);
   if (player == NOTHING) {
     notify_format(creator, T("Failure creating '%s' (bad name)"), player_name);
     return NOTHING;
@@ -196,7 +210,7 @@ do_quota(dbref player, const char *arg1, const char *arg2, int set_q)
   if (adjust)
     limit = owned + get_current_quota(who) + atoi(arg2);
   else
-    limit = parse_integer(arg2);
+    limit = atoi(arg2);
   if (limit < owned)            /* always have enough quota for your objects */
     limit = owned;
 
@@ -446,11 +460,12 @@ do_teleport(dbref player, const char *arg1, const char *arg2, int silent,
     if (IsPlayer(destination) && Tel_Anywhere(player) && IsPlayer(victim)
         && !inside) {
       if (!silent && loc != Location(destination))
-        did_it(victim, victim, NULL, NULL, "OXTPORT", NULL, NULL, loc);
+        did_it_with(victim, victim, NULL, NULL, "OXTPORT", NULL, NULL, loc,
+                    player, NOTHING, NA_INTER_HEAR);
       safe_tel(victim, Location(destination), silent);
       if (!silent && loc != Location(destination))
-        did_it(victim, victim, "TPORT", NULL, "OTPORT", NULL, "ATPORT",
-               Location(destination));
+        did_it_with(victim, victim, "TPORT", NULL, "OTPORT", NULL, "ATPORT",
+                    Location(destination), player, loc, NA_INTER_HEAR);
       return;
     }
     /* check needed for NOTHING. Especially important for unlinked exits */
@@ -534,11 +549,12 @@ do_teleport(dbref player, const char *arg1, const char *arg2, int silent,
               (destination == Owner(victim)) ||
               (!Fixed(Owner(victim)) && !Fixed(player)))) {
         if (!silent && loc != destination)
-          did_it(victim, victim, NULL, NULL, "OXTPORT", NULL, NULL, loc);
+          did_it_with(victim, victim, NULL, NULL, "OXTPORT", NULL, NULL, loc,
+                      player, NOTHING, NA_INTER_HEAR);
         safe_tel(victim, destination, silent);
         if (!silent && loc != destination)
-          did_it(victim, victim, "TPORT", NULL, "OTPORT", NULL, "ATPORT",
-                 destination);
+          did_it_with(victim, victim, "TPORT", NULL, "OTPORT", NULL, "ATPORT",
+                      destination, player, loc, NA_INTER_HEAR);
         if ((victim != player) && !(Puppet(victim) &&
                                     (Owner(victim) == Owner(player)))) {
           if (!Quiet(player) && !(Quiet(victim) && (Owner(victim) == player)))
@@ -583,7 +599,7 @@ do_force(dbref player, const char *what, char *command)
   int j;
 
   if ((victim = match_controlled(player, what)) == NOTHING) {
-    notify(player, "Sorry.");
+    notify(player, T("Sorry."));
     return;
   }
   if (options.log_forces) {
@@ -1024,7 +1040,6 @@ do_kick(dbref player, const char *num)
 void
 do_debug_examine(dbref player, const char *name)
 {
-  MAIL *mp;
   dbref thing;
 
   if (!Hasprivs(player)) {
@@ -1048,8 +1063,7 @@ do_debug_examine(dbref player, const char *name)
 
   switch (Typeof(thing)) {
   case TYPE_PLAYER:
-    mp = desc_mail(thing);
-    notify_format(player, T("First mail sender: %d"), mp ? mp->from : NOTHING);
+    break;
   case TYPE_THING:
     notify_format(player, "Location: %d", Location(thing));
     notify_format(player, "Home: %d", Home(thing));
@@ -1200,10 +1214,10 @@ do_search(dbref player, const char *arg1, char **arg3)
     int nthings = 0, nexits = 0, nrooms = 0, nplayers = 0;
     dbref *things, *exits, *rooms, *players;
 
-    things = (dbref *) mush_malloc(sizeof(dbref) * nresults, "dbref_list");
-    exits = (dbref *) mush_malloc(sizeof(dbref) * nresults, "dbref_list");
-    rooms = (dbref *) mush_malloc(sizeof(dbref) * nresults, "dbref_list");
-    players = (dbref *) mush_malloc(sizeof(dbref) * nresults, "dbref_list");
+    things = mush_calloc(nresults, sizeof(dbref), "dbref_list");
+    exits = mush_calloc(nresults, sizeof(dbref), "dbref_list");
+    rooms = mush_calloc(nresults, sizeof(dbref), "dbref_list");
+    players = mush_calloc(nresults, sizeof(dbref), "dbref_list");
 
     for (n = 0; n < nresults; n++) {
       switch (Typeof(results[n])) {
@@ -1226,10 +1240,11 @@ do_search(dbref player, const char *arg1, char **arg3)
     }
 
     if (nrooms) {
-      notify(player, "\nROOMS:");
+      notify(player, T("\nROOMS:"));
       for (n = 0; n < nrooms; n++) {
         tbp = tbuf;
-        safe_format(tbuf, &tbp, "%s [owner: ", object_header(player, rooms[n]));
+        safe_format(tbuf, &tbp, T("%s [owner: "),
+                    object_header(player, rooms[n]));
         safe_str(object_header(player, Owner(rooms[n])), tbuf, &tbp);
         safe_chr(']', tbuf, &tbp);
         *tbp = '\0';
@@ -1240,7 +1255,7 @@ do_search(dbref player, const char *arg1, char **arg3)
     if (nexits) {
       dbref from, to;
 
-      notify(player, "\nEXITS:");
+      notify(player, T("\nEXITS:"));
       for (n = 0; n < nexits; n++) {
         tbp = tbuf;
         if (Source(exits[n]) == NOTHING)
@@ -1248,11 +1263,12 @@ do_search(dbref player, const char *arg1, char **arg3)
         else
           from = Source(exits[n]);
         to = Destination(exits[n]);
-        safe_format(tbuf, &tbp, "%s [from ", object_header(player, exits[n]));
-        safe_str((from == NOTHING) ? "NOWHERE" : object_header(player, from),
+        safe_format(tbuf, &tbp, T("%s [from "),
+                    object_header(player, exits[n]));
+        safe_str((from == NOTHING) ? T("NOWHERE") : object_header(player, from),
                  tbuf, &tbp);
-        safe_str(" to ", tbuf, &tbp);
-        safe_str((to == NOTHING) ? "NOWHERE" : object_header(player, to),
+        safe_str(T(" to "), tbuf, &tbp);
+        safe_str((to == NOTHING) ? T("NOWHERE") : object_header(player, to),
                  tbuf, &tbp);
         safe_chr(']', tbuf, &tbp);
         *tbp = '\0';
@@ -1261,10 +1277,10 @@ do_search(dbref player, const char *arg1, char **arg3)
     }
 
     if (nthings) {
-      notify(player, "\nTHINGS:");
+      notify(player, T("\nTHINGS:"));
       for (n = 0; n < nthings; n++) {
         tbp = tbuf;
-        safe_format(tbuf, &tbp, "%s [owner: ",
+        safe_format(tbuf, &tbp, T("%s [owner: "),
                     object_header(player, things[n]));
         safe_str(object_header(player, Owner(things[n])), tbuf, &tbp);
         safe_chr(']', tbuf, &tbp);
@@ -1275,7 +1291,7 @@ do_search(dbref player, const char *arg1, char **arg3)
 
     if (nplayers) {
       int is_wizard = Search_All(player) || See_All(player);
-      notify(player, "\nPLAYERS:");
+      notify(player, T("\nPLAYERS:"));
       for (n = 0; n < nplayers; n++) {
         tbp = tbuf;
         safe_str(object_header(player, players[n]), tbuf, &tbp);
@@ -1293,10 +1309,10 @@ do_search(dbref player, const char *arg1, char **arg3)
                   T
                   ("Totals: Rooms...%d  Exits...%d  Things...%d  Players...%d"),
                   nrooms, nexits, nthings, nplayers);
-    mush_free((Malloc_t) rooms, "dbref_list");
-    mush_free((Malloc_t) exits, "dbref_list");
-    mush_free((Malloc_t) things, "dbref_list");
-    mush_free((Malloc_t) players, "dbref_list");
+    mush_free(rooms, "dbref_list");
+    mush_free(exits, "dbref_list");
+    mush_free(things, "dbref_list");
+    mush_free(players, "dbref_list");
   }
   if (results)
     mush_free(results, "search_results");
@@ -1443,7 +1459,7 @@ do_sitelock(dbref player, const char *site, const char *opts, const char *who,
     return;
   }
   if (opts && *opts) {
-    int can, cant;
+    uint32_t can, cant;
     dbref whod = AMBIGUOUS;
     /* Options form of the command. */
     if (!site || !*site) {
@@ -1463,20 +1479,23 @@ do_sitelock(dbref player, const char *site, const char *opts, const char *who,
       }
     }
 
-    add_access_sitelock(player, site, whod, can, cant);
-    write_access_file();
-    if (whod != AMBIGUOUS) {
-      notify_format(player,
-                    T("Site %s access options for %s(%s) set to %s"),
-                    site, Name(whod), unparse_dbref(whod), opts);
-      do_log(LT_WIZ, player, NOTHING,
-             T("*** SITELOCK *** %s for %s(%s) --> %s"), site,
-             Name(whod), unparse_dbref(whod), opts);
-    } else {
-      notify_format(player, T("Site %s access options set to %s"), site, opts);
-      do_log(LT_WIZ, player, NOTHING, "*** SITELOCK *** %s --> %s", site, opts);
+    if (add_access_sitelock(player, site, whod, can, cant)) {
+      write_access_file();
+      if (whod != AMBIGUOUS) {
+        notify_format(player,
+                      T("Site %s access options for %s(%s) set to %s"),
+                      site, Name(whod), unparse_dbref(whod), opts);
+        do_log(LT_WIZ, player, NOTHING,
+               T("*** SITELOCK *** %s for %s(%s) --> %s"), site,
+               Name(whod), unparse_dbref(whod), opts);
+      } else {
+        notify_format(player, T("Site %s access options set to %s"), site,
+                      opts);
+        do_log(LT_WIZ, player, NOTHING, "*** SITELOCK *** %s --> %s", site,
+               opts);
+      }
+      return;
     }
-    return;
   } else {
     /* Backward-compatible non-options form of the command,
      * or @sitelock/name
@@ -1487,16 +1506,18 @@ do_sitelock(dbref player, const char *site, const char *opts, const char *who,
       do_list_access(player);
       return;
     case SITELOCK_ADD:
-      add_access_sitelock(player, site, AMBIGUOUS, 0, ACS_CREATE);
-      write_access_file();
-      notify_format(player, T("Site %s locked"), site);
-      do_log(LT_WIZ, player, NOTHING, "*** SITELOCK *** %s", site);
+      if (add_access_sitelock(player, site, AMBIGUOUS, 0, ACS_CREATE)) {
+        write_access_file();
+        notify_format(player, T("Site %s locked"), site);
+        do_log(LT_WIZ, player, NOTHING, "*** SITELOCK *** %s", site);
+      }
       break;
     case SITELOCK_BAN:
-      add_access_sitelock(player, site, AMBIGUOUS, 0, ACS_DEFAULT);
-      write_access_file();
-      notify_format(player, T("Site %s banned"), site);
-      do_log(LT_WIZ, player, NOTHING, "*** SITELOCK *** %s", site);
+      if (add_access_sitelock(player, site, AMBIGUOUS, 0, ACS_DEFAULT)) {
+        write_access_file();
+        notify_format(player, T("Site %s banned"), site);
+        do_log(LT_WIZ, player, NOTHING, "*** SITELOCK *** %s", site);
+      }
       break;
     case SITELOCK_CHECK:{
         struct access *ap;
@@ -1725,65 +1746,6 @@ FUNCTION(fun_playermem)
   safe_integer(tot, buff, bp);
 }
 
-
-/** Reboot the game without disconnecting players.
- * \verbatim
- * This implements @shutdown/reboot, which performs a dump, saves
- * information about which player is associated with which socket,
- * and then re-execs the mush process without closing the sockets.
- * \endverbatim
- * \param player the enactor.
- * \param flag if 0, normal dump; if 1, paranoid dump.
- */
-void
-do_reboot(dbref player, int flag)
-{
-  if (player == NOTHING) {
-    flag_broadcast(0, 0,
-                   T
-                   ("GAME: Reboot w/o disconnect from game account, please wait."));
-  } else {
-    flag_broadcast(0, 0,
-                   T
-                   ("GAME: Reboot w/o disconnect by %s, please wait."),
-                   Name(Owner(player)));
-  }
-  if (flag) {
-    globals.paranoid_dump = 1;
-    globals.paranoid_checkpt = db_top / 5;
-    if (globals.paranoid_checkpt < 1)
-      globals.paranoid_checkpt = 1;
-  }
-#ifdef HAS_OPENSSL
-  close_ssl_connections();
-#endif
-  sql_shutdown();
-  shutdown_queues();
-  fork_and_dump(0);
-#ifndef PROFILING
-#ifndef WIN32
-  /* Some broken libcs appear to retain the itimer across exec!
-   * So we make sure that if we get a SIGPROF in our next incarnation,
-   * we ignore it until our proper handler is set up.
-   */
-  ignore_signal(SIGPROF);
-#endif
-#endif
-  dump_reboot_db();
-#ifdef INFO_SLAVE
-  kill_info_slave();
-#endif
-  local_shutdown();
-  end_all_logs();
-#ifndef WIN32
-  execl("netmush", "netmush", confname, NULL);
-#else
-  execl("pennmush.exe", "pennmush.exe", "/run", NULL);
-#endif                          /* WIN32 */
-  exit(1);                      /* Shouldn't ever get here, but just in case... */
-}
-
-
 static int
 fill_search_spec(dbref player, const char *owner, int nargs, const char **args,
                  struct search_spec *spec)
@@ -1803,6 +1765,9 @@ fill_search_spec(dbref player, const char *owner, int nargs, const char **args,
   spec->high = db_top - 1;
   spec->start = 1;              /* 1-indexed */
   spec->count = 0;
+  spec->lock = TRUE_BOOLEXP;
+  strcpy(spec->cmdstring, "");
+  strcpy(spec->listenstring, "");
 
   is_wiz = Search_All(player) || See_All(player);
   /* set limits on who we search */
@@ -1953,8 +1918,14 @@ fill_search_spec(dbref player, const char *owner, int nargs, const char **args,
         notify(player, T("Unknown zone."));
         return -1;
       }
+    } else if (string_prefix("elock", class)) {
+      spec->lock = parse_boolexp(player, restriction, "Search");
     } else if (string_prefix("eval", class)) {
       strcpy(spec->eval, restriction);
+    } else if (string_prefix("command", class)) {
+      strcpy(spec->cmdstring, restriction);
+    } else if (string_prefix("listen", class)) {
+      strcpy(spec->listenstring, restriction);
     } else if (string_prefix("ethings", class) ||
                string_prefix("eobjects", class)) {
       strcpy(spec->eval, restriction);
@@ -2002,7 +1973,7 @@ fill_search_spec(dbref player, const char *owner, int nargs, const char **args,
 /* Does the actual searching */
 static int
 raw_search(dbref player, const char *owner, int nargs, const char **args,
-           dbref **result, PE_Info * pe_info)
+           dbref **result, PE_Info *pe_info)
 {
   size_t result_size;
   size_t nresults = 0;
@@ -2010,6 +1981,9 @@ raw_search(dbref player, const char *owner, int nargs, const char **args,
   int is_wiz;
   struct search_spec spec;
   int count = 0;
+  int ret = 0;
+  ATTR *a;
+  char lbuff[BUFFER_LEN];
 
   /* make sure player has money to do the search */
   if (!payfor(player, FIND_COST)) {
@@ -2020,6 +1994,8 @@ raw_search(dbref player, const char *owner, int nargs, const char **args,
 
   if (fill_search_spec(player, owner, nargs, args, &spec) < 0) {
     giveto(player, FIND_COST);
+    if (spec.lock != TRUE_BOOLEXP)
+      free_boolexp(spec.lock);
     return -1;
   }
 
@@ -2030,12 +2006,13 @@ raw_search(dbref player, const char *owner, int nargs, const char **args,
         (ZMaster(spec.owner) && eval_lock(player, spec.owner, Zone_Lock)))) {
     giveto(player, FIND_COST);
     notify(player, T("You need a search warrant to do that."));
+    if (spec.lock != TRUE_BOOLEXP)
+      free_boolexp(spec.lock);
     return -1;
   }
 
   result_size = (db_top / 4) + 1;
-  *result =
-    (dbref *) mush_malloc(sizeof(dbref) * result_size, "search_results");
+  *result = mush_calloc(result_size, sizeof(dbref), "search_results");
   if (!*result)
     mush_panic(T("Couldn't allocate memory in search!"));
 
@@ -2059,6 +2036,29 @@ raw_search(dbref player, const char *owner, int nargs, const char **args,
     if (*spec.powers
         && !flaglist_check_long("POWER", player, n, spec.powers, 1))
       continue;
+    if (spec.lock != TRUE_BOOLEXP && !eval_boolexp(n, spec.lock, player))
+      continue;
+    if (spec.cmdstring[0] &&
+        !atr_comm_match(n, player, '$', ':', spec.cmdstring, 1, 0,
+                        NULL, NULL, NULL))
+      continue;
+    if (spec.listenstring[0]) {
+      ret = 0;
+      /* do @listen stuff */
+      a = atr_get_noparent(n, "LISTEN");
+      if (a) {
+        strcpy(lbuff, atr_value(a));
+        ret = AF_Regexp(a)
+          ? regexp_match_case_r(lbuff, spec.listenstring,
+                                AF_Case(a), NULL, 0, NULL, 0)
+          : wild_match_case_r(lbuff, spec.listenstring,
+                              AF_Case(a), NULL, 0, NULL, 0);
+      }
+      if (!ret &&
+          !atr_comm_match(n, player, '^', ':', spec.listenstring, 1, 0,
+                          NULL, NULL, NULL))
+        continue;
+    }
     if (*spec.eval) {
       char *ebuf1;
       const char *ebuf2;
@@ -2088,8 +2088,7 @@ raw_search(dbref player, const char *owner, int nargs, const char **args,
     if (nresults >= result_size) {
       dbref *newresults;
       result_size *= 2;
-      newresults =
-        (dbref *) realloc((Malloc_t) *result, sizeof(dbref) * result_size);
+      newresults = (dbref *) realloc(*result, sizeof(dbref) * result_size);
       if (!newresults)
         mush_panic(T("Couldn't reallocate memory in search!"));
       *result = newresults;
@@ -2098,5 +2097,7 @@ raw_search(dbref player, const char *owner, int nargs, const char **args,
     (*result)[nresults++] = (dbref) n;
   }
 
+  if (spec.lock != TRUE_BOOLEXP)
+    free_boolexp(spec.lock);
   return (int) nresults;
 }
