@@ -1,10 +1,4 @@
-use PennMUSH;
-use MUSHConnection;
-use TestHarness;
-
-$mush = PennMUSH->new();
-$god = $mush->loginGod();
-
+run tests:
 test('firstof.1', $god, 'think firstof(0,0,2)', '2');
 test('firstof.2', $god, 'think firstof(2,0,0)', '2');
 test('firstof.3', $god, 'think firstof(0,0,0)', '0');
